@@ -11,10 +11,12 @@
 #include <wx/file.h>
 #include <wx/filename.h>
 
+#ifndef __WXMSW__
 #include <idna.h>
 extern "C" {
 #include <idn-free.h>
 }
+#endif
 
 #include <errno.h>
 
@@ -32,7 +34,7 @@ BEGIN_EVENT_TABLE(CControlSocket, wxEvtHandler)
 	EVT_COMMAND(wxID_ANY, fzOBTAINLOCK, CControlSocket::OnObtainLock)
 END_EVENT_TABLE()
 
-COpData::COpData(enum Command op_Id)
+COpData::COpData(Command op_Id)
 	: opId(op_Id)
 {
 	opState = 0;
@@ -79,13 +81,13 @@ CControlSocket::~CControlSocket()
 
 int CControlSocket::Disconnect()
 {
-	LogMessage(Status, _("Disconnected from server"));
+	LogMessage(MessageType::Status, _("Disconnected from server"));
 
 	DoClose();
 	return FZ_REPLY_OK;
 }
 
-enum Command CControlSocket::GetCurrentCommandId() const
+Command CControlSocket::GetCurrentCommandId() const
 {
 	if (m_pCurOpData)
 		return m_pCurOpData->opId;
@@ -107,11 +109,11 @@ void CControlSocket::LogTransferResultMessage(int nErrorCode, CFileTransferOpDat
 		wxLongLong transferred = m_pTransferStatus->currentOffset - m_pTransferStatus->startOffset;
 		wxString size = CSizeFormatBase::Format(m_pEngine->GetOptions(), transferred, true);
 
-		MessageType msgType = ::Error;
+		MessageType msgType = MessageType::Error;
 		wxString msg;
 		if (nErrorCode == FZ_REPLY_OK)
 		{
-			msgType = Status;
+			msgType = MessageType::Status;
 			msg = _("File transfer successful, transferred %s in %s");
 		}
 		else if ((nErrorCode & FZ_REPLY_CANCELED) == FZ_REPLY_CANCELED)
@@ -120,33 +122,33 @@ void CControlSocket::LogTransferResultMessage(int nErrorCode, CFileTransferOpDat
 			msg = _("Critical file transfer error after transferring %s in %s");
 		else
 			msg = _("File transfer failed after transferring %s in %s");
-		LogMessage(msgType, msg, size.c_str(), time.c_str());
+		LogMessage(msgType, msg, size, time);
 	}
 	else
 	{
 		if ((nErrorCode & FZ_REPLY_CANCELED) == FZ_REPLY_CANCELED)
-			LogMessage(::Error, _("File transfer aborted by user"));
+			LogMessage(MessageType::Error, _("File transfer aborted by user"));
 		else if (nErrorCode == FZ_REPLY_OK)
 		{
 			if (pData->transferInitiated)
-				LogMessage(Status, _("File transfer successful"));
+				LogMessage(MessageType::Status, _("File transfer successful"));
 			else
-				LogMessage(Status, _("File transfer skipped"));
+				LogMessage(MessageType::Status, _("File transfer skipped"));
 		}
 		else if ((nErrorCode & FZ_REPLY_CRITICALERROR) == FZ_REPLY_CRITICALERROR)
-			LogMessage(::Error, _("Critical file transfer error"));
+			LogMessage(MessageType::Error, _("Critical file transfer error"));
 		else
-			LogMessage(::Error, _("File transfer failed"));
+			LogMessage(MessageType::Error, _("File transfer failed"));
 	}
 }
 
 int CControlSocket::ResetOperation(int nErrorCode)
 {
-	LogMessage(Debug_Verbose, _T("CControlSocket::ResetOperation(%d)"), nErrorCode);
+	LogMessage(MessageType::Debug_Verbose, _T("CControlSocket::ResetOperation(%d)"), nErrorCode);
 
 	if (nErrorCode & FZ_REPLY_WOULDBLOCK)
 	{
-		LogMessage(::Debug_Warning, _T("ResetOperation with FZ_REPLY_WOULDBLOCK in nErrorCode (%d)"), nErrorCode);
+		LogMessage(MessageType::Debug_Warning, _T("ResetOperation with FZ_REPLY_WOULDBLOCK in nErrorCode (%d)"), nErrorCode);
 	}
 
 	if (m_pCurOpData && m_pCurOpData->holdsLock)
@@ -168,40 +170,44 @@ int CControlSocket::ResetOperation(int nErrorCode)
 			return ResetOperation(nErrorCode);
 	}
 
+	wxString prefix;
 	if ((nErrorCode & FZ_REPLY_CRITICALERROR) == FZ_REPLY_CRITICALERROR &&
-		(!m_pCurOpData || m_pCurOpData->opId != cmd_transfer))
+		(!m_pCurOpData || m_pCurOpData->opId != Command::transfer))
 	{
-		LogMessage(::Error, _("Critical error"));
+		prefix = _("Critical error:") + _T(" ");
 	}
 
 	if (m_pCurOpData)
 	{
-		const enum Command commandId = m_pCurOpData->opId;
+		const Command commandId = m_pCurOpData->opId;
 		switch (commandId)
 		{
-		case cmd_none:
+		case Command::none:
+			if( !prefix.empty() ) {
+				LogMessage(MessageType::Error, _("Critical error"));
+			}
 			break;
-		case cmd_connect:
+		case Command::connect:
 			if ((nErrorCode & FZ_REPLY_CANCELED) == FZ_REPLY_CANCELED)
-				LogMessage(::Error, _("Connection attempt interrupted by user"));
+				LogMessage(MessageType::Error, prefix + _("Connection attempt interrupted by user"));
 			else if (nErrorCode != FZ_REPLY_OK)
-				LogMessage(::Error, _("Could not connect to server"));
+				LogMessage(MessageType::Error, prefix + _("Could not connect to server"));
 			break;
-		case cmd_list:
+		case Command::list:
 			if ((nErrorCode & FZ_REPLY_CANCELED) == FZ_REPLY_CANCELED)
-				LogMessage(::Error, _("Directory listing aborted by user"));
+				LogMessage(MessageType::Error, prefix + _("Directory listing aborted by user"));
 			else if (nErrorCode != FZ_REPLY_OK)
-				LogMessage(::Error, _("Failed to retrieve directory listing"));
+				LogMessage(MessageType::Error, prefix + _("Failed to retrieve directory listing"));
 			else
-				LogMessage(Status, _("Directory listing successful"));
+				LogMessage(MessageType::Status, _("Directory listing successful"));
 			break;
-		case cmd_transfer:
+		case Command::transfer:
 			{
 				CFileTransferOpData *pData = static_cast<CFileTransferOpData *>(m_pCurOpData);
 				if (!pData->download && pData->transferInitiated)
 				{
 					if (!m_pCurrentServer)
-						LogMessage(__TFILE__, __LINE__, this, Debug_Warning, _T("m_pCurrentServer is 0"));
+						LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Warning, _T("m_pCurrentServer is 0"));
 					else
 					{
 						CDirectoryCache cache;
@@ -216,7 +222,7 @@ int CControlSocket::ResetOperation(int nErrorCode)
 			break;
 		default:
 			if ((nErrorCode & FZ_REPLY_CANCELED) == FZ_REPLY_CANCELED)
-				LogMessage(::Error, _("Interrupted by user"));
+				LogMessage(MessageType::Error, prefix + _("Interrupted by user"));
 			break;
 		}
 
@@ -228,9 +234,8 @@ int CControlSocket::ResetOperation(int nErrorCode)
 
 	SetWait(false);
 
-	if (m_invalidateCurrentPath)
-	{
-		m_CurrentPath.Clear();
+	if (m_invalidateCurrentPath) {
+		m_CurrentPath.clear();
 		m_invalidateCurrentPath = false;
 	}
 
@@ -239,7 +244,7 @@ int CControlSocket::ResetOperation(int nErrorCode)
 
 int CControlSocket::DoClose(int nErrorCode /*=FZ_REPLY_DISCONNECTED*/)
 {
-	LogMessage(Debug_Debug, _T("CControlSocket::DoClose(%d)"), nErrorCode);
+	LogMessage(MessageType::Debug_Debug, _T("CControlSocket::DoClose(%d)"), nErrorCode);
 	if (m_closed)
 	{
 		wxASSERT(!m_pCurOpData);
@@ -256,8 +261,26 @@ int CControlSocket::DoClose(int nErrorCode /*=FZ_REPLY_DISCONNECTED*/)
 	return nErrorCode;
 }
 
-wxString CControlSocket::ConvertDomainName(wxString domain)
+wxString CControlSocket::ConvertDomainName(wxString const& domain)
 {
+#ifdef __WXMSW__
+	int len = IdnToAscii(IDN_ALLOW_UNASSIGNED, domain, domain.size() + 1, 0, 0);
+	if( !len ) {
+		LogMessage(MessageType::Debug_Warning, _T("Could not convert domain name"));
+		return domain;
+	}
+
+	wchar_t* output = new wchar_t[len];
+	int res = IdnToAscii(IDN_ALLOW_UNASSIGNED, domain, domain.size() + 1, output, len);
+	if( !res ) {
+		LogMessage(MessageType::Debug_Warning, _T("Could not convert domain name"));
+		return domain;
+	}
+
+	wxString ret(output);
+	delete [] output;
+	return ret;
+#else
 	const wxWCharBuffer buffer = wxConvCurrent->cWX2WC(domain);
 
 	int len = 0;
@@ -272,7 +295,7 @@ wxString CControlSocket::ConvertDomainName(wxString domain)
 	if (idna_to_ascii_8z(utf8, &output, IDNA_ALLOW_UNASSIGNED))
 	{
 		delete [] utf8;
-		LogMessage(::Debug_Warning, _T("Could not convert domain name"));
+		LogMessage(MessageType::Debug_Warning, _T("Could not convert domain name"));
 		return domain;
 	}
 	delete [] utf8;
@@ -280,13 +303,14 @@ wxString CControlSocket::ConvertDomainName(wxString domain)
 	wxString result = wxConvCurrent->cMB2WX(output);
 	idn_free(output);
 	return result;
+#endif
 }
 
 void CControlSocket::Cancel()
 {
-	if (GetCurrentCommandId() != cmd_none)
+	if (GetCurrentCommandId() != Command::none)
 	{
-		if (GetCurrentCommandId() == cmd_connect)
+		if (GetCurrentCommandId() == Command::connect)
 			DoClose(FZ_REPLY_CANCELED);
 		else
 			ResetOperation(FZ_REPLY_CANCELED);
@@ -382,46 +406,40 @@ bool CControlSocket::ParsePwdReply(wxString reply, bool unquoted /*=false*/, con
 	{
 		int pos1 = reply.Find('"');
 		int pos2 = reply.Find('"', true);
-		if (pos1 == -1 || pos1 >= pos2)
-		{
+		if (pos1 == -1 || pos1 >= pos2) {
 			pos1 = reply.Find('\'');
 			pos2 = reply.Find('\'', true);
 
 			if (pos1 != -1 && pos1 < pos2)
-				LogMessage(__TFILE__, __LINE__, this, Debug_Info, _T("Broken server sending single-quoted path instead of double-quoted path."));
+				LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("Broken server sending single-quoted path instead of double-quoted path."));
 		}
-		if (pos1 == -1 || pos1 >= pos2)
-		{
-			LogMessage(__TFILE__, __LINE__, this, Debug_Info, _T("Broken server, no quoted path found in pwd reply, trying first token as path"));
+		if (pos1 == -1 || pos1 >= pos2) {
+			LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("Broken server, no quoted path found in pwd reply, trying first token as path"));
 			pos1 = reply.Find(' ');
-			if (pos1 != -1)
-			{
+			if (pos1 != -1) {
 				reply = reply.Mid(pos1 + 1);
 				pos2 = reply.Find(' ');
 				if (pos2 != -1)
 					reply = reply.Left(pos2);
 			}
 			else
-				reply = _T("");
+				reply.clear();
 		}
-		else
-		{
+		else {
 			reply = reply.Mid(pos1 + 1, pos2 - pos1 - 1);
 			reply.Replace(_T("\"\""), _T("\""));
 		}
 	}
 
 	m_CurrentPath.SetType(m_pCurrentServer->GetType());
-	if (reply == _T("") || !m_CurrentPath.SetPath(reply))
-	{
-		if (reply != _T(""))
-			LogMessage(::Error, _("Failed to parse returned path."));
+	if (reply.empty() || !m_CurrentPath.SetPath(reply)) {
+		if (reply.empty())
+			LogMessage(MessageType::Error, _("Server returned empty path."));
 		else
-			LogMessage(::Error, _("Server returned empty path."));
+			LogMessage(MessageType::Error, _("Failed to parse returned path."));
 
-		if (!defaultPath.IsEmpty())
-		{
-			LogMessage(Debug_Warning, _T("Assuming path is '%s'."), defaultPath.GetPath().c_str());
+		if (!defaultPath.empty()) {
+			LogMessage(MessageType::Debug_Warning, _T("Assuming path is '%s'."), defaultPath.GetPath());
 			m_CurrentPath = defaultPath;
 			return true;
 		}
@@ -435,7 +453,7 @@ int CControlSocket::CheckOverwriteFile()
 {
 	if (!m_pCurOpData)
 	{
-		LogMessage(__TFILE__, __LINE__, this, Debug_Info, _T("Empty m_pCurOpData"));
+		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("Empty m_pCurOpData"));
 		ResetOperation(FZ_REPLY_INTERNALERROR);
 		return FZ_REPLY_ERROR;
 	}
@@ -453,7 +471,7 @@ int CControlSocket::CheckOverwriteFile()
 	bool matchedCase;
 	CDirectoryCache cache;
 	CServerPath remotePath;
-	if (pData->tryAbsolutePath || m_CurrentPath.IsEmpty())
+	if (pData->tryAbsolutePath || m_CurrentPath.empty())
 		remotePath = pData->remotePath;
 	else
 		remotePath = m_CurrentPath;
@@ -486,25 +504,14 @@ int CControlSocket::CheckOverwriteFile()
 	else
 		pNotification->canResume = false;
 
-	wxStructStat buf;
-	int result;
-	result = wxStat(pData->localFile, &buf);
-	if (!result)
-	{
-		pNotification->localTime = wxDateTime(buf.st_mtime);
-		if (!pNotification->localTime.IsValid())
-			pNotification->localTime = wxDateTime(buf.st_ctime);
-	}
+	pNotification->localTime = CLocalFileSystem::GetModificationTime(pData->localFile);
 
 	if (pData->fileTime.IsValid())
 		pNotification->remoteTime = pData->fileTime;
 
-	if (found)
-	{
-		if (!pData->fileTime.IsValid())
-		{
-			if (entry.has_date())
-			{
+	if (found) {
+		if (!pData->fileTime.IsValid()) {
+			if (entry.has_date()) {
 				pNotification->remoteTime = entry.time;
 				pData->fileTime = entry.time;
 			}
@@ -517,7 +524,7 @@ int CControlSocket::CheckOverwriteFile()
 }
 
 CFileTransferOpData::CFileTransferOpData(bool is_download, const wxString& local_file, const wxString& remote_file, const CServerPath& remote_path) :
-	COpData(cmd_transfer),
+	COpData(Command::transfer),
 	localFile(local_file), remoteFile(remote_file), remotePath(remote_path),
 	download(is_download),
 	localFileSize(-1), remoteFileSize(-1),
@@ -544,7 +551,7 @@ wxString CControlSocket::ConvToLocal(const char* buffer)
 		// Fall back to local charset on error
 		if (m_pCurrentServer->GetEncodingType() != ENCODING_UTF8)
 		{
-			LogMessage(Status, _("Invalid character sequence received, disabling UTF-8. Select UTF-8 option in site manager to force UTF-8."));
+			LogMessage(MessageType::Status, _("Invalid character sequence received, disabling UTF-8. Select UTF-8 option in site manager to force UTF-8."));
 			m_useUTF8 = false;
 		}
 	}
@@ -562,7 +569,7 @@ wxString CControlSocket::ConvToLocal(const char* buffer)
 
 	wxCSConv conv(_T("ISO-8859-1"));
 	wxString str = conv.cMB2WX(buffer);
-	if (str == _T(""))
+	if (str.empty())
 		str = wxConvCurrent->cMB2WX(buffer);
 
 	return str;
@@ -576,21 +583,7 @@ wxChar* CControlSocket::ConvToLocalBuffer(const char* buffer, wxMBConv& conv)
 
 	wchar_t* unicode = new wchar_t[len + 1];
 	conv.MB2WC(unicode, buffer, len + 1);
-#if wxUSE_UNICODE
 	return unicode;
-#else
-	len = wxConvCurrent->WC2MB(0, unicode, 0);
-	if (!len)
-	{
-		delete [] unicode;
-		return 0;
-	}
-
-	wxChar* output = new wxChar[len + 1];
-	wxConvCurrent->WC2MB(output, unicode, len + 1);
-	delete [] unicode;
-	return output;
-#endif
 }
 
 wxChar* CControlSocket::ConvToLocalBuffer(const char* buffer)
@@ -604,7 +597,7 @@ wxChar* CControlSocket::ConvToLocalBuffer(const char* buffer)
 		// Fall back to local charset on error
 		if (m_pCurrentServer->GetEncodingType() != ENCODING_UTF8)
 		{
-			LogMessage(Status, _("Invalid character sequence received, disabling UTF-8. Select UTF-8 option in site manager to force UTF-8."));
+			LogMessage(MessageType::Status, _("Invalid character sequence received, disabling UTF-8. Select UTF-8 option in site manager to force UTF-8."));
 			m_useUTF8 = false;
 		}
 	}
@@ -617,13 +610,7 @@ wxChar* CControlSocket::ConvToLocalBuffer(const char* buffer)
 	}
 
 	// Fallback: Conversion using current locale
-#if wxUSE_UNICODE
 	wxChar* res = ConvToLocalBuffer(buffer, *wxConvCurrent);
-#else
-	// No conversion needed, just copy
-	wxChar* res = new wxChar[strlen(buffer) + 1];
-	strcpy(res, buffer);
-#endif
 
 	return res;
 }
@@ -632,30 +619,14 @@ wxCharBuffer CControlSocket::ConvToServer(const wxString& str, bool force_utf8 /
 {
 	if (m_useUTF8 || force_utf8)
 	{
-#if wxUSE_UNICODE
 		wxCharBuffer buffer = wxConvUTF8.cWX2MB(str);
-#else
-		wxWCharBuffer unicode = wxConvCurrent->cMB2WC(str);
-		wxCharBuffer buffer;
-		if (unicode)
-			 buffer = wxConvUTF8.cWC2MB(unicode);
-#endif
-
 		if (buffer || force_utf8)
 			return buffer;
 	}
 
 	if (m_pCSConv)
 	{
-#if wxUSE_UNICODE
 		wxCharBuffer buffer = m_pCSConv->cWX2MB(str);
-#else
-		wxWCharBuffer unicode = wxConvCurrent->cMB2WC(str);
-		wxCharBuffer buffer;
-		if (unicode)
-			 buffer = m_pCSConv->cWC2MB(unicode);
-#endif
-
 		if (buffer)
 			return buffer;
 	}
@@ -667,7 +638,7 @@ wxCharBuffer CControlSocket::ConvToServer(const wxString& str, bool force_utf8 /
 	return buffer;
 }
 
-void CControlSocket::OnTimer(wxTimerEvent& event)
+void CControlSocket::OnTimer(wxTimerEvent&)
 {
 	int timeout = m_pEngine->GetOptions()->GetOptionVal(OPTION_TIMEOUT);
 	if (!timeout)
@@ -681,7 +652,7 @@ void CControlSocket::OnTimer(wxTimerEvent& event)
 
 	if (m_stopWatch.Time() > (timeout * 1000))
 	{
-		LogMessage(::Error, _("Connection timed out"));
+		LogMessage(MessageType::Error, _("Connection timed out"));
 		DoClose(FZ_REPLY_TIMEOUT);
 		wxASSERT(!m_timer.IsRunning());
 	}
@@ -712,7 +683,7 @@ int CControlSocket::SendNextCommand()
 	return FZ_REPLY_ERROR;
 }
 
-int CControlSocket::ParseSubcommandResult(int prevResult)
+int CControlSocket::ParseSubcommandResult(int)
 {
 	ResetOperation(FZ_REPLY_INTERNALERROR);
 	return FZ_REPLY_ERROR;
@@ -835,14 +806,14 @@ void CControlSocket::UnlockCache()
 	// Find other instance waiting for the lock
 	if (!m_pCurrentServer)
 	{
-		LogMessage(Debug_Warning, _T("UnlockCache called with !m_pCurrentServer"));
+		LogMessage(MessageType::Debug_Warning, _T("UnlockCache called with !m_pCurrentServer"));
 		return;
 	}
 	for (std::list<t_lockInfo>::const_iterator iter = m_lockInfoList.begin(); iter != m_lockInfoList.end(); ++iter)
 	{
 		if (!iter->pControlSocket->m_pCurrentServer)
 		{
-			LogMessage(Debug_Warning, _T("UnlockCache found other instance with !m_pCurrentServer"));
+			LogMessage(MessageType::Debug_Warning, _T("UnlockCache found other instance with !m_pCurrentServer"));
 			continue;
 		}
 
@@ -895,7 +866,7 @@ enum CControlSocket::locking_reason CControlSocket::ObtainLockFromEvent()
 	return own->reason;
 }
 
-void CControlSocket::OnObtainLock(wxCommandEvent& event)
+void CControlSocket::OnObtainLock(wxCommandEvent&)
 {
 	if (ObtainLockFromEvent() == lock_unknown)
 		return;
@@ -916,8 +887,8 @@ bool CControlSocket::IsWaitingForLock()
 
 void CControlSocket::InvalidateCurrentWorkingDir(const CServerPath& path)
 {
-	wxASSERT(!path.IsEmpty());
-	if (m_CurrentPath.IsEmpty())
+	wxASSERT(!path.empty());
+	if (m_CurrentPath.empty())
 		return;
 
 	if (m_CurrentPath == path || path.IsParentOf(m_CurrentPath, false))
@@ -925,7 +896,7 @@ void CControlSocket::InvalidateCurrentWorkingDir(const CServerPath& path)
 		if (m_pCurOpData)
 			m_invalidateCurrentPath = true;
 		else
-			m_CurrentPath.Clear();
+			m_CurrentPath.clear();
 	}
 }
 
@@ -999,8 +970,8 @@ bool CRealControlSocket::Send(const char *buffer, int len)
 		{
 			if (error != EAGAIN)
 			{
-				LogMessage(::Error, _("Could not write to socket: %s"), CSocket::GetErrorDescription(error).c_str());
-				LogMessage(::Error, _("Disconnected from server"));
+				LogMessage(MessageType::Error, _("Could not write to socket: %s"), CSocket::GetErrorDescription(error));
+				LogMessage(MessageType::Error, _("Disconnected from server"));
 				DoClose();
 				return false;
 			}
@@ -1034,17 +1005,17 @@ void CRealControlSocket::OnSocketEvent(CSocketEvent &event)
 	case CSocketEvent::hostaddress:
 		{
 			const wxString& address = event.GetData();
-			LogMessage(Status, _("Connecting to %s..."), address.c_str());
+			LogMessage(MessageType::Status, _("Connecting to %s..."), address);
 		}
 		break;
 	case CSocketEvent::connection_next:
 		if (event.GetError())
-			LogMessage(Status, _("Connection attempt failed with \"%s\", trying next address."), CSocket::GetErrorDescription(event.GetError()).c_str());
+			LogMessage(MessageType::Status, _("Connection attempt failed with \"%s\", trying next address."), CSocket::GetErrorDescription(event.GetError()));
 		break;
 	case CSocketEvent::connection:
 		if (event.GetError())
 		{
-			LogMessage(Status, _("Connection attempt failed with \"%s\"."), CSocket::GetErrorDescription(event.GetError()).c_str());
+			LogMessage(MessageType::Status, _("Connection attempt failed with \"%s\"."), CSocket::GetErrorDescription(event.GetError()));
 			OnClose(event.GetError());
 		}
 		else
@@ -1067,7 +1038,7 @@ void CRealControlSocket::OnSocketEvent(CSocketEvent &event)
 		OnClose(event.GetError());
 		break;
 	default:
-		LogMessage(Debug_Warning, _T("Unhandled socket event %d"), event.GetType());
+		LogMessage(MessageType::Debug_Warning, _T("Unhandled socket event %d"), event.GetType());
 		break;
 	}
 }
@@ -1097,9 +1068,9 @@ void CRealControlSocket::OnSend()
 		{
 			if (error != EAGAIN)
 			{
-				LogMessage(::Error, _("Could not write to socket: %s"), CSocket::GetErrorDescription(error).c_str());
-				if (GetCurrentCommandId() != cmd_connect)
-					LogMessage(::Error, _("Disconnected from server"));
+				LogMessage(MessageType::Error, _("Could not write to socket: %s"), CSocket::GetErrorDescription(error));
+				if (GetCurrentCommandId() != Command::connect)
+					LogMessage(MessageType::Error, _("Disconnected from server"));
 				DoClose();
 			}
 			return;
@@ -1127,14 +1098,14 @@ void CRealControlSocket::OnSend()
 
 void CRealControlSocket::OnClose(int error)
 {
-	LogMessage(Debug_Verbose, _T("CRealControlSocket::OnClose(%d)"), error);
+	LogMessage(MessageType::Debug_Verbose, _T("CRealControlSocket::OnClose(%d)"), error);
 
-	if (GetCurrentCommandId() != cmd_connect)
+	if (GetCurrentCommandId() != Command::connect)
 	{
 		if (!error)
-			LogMessage(::Error, _("Connection closed by server"));
+			LogMessage(MessageType::Error, _("Connection closed by server"));
 		else
-			LogMessage(::Error, _("Disconnected from server: %s"), CSocket::GetErrorDescription(error).c_str());
+			LogMessage(MessageType::Error, _("Disconnected from server: %s"), CSocket::GetErrorDescription(error));
 	}
 	DoClose();
 }
@@ -1145,7 +1116,7 @@ int CRealControlSocket::Connect(const CServer &server)
 
 	if (server.GetEncodingType() == ENCODING_CUSTOM)
 	{
-		LogMessage(Debug_Info, _T("Using custom encoding: %s"), server.GetCustomEncoding().c_str());
+		LogMessage(MessageType::Debug_Info, _T("Using custom encoding: %s"), server.GetCustomEncoding());
 		m_pCSConv = new wxCSConv(server.GetCustomEncoding());
 	}
 
@@ -1164,9 +1135,8 @@ int CRealControlSocket::ContinueConnect()
 	unsigned int port = 0;
 
 	const int proxy_type = m_pEngine->GetOptions()->GetOptionVal(OPTION_PROXY_TYPE);
-	if (proxy_type > CProxySocket::unknown && proxy_type < CProxySocket::proxytype_count && !m_pCurrentServer->GetBypassProxy())
-	{
-		LogMessage(::Status, _("Connecting to %s through proxy"), m_pCurrentServer->FormatHost().c_str());
+	if (proxy_type > CProxySocket::unknown && proxy_type < CProxySocket::proxytype_count && !m_pCurrentServer->GetBypassProxy()) {
+		LogMessage(MessageType::Status, _("Connecting to %s through proxy"), m_pCurrentServer->FormatHost());
 
 		host = m_pEngine->GetOptions()->GetOption(OPTION_PROXY_HOST);
 		port = m_pEngine->GetOptions()->GetOptionVal(OPTION_PROXY_PORT);
@@ -1179,36 +1149,31 @@ int CRealControlSocket::ContinueConnect()
 											  m_pEngine->GetOptions()->GetOption(OPTION_PROXY_USER),
 											  m_pEngine->GetOptions()->GetOption(OPTION_PROXY_PASS));
 
-		if (res != EINPROGRESS)
-		{
-			LogMessage(::Error, _("Could not start proxy handshake: %s"), CSocket::GetErrorDescription(res).c_str());
+		if (res != EINPROGRESS) {
+			LogMessage(MessageType::Error, _("Could not start proxy handshake: %s"), CSocket::GetErrorDescription(res));
 			DoClose();
 			return FZ_REPLY_ERROR;
 		}
 	}
-	else
-	{
-		if (m_pCurOpData && m_pCurOpData->opId == cmd_connect)
-		{
+	else {
+		if (m_pCurOpData && m_pCurOpData->opId == Command::connect) {
 			CConnectOpData* pData(static_cast<CConnectOpData*>(m_pCurOpData));
 			host = ConvertDomainName(pData->host);
 			port = pData->port;
 		}
-		if (host == _T(""))
-		{
+		if (host.empty()) {
 			host = m_pCurrentServer->GetHost();
 			port = m_pCurrentServer->GetPort();
 		}
 	}
 	if (!IsIpAddress(host))
-		LogMessage(Status, _("Resolving address of %s"), host.c_str());
+		LogMessage(MessageType::Status, _("Resolving address of %s"), host);
 
 	int res = m_pSocket->Connect(host, port);
 
 	// Treat success same as EINPROGRESS, we wait for connect notification in any case
-	if (res && res != EINPROGRESS)
-	{
-		LogMessage(::Error, _("Could not connect to server: %s"), CSocket::GetErrorDescription(res).c_str());
+	if (res && res != EINPROGRESS) {
+		LogMessage(MessageType::Error, _("Could not connect to server: %s"), CSocket::GetErrorDescription(res));
 		DoClose();
 		return FZ_REPLY_ERROR;
 	}
@@ -1248,9 +1213,9 @@ bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNot
 {
 	wxASSERT(pFileExistsNotification);
 
-	if (!m_pCurOpData || m_pCurOpData->opId != cmd_transfer)
+	if (!m_pCurOpData || m_pCurOpData->opId != Command::transfer)
 	{
-		LogMessage(__TFILE__, __LINE__, this, Debug_Info, _T("No or invalid operation in progress, ignoring request reply %f"), pFileExistsNotification->GetRequestID());
+		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("No or invalid operation in progress, ignoring request reply %f"), pFileExistsNotification->GetRequestID());
 		return false;
 	}
 
@@ -1273,11 +1238,11 @@ bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNot
 			if (pData->download)
 			{
 				wxString filename = pData->remotePath.FormatFilename(pData->remoteFile);
-				LogMessage(Status, _("Skipping download of %s"), filename.c_str());
+				LogMessage(MessageType::Status, _("Skipping download of %s"), filename);
 			}
 			else
 			{
-				LogMessage(Status, _("Skipping upload of %s"), pData->localFile.c_str());
+				LogMessage(MessageType::Status, _("Skipping upload of %s"), pData->localFile);
 			}
 			ResetOperation(FZ_REPLY_OK);
 		}
@@ -1287,16 +1252,13 @@ bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNot
 		Second compare flags the remaining case in which we need to send command : both size unknown */
 		if ((pFileExistsNotification->localSize != pFileExistsNotification->remoteSize) || (pFileExistsNotification->localSize == -1))
 			SendNextCommand();
-		else
-		{
-			if (pData->download)
-			{
+		else {
+			if (pData->download) {
 				wxString filename = pData->remotePath.FormatFilename(pData->remoteFile);
-				LogMessage(Status, _("Skipping download of %s"), filename.c_str());
+				LogMessage(MessageType::Status, _("Skipping download of %s"), filename);
 			}
-			else
-			{
-				LogMessage(Status, _("Skipping upload of %s"), pData->localFile.c_str());
+			else {
+				LogMessage(MessageType::Status, _("Skipping upload of %s"), pData->localFile);
 			}
 			ResetOperation(FZ_REPLY_OK);
 		}
@@ -1317,11 +1279,11 @@ bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNot
 			if (pData->download)
 			{
 				wxString filename = pData->remotePath.FormatFilename(pData->remoteFile);
-				LogMessage(Status, _("Skipping download of %s"), filename.c_str());
+				LogMessage(MessageType::Status, _("Skipping download of %s"), filename);
 			}
 			else
 			{
-				LogMessage(Status, _("Skipping upload of %s"), pData->localFile.c_str());
+				LogMessage(MessageType::Status, _("Skipping upload of %s"), pData->localFile);
 			}
 			ResetOperation(FZ_REPLY_OK);
 		}
@@ -1372,7 +1334,7 @@ bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNot
 			}
 			else
 			{
-				pData->fileTime = wxDateTime();
+				pData->fileTime = CDateTime();
 				pData->remoteFileSize = -1;
 			}
 
@@ -1383,16 +1345,16 @@ bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNot
 		if (pData->download)
 		{
 			wxString filename = pData->remotePath.FormatFilename(pData->remoteFile);
-			LogMessage(Status, _("Skipping download of %s"), filename.c_str());
+			LogMessage(MessageType::Status, _("Skipping download of %s"), filename);
 		}
 		else
 		{
-			LogMessage(Status, _("Skipping upload of %s"), pData->localFile.c_str());
+			LogMessage(MessageType::Status, _("Skipping upload of %s"), pData->localFile);
 		}
 		ResetOperation(FZ_REPLY_OK);
 		break;
 	default:
-		LogMessage(__TFILE__, __LINE__, this, Debug_Warning, _T("Unknown file exists action: %d"), pFileExistsNotification->overwriteAction);
+		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Warning, _T("Unknown file exists action: %d"), pFileExistsNotification->overwriteAction);
 		ResetOperation(FZ_REPLY_INTERNALERROR);
 		return false;
 	}
@@ -1485,4 +1447,4 @@ int CControlSocket::Rename(const CRenameCommand&)
 int CControlSocket::Chmod(const CChmodCommand&)
 {
 	return FZ_REPLY_NOTSUPPORTED;
-}
\ No newline at end of file
+}
