@@ -27,9 +27,9 @@ int CFileZillaEngine::Init(wxEvtHandler *pEventHandler, COptionsBase *pOptions)
 	return FZ_REPLY_OK;
 }
 
-int CFileZillaEngine::Command(const CCommand &command)
+int CFileZillaEngine::Execute(const CCommand &command)
 {
-	if (command.GetId() != cmd_cancel && IsBusy())
+	if (command.GetId() != Command::cancel && IsBusy())
 		return FZ_REPLY_BUSY;
 
 	m_bIsInCommand = true;
@@ -37,37 +37,37 @@ int CFileZillaEngine::Command(const CCommand &command)
 	int res = FZ_REPLY_INTERNALERROR;
 	switch (command.GetId())
 	{
-	case cmd_connect:
+	case Command::connect:
 		res = Connect(reinterpret_cast<const CConnectCommand &>(command));
 		break;
-	case cmd_disconnect:
+	case Command::disconnect:
 		res = Disconnect(reinterpret_cast<const CDisconnectCommand &>(command));
 		break;
-	case cmd_cancel:
+	case Command::cancel:
 		res = Cancel(reinterpret_cast<const CCancelCommand &>(command));
 		break;
-	case cmd_list:
+	case Command::list:
 		res = List(reinterpret_cast<const CListCommand &>(command));
 		break;
-	case cmd_transfer:
+	case Command::transfer:
 		res = FileTransfer(reinterpret_cast<const CFileTransferCommand &>(command));
 		break;
-	case cmd_raw:
+	case Command::raw:
 		res = RawCommand(reinterpret_cast<const CRawCommand&>(command));
 		break;
-	case cmd_delete:
+	case Command::del:
 		res = Delete(reinterpret_cast<const CDeleteCommand&>(command));
 		break;
-	case cmd_removedir:
+	case Command::removedir:
 		res = RemoveDir(reinterpret_cast<const CRemoveDirCommand&>(command));
 		break;
-	case cmd_mkdir:
+	case Command::mkdir:
 		res = Mkdir(reinterpret_cast<const CMkdirCommand&>(command));
 		break;
-	case cmd_rename:
+	case Command::rename:
 		res = Rename(reinterpret_cast<const CRenameCommand&>(command));
 		break;
-	case cmd_chmod:
+	case Command::chmod:
 		res = Chmod(reinterpret_cast<const CChmodCommand&>(command));
 		break;
 	default:
@@ -79,7 +79,7 @@ int CFileZillaEngine::Command(const CCommand &command)
 
 	m_bIsInCommand = false;
 
-	if (command.GetId() != cmd_disconnect)
+	if (command.GetId() != Command::disconnect)
 		res |= m_nControlSocketError;
 	else if (res & FZ_REPLY_DISCONNECTED)
 		res = FZ_REPLY_OK;
@@ -102,16 +102,10 @@ CNotification* CFileZillaEngine::GetNextNotification()
 {
 	wxCriticalSectionLocker lock(m_lock);
 
-	if (!m_maySendNotificationEvent)
-	{
+	if (m_NotificationList.empty()) {
 		m_maySendNotificationEvent = true;
-		if (m_NotificationList.empty())
-			return 0;
-
-		m_NotificationList.push_back(static_cast<CNotification*>(0));
-	}
-	else if (m_NotificationList.empty())
 		return 0;
+	}
 	CNotification* pNotification = m_NotificationList.front();
 	m_NotificationList.pop_front();
 
@@ -123,7 +117,7 @@ const CCommand *CFileZillaEngine::GetCurrentCommand() const
 	return CFileZillaEnginePrivate::GetCurrentCommand();
 }
 
-enum Command CFileZillaEngine::GetCurrentCommandId() const
+Command CFileZillaEngine::GetCurrentCommandId() const
 {
 	return CFileZillaEnginePrivate::GetCurrentCommandId();
 }
