@@ -21,16 +21,13 @@ CDirectoryCache::~CDirectoryCache()
 	if (m_nRefCount)
 		return;
 
-	for (std::list<CServerEntry>::iterator sit = m_serverList.begin(); sit != m_serverList.end(); ++sit)
-	{
-		for (tCacheIter cit = sit->cacheList.begin(); cit != sit->cacheList.end(); ++cit)
-		{
+	for( auto & serverEntry : m_serverList ) {
+		for (auto & cacheEntry : serverEntry.cacheList ) {
 #ifdef __WXDEBUG__
-			m_totalFileCount -= cit->listing.GetCount();
+			m_totalFileCount -= cacheEntry.listing.GetCount();
 #endif
-			tLruList::iterator* lruIt = (tLruList::iterator*)cit->lruIt;
-			if (lruIt)
-			{
+			tLruList::iterator* lruIt = (tLruList::iterator*)cacheEntry.lruIt;
+			if (lruIt) {
 				m_leastRecentlyUsedList.erase(*lruIt);
 				delete lruIt;
 			}
@@ -50,9 +47,8 @@ void CDirectoryCache::Store(const CDirectoryListing &listing, const CServer &ser
 
 	tCacheIter cit;
 	bool unused;
-	if (Lookup(cit, sit, listing.path, true, unused))
-	{
-		cit->modificationTime = CTimeEx::Now();
+	if (Lookup(cit, sit, listing.path, true, unused)) {
+		cit->modificationTime = CMonotonicTime::Now();
 
 		m_totalFileCount -= cit->listing.GetCount();
 		cit->listing = listing;
@@ -60,12 +56,7 @@ void CDirectoryCache::Store(const CDirectoryListing &listing, const CServer &ser
 		return;
 	}
 
-	// Create new entry and store listing in cache
-	CCacheEntry entry;
-	entry.modificationTime = CTimeEx::Now();
-	entry.listing = listing;
-
-	sit->cacheList.push_front(entry);
+	sit->cacheList.emplace_front(listing);
 
 	UpdateLru(sit, sit->cacheList.begin());
 
@@ -79,8 +70,7 @@ bool CDirectoryCache::Lookup(CDirectoryListing &listing, const CServer &server,
 		return false;
 
 	tCacheIter iter;
-	if (Lookup(iter, sit, path, allowUnsureEntries, is_outdated))
-	{
+	if (Lookup(iter, sit, path, allowUnsureEntries, is_outdated)) {
 		listing = iter->listing;
 		return true;
 	}
@@ -90,8 +80,7 @@ bool CDirectoryCache::Lookup(CDirectoryListing &listing, const CServer &server,
 
 bool CDirectoryCache::Lookup(tCacheIter &cacheIter, tServerIter &sit, const CServerPath &path, bool allowUnsureEntries, bool& is_outdated)
 {
-	for (tCacheIter iter = sit->cacheList.begin(); iter != sit->cacheList.end(); ++iter)
-	{
+	for (tCacheIter iter = sit->cacheList.begin(); iter != sit->cacheList.end(); ++iter) {
 		const CCacheEntry &entry = *iter;
 
 		if (entry.listing.path != path)
@@ -99,11 +88,11 @@ bool CDirectoryCache::Lookup(tCacheIter &cacheIter, tServerIter &sit, const CSer
 
 		UpdateLru(sit, iter);
 
-		if (!allowUnsureEntries && entry.listing.m_hasUnsureEntries)
+		if (!allowUnsureEntries && entry.listing.get_unsure_flags())
 			return false;
 
 		cacheIter = iter;
-		is_outdated = (wxDateTime::Now() - entry.listing.m_firstListTime.GetTime()).GetSeconds() > CACHE_TIMEOUT;
+		is_outdated = (CDateTime::Now() - entry.listing.m_firstListTime.GetTime()).GetSeconds() > CACHE_TIMEOUT;
 		return true;
 	}
 
@@ -117,9 +106,8 @@ bool CDirectoryCache::DoesExist(const CServer &server, const CServerPath &path,
 		return false;
 
 	tCacheIter iter;
-	if (Lookup(iter, sit, path, true, is_outdated))
-	{
-		hasUnsureEntries = iter->listing.m_hasUnsureEntries;
+	if (Lookup(iter, sit, path, true, is_outdated)) {
+		hasUnsureEntries = iter->listing.get_unsure_flags();
 		return true;
 	}
 
@@ -129,16 +117,14 @@ bool CDirectoryCache::DoesExist(const CServer &server, const CServerPath &path,
 bool CDirectoryCache::LookupFile(CDirentry &entry, const CServer &server, const CServerPath &path, const wxString& file, bool &dirDidExist, bool &matchedCase)
 {
 	tServerIter sit = GetServerEntry(server);
-	if (sit == m_serverList.end())
-	{
+	if (sit == m_serverList.end()) {
 		dirDidExist = false;
 		return false;
 	}
 
 	tCacheIter iter;
 	bool unused;
-	if (!Lookup(iter, sit, path, true, unused))
-	{
+	if (!Lookup(iter, sit, path, true, unused)) {
 		dirDidExist = false;
 		return false;
 	}
@@ -148,15 +134,13 @@ bool CDirectoryCache::LookupFile(CDirentry &entry, const CServer &server, const
 	const CDirectoryListing &listing = cacheEntry.listing;
 
 	int i = listing.FindFile_CmpCase(file);
-	if (i != -1)
-	{
+	if (i >= 0) {
 		entry = listing[i];
 		matchedCase = true;
 		return true;
 	}
-	listing.FindFile_CmpNoCase(file);
-	if (i != -1)
-	{
+	i = listing.FindFile_CmpNoCase(file);
+	if (i >= 0) {
 		entry = listing[i];
 		matchedCase = false;
 		return true;
@@ -187,25 +171,22 @@ bool CDirectoryCache::InvalidateFile(const CServer &server, const CServerPath &p
 	if (sit == m_serverList.end())
 		return false;
 
-	for (tCacheIter iter = sit->cacheList.begin(); iter != sit->cacheList.end(); ++iter)
-	{
+	for (tCacheIter iter = sit->cacheList.begin(); iter != sit->cacheList.end(); ++iter) {
 		CCacheEntry &entry = *iter;
 		if (path.CmpNoCase(entry.listing.path))
 			continue;
 
 		UpdateLru(sit, iter);
 
-		for (unsigned int i = 0; i < entry.listing.GetCount(); i++)
-		{
-			if (!filename.CmpNoCase(((const CCacheEntry&)entry).listing[i].name))
-			{
+		for (unsigned int i = 0; i < entry.listing.GetCount(); i++) {
+			if (!filename.CmpNoCase(((const CCacheEntry&)entry).listing[i].name)) {
 				if (wasDir)
 					*wasDir = entry.listing[i].is_dir();
 				entry.listing[i].flags |= CDirentry::flag_unsure;
 			}
 		}
-		entry.listing.m_hasUnsureEntries |= CDirectoryListing::unsure_unknown;
-		entry.modificationTime = CTimeEx::Now();
+		entry.listing.m_flags |= CDirectoryListing::unsure_unknown;
+		entry.modificationTime = CMonotonicTime::Now();
 	}
 
 	return true;
@@ -247,11 +228,11 @@ bool CDirectoryCache::UpdateFile(const CServer &server, const CServerPath &path,
 		{
 			enum Filetype old_type = entry.listing[i].is_dir() ? dir : file;
 			if (type != old_type)
-				entry.listing.m_hasUnsureEntries |= CDirectoryListing::unsure_invalid;
+				entry.listing.m_flags |= CDirectoryListing::unsure_invalid;
 			else if (type == dir)
-				entry.listing.m_hasUnsureEntries |= CDirectoryListing::unsure_dir_changed;
+				entry.listing.m_flags |= CDirectoryListing::unsure_dir_changed;
 			else
-				entry.listing.m_hasUnsureEntries |= CDirectoryListing::unsure_file_changed;
+				entry.listing.m_flags |= CDirectoryListing::unsure_file_changed;
 		}
 		else if (type != unknown && mayCreate)
 		{
@@ -267,21 +248,21 @@ bool CDirectoryCache::UpdateFile(const CServer &server, const CServerPath &path,
 			switch (type)
 			{
 			case dir:
-				entry.listing.m_hasUnsureEntries |= CDirectoryListing::unsure_dir_added;
+				entry.listing.m_flags |= CDirectoryListing::unsure_dir_added;
 				break;
 			case file:
-				entry.listing.m_hasUnsureEntries |= CDirectoryListing::unsure_file_added;
+				entry.listing.m_flags |= CDirectoryListing::unsure_file_added;
 				break;
 			default:
-				entry.listing.m_hasUnsureEntries |= CDirectoryListing::unsure_invalid;
+				entry.listing.m_flags |= CDirectoryListing::unsure_invalid;
 				break;
 			}
 
 			++m_totalFileCount;
 		}
 		else
-			entry.listing.m_hasUnsureEntries |= CDirectoryListing::unsure_unknown;
-		entry.modificationTime = CTimeEx::Now();
+			entry.listing.m_flags |= CDirectoryListing::unsure_unknown;
+		entry.modificationTime = CMonotonicTime::Now();
 
 		updated = true;
 	}
@@ -329,9 +310,9 @@ bool CDirectoryCache::RemoveFile(const CServer &server, const CServerPath &path,
 				if (!filename.CmpNoCase(entry.listing[i].name))
 					iter->listing[i].flags |= CDirentry::flag_unsure;
 			}
-			iter->listing.m_hasUnsureEntries |= CDirectoryListing::unsure_invalid;
+			iter->listing.m_flags |= CDirectoryListing::unsure_invalid;
 		}
-		iter->modificationTime = CTimeEx::Now();
+		iter->modificationTime = CMonotonicTime::Now();
 	}
 
 	return true;
@@ -339,7 +320,7 @@ bool CDirectoryCache::RemoveFile(const CServer &server, const CServerPath &path,
 
 void CDirectoryCache::InvalidateServer(const CServer& server)
 {
-	for (std::list<CServerEntry>::iterator iter = m_serverList.begin(); iter != m_serverList.end(); ++iter)
+	for (auto iter = m_serverList.begin(); iter != m_serverList.end(); ++iter)
 	{
 		if (iter->server != server)
 			continue;
@@ -361,7 +342,7 @@ void CDirectoryCache::InvalidateServer(const CServer& server)
 	}
 }
 
-bool CDirectoryCache::GetChangeTime(CTimeEx& time, const CServer &server, const CServerPath &path)
+bool CDirectoryCache::GetChangeTime(CMonotonicTime& time, const CServer &server, const CServerPath &path)
 {
 	tServerIter sit = GetServerEntry(server);
 	if (sit == m_serverList.end())
@@ -369,8 +350,7 @@ bool CDirectoryCache::GetChangeTime(CTimeEx& time, const CServer &server, const
 
 	tCacheIter iter;
 	bool unused;
-	if (Lookup(iter, sit, path, true, unused))
-	{
+	if (Lookup(iter, sit, path, true, unused)) {
 		time = iter->modificationTime;
 		return true;
 	}
@@ -378,7 +358,7 @@ bool CDirectoryCache::GetChangeTime(CTimeEx& time, const CServer &server, const
 	return false;
 }
 
-void CDirectoryCache::RemoveDir(const CServer& server, const CServerPath& path, const wxString& filename, const CServerPath& target)
+void CDirectoryCache::RemoveDir(const CServer& server, const CServerPath& path, const wxString& filename, const CServerPath&)
 {
 	// TODO: This is not 100% foolproof and may not work properly
 	// Perhaps just throw away the complete cache?
@@ -389,18 +369,15 @@ void CDirectoryCache::RemoveDir(const CServer& server, const CServerPath& path,
 
 	CServerPath absolutePath = path;
 	if (!absolutePath.AddSegment(filename))
-		absolutePath.Clear();
+		absolutePath.clear();
 
-	for (tCacheIter iter = sit->cacheList.begin(); iter != sit->cacheList.end(); )
-	{
+	for (tCacheIter iter = sit->cacheList.begin(); iter != sit->cacheList.end(); ) {
 		CCacheEntry &entry = *iter;
 		// Delete exact matches and subdirs
-		if (!absolutePath.IsEmpty() && (entry.listing.path == absolutePath || absolutePath.IsParentOf(entry.listing.path, true)))
-		{
+		if (!absolutePath.empty() && (entry.listing.path == absolutePath || absolutePath.IsParentOf(entry.listing.path, true))) {
 			m_totalFileCount -= entry.listing.GetCount();
 			tLruList::iterator* lruIt = (tLruList::iterator*)iter->lruIt;
-			if (lruIt)
-			{
+			if (lruIt) {
 				m_leastRecentlyUsedList.erase(*lruIt);
 				delete lruIt;
 			}
@@ -447,29 +424,24 @@ void CDirectoryCache::Rename(const CServer& server, const CServerPath& pathFrom,
 				{
 					listing[i].name = fileTo;
 					listing[i].flags |= CDirentry::flag_unsure;
-					listing.m_hasUnsureEntries |= CDirectoryListing::unsure_unknown;
+					listing.m_flags |= CDirectoryListing::unsure_unknown;
 					listing.ClearFindMap();
 				}
 			}
 			return;
 		}
-		else
-		{
+		else {
 			unsigned int i;
-			for (i = 0; i < listing.GetCount(); i++)
-			{
+			for (i = 0; i < listing.GetCount(); i++) {
 				if (listing[i].name == fileFrom)
 					break;
 			}
-			if (i != listing.GetCount())
-			{
-				if (listing[i].is_dir())
-				{
+			if (i != listing.GetCount()) {
+				if (listing[i].is_dir()) {
 					RemoveDir(server, pathFrom, fileFrom, CServerPath());
 					UpdateFile(server, pathTo, fileTo, true, dir);
 				}
-				else
-				{
+				else {
 					RemoveFile(server, pathFrom, fileFrom);
 					UpdateFile(server, pathTo, fileTo, true, file);
 				}
@@ -484,14 +456,11 @@ void CDirectoryCache::Rename(const CServer& server, const CServerPath& pathFrom,
 
 CDirectoryCache::tServerIter CDirectoryCache::CreateServerEntry(const CServer& server)
 {
-	for (tServerIter iter = m_serverList.begin(); iter != m_serverList.end(); ++iter)
-	{
+	for (tServerIter iter = m_serverList.begin(); iter != m_serverList.end(); ++iter) {
 		if (iter->server == server)
 			return iter;
 	}
-	CServerEntry entry;
-	entry.server = server;
-	m_serverList.push_back(entry);
+	m_serverList.emplace_back(server);
 
 	return --m_serverList.end();
 }
@@ -499,8 +468,7 @@ CDirectoryCache::tServerIter CDirectoryCache::CreateServerEntry(const CServer& s
 CDirectoryCache::tServerIter CDirectoryCache::GetServerEntry(const CServer& server)
 {
 	tServerIter iter;
-	for (iter = m_serverList.begin(); iter != m_serverList.end(); ++iter)
-	{
+	for (iter = m_serverList.begin(); iter != m_serverList.end(); ++iter) {
 		if (iter->server == server)
 			break;
 	}
@@ -511,13 +479,12 @@ CDirectoryCache::tServerIter CDirectoryCache::GetServerEntry(const CServer& serv
 void CDirectoryCache::UpdateLru(tServerIter const& sit, tCacheIter const& cit)
 {
 	tLruList::iterator* lruIt = (tLruList::iterator*)cit->lruIt;
-	if (lruIt)
-	{
-		m_leastRecentlyUsedList.erase(*lruIt);
-		*lruIt = m_leastRecentlyUsedList.insert(m_leastRecentlyUsedList.end(), std::make_pair(sit, cit));
+	if (lruIt) {
+		m_leastRecentlyUsedList.splice(m_leastRecentlyUsedList.end(), m_leastRecentlyUsedList, *lruIt);
+		**lruIt = std::make_pair(sit, cit);
 	}
 	else
-		cit->lruIt = (void*)new tLruList::iterator(m_leastRecentlyUsedList.insert(m_leastRecentlyUsedList.end(), std::make_pair(sit, cit)));
+		cit->lruIt = (void*)new tLruList::iterator(m_leastRecentlyUsedList.emplace(m_leastRecentlyUsedList.end(), sit, cit));
 }
 
 void CDirectoryCache::Prune()
