@@ -2,6 +2,9 @@
 #include "directorylistingparser.h"
 #include "ControlSocket.h"
 
+#include <algorithm>
+#include <vector>
+
 #ifdef _DEBUG
 #define new DEBUG_NEW
 #endif
@@ -17,6 +20,29 @@ static char data[][150]={
 
 #endif
 
+namespace {
+struct ObjectCache
+{
+	CRefcountObject<wxString> const& get(wxString const& v)
+	{
+		auto it = std::lower_bound( cache.begin(), cache.end(), v );
+		
+		if( it == cache.end() || !(*it == v) ) {
+			it = cache.emplace(it, v);
+		}
+		return *it;
+	}
+
+	// Vector coupled with binary search and sorted insertion is fastest
+	// alternative as we expect a relatively low amount of inserts.
+	// Note that we cannot use set, as it it cannot search based on a different type.
+	std::vector<CRefcountObject<wxString>> cache;
+};
+
+
+ObjectCache objcache;
+}
+
 class CToken
 {
 protected:
@@ -63,51 +89,16 @@ public:
 		return m_len;
 	}
 
-	wxString GetString(unsigned int type = 0)
+	wxString GetString()
 	{
 		if (!m_pToken)
-			return _T("");
+			return wxString();
 
-		if (type > 2)
-			return _T("");
-
-		if (!m_str[type].IsEmpty())
-			return m_str[type];
-
-		if (!type)
-		{
-			wxString str(m_pToken, m_len);
-			m_str[type] = str;
-			return str;
-		}
-		else if (type == 1)
-		{
-			if (!IsRightNumeric() || IsNumeric())
-				return _T("");
-
-			int pos = m_len - 1;
-			while (m_pToken[pos] >= '0' && m_pToken[pos] <= '9')
-				--pos;
-
-			wxString str(m_pToken, pos + 1);
-			m_str[type] = str;
-			return str;
-		}
-		else if (type == 2)
-		{
-			if (!IsLeftNumeric() || IsNumeric())
-				return _T("");
-
-			int pos = 0;
-			while (m_pToken[pos] >= '0' && m_pToken[pos] <= '9')
-				++pos;
-
-			wxString str(m_pToken + pos, m_len - pos);
-			m_str[type] = str;
-			return str;
+		if( m_str.empty() ) {
+			m_str.assign(m_pToken, m_len);
 		}
 
-		return _T("");
+		return m_str;
 	}
 
 	bool IsNumeric(t_numberBase base = decimal)
@@ -304,7 +295,7 @@ protected:
 	TokenInformation m_leftNumeric;
 	TokenInformation m_rightNumeric;
 	wxLongLong m_number;
-	wxString m_str[3];
+	wxString m_str;
 };
 
 class CLine
@@ -354,7 +345,7 @@ public:
 					CToken *pToken = new CToken(m_pLine + start, m_parsePos - start);
 					m_Tokens.push_back(pToken);
 
-					while ((m_pLine[m_parsePos] == ' ' || m_pLine[m_parsePos] == '\t') && m_parsePos < m_len)
+					while (m_parsePos < m_len && (m_pLine[m_parsePos] == ' ' || m_pLine[m_parsePos] == '\t'))
 						++m_parsePos;
 
 					if (m_Tokens.size() > n)
@@ -524,6 +515,7 @@ CDirectoryListingParser::CDirectoryListingParser(CControlSocket* pControlSocket,
 		m_MonthNamesMap[_T("mars")] = 3;
 		m_MonthNamesMap[_T("mrs")] = 3;
 		m_MonthNamesMap[_T("avr")] = 4;
+		m_MonthNamesMap[_T("avril")] = 4;
 		m_MonthNamesMap[_T("juin")] = 6;
 		m_MonthNamesMap[_T("juil")] = 7;
 		m_MonthNamesMap[_T("jui")] = 7;
@@ -565,6 +557,10 @@ CDirectoryListingParser::CDirectoryListingParser(CControlSocket* pControlSocket,
 		m_MonthNamesMap[_T("sie")] = 8;
 		m_MonthNamesMap[_T("wrz")] = 9;
 		m_MonthNamesMap[_T("pa\x9f")] = 10;
+		m_MonthNamesMap[_T("pa\xbc")] = 10; // ISO-8859-2
+		m_MonthNamesMap[_T("paz")] = 10; // ASCII
+		m_MonthNamesMap[_T("pa\xc5\xba")] = 10; // UTF-8
+		m_MonthNamesMap[_T("pa\x017a")] = 10; // some servers send this
 		m_MonthNamesMap[_T("lis")] = 11;
 		m_MonthNamesMap[_T("gru")] = 12;
 
@@ -607,13 +603,11 @@ CDirectoryListingParser::CDirectoryListingParser(CControlSocket* pControlSocket,
 		m_MonthNamesMap[_T("avg")] = 8;
 
 		//Icelandic
-#if wxUSE_UNICODE
 		m_MonthNamesMap[_T("ma\x00ed")] = 5;
 		m_MonthNamesMap[_T("j\x00fan")] = 6;
 		m_MonthNamesMap[_T("j\x00fal")] = 7;
 		m_MonthNamesMap[_T("\x00e1g")] = 8;
 		m_MonthNamesMap[_T("n\x00f3v")] = 11;
-#endif
 		m_MonthNamesMap[_T("des")] = 12;
 
 		//Lithuanian
@@ -641,19 +635,19 @@ CDirectoryListingParser::CDirectoryListingParser(CControlSocket* pControlSocket,
 		// Some servers send a combination of month name and number,
 		// Add corresponding numbers to the month names.
 		std::map<wxString, int> combo;
-		for (std::map<wxString, int>::iterator iter = m_MonthNamesMap.begin(); iter != m_MonthNamesMap.end(); ++iter)
+		for (auto iter = m_MonthNamesMap.begin(); iter != m_MonthNamesMap.end(); ++iter)
 		{
 			// January could be 1 or 0, depends how the server counts
-			combo[wxString::Format(_T("%s%02d"), iter->first.c_str(), iter->second)] = iter->second;
-			combo[wxString::Format(_T("%s%02d"), iter->first.c_str(), iter->second - 1)] = iter->second;
+			combo[wxString::Format(_T("%s%02d"), iter->first, iter->second)] = iter->second;
+			combo[wxString::Format(_T("%s%02d"), iter->first, iter->second - 1)] = iter->second;
 			if (iter->second < 10)
-				combo[wxString::Format(_T("%s%d"), iter->first.c_str(), iter->second)] = iter->second;
+				combo[wxString::Format(_T("%s%d"), iter->first, iter->second)] = iter->second;
 			else
-				combo[wxString::Format(_T("%s%d"), iter->first.c_str(), iter->second % 10)] = iter->second;
+				combo[wxString::Format(_T("%s%d"), iter->first, iter->second % 10)] = iter->second;
 			if (iter->second <= 10)
-				combo[wxString::Format(_T("%s%d"), iter->first.c_str(), iter->second - 1)] = iter->second;
+				combo[wxString::Format(_T("%s%d"), iter->first, iter->second - 1)] = iter->second;
 			else
-				combo[wxString::Format(_T("%s%d"), iter->first.c_str(), (iter->second - 1) % 10)] = iter->second;
+				combo[wxString::Format(_T("%s%d"), iter->first, (iter->second - 1) % 10)] = iter->second;
 		}
 		m_MonthNamesMap.insert(combo.begin(), combo.end());
 
@@ -685,7 +679,7 @@ CDirectoryListingParser::CDirectoryListingParser(CControlSocket* pControlSocket,
 
 CDirectoryListingParser::~CDirectoryListingParser()
 {
-	for (std::list<t_list>::iterator iter = m_DataList.begin(); iter != m_DataList.end(); ++iter)
+	for (auto iter = m_DataList.begin(); iter != m_DataList.end(); ++iter)
 		delete [] iter->p;
 
 	delete m_prevLine;
@@ -736,11 +730,11 @@ CDirectoryListing CDirectoryListingParser::Parse(const CServerPath &path)
 {
 	CDirectoryListing listing;
 	listing.path = path;
-	listing.m_firstListTime = CTimeEx::Now();
+	listing.m_firstListTime = CMonotonicTime::Now();
 
 	if (!ParseData(false))
 	{
-		listing.m_failed = true;
+		listing.m_flags |= CDirectoryListing::listing_failed;
 		return listing;
 	}
 
@@ -858,7 +852,7 @@ bool CDirectoryListingParser::ParseLine(CLine *pLine, const enum ServerType serv
 		{
 			m_maybeMultilineVms = token.Find(';') != -1;
 			if (m_fileListOnly)
-				m_fileList.push_back(token.GetString());
+				m_fileList.emplace_back(token.GetString());
 		}
 	}
 	else
@@ -883,17 +877,9 @@ done:
 			entry.name = entry.name.Left(pos);
 	}
 
-	{
-		int offset = m_server.GetTimezoneOffset();
-		if (offset && entry.has_time())
-		{
-			// Apply timezone offset
-			wxTimeSpan span(0, offset, 0, 0);
-			entry.time.Add(span);
-		}
-	}
+	entry.time += wxTimeSpan( 0, m_server.GetTimezoneOffset(), 0, 0);
 
-	m_entryList.push_back(entry);
+	m_entryList.emplace_back(std::move(entry));
 
 skip:
 	m_maybeMultilineVms = false;
@@ -920,7 +906,7 @@ bool CDirectoryListingParser::ParseAsUnix(CLine *pLine, CDirentry &entry, bool e
 		chr != '-')
 		return false;
 
-	entry.permissions = token.GetString();
+	wxString permissions = token.GetString();
 
 	entry.flags = 0;
 
@@ -936,9 +922,10 @@ bool CDirectoryListingParser::ParseAsUnix(CLine *pLine, CDirentry &entry, bool e
 	{
 		if (!pLine->GetToken(++index, token))
 			return false;
-		entry.permissions += _T(" ") + token.GetString();
+		permissions += _T(" ") + token.GetString();
 		netware = true;
 	}
+	entry.permissions = objcache.get(permissions);
 
 	int numOwnerGroup = 3;
 	if (!netware)
@@ -958,14 +945,14 @@ bool CDirectoryListingParser::ParseAsUnix(CLine *pLine, CDirentry &entry, bool e
 		// Reset index
 		index = startindex;
 
-		entry.ownerGroup.clear();
+		wxString ownerGroup;
 		for (int i = 0; i < numOwnerGroup; ++i)
 		{
 			if (!pLine->GetToken(++index, token))
 				return false;
 			if (i)
-				entry.ownerGroup += _T(" ");
-			entry.ownerGroup += token.GetString();
+				ownerGroup += _T(" ");
+			ownerGroup += token.GetString();
 		}
 
 		if (!pLine->GetToken(++index, token))
@@ -982,14 +969,21 @@ bool CDirectoryListingParser::ParseAsUnix(CLine *pLine, CDirentry &entry, bool e
 		// Append missing group to ownerGroup
 		if (!token.IsNumeric() && token.IsRightNumeric())
 		{
-			if (!entry.ownerGroup.IsEmpty())
-				entry.ownerGroup += _T(" ");
-			entry.ownerGroup += token.GetString(1);
+			if (!ownerGroup.empty())
+				ownerGroup += _T(" ");
+
+			wxString const group = token.GetString();
+			int i;
+			for( i = group.size() - 1;
+				 i >= 0 && group[i] >= '0' && group[i] <= '9';
+				 --i ) {}
+
+			ownerGroup += group.Left(i + 1);
 		}
 
 		if (expect_date)
 		{
-			entry.flags &= ~CDirentry::flag_timestamp_mask;
+			entry.time = CDateTime();
 			if (!ParseUnixDateTime(pLine, index, entry))
 				continue;
 		}
@@ -1012,14 +1006,14 @@ bool CDirectoryListingParser::ParseAsUnix(CLine *pLine, CDirentry &entry, bool e
 			int pos;
 			if ((pos = entry.name.Find(_T(" -> "))) != -1)
 			{
-				entry.target = entry.name.Mid(pos + 4);
+				entry.target = CSparseOptional<wxString>(entry.name.Mid(pos + 4));
 				entry.name = entry.name.Left(pos);
 			}
 		}
 
-		if (entry.has_time())
-			entry.time.Add(m_timezoneOffset);
+		entry.time += m_timezoneOffset;
 
+		entry.ownerGroup = objcache.get(ownerGroup);
 		return true;
 	}
 	while (numOwnerGroup--);
@@ -1031,7 +1025,6 @@ bool CDirectoryListingParser::ParseUnixDateTime(CLine *pLine, int &index, CDiren
 {
 	bool mayHaveTime = true;
 	bool bHasYearAndTime = false;
-	bool hasTime = false;
 
 	CToken token;
 
@@ -1040,11 +1033,11 @@ bool CDirectoryListingParser::ParseUnixDateTime(CLine *pLine, int &index, CDiren
 	if (!pLine->GetToken(++index, token))
 		return false;
 
-	int year = 0;
-	int month = 0;
-	int day = 0;
-	long hour = 0;
-	long minute = 0;
+	int year = -1;
+	int month = -1;
+	int day = -1;
+	long hour = -1;
+	long minute = -1;
 
 	// Some servers use the following date formats:
 	// 26-05 2002, 2002-10-14, 01-jun-99 or 2004.07.15
@@ -1076,9 +1069,10 @@ bool CDirectoryListingParser::ParseUnixDateTime(CLine *pLine, int &index, CDiren
 			if (token[pos] == '.')
 				return true;
 
-			year = entry.time.GetYear();
-			month = entry.time.GetMonth() - wxDateTime::Jan + 1;
-			day = entry.time.GetDay();
+			wxDateTime::Tm t = entry.time.Degenerate().GetTm();
+			year = t.year;
+			month = t.mon + 1;
+			day = t.mday;
 		}
 	}
 	else if (token.IsNumeric())
@@ -1115,8 +1109,7 @@ bool CDirectoryListingParser::ParseUnixDateTime(CLine *pLine, int &index, CDiren
 			dateMonth = token;
 	}
 
-	if (!day)
-	{
+	if( day < 1 ) {
 		// Get day field
 		if (!pLine->GetToken(++index, token))
 			return false;
@@ -1146,8 +1139,7 @@ bool CDirectoryListingParser::ParseUnixDateTime(CLine *pLine, int &index, CDiren
 		day = dateDay;
 	}
 
-	if (!month)
-	{
+	if( month < 1 ) {
 		wxString strMonth = dateMonth.GetString();
 		if (dateMonth.IsLeftNumeric() && (unsigned int)strMonth[strMonth.Length() - 1] > 127)
 		{
@@ -1190,11 +1182,9 @@ bool CDirectoryListingParser::ParseUnixDateTime(CLine *pLine, int &index, CDiren
 		if (minute < 0 || minute > 59)
 			return false;
 
-		hasTime = true;
-
 		// Some servers use times only for files newer than 6 months
-		if (!year)
-		{
+		if( year <= 0 ) {
+			wxASSERT( month != -1 && day != -1 );
 			year = wxDateTime::Now().GetYear();
 			int currentDayOfYear = wxDateTime::Now().GetDay() + 31 * (wxDateTime::Now().GetMonth() - wxDateTime::Jan);
 			int fileDayOfYear = (month - 1) * 31 + day;
@@ -1209,7 +1199,7 @@ bool CDirectoryListingParser::ParseUnixDateTime(CLine *pLine, int &index, CDiren
 				year -= 1;
 		}
 	}
-	else if (!year)
+	else if (year <= 0)
 	{
 		// token is a year
 		if (!token.IsNumeric() && !token.IsLeftNumeric())
@@ -1245,8 +1235,6 @@ bool CDirectoryListingParser::ParseUnixDateTime(CLine *pLine, int &index, CDiren
 					return false;
 				if (minute < 0 || minute > 59)
 					return false;
-
-				hasTime = true;
 			}
 			else
 				--index;
@@ -1255,13 +1243,9 @@ bool CDirectoryListingParser::ParseUnixDateTime(CLine *pLine, int &index, CDiren
 	else
 		--index;
 
-	entry.time = wxDateTime();
-	if (!VerifySetDate(entry.time, year, (wxDateTime::Month)(wxDateTime::Jan + month - 1), day, hour, minute))
+	if( !entry.time.Set( year, month, day, hour, minute ) ) {
 		return false;
-
-	entry.flags |= CDirentry::flag_timestamp_date;
-	if (hasTime)
-		entry.flags |= CDirentry::flag_timestamp_time;
+	}
 
 	return true;
 }
@@ -1418,10 +1402,9 @@ bool CDirectoryListingParser::ParseShortDate(CToken &token, CDirentry &entry, bo
 	if (!gotMonth || !gotDay || !gotYear)
 		return false;
 
-	entry.time = wxDateTime();
-	if (!VerifySetDate(entry.time, year, (wxDateTime::Month)(wxDateTime::Jan + month - 1), day))
+	if( !entry.time.Set( year, month, day ) ) {
 		return false;
-	entry.flags |= CDirentry::flag_timestamp_date;
+	}
 
 	return true;
 }
@@ -1483,12 +1466,10 @@ bool CDirectoryListingParser::ParseAsDos(CLine *pLine, CDirentry &entry)
 		return false;
 	entry.name = token.GetString();
 
-	entry.target = _T("");
-	entry.ownerGroup = _T("");
-	entry.permissions = _T("");
-
-	if (entry.has_time())
-		entry.time.Add(m_timezoneOffset);
+	entry.target.clear();
+	entry.ownerGroup = objcache.get(wxString());
+	entry.permissions = objcache.get(wxString());
+	entry.time += m_timezoneOffset;
 
 	return true;
 }
@@ -1521,17 +1502,12 @@ bool CDirectoryListingParser::ParseTime(CToken &token, CDirentry &entry)
 	if (minute < 0 || minute > 59)
 		return false;
 
-	wxLongLong seconds;
-	bool hasSeconds = false;
-	if (pos2 == -1)
-		seconds = 0;
-	else
-	{
+	wxLongLong seconds = -1;
+	if (pos2 != -1) {
 		// Parse seconds
 		seconds = token.GetNumber(pos2 + 1, -1);
-		if (seconds < 0 || seconds > 59)
+		if (seconds < 0 || seconds > 60)
 			return false;
-		hasSeconds = true;
 	}
 
 	// Convert to 24h format
@@ -1547,14 +1523,7 @@ bool CDirectoryListingParser::ParseTime(CToken &token, CDirentry &entry)
 				hour = 0;
 	}
 
-	wxTimeSpan span(hour.GetLo(), minute.GetLo(), seconds.GetLo());
-	entry.time.Add(span);
-
-	entry.flags |= CDirentry::flag_timestamp_time;
-	if (hasSeconds)
-		entry.flags |= CDirentry::flag_timestamp_seconds;
-
-	return true;
+	return entry.time.ImbueTime( hour.GetLo(), minute.GetLo(), seconds.GetLo() );
 }
 
 bool CDirectoryListingParser::ParseAsEplf(CLine *pLine, CDirentry &entry)
@@ -1573,8 +1542,8 @@ bool CDirectoryListingParser::ParseAsEplf(CLine *pLine, CDirentry &entry)
 	entry.name = token.GetString().Mid(pos + 1);
 
 	entry.flags = 0;
-	entry.ownerGroup = _T("");
-	entry.permissions = _T("");
+	entry.ownerGroup = objcache.get(wxString());
+	entry.permissions = objcache.get(wxString());
 	entry.size = -1;
 
 	int fact = 1;
@@ -1604,12 +1573,10 @@ bool CDirectoryListingParser::ParseAsEplf(CLine *pLine, CDirentry &entry)
 			wxLongLong number = token.GetNumber(fact + 1, len - 1);
 			if (number < 0)
 				return false;
-			entry.time = wxDateTime((time_t)number.GetValue());
-
-			entry.flags |= CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time | CDirentry::flag_timestamp_seconds;
+			entry.time = CDateTime(wxDateTime((time_t)number.GetValue()), CDateTime::seconds);
 		}
 		else if (type == 'u' && len > 2 && token[fact + 1] == 'p')
-			entry.permissions = token.GetString().Mid(fact + 2, len - 2);
+			entry.permissions = objcache.get(token.GetString().Mid(fact + 2, len - 2));
 
 		fact += len + 1;
 	}
@@ -1666,17 +1633,16 @@ bool CDirectoryListingParser::ParseAsVms(CLine *pLine, CDirentry &entry)
 	if (!pLine->GetToken(++index, token))
 		return false;
 
-	entry.ownerGroup = _T("");
+	wxString ownerGroup;
+	wxString permissions;
 
 	// This field can either be the filesize, a username (at least that's what I think) enclosed in [] or a date.
-	if (!token.IsNumeric() && !token.IsLeftNumeric())
-	{
+	if (!token.IsNumeric() && !token.IsLeftNumeric()) {
 		// Must be username
-
 		const int len = token.GetLength();
 		if (len < 3 || token[0] != '[' || token[len - 1] != ']')
 			return false;
-		entry.ownerGroup = token.GetString().Mid(1, len - 2);
+		ownerGroup = token.GetString().Mid(1, len - 2);
 
 		if (!pLine->GetToken(++index, token))
 			return false;
@@ -1691,10 +1657,8 @@ bool CDirectoryListingParser::ParseAsVms(CLine *pLine, CDirentry &entry)
 	if (!pos)
 		return false;
 
-	if (token.IsNumeric() || (pos != -1 && token.Find('/', pos + 1) == -1))
-	{
+	if (token.IsNumeric() || (pos != -1 && token.Find('/', pos + 1) == -1)) {
 		// Definitely size
-
 		CToken sizeToken;
 		if (pos == -1)
 			sizeToken = token;
@@ -1707,16 +1671,9 @@ bool CDirectoryListingParser::ParseAsVms(CLine *pLine, CDirentry &entry)
 		if (!pLine->GetToken(++index, token))
 			return false;
 	}
-	else if (pos == -1 && token.IsLeftNumeric())
-	{
+	else if (pos == -1 && token.IsLeftNumeric()) {
 		// Perhaps size
-		CToken sizeToken;
-		if (pos == -1)
-			sizeToken = token;
-		else
-			sizeToken = CToken(token.GetToken(), pos);
-		if (ParseComplexFileSize(sizeToken, entry.size, 512))
-		{
+		if (ParseComplexFileSize(token, entry.size, 512)) {
 			gotSize = true;
 
 			if (!pLine->GetToken(++index, token))
@@ -1732,8 +1689,7 @@ bool CDirectoryListingParser::ParseAsVms(CLine *pLine, CDirentry &entry)
 	if (!pLine->GetToken(++index, token))
 		return true;
 
-	if (!ParseTime(token, entry))
-	{
+	if (!ParseTime(token, entry)) {
 		int len = token.GetLength();
 		if (token[0] == '[' && token[len - 1] != ']')
 			return false;
@@ -1746,8 +1702,7 @@ bool CDirectoryListingParser::ParseAsVms(CLine *pLine, CDirentry &entry)
 		--index;
 	}
 
-	if (!gotSize)
-	{
+	if (!gotSize) {
 		// Get size
 		if (!pLine->GetToken(++index, token))
 			return false;
@@ -1769,32 +1724,28 @@ bool CDirectoryListingParser::ParseAsVms(CLine *pLine, CDirentry &entry)
 	}
 
 	// Owner / group and permissions
-	entry.permissions = _T("");
-	while (pLine->GetToken(++index, token))
-	{
+	while (pLine->GetToken(++index, token)) {
 		const int len = token.GetLength();
-		if (len > 2 && token[0] == '(' && token[len - 1] == ')')
-		{
-			if (!entry.permissions.IsEmpty())
-				entry.permissions += _T(" ");
-			entry.permissions += token.GetString().Mid(1, len - 2);
+		if (len > 2 && token[0] == '(' && token[len - 1] == ')') {
+			if (!permissions.empty())
+				permissions += _T(" ");
+			permissions += token.GetString().Mid(1, len - 2);
 		}
-		else if (len > 2 && token[0] == '[' && token[len - 1] == ']')
-		{
-			if (!entry.ownerGroup.IsEmpty())
-				entry.ownerGroup += _T(" ");
-			entry.ownerGroup += token.GetString().Mid(1, len - 2);
+		else if (len > 2 && token[0] == '[' && token[len - 1] == ']') {
+			if (!ownerGroup.empty())
+				ownerGroup += _T(" ");
+			ownerGroup += token.GetString().Mid(1, len - 2);
 		}
-		else
-		{
-			if (!entry.permissions.IsEmpty())
-				entry.permissions += _T(" ");
-			entry.ownerGroup += token.GetString();
+		else {
+			if (!ownerGroup.empty())
+				ownerGroup += _T(" ");
+			ownerGroup += token.GetString();
 		}
 	}
+	entry.permissions = objcache.get(permissions);
+	entry.ownerGroup = objcache.get(ownerGroup);
 
-	if (entry.has_time())
-		entry.time.Add(m_timezoneOffset);
+	entry.time += m_timezoneOffset;
 
 	return true;
 }
@@ -1808,7 +1759,7 @@ bool CDirectoryListingParser::ParseAsIbm(CLine *pLine, CDirentry &entry)
 	if (!pLine->GetToken(index, token))
 		return false;
 
-	entry.ownerGroup = token.GetString();
+	entry.ownerGroup = objcache.get(token.GetString());
 
 	// Get size
 	if (!pLine->GetToken(++index, token))
@@ -1846,8 +1797,7 @@ bool CDirectoryListingParser::ParseAsIbm(CLine *pLine, CDirentry &entry)
 		entry.flags |= CDirentry::flag_dir;
 	}
 
-	if (entry.has_time())
-		entry.time.Add(m_timezoneOffset);
+	entry.time += m_timezoneOffset;
 
 	return true;
 }
@@ -1873,18 +1823,18 @@ bool CDirectoryListingParser::ParseOther(CLine *pLine, CDirentry &entry)
 
 	// If token is a number, than it's the numerical Unix style format,
 	// else it's the VShell, OS/2 or nortel.VxWorks format
-	if (token.IsNumeric())
-	{
-		entry.permissions = firstToken.GetString();
+	if (token.IsNumeric()) {
+		entry.permissions = objcache.get(firstToken.GetString());
 		if (firstToken.GetLength() >= 2 && firstToken[1] == '4')
 			entry.flags |= CDirentry::flag_dir;
 
-		entry.ownerGroup += token.GetString();
+		wxString ownerGroup = token.GetString();
 
 		if (!pLine->GetToken(++index, token))
 			return false;
 
-		entry.ownerGroup += _T(" ") + token.GetString();
+		ownerGroup += _T(" ") + token.GetString();
+		entry.ownerGroup = objcache.get(ownerGroup);
 
 		// Get size
 		if (!pLine->GetToken(++index, token))
@@ -1902,20 +1852,16 @@ bool CDirectoryListingParser::ParseOther(CLine *pLine, CDirentry &entry)
 		wxLongLong number = token.GetNumber();
 		if (number < 0)
 			return false;
-		entry.time = wxDateTime((time_t)number.GetValue());
-
-		entry.flags |= CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time | CDirentry::flag_timestamp_seconds;
+		entry.time = CDateTime(wxDateTime((time_t)number.GetValue()), CDateTime::seconds);
 
 		// Get filename
 		if (!pLine->GetToken(++index, token, true))
 			return false;
 
 		entry.name = token.GetString();
-
-		entry.target = _T("");
+		entry.target.clear();
 	}
-	else
-	{
+	else {
 		// Possible conflict with multiline VMS listings
 		if (m_maybeMultilineVms)
 			return false;
@@ -1994,11 +1940,9 @@ bool CDirectoryListingParser::ParseOther(CLine *pLine, CDirentry &entry)
 			else if (year < 1000)
 				year += 1900;
 
-			entry.time = wxDateTime();
-			if (!VerifySetDate(entry.time, year.GetLo(), (wxDateTime::Month)(month - 1), day.GetLo()))
+			if( !entry.time.Set( year.GetLo(), month, day.GetLo() ) ) {
 				return false;
-
-			entry.flags |= CDirentry::flag_timestamp_date;
+			}
 
 			// Get time
 			if (!pLine->GetToken(++index, token))
@@ -2019,12 +1963,10 @@ bool CDirectoryListingParser::ParseOther(CLine *pLine, CDirentry &entry)
 				entry.name.RemoveLast();
 			}
 		}
-		entry.target = _T("");
-		entry.ownerGroup = _T("");
-		entry.permissions = _T("");
-
-		if (entry.has_time())
-			entry.time.Add(m_timezoneOffset);
+		entry.target.clear();
+		entry.ownerGroup = objcache.get(wxString());
+		entry.permissions = objcache.get(wxString());
+		entry.time += m_timezoneOffset;
 	}
 
 	return true;
@@ -2034,11 +1976,7 @@ bool CDirectoryListingParser::AddData(char *pData, int len)
 {
 	ConvertEncoding( pData, len );
 
-	t_list item;
-	item.p = pData;
-	item.len = len;
-
-	m_DataList.push_back(item);
+	m_DataList.emplace_back(pData, len);
 	m_totalData += len;
 
 	if (m_totalData < 512)
@@ -2050,7 +1988,7 @@ bool CDirectoryListingParser::AddData(char *pData, int len)
 bool CDirectoryListingParser::AddLine(const wxChar* pLine)
 {
 	if (m_pControlSocket)
-		m_pControlSocket->LogMessageRaw(RawList, pLine);
+		m_pControlSocket->LogMessageRaw(MessageType::RawList, pLine);
 
 	while (*pLine == ' ' || *pLine == '\t')
 		++pLine;
@@ -2076,7 +2014,7 @@ CLine *CDirectoryListingParser::GetLine(bool breakAtEnd /*=false*/, bool &error)
 	while (!m_DataList.empty())
 	{
 		// Trim empty lines and spaces
-		std::list<t_list>::iterator iter = m_DataList.begin();
+		auto iter = m_DataList.begin();
 		int len = iter->len;
 		while (iter->p[m_currentOffset]=='\r' || iter->p[m_currentOffset]=='\n' || iter->p[m_currentOffset]==' ' || iter->p[m_currentOffset]=='\t')
 		{
@@ -2120,7 +2058,7 @@ CLine *CDirectoryListingParser::GetLine(bool breakAtEnd /*=false*/, bool &error)
 				{
 					if (reslen > 10000)
 					{
-						m_pControlSocket->LogMessage(::Error, _("Received a line exceeding 10000 characters, aborting."));
+						m_pControlSocket->LogMessage(MessageType::Error, _("Received a line exceeding 10000 characters, aborting."));
 						error = true;
 						return 0;
 					}
@@ -2135,7 +2073,7 @@ CLine *CDirectoryListingParser::GetLine(bool breakAtEnd /*=false*/, bool &error)
 
 		if (reslen > 10000)
 		{
-			m_pControlSocket->LogMessage(::Error, _("Received a line exceeding 10000 characters, aborting."));
+			m_pControlSocket->LogMessage(MessageType::Error, _("Received a line exceeding 10000 characters, aborting."));
 			error = true;
 			return 0;
 		}
@@ -2148,7 +2086,7 @@ CLine *CDirectoryListingParser::GetLine(bool breakAtEnd /*=false*/, bool &error)
 		int respos = 0;
 
 		// Copy line data
-		std::list<t_list>::iterator i = m_DataList.begin();
+		auto i = m_DataList.begin();
 		while (i != iter && reslen)
 		{
 			int copylen = i->len - startpos;
@@ -2185,15 +2123,15 @@ CLine *CDirectoryListingParser::GetLine(bool breakAtEnd /*=false*/, bool &error)
 		if (m_pControlSocket)
 		{
 			buffer = m_pControlSocket->ConvToLocalBuffer(res);
-			m_pControlSocket->LogMessageRaw(RawList, buffer);
+			m_pControlSocket->LogMessageRaw(MessageType::RawList, buffer);
 		}
 		else
 		{
 			wxString str(res, wxConvUTF8);
-			if (str.IsEmpty())
+			if (str.empty())
 			{
 				str = wxString(res, wxConvLocal);
-				if (str.IsEmpty())
+				if (str.empty())
 					str = wxString(res, wxConvISO8859_1);
 			}
 			buffer = new wxChar[str.Len() + 1];
@@ -2256,11 +2194,9 @@ bool CDirectoryListingParser::ParseAsWfFtp(CLine *pLine, CDirentry &entry)
 	if (!ParseTime(token, entry))
 		return false;
 
-	entry.ownerGroup = _T("");
-	entry.permissions = _T("");
-
-	if (entry.has_time())
-		entry.time.Add(m_timezoneOffset);
+	entry.ownerGroup = objcache.get(wxString());
+	entry.permissions = objcache.get(wxString());
+	entry.time += m_timezoneOffset;
 
 	return true;
 }
@@ -2298,8 +2234,8 @@ bool CDirectoryListingParser::ParseAsIBM_MVS(CLine *pLine, CDirentry &entry)
 			return false;
 
 		entry.size = -1;
-		entry.ownerGroup = _T("");
-		entry.permissions = _T("");
+		entry.ownerGroup = objcache.get(wxString());
+		entry.permissions = objcache.get(wxString());
 
 		return true;
 	}
@@ -2359,8 +2295,8 @@ bool CDirectoryListingParser::ParseAsIBM_MVS(CLine *pLine, CDirentry &entry)
 
 	entry.name = token.GetString();
 
-	entry.ownerGroup = _T("");
-	entry.permissions = _T("");
+	entry.ownerGroup = objcache.get(wxString());
+	entry.permissions = objcache.get(wxString());
 
 	return true;
 }
@@ -2422,11 +2358,9 @@ bool CDirectoryListingParser::ParseAsIBM_MVS_PDS(CLine *pLine, CDirentry &entry)
 	if (!pLine->GetToken(index++, token, true))
 		return false;
 
-	entry.ownerGroup = _T("");
-	entry.permissions = _T("");
-
-	if (entry.has_time())
-		entry.time.Add(m_timezoneOffset);
+	entry.ownerGroup = objcache.get(wxString());
+	entry.permissions = objcache.get(wxString());
+	entry.time += m_timezoneOffset;
 
 	return true;
 }
@@ -2450,8 +2384,8 @@ bool CDirectoryListingParser::ParseAsIBM_MVS_Migrated(CLine *pLine, CDirentry &e
 	entry.name = token.GetString();
 
 	entry.flags = 0;
-	entry.ownerGroup = _T("");
-	entry.permissions = _T("");
+	entry.ownerGroup = objcache.get(wxString());
+	entry.permissions = objcache.get(wxString());
 	entry.size = -1;
 
 	if (pLine->GetToken(++index, token))
@@ -2470,8 +2404,8 @@ bool CDirectoryListingParser::ParseAsIBM_MVS_PDS2(CLine *pLine, CDirentry &entry
 	entry.name = token.GetString();
 
 	entry.flags = 0;
-	entry.ownerGroup = _T("");
-	entry.permissions = _T("");
+	entry.ownerGroup = objcache.get(wxString());
+	entry.permissions = objcache.get(wxString());
 	entry.size = -1;
 
 	if (!pLine->GetToken(++index, token))
@@ -2544,8 +2478,8 @@ bool CDirectoryListingParser::ParseAsIBM_MVS_Tape(CLine *pLine, CDirentry &entry
 
 	entry.name = token.GetString();
 	entry.flags = 0;
-	entry.ownerGroup = _T("");
-	entry.permissions = _T("");
+	entry.ownerGroup = objcache.get(wxString());
+	entry.permissions = objcache.get(wxString());
 	entry.size = -1;
 
 	if (pLine->GetToken(index++, token))
@@ -2657,17 +2591,17 @@ int CDirectoryListingParser::ParseAsMlsd(CLine *pLine, CDirentry &entry)
 		return 0;
 
 	wxString facts = token.GetString();
-	if (facts.IsEmpty())
+	if (facts.empty())
 		return 0;
 
 	entry.flags = 0;
 	entry.size = -1;
-	entry.ownerGroup = _T("");
-	entry.permissions = _T("");
 
 	wxString owner, group, uid, gid;
+	wxString ownerGroup;
+	wxString permissions;
 
-	while (!facts.IsEmpty())
+	while (!facts.empty())
 	{
 		int delim = facts.Find(';');
 		if (delim < 3)
@@ -2687,17 +2621,27 @@ int CDirectoryListingParser::ParseAsMlsd(CLine *pLine, CDirentry &entry)
 		wxString value = facts.Mid(pos + 1, delim - pos - 1);
 		if (factname == _T("type"))
 		{
-			if (!value.CmpNoCase(_T("dir")))
+			int pos = value.Find(':');
+			wxString valuePrefix;
+			if (pos == -1)
+				valuePrefix = value;
+			else
+				valuePrefix = value.Left(pos);
+			MakeLowerAscii(valuePrefix);
+
+			if (valuePrefix == _T("dir") && pos == -1)
 				entry.flags |= CDirentry::flag_dir;
-			else if (!value.Left(13).CmpNoCase(_T("OS.unix=slink")))
+			else if (valuePrefix == _T("os.unix=slink") || valuePrefix == _T("os.unix=symlink"))
 			{
 				entry.flags |= CDirentry::flag_dir | CDirentry::flag_link;
-				if (value[13] == ':' && value[14] != 0)
-					entry.target = value.Mid(14);
+				if (pos != -1)
+					entry.target = CSparseOptional<wxString>(value.Mid(pos));
 			}
-			else if (!value.CmpNoCase(_T("cdir")) ||
-					 !value.CmpNoCase(_T("pdir")))
+			else if ((valuePrefix == _T("cdir") || valuePrefix == _T("pdir")) && pos == -1)
+			{
+				// Current and parent directory, don't parse it
 				return 2;
+			}
 		}
 		else if (factname == _T("size"))
 		{
@@ -2720,33 +2664,31 @@ int CDirectoryListingParser::ParseAsMlsd(CLine *pLine, CDirentry &entry)
 			if (!time)
 				return 0;
 
-			if (*time)
-			{
+			if (*time) {
 				if (!dateTime.ParseFormat(time, _T("%H%M%S"), dateTime))
 					return 0;
-				entry.flags |= CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time | CDirentry::flag_timestamp_seconds;
+				entry.time = CDateTime(dateTime.FromTimezone(wxDateTime::UTC), CDateTime::seconds);
+			}
+			else {
+				entry.time = CDateTime(dateTime.FromTimezone(wxDateTime::UTC), CDateTime::days);
 			}
-			else
-				entry.flags |= CDirentry::flag_timestamp_date;
-
-			entry.time = dateTime.FromTimezone(wxDateTime::GMT0);
 		}
 		else if (factname == _T("perm"))
 		{
 			if (!value.empty())
 			{
-				if (!entry.permissions.empty())
-					entry.permissions = value + _T(" (") + entry.permissions + _T(")");
+				if (!permissions.empty())
+					permissions = value + _T(" (") + permissions + _T(")");
 				else
-					entry.permissions = value;
+					permissions = value;
 			}
 		}
 		else if (factname == _T("unix.mode"))
 		{
-			if (!entry.permissions.empty())
-				entry.permissions = entry.permissions + _T(" (") + value + _T(")");
+			if (!permissions.empty())
+				permissions = permissions + _T(" (") + value + _T(")");
 			else
-				entry.permissions = value;
+				permissions = value;
 		}
 		else if (factname == _T("unix.owner") || factname == _T("unix.user"))
 			owner = value;
@@ -2763,13 +2705,16 @@ int CDirectoryListingParser::ParseAsMlsd(CLine *pLine, CDirentry &entry)
 	// The order of the facts is undefined, so assemble ownerGroup in correct
 	// order
 	if (!owner.empty())
-		entry.ownerGroup += owner;
+		ownerGroup += owner;
 	else if (!uid.empty())
-		entry.ownerGroup += uid;
+		ownerGroup += uid;
 	if (!group.empty())
-		entry.ownerGroup += _T(" ") + group;
+		ownerGroup += _T(" ") + group;
 	else if (!gid.empty())
-		entry.ownerGroup += _T(" ") + gid;
+		ownerGroup += _T(" ") + gid;
+
+	entry.ownerGroup = objcache.get(ownerGroup);
+	entry.permissions = objcache.get(permissions);
 
 	if (!pLine->GetToken(1, token, true, true))
 		return 0;
@@ -2799,7 +2744,7 @@ bool CDirectoryListingParser::ParseAsOS9(CLine *pLine, CDirentry &entry)
 	if (!token.IsNumeric(pos + 1, token.GetLength() - pos - 1))
 		return false;
 
-	entry.ownerGroup = token.GetString();
+	entry.ownerGroup = objcache.get(token.GetString());
 
 	entry.flags = 0;
 
@@ -2818,7 +2763,7 @@ bool CDirectoryListingParser::ParseAsOS9(CLine *pLine, CDirentry &entry)
 	if (!pLine->GetToken(index++, token))
 		return false;
 
-	entry.permissions = token.GetString();
+	entry.permissions = objcache.get(token.GetString());
 
 	if (token[0] == 'd')
 		entry.flags |= CDirentry::flag_dir;
@@ -2847,7 +2792,7 @@ bool CDirectoryListingParser::ParseAsOS9(CLine *pLine, CDirentry &entry)
 
 void CDirectoryListingParser::Reset()
 {
-	for (std::list<t_list>::iterator iter = m_DataList.begin(); iter != m_DataList.end(); ++iter)
+	for (auto iter = m_DataList.begin(); iter != m_DataList.end(); ++iter)
 		delete [] iter->p;
 	m_DataList.clear();
 
@@ -2929,17 +2874,15 @@ bool CDirectoryListingParser::ParseAsZVM(CLine* pLine, CDirentry &entry)
 	if (!pLine->GetToken(++index, token))
 		return false;
 
-	entry.ownerGroup = token.GetString();
+	entry.ownerGroup = objcache.get(token.GetString());
 
 	// No further token!
 	if (pLine->GetToken(++index, token))
 		return false;
 
-	entry.permissions = _T("");
-	entry.target = _T("");
-
-	if (entry.has_time())
-		entry.time.Add(m_timezoneOffset);
+	entry.permissions = objcache.get(wxString());
+	entry.target.clear();
+	entry.time += m_timezoneOffset;
 
 	return true;
 }
@@ -2986,20 +2929,21 @@ bool CDirectoryListingParser::ParseAsHPNonstop(CLine *pLine, CDirentry &entry)
 	// Owner
 	if (!pLine->GetToken(++index, token))
 		return false;
-	entry.ownerGroup = token.GetString();
+	wxString ownerGroup = token.GetString();
 
 	if (token[token.GetLength() - 1] == ',')
 	{
 		// Owner, part 2
 		if (!pLine->GetToken(++index, token))
 			return false;
-		entry.ownerGroup += _T(" ") + token.GetString();
+		ownerGroup += _T(" ") + token.GetString();
 	}
+	entry.ownerGroup = objcache.get(ownerGroup);
 
 	// Permissions
 	if (!pLine->GetToken(++index, token))
 		return false;
-	entry.permissions = token.GetString();
+	entry.permissions = objcache.get(token.GetString());
 
 	// Nothing
 	if (pLine->GetToken(++index, token))
@@ -3010,7 +2954,7 @@ bool CDirectoryListingParser::ParseAsHPNonstop(CLine *pLine, CDirentry &entry)
 
 bool CDirectoryListingParser::GetMonthFromName(const wxString& name, int &month)
 {
-	std::map<wxString, int>::iterator iter = m_MonthNamesMap.find(name.Lower());
+	auto iter = m_MonthNamesMap.find(name.Lower());
 	if (iter == m_MonthNamesMap.end())
 	{
 		wxString lower(name);
@@ -3027,7 +2971,7 @@ bool CDirectoryListingParser::GetMonthFromName(const wxString& name, int &month)
 
 char ebcdic_table[256] = {
 	' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  // 0
-	' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  '\n', // 1
+	' ',  ' ',  ' ',  ' ',  ' ',  '\n', ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  '\n', // 1
 	' ',  ' ',  ' ',  ' ',  ' ',  '\n', ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  // 2
 	' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  // 3
 	' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  ' ',  '.',  '<',  '(',  '+',  '|',  // 4
@@ -3075,7 +3019,7 @@ void CDirectoryListingParser::DeduceEncoding()
 	for (int i = '0'; i <= '9'; ++i ) {
 		count_normal += count[i];
 	}
-	for (int i = 'a'; i <= 'Z'; ++i ) {
+	for (int i = 'a'; i <= 'z'; ++i ) {
 		count_normal += count[i];
 	}
 	for (int i = 'A'; i <= 'Z'; ++i ) {
@@ -3091,13 +3035,13 @@ void CDirectoryListingParser::DeduceEncoding()
 	for (int i = 0xa2; i <= 0xa9; ++i ) {
 		count_ebcdic += count[i];
 	}
-	for (int i = 0xc1; i <= 0x89; ++i ) {
+	for (int i = 0xc1; i <= 0xc9; ++i ) {
 		count_ebcdic += count[i];
 	}
-	for (int i = 0xd1; i <= 0x99; ++i ) {
+	for (int i = 0xd1; i <= 0xd9; ++i ) {
 		count_ebcdic += count[i];
 	}
-	for (int i = 0xe2; i <= 0xa9; ++i ) {
+	for (int i = 0xe2; i <= 0xe9; ++i ) {
 		count_ebcdic += count[i];
 	}
 	for (int i = 0xf0; i <= 0xf9; ++i ) {
@@ -3105,11 +3049,11 @@ void CDirectoryListingParser::DeduceEncoding()
 	}
 
 
-	if ((count[0x1f] || count[0x25]) && !count[0x0a] && count['@'] && count['@'] > count[' '] && count_ebcdic > count_normal)
+	if ((count[0x1f] || count[0x15] || count[0x25]) && !count[0x0a] && count[static_cast<unsigned char>('@')] && count[static_cast<unsigned char>('@')] > count[static_cast<unsigned char>(' ')] && count_ebcdic > count_normal)
 	{
-		m_pControlSocket->LogMessage(::Status, _("Received a directory listing which appears to be encoded in EBCDIC."));
+		m_pControlSocket->LogMessage(MessageType::Status, _("Received a directory listing which appears to be encoded in EBCDIC."));
 		m_listingEncoding = listingEncoding::ebcdic;
-		for (std::list<t_list>::iterator it = m_DataList.begin(); it != m_DataList.end(); ++it)
+		for (auto it = m_DataList.begin(); it != m_DataList.end(); ++it)
 			ConvertEncoding(it->p, it->len);
 	}
 	else
