@@ -2,6 +2,7 @@
 
 #include "ControlSocket.h"
 #include "httpcontrolsocket.h"
+#include "local_filesys.h"
 #include "tlssocket.h"
 
 #include <wx/file.h>
@@ -51,8 +52,7 @@ public:
 	bool m_gotHeader;
 	int m_responseCode;
 	wxString m_responseString;
-	wxString m_newLocation;
-	wxString m_newHostWithPort;
+	wxURI m_newLocation;
 	int m_redirectionCount;
 
 	wxLongLong m_totalSize;
@@ -110,26 +110,26 @@ CHttpControlSocket::~CHttpControlSocket()
 
 int CHttpControlSocket::SendNextCommand()
 {
-	LogMessage(Debug_Verbose, _T("CHttpControlSocket::SendNextCommand()"));
+	LogMessage(MessageType::Debug_Verbose, _T("CHttpControlSocket::SendNextCommand()"));
 	if (!m_pCurOpData)
 	{
-		LogMessage(__TFILE__, __LINE__, this, Debug_Warning, _T("SendNextCommand called without active operation"));
+		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Warning, _T("SendNextCommand called without active operation"));
 		ResetOperation(FZ_REPLY_ERROR);
 		return FZ_REPLY_ERROR;
 	}
 
 	if (m_pCurOpData->waitForAsyncRequest)
 	{
-		LogMessage(__TFILE__, __LINE__, this, Debug_Info, _T("Waiting for async request, ignoring SendNextCommand"));
+		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("Waiting for async request, ignoring SendNextCommand"));
 		return FZ_REPLY_WOULDBLOCK;
 	}
 
 	switch (m_pCurOpData->opId)
 	{
-	case cmd_transfer:
+	case Command::transfer:
 		return FileTransferSend();
 	default:
-		LogMessage(__TFILE__, __LINE__, this, ::Debug_Warning, _T("Unknown opID (%d) in SendNextCommand"), m_pCurOpData->opId);
+		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Warning, _T("Unknown opID (%d) in SendNextCommand"), m_pCurOpData->opId);
 		ResetOperation(FZ_REPLY_INTERNALERROR);
 		break;
 	}
@@ -140,11 +140,11 @@ int CHttpControlSocket::SendNextCommand()
 
 int CHttpControlSocket::ContinueConnect()
 {
-	LogMessage(__TFILE__, __LINE__, this, Debug_Verbose, _T("CHttpControlSocket::ContinueConnect() m_pEngine=%p"), m_pEngine);
-	if (GetCurrentCommandId() != cmd_connect ||
+	LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Verbose, _T("CHttpControlSocket::ContinueConnect() m_pEngine=%p"), m_pEngine);
+	if (GetCurrentCommandId() != Command::connect ||
 		!m_pCurrentServer)
 	{
-		LogMessage(Debug_Warning, _T("Invalid context for call to ContinueConnect(), cmd=%d, m_pCurrentServer=%p"), GetCurrentCommandId(), m_pCurrentServer);
+		LogMessage(MessageType::Debug_Warning, _T("Invalid context for call to ContinueConnect(), cmd=%d, m_pCurrentServer=%p"), GetCurrentCommandId(), m_pCurrentServer);
 		return DoClose(FZ_REPLY_INTERNALERROR);
 	}
 
@@ -158,7 +158,7 @@ bool CHttpControlSocket::SetAsyncRequestReply(CAsyncRequestNotification *pNotifi
 	{
 		if (!m_pCurOpData->waitForAsyncRequest)
 		{
-			LogMessage(__TFILE__, __LINE__, this, Debug_Info, _T("Not waiting for request reply, ignoring request reply %d"), pNotification->GetRequestID());
+			LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("Not waiting for request reply, ignoring request reply %d"), pNotification->GetRequestID());
 			return false;
 		}
 		m_pCurOpData->waitForAsyncRequest = false;
@@ -168,28 +168,21 @@ bool CHttpControlSocket::SetAsyncRequestReply(CAsyncRequestNotification *pNotifi
 	{
 	case reqId_fileexists:
 		{
-			if (!m_pCurOpData || m_pCurOpData->opId != cmd_transfer)
+			if (!m_pCurOpData || m_pCurOpData->opId != Command::transfer)
 			{
-				LogMessage(__TFILE__, __LINE__, this, Debug_Info, _T("No or invalid operation in progress, ignoring request reply %f"), pNotification->GetRequestID());
+				LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("No or invalid operation in progress, ignoring request reply %f"), pNotification->GetRequestID());
 				return false;
 			}
 
 			CFileExistsNotification *pFileExistsNotification = reinterpret_cast<CFileExistsNotification *>(pNotification);
-			switch (pFileExistsNotification->overwriteAction)
-			{
-			case CFileExistsNotification::resume:
-				ResetOperation(FZ_REPLY_CRITICALERROR | FZ_REPLY_NOTSUPPORTED);
-				break;
-			default:
-				return SetFileExistsAction(pFileExistsNotification);
-			}
+			return SetFileExistsAction(pFileExistsNotification);
 		}
 		break;
 	case reqId_certificate:
 		{
 			if (!m_pTlsSocket || m_pTlsSocket->GetState() != CTlsSocket::verifycert)
 			{
-				LogMessage(__TFILE__, __LINE__, this, Debug_Info, _T("No or invalid operation in progress, ignoring request reply %d"), pNotification->GetRequestID());
+				LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("No or invalid operation in progress, ignoring request reply %d"), pNotification->GetRequestID());
 				return false;
 			}
 
@@ -198,7 +191,7 @@ bool CHttpControlSocket::SetAsyncRequestReply(CAsyncRequestNotification *pNotifi
 		}
 		break;
 	default:
-		LogMessage(__TFILE__, __LINE__, this, Debug_Warning, _T("Unknown request %d"), pNotification->GetRequestID());
+		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Warning, _T("Unknown request %d"), pNotification->GetRequestID());
 		ResetOperation(FZ_REPLY_INTERNALERROR);
 		return false;
 	}
@@ -239,7 +232,7 @@ int CHttpControlSocket::DoReceive()
 
 		m_pEngine->SetActive(CFileZillaEngine::recv);
 
-		if (!m_pCurOpData || m_pCurOpData->opId == cmd_connect)
+		if (!m_pCurOpData || m_pCurOpData->opId == Command::connect)
 		{
 			// Just ignore all further data
 			m_recvBufferPos = 0;
@@ -294,7 +287,7 @@ int CHttpControlSocket::DoReceive()
 
 void CHttpControlSocket::OnConnect()
 {
-	wxASSERT(GetCurrentCommandId() == cmd_connect);
+	wxASSERT(GetCurrentCommandId() == Command::connect);
 
 	CHttpConnectOpData *pData = static_cast<CHttpConnectOpData *>(m_pCurOpData);
 
@@ -302,7 +295,7 @@ void CHttpControlSocket::OnConnect()
 	{
 		if (!m_pTlsSocket)
 		{
-			LogMessage(Status, _("Connection established, initializing TLS..."));
+			LogMessage(MessageType::Status, _("Connection established, initializing TLS..."));
 
 			delete m_pBackend;
 			m_pTlsSocket = new CTlsSocket(this, m_pSocket, this);
@@ -310,15 +303,15 @@ void CHttpControlSocket::OnConnect()
 
 			if (!m_pTlsSocket->Init())
 			{
-				LogMessage(::Error, _("Failed to initialize TLS."));
+				LogMessage(MessageType::Error, _("Failed to initialize TLS."));
 				DoClose();
 				return;
 			}
 
 			const wxString trusted_rootcert = m_pEngine->GetOptions()->GetOption(OPTION_INTERNAL_ROOTCERT);
-			if (trusted_rootcert != _T("") && !m_pTlsSocket->AddTrustedRootCertificate(trusted_rootcert))
+			if (!trusted_rootcert.empty() && !m_pTlsSocket->AddTrustedRootCertificate(trusted_rootcert))
 			{
-				LogMessage(::Error, _("Failed to parse trusted root cert."));
+				LogMessage(MessageType::Error, _("Failed to parse trusted root cert."));
 				DoClose();
 				return;
 			}
@@ -329,7 +322,7 @@ void CHttpControlSocket::OnConnect()
 		}
 		else
 		{
-			LogMessage(Status, _("TLS/SSL connection established, sending HTTP request"));
+			LogMessage(MessageType::Status, _("TLS/SSL connection established, sending HTTP request"));
 			ResetOperation(FZ_REPLY_OK);
 		}
 
@@ -337,7 +330,7 @@ void CHttpControlSocket::OnConnect()
 	}
 	else
 	{
-		LogMessage(Status, _("Connection established, sending HTTP request"));
+		LogMessage(MessageType::Status, _("Connection established, sending HTTP request"));
 		ResetOperation(FZ_REPLY_OK);
 	}
 }
@@ -351,11 +344,11 @@ enum filetransferStates
 
 int CHttpControlSocket::FileTransfer(const wxString localFile, const CServerPath &remotePath,
 							  const wxString &remoteFile, bool download,
-							  const CFileTransferCommand::t_transferSettings& transferSettings)
+							  const CFileTransferCommand::t_transferSettings&)
 {
-	LogMessage(Debug_Verbose, _T("CHttpControlSocket::FileTransfer()"));
+	LogMessage(MessageType::Debug_Verbose, _T("CHttpControlSocket::FileTransfer()"));
 
-	LogMessage(Status, _("Downloading %s"), remotePath.FormatFilename(remoteFile).c_str());
+	LogMessage(MessageType::Status, _("Downloading %s"), remotePath.FormatFilename(remoteFile));
 
 	if (!download)
 	{
@@ -365,7 +358,7 @@ int CHttpControlSocket::FileTransfer(const wxString localFile, const CServerPath
 
 	if (m_pCurOpData)
 	{
-		LogMessage(__TFILE__, __LINE__, this, Debug_Info, _T("deleting nonzero pData"));
+		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("deleting nonzero pData"));
 		delete m_pCurOpData;
 	}
 
@@ -373,8 +366,11 @@ int CHttpControlSocket::FileTransfer(const wxString localFile, const CServerPath
 	m_pCurOpData = pData;
 	m_pHttpOpData = pData;
 
-	if (localFile != _T(""))
-	{
+	m_current_uri = wxURI(m_pCurrentServer->FormatServer() + pData->remotePath.FormatFilename(pData->remoteFile));
+
+	if (!localFile.empty()) {
+		pData->localFileSize = CLocalFileSystem::GetSize(pData->localFile).GetValue();
+
 		pData->opState = filetransfer_waitfileexists;
 		int res = CheckOverwriteFile();
 		if (res != FZ_REPLY_OK)
@@ -382,16 +378,9 @@ int CHttpControlSocket::FileTransfer(const wxString localFile, const CServerPath
 
 		pData->opState = filetransfer_transfer;
 
-		pData->pFile = new wxFile();
-
-		CreateLocalDir(pData->localFile);
-
-		if (!pData->pFile->Open(pData->localFile, wxFile::write))
-		{
-			LogMessage(::Error, _("Failed to open \"%s\" for writing"), pData->localFile.c_str());
-			ResetOperation(FZ_REPLY_ERROR);
-			return FZ_REPLY_ERROR;
-		}
+		res = OpenFile(pData);
+		if( res != FZ_REPLY_OK )
+			return res;
 	}
 	else
 		pData->opState = filetransfer_transfer;
@@ -405,11 +394,11 @@ int CHttpControlSocket::FileTransfer(const wxString localFile, const CServerPath
 
 int CHttpControlSocket::FileTransferSubcommandResult(int prevResult)
 {
-	LogMessage(Debug_Verbose, _T("CHttpControlSocket::FileTransferSubcommandResult(%d)"), prevResult);
+	LogMessage(MessageType::Debug_Verbose, _T("CHttpControlSocket::FileTransferSubcommandResult(%d)"), prevResult);
 
 	if (!m_pCurOpData)
 	{
-		LogMessage(__TFILE__, __LINE__, this, Debug_Info, _T("Empty m_pCurOpData"));
+		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("Empty m_pCurOpData"));
 		ResetOperation(FZ_REPLY_INTERNALERROR);
 		return FZ_REPLY_ERROR;
 	}
@@ -425,11 +414,17 @@ int CHttpControlSocket::FileTransferSubcommandResult(int prevResult)
 
 int CHttpControlSocket::FileTransferSend()
 {
-	LogMessage(Debug_Verbose, _T("CHttpControlSocket::FileTransferSend()"));
+	LogMessage(MessageType::Debug_Verbose, _T("CHttpControlSocket::FileTransferSend()"));
 
 	if (!m_pCurOpData)
 	{
-		LogMessage(__TFILE__, __LINE__, this, Debug_Info, _T("Empty m_pCurOpData"));
+		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("Empty m_pCurOpData"));
+		ResetOperation(FZ_REPLY_INTERNALERROR);
+		return FZ_REPLY_ERROR;
+	}
+
+	if( !m_current_uri.HasScheme() || !m_current_uri.HasServer() || !m_current_uri.HasPath() ) {
+		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Warning, _T("Invalid URI: %s"), m_current_uri.BuildURI());
 		ResetOperation(FZ_REPLY_INTERNALERROR);
 		return FZ_REPLY_ERROR;
 	}
@@ -439,42 +434,32 @@ int CHttpControlSocket::FileTransferSend()
 	if (pData->opState == filetransfer_waitfileexists)
 	{
 		pData->opState = filetransfer_transfer;
-		pData->pFile = new wxFile();
 
-		CreateLocalDir(pData->localFile);
+		int res = OpenFile(pData);
+		if( res != FZ_REPLY_OK)
+			return res;
 
-		if (!pData->pFile->Open(pData->localFile, wxFile::write))
-		{
-			LogMessage(::Error, _("Failed to open \"%s\" for writing"), pData->localFile.c_str());
-			ResetOperation(FZ_REPLY_ERROR);
-			return FZ_REPLY_ERROR;
-		}
-
-		int res = InternalConnect(m_pCurrentServer->GetHost(), m_pCurrentServer->GetPort(), m_pCurrentServer->GetProtocol() == HTTPS);
+		res = InternalConnect(m_pCurrentServer->GetHost(), m_pCurrentServer->GetPort(), m_pCurrentServer->GetProtocol() == HTTPS);
 		if (res != FZ_REPLY_OK)
 			return res;
 	}
 
-	wxString location;
-	wxString hostWithPort;
-	if (pData->m_newLocation == _T(""))
-	{
-		if (m_pCurrentServer->GetProtocol() == HTTPS)
-			location = _T("https://") + wxString(m_pCurrentServer->FormatHost()) + wxString(pData->remotePath.FormatFilename(pData->remoteFile).c_str());
-		else
-			location = _T("http://") + wxString(m_pCurrentServer->FormatHost()) + wxString(pData->remotePath.FormatFilename(pData->remoteFile).c_str());
-		hostWithPort = wxString::Format(_T("%s:%d"), m_pCurrentServer->FormatHost(true).c_str(), m_pCurrentServer->GetPort());
+	wxString location = m_current_uri.GetPath();
+	if( m_current_uri.HasQuery() ) {
+		location += _T("?") + m_current_uri.GetQuery();
 	}
-	else
-	{
-		location = pData->m_newLocation;
-		hostWithPort = pData->m_newHostWithPort;
+	wxString action = wxString::Format(_T("GET %s HTTP/1.1"), location );
+	LogMessageRaw(MessageType::Command, action);
+
+	wxString hostWithPort = m_current_uri.GetServer();
+	if( m_current_uri.HasPort() ) {
+		hostWithPort += _T(":") + m_current_uri.GetPort();
 	}
-
-	wxString action = wxString::Format(_T("GET %s HTTP/1.1"), location.c_str());
-	LogMessageRaw(Command, action);
-
-	wxString command = wxString::Format(_T("%s\r\nHost: %s\r\nUser-Agent: %s\r\nConnection: close\r\n\r\n"), action.c_str(), hostWithPort.c_str(), wxString(PACKAGE_STRING, wxConvLocal).c_str());
+	wxString command = wxString::Format(_T("%s\r\nHost: %s\r\nUser-Agent: %s\r\nConnection: close\r\n"), action, hostWithPort, wxString(PACKAGE_STRING, wxConvLocal));
+	if( pData->resume ) {
+		command += wxString::Format(_T("Range: bytes=%") + wxString(wxFileOffsetFmtSpec) + _T("d-\r\n"), pData->localFileSize);
+	}
+	command += _T("\r\n");
 
 	const wxWX2MBbuf str = command.mb_str();
 	if (!Send(str, strlen(str)))
@@ -485,7 +470,7 @@ int CHttpControlSocket::FileTransferSend()
 
 int CHttpControlSocket::InternalConnect(wxString host, unsigned short port, bool tls)
 {
-	LogMessage(Debug_Verbose, _T("CHttpControlSocket::InternalConnect()"));
+	LogMessage(MessageType::Debug_Verbose, _T("CHttpControlSocket::InternalConnect()"));
 
 	CHttpConnectOpData* pData = new CHttpConnectOpData;
 	pData->pNextOpData = m_pCurOpData;
@@ -494,7 +479,7 @@ int CHttpControlSocket::InternalConnect(wxString host, unsigned short port, bool
 	pData->tls = tls;
 
 	if (!IsIpAddress(host))
-		LogMessage(Status, _("Resolving address of %s"), host.c_str());
+		LogMessage(MessageType::Status, _("Resolving address of %s"), host);
 
 	pData->host = host;
 	return DoInternalConnect();
@@ -502,11 +487,11 @@ int CHttpControlSocket::InternalConnect(wxString host, unsigned short port, bool
 
 int CHttpControlSocket::DoInternalConnect()
 {
-	LogMessage(Debug_Verbose, _T("CHttpControlSocket::DoInternalConnect()"));
+	LogMessage(MessageType::Debug_Verbose, _T("CHttpControlSocket::DoInternalConnect()"));
 
 	if (!m_pCurOpData)
 	{
-		LogMessage(__TFILE__, __LINE__, this, Debug_Info, _T("Empty m_pCurOpData"));
+		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("Empty m_pCurOpData"));
 		ResetOperation(FZ_REPLY_INTERNALERROR);
 		return FZ_REPLY_ERROR;
 	}
@@ -528,11 +513,11 @@ int CHttpControlSocket::DoInternalConnect()
 
 int CHttpControlSocket::FileTransferParseResponse(char* p, unsigned int len)
 {
-	LogMessage(Debug_Verbose, _T("CHttpControlSocket::FileTransferParseResponse(%p, %d)"), p, len);
+	LogMessage(MessageType::Debug_Verbose, _T("CHttpControlSocket::FileTransferParseResponse(%p, %d)"), p, len);
 
 	if (!m_pCurOpData)
 	{
-		LogMessage(__TFILE__, __LINE__, this, Debug_Info, _T("Empty m_pCurOpData"));
+		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("Empty m_pCurOpData"));
 		ResetOperation(FZ_REPLY_INTERNALERROR);
 		return FZ_REPLY_ERROR;
 	}
@@ -551,7 +536,7 @@ int CHttpControlSocket::FileTransferParseResponse(char* p, unsigned int len)
 		SetTransferStatusStartTime();
 	}
 
-	if (pData->localFile == _T(""))
+	if (pData->localFile.empty())
 	{
 		char* q = new char[len];
 		memcpy(q, p, len);
@@ -563,7 +548,7 @@ int CHttpControlSocket::FileTransferParseResponse(char* p, unsigned int len)
 
 		if (pData->pFile->Write(p, len) != len)
 		{
-			LogMessage(::Error, _("Failed to write to file %s"), pData->localFile.c_str());
+			LogMessage(MessageType::Error, _("Failed to write to file %s"), pData->localFile);
 			ResetOperation(FZ_REPLY_ERROR);
 			return FZ_REPLY_ERROR;
 		}
@@ -590,7 +575,7 @@ int CHttpControlSocket::ParseHeader(CHttpOpData* pData)
 			{
 				if (m_pRecvBuffer[i + 1] != '\n')
 				{
-					LogMessage(::Error, _("Malformed reply, server not sending proper line endings"));
+					LogMessage(MessageType::Error, _("Malformed reply, server not sending proper line endings"));
 					ResetOperation(FZ_REPLY_ERROR);
 					return FZ_REPLY_ERROR;
 				}
@@ -602,7 +587,7 @@ int CHttpControlSocket::ParseHeader(CHttpOpData* pData)
 			if (m_recvBufferPos == m_recvBufferLen)
 			{
 				// We don't support header lines larger than 4096
-				LogMessage(::Error, _("Too long header line"));
+				LogMessage(MessageType::Error, _("Too long header line"));
 				ResetOperation(FZ_REPLY_ERROR);
 				return FZ_REPLY_ERROR;
 			}
@@ -611,8 +596,8 @@ int CHttpControlSocket::ParseHeader(CHttpOpData* pData)
 
 		m_pRecvBuffer[i] = 0;
 		const wxString& line = wxString(m_pRecvBuffer, wxConvLocal);
-		if (line != _T(""))
-			LogMessageRaw(Response, line);
+		if (!line.empty())
+			LogMessageRaw(MessageType::Response, line);
 
 		if (pData->m_responseCode == -1)
 		{
@@ -620,7 +605,7 @@ int CHttpControlSocket::ParseHeader(CHttpOpData* pData)
 			if (m_recvBufferPos < 16 || memcmp(m_pRecvBuffer, "HTTP/1.", 7))
 			{
 				// Invalid HTTP Status-Line
-				LogMessage(::Error, _("Invalid HTTP Response"));
+				LogMessage(MessageType::Error, _("Invalid HTTP Response"));
 				ResetOperation(FZ_REPLY_ERROR);
 				return FZ_REPLY_ERROR;
 			}
@@ -630,13 +615,28 @@ int CHttpControlSocket::ParseHeader(CHttpOpData* pData)
 				m_pRecvBuffer[11] < '0' || m_pRecvBuffer[11] > '9')
 			{
 				// Invalid response code
-				LogMessage(::Error, _("Invalid response code"));
+				LogMessage(MessageType::Error, _("Invalid response code"));
 				ResetOperation(FZ_REPLY_ERROR);
 				return FZ_REPLY_ERROR;
 			}
 
 			pData->m_responseCode = (m_pRecvBuffer[9] - '0') * 100 + (m_pRecvBuffer[10] - '0') * 10 + m_pRecvBuffer[11] - '0';
 
+			if( pData->m_responseCode == 416 ) {
+				CHttpFileTransferOpData* pTransfer = reinterpret_cast<CHttpFileTransferOpData*>(pData->m_pOpData);
+				if( pTransfer->resume ) {
+					// Sad, the server does not like our attempt to resume.
+					// Get full file instead.
+					pTransfer->resume = false;
+					int res = OpenFile(pTransfer);
+					if( res != FZ_REPLY_OK ) {
+						return res;
+					}
+					pData->m_newLocation = m_current_uri;
+					pData->m_responseCode = 300;
+				}
+			}
+
 			if (pData->m_responseCode >= 400)
 			{
 				// Failed request
@@ -647,7 +647,7 @@ int CHttpControlSocket::ParseHeader(CHttpOpData* pData)
 			if (pData->m_responseCode == 305)
 			{
 				// Unsupported redirect
-				LogMessage(::Error, _("Unsupported redirect"));
+				LogMessage(MessageType::Error, _("Unsupported redirect"));
 				ResetOperation(FZ_REPLY_ERROR);
 				return FZ_REPLY_ERROR;
 			}
@@ -661,9 +661,8 @@ int CHttpControlSocket::ParseHeader(CHttpOpData* pData)
 				// Redirect if neccessary
 				if (pData->m_responseCode >= 300)
 				{
-					if (pData->m_redirectionCount++ == 5)
-					{
-						LogMessage(::Error, _("Too many redirects"));
+					if (pData->m_redirectionCount++ == 6) {
+						LogMessage(MessageType::Error, _("Too many redirects"));
 						ResetOperation(FZ_REPLY_ERROR);
 						return FZ_REPLY_ERROR;
 					}
@@ -671,62 +670,48 @@ int CHttpControlSocket::ParseHeader(CHttpOpData* pData)
 					ResetSocket();
 					ResetHttpData(pData);
 
-					wxString host;
-					enum ServerProtocol protocol;
-					int pos;
-					if ((pos = pData->m_newLocation.Find(_T("://"))) != -1)
-					{
-						protocol = CServer::GetProtocolFromPrefix(pData->m_newLocation.Left(pos));
-						host = pData->m_newLocation.Mid(pos + 3);
-					}
-					else
-					{
-						protocol = HTTP;
-						host = pData->m_newLocation;
-					}
-
-					if ((pos = host.Find(_T("/"))) != -1)
-						host = host.Left(pos);
-
-					unsigned long port;
-					if ((pos = host.Find(':', true)) != -1)
-					{
-						wxString strport = host.Mid(pos + 1);
-						if (!strport.ToULong(&port) || port < 1 || port > 65535)
-						{
-							if (protocol == HTTPS)
-								port = 443;
-							else
-								port = 80;
-						}
-						host = host.Left(pos);
-					}
-					else
-					{
-						if (protocol == HTTPS)
-							port = 443;
-						else
-							port = 80;
-					}
-
-					if (host == _T(""))
-					{
-						// Unsupported redirect
-						LogMessage(::Error, _("Redirection to invalid address"));
+					if( !pData->m_newLocation.HasScheme() || !pData->m_newLocation.HasServer() || !pData->m_newLocation.HasPath() ) {
+						LogMessage(MessageType::Error, _("Redirection to invalid or unsupported URI: %s"), m_current_uri.BuildURI());
 						ResetOperation(FZ_REPLY_ERROR);
 						return FZ_REPLY_ERROR;
 					}
-					pData->m_newHostWithPort = wxString::Format(_T("%s:%d"), host.c_str(), (int)port);
+
+					enum ServerProtocol protocol = CServer::GetProtocolFromPrefix(pData->m_newLocation.GetScheme());
+					if( protocol != HTTP && protocol != HTTPS ) {
+						LogMessage(MessageType::Error, _("Redirection to invalid or unsupported address: %s"), pData->m_newLocation.BuildURI());
+						ResetOperation(FZ_REPLY_ERROR);
+						return FZ_REPLY_ERROR;
+					}
+
+					long port = CServer::GetDefaultPort(protocol);
+					if( pData->m_newLocation.HasPort() && (!pData->m_newLocation.GetPort().ToLong(&port) || port < 1 || port > 65535) ) {
+						LogMessage(MessageType::Error, _("Redirection to invalid or unsupported address: %s"), pData->m_newLocation.BuildURI());
+						ResetOperation(FZ_REPLY_ERROR);
+						return FZ_REPLY_ERROR;
+					}
+
+					m_current_uri = pData->m_newLocation;
 
 					// International domain names
-					host = ConvertDomainName(host);
+					wxString host = ConvertDomainName(m_current_uri.GetServer());
 
-					int res = InternalConnect(host, port, protocol == HTTPS);
+					int res = InternalConnect(host, static_cast<unsigned short>(port), protocol == HTTPS);
 					if (res == FZ_REPLY_WOULDBLOCK)
 						res |= FZ_REPLY_REDIRECTED;
 					return res;
 				}
 
+				if( pData->m_pOpData && pData->m_pOpData->opId == Command::transfer) {
+					CHttpFileTransferOpData* pTransfer = reinterpret_cast<CHttpFileTransferOpData*>(pData->m_pOpData);
+					if( pTransfer->resume && pData->m_responseCode != 206 ) {
+						pTransfer->resume = false;
+						int res = OpenFile(pTransfer);
+						if( res != FZ_REPLY_OK ) {
+							return res;
+						}
+					}
+				}
+
 				pData->m_gotHeader = true;
 
 				memmove(m_pRecvBuffer, m_pRecvBuffer + 2, m_recvBufferPos - 2);
@@ -750,7 +735,8 @@ int CHttpControlSocket::ParseHeader(CHttpOpData* pData)
 			}
 			if (m_recvBufferPos > 12 && !memcmp(m_pRecvBuffer, "Location: ", 10))
 			{
-				pData->m_newLocation = wxString(m_pRecvBuffer + 10, wxConvLocal);
+				pData->m_newLocation = wxURI(wxString(m_pRecvBuffer + 10, wxConvLocal));
+				pData->m_newLocation.Resolve(m_current_uri);
 			}
 			else if (m_recvBufferPos > 21 && !memcmp(m_pRecvBuffer, "Transfer-Encoding: ", 19))
 			{
@@ -769,7 +755,7 @@ int CHttpControlSocket::ParseHeader(CHttpOpData* pData)
 				{
 					if (*p < '0' || *p > '9')
 					{
-						LogMessage(::Error, _("Malformed header: %s"), _("Invalid Content-Length"));
+						LogMessage(MessageType::Error, _("Malformed header: %s"), _("Invalid Content-Length"));
 						ResetOperation(FZ_REPLY_ERROR);
 						return FZ_REPLY_ERROR;
 					}
@@ -824,7 +810,7 @@ int CHttpControlSocket::OnChunkedData(CHttpOpData* pData)
 			{
 				if (p[i + 1] != '\n')
 				{
-					LogMessage(::Error, _("Malformed chunk data: %s"), _("Wrong line endings"));
+					LogMessage(MessageType::Error, _("Malformed chunk data: %s"), _("Wrong line endings"));
 					ResetOperation(FZ_REPLY_ERROR);
 					return FZ_REPLY_ERROR;
 				}
@@ -836,7 +822,7 @@ int CHttpControlSocket::OnChunkedData(CHttpOpData* pData)
 			if (len == m_recvBufferLen)
 			{
 				// We don't support lines larger than 4096
-				LogMessage(::Error, _("Malformed chunk data: %s"), _("Line length exceeded"));
+				LogMessage(MessageType::Error, _("Malformed chunk data: %s"), _("Line length exceeded"));
 				ResetOperation(FZ_REPLY_ERROR);
 				return FZ_REPLY_ERROR;
 			}
@@ -851,7 +837,7 @@ int CHttpControlSocket::OnChunkedData(CHttpOpData* pData)
 			{
 				// The chunk data has to end with CRLF. If i is nonzero,
 				// it didn't end with just CRLF.
-				LogMessage(::Error, _("Malformed chunk data: %s"), _("Chunk data improperly terminated"));
+				LogMessage(MessageType::Error, _("Malformed chunk data: %s"), _("Chunk data improperly terminated"));
 				ResetOperation(FZ_REPLY_ERROR);
 				return FZ_REPLY_ERROR;
 			}
@@ -870,34 +856,23 @@ int CHttpControlSocket::OnChunkedData(CHttpOpData* pData)
 		else
 		{
 			// Read chunk size
-			char* q = p;
-			while (*q)
-			{
-				if (*q >= '0' && *q <= '9')
-				{
-					pData->m_chunkData.size *= 16;
+			for( char* q = p; *q && *q != ';' && *q != ' '; ++q ) {
+				pData->m_chunkData.size *= 16;
+				if (*q >= '0' && *q <= '9') {
 					pData->m_chunkData.size += *q - '0';
 				}
-				else if (*q >= 'A' && *q <= 'F')
-				{
-					pData->m_chunkData.size *= 16;
+				else if (*q >= 'A' && *q <= 'F') {
 					pData->m_chunkData.size += *q - 'A' + 10;
 				}
-				else if (*q >= 'a' && *q <= 'f')
-				{
-					pData->m_chunkData.size *= 16;
+				else if (*q >= 'a' && *q <= 'f') {
 					pData->m_chunkData.size += *q - 'a' + 10;
 				}
-				else if (*q == ';' || *q == ' ')
-					break;
-				else
-				{
+				else {
 					// Invalid size
-					LogMessage(::Error, _("Malformed chunk data: %s"), _("Invalid chunk size"));
+					LogMessage(MessageType::Error, _("Malformed chunk data: %s"), _("Invalid chunk size"));
 					ResetOperation(FZ_REPLY_ERROR);
 					return FZ_REPLY_ERROR;
 				}
-				q++;
 			}
 			if (pData->m_chunkData.size == 0)
 				pData->m_chunkData.getTrailer = true;
@@ -921,7 +896,7 @@ int CHttpControlSocket::OnChunkedData(CHttpOpData* pData)
 
 int CHttpControlSocket::ResetOperation(int nErrorCode)
 {
-	if (m_pCurOpData && m_pCurOpData->opId == cmd_transfer)
+	if (m_pCurOpData && m_pCurOpData->opId == Command::transfer)
 	{
 		CHttpFileTransferOpData *pData = static_cast<CHttpFileTransferOpData *>(m_pCurOpData);
 		delete pData->pFile;
@@ -933,9 +908,9 @@ int CHttpControlSocket::ResetOperation(int nErrorCode)
 		if (m_pBackend)
 		{
 			if (nErrorCode == FZ_REPLY_OK)
-				LogMessage(Status, _("Disconnected from server"));
+				LogMessage(MessageType::Status, _("Disconnected from server"));
 			else
-				LogMessage(::Error, _("Disconnected from server"));
+				LogMessage(MessageType::Error, _("Disconnected from server"));
 		}
 		ResetSocket();
 		m_pHttpOpData = 0;
@@ -946,18 +921,15 @@ int CHttpControlSocket::ResetOperation(int nErrorCode)
 
 void CHttpControlSocket::OnClose(int error)
 {
-	LogMessage(Debug_Verbose, _T("CHttpControlSocket::OnClose(%d)"), error);
+	LogMessage(MessageType::Debug_Verbose, _T("CHttpControlSocket::OnClose(%d)"), error);
 
 	if (error)
 	{
-		LogMessage(::Error, _("Disconnected from server: %s"), CSocket::GetErrorDescription(error).c_str());
+		LogMessage(MessageType::Error, _("Disconnected from server: %s"), CSocket::GetErrorDescription(error));
 		ResetOperation(FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED);
 		return;
 	}
 
-	if (DoReceive() == FZ_REPLY_REDIRECTED)
-		return; // Socket got closed already
-
 	// HTTP socket isn't connected outside operations
 	if (!m_pCurOpData)
 		return;
@@ -1016,14 +988,14 @@ void CHttpControlSocket::ResetHttpData(CHttpOpData* pData)
 int CHttpControlSocket::ProcessData(char* p, int len)
 {
 	int res;
-	enum Command commandId = GetCurrentCommandId();
+	Command commandId = GetCurrentCommandId();
 	switch (commandId)
 	{
-	case cmd_transfer:
+	case Command::transfer:
 		res = FileTransferParseResponse(p, len);
 		break;
 	default:
-		LogMessage(Debug_Warning, _T("No action for parsing data for command %d"), (int)commandId);
+		LogMessage(MessageType::Debug_Warning, _T("No action for parsing data for command %d"), (int)commandId);
 		ResetOperation(FZ_REPLY_INTERNALERROR);
 		res = FZ_REPLY_ERROR;
 		break;
@@ -1036,20 +1008,20 @@ int CHttpControlSocket::ProcessData(char* p, int len)
 
 int CHttpControlSocket::ParseSubcommandResult(int prevResult)
 {
-	LogMessage(Debug_Verbose, _T("CHttpControlSocket::SendNextCommand(%d)"), prevResult);
+	LogMessage(MessageType::Debug_Verbose, _T("CHttpControlSocket::SendNextCommand(%d)"), prevResult);
 	if (!m_pCurOpData)
 	{
-		LogMessage(__TFILE__, __LINE__, this, Debug_Warning, _T("SendNextCommand called without active operation"));
+		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Warning, _T("SendNextCommand called without active operation"));
 		ResetOperation(FZ_REPLY_ERROR);
 		return FZ_REPLY_ERROR;
 	}
 
 	switch (m_pCurOpData->opId)
 	{
-	case cmd_transfer:
+	case Command::transfer:
 		return FileTransferSubcommandResult(prevResult);
 	default:
-		LogMessage(__TFILE__, __LINE__, this, ::Debug_Warning, _T("Unknown opID (%d) in SendNextCommand"), m_pCurOpData->opId);
+		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Warning, _T("Unknown opID (%d) in SendNextCommand"), m_pCurOpData->opId);
 		ResetOperation(FZ_REPLY_INTERNALERROR);
 		break;
 	}
@@ -1062,3 +1034,23 @@ int CHttpControlSocket::Disconnect()
 	DoClose();
 	return FZ_REPLY_OK;
 }
+
+int CHttpControlSocket::OpenFile( CHttpFileTransferOpData* pData)
+{
+	delete pData->pFile;
+	pData->pFile = new wxFile();
+	CreateLocalDir(pData->localFile);
+
+	if (!pData->pFile->Open(pData->localFile, pData->resume ? wxFile::write_append : wxFile::write))
+	{
+		LogMessage(MessageType::Error, _("Failed to open \"%s\" for writing"), pData->localFile);
+		ResetOperation(FZ_REPLY_ERROR);
+		return FZ_REPLY_ERROR;
+	}
+	wxFileOffset end = pData->pFile->SeekEnd();
+	if( !end ) {
+		pData->resume = false;
+	}
+	pData->localFileSize = CLocalFileSystem::GetSize(pData->localFile).GetValue();
+	return FZ_REPLY_OK;
+}
