@@ -30,16 +30,6 @@ static const CServerTypeTraits traits[SERVERTYPE_MAX] = {
 	{ _T("/"),   true,     0,    0,    false, 0, 0,   true,  true  }  // Cygwin is like Unix but has optional prefix of form "//server"
 };
 
-CServerPathData::CServerPathData()
-{
-}
-
-CServerPathData::CServerPathData(const CServerPathData& v)
-	: m_segments(v.m_segments)
-	, m_prefix(v.m_prefix)
-{
-}
-
 bool CServerPathData::operator==(const CServerPathData& cmp) const
 {
 	if (m_prefix != cmp.m_prefix)
@@ -62,11 +52,11 @@ CServerPath::CServerPath(const CServerPath &path, wxString subdir)
 	, m_type(path.m_type)
 	, m_data(path.m_data)
 {
-	if (subdir == _T(""))
+	if (subdir.empty())
 		return;
 
 	if (!ChangePath(subdir))
-		Clear();
+		clear();
 }
 
 CServerPath::CServerPath(const CServerPath &path)
@@ -76,17 +66,13 @@ CServerPath::CServerPath(const CServerPath &path)
 {
 }
 
-CServerPath::CServerPath(wxString path, ServerType type /*=DEFAULT*/)
+CServerPath::CServerPath(wxString const& path, ServerType type /*=DEFAULT*/)
 	: m_type(type)
 {
 	SetPath(path);
 }
 
-CServerPath::~CServerPath()
-{
-}
-
-void CServerPath::Clear()
+void CServerPath::clear()
 {
 	m_bEmpty = true;
 	m_type = DEFAULT;
@@ -103,7 +89,7 @@ bool CServerPath::SetPath(wxString &newPath, bool isFile)
 	wxString path = newPath;
 	wxString file;
 
-	if (path == _T(""))
+	if (path.empty())
 		return false;
 
 	if (m_type == DEFAULT)
@@ -150,31 +136,28 @@ bool CServerPath::SetPath(wxString &newPath, bool isFile)
 wxString CServerPath::GetPath() const
 {
 	if (m_bEmpty)
-		return _T("");
+		return wxString();
 
 	wxString path;
 
-	if (!traits[m_type].prefixmode)
-		path = m_data->m_prefix;
+	if (!traits[m_type].prefixmode && m_data->m_prefix)
+		path = *m_data->m_prefix;
 
 	if (traits[m_type].left_enclosure != 0)
 		path += traits[m_type].left_enclosure;
-	if (m_data->m_segments.empty() && (!traits[m_type].has_root || m_data->m_prefix.empty() || traits[m_type].separator_after_prefix))
+	if (m_data->m_segments.empty() && (!traits[m_type].has_root || !m_data->m_prefix || traits[m_type].separator_after_prefix))
 		path += traits[m_type].separators[0];
 
-	for (tConstSegmentIter iter = m_data->m_segments.begin(); iter != m_data->m_segments.end(); ++iter)
-	{
+	for (tConstSegmentIter iter = m_data->m_segments.begin(); iter != m_data->m_segments.end(); ++iter) {
 		const wxString& segment = *iter;
 		if (iter != m_data->m_segments.begin())
 			path += traits[m_type].separators[0];
-		else if (traits[m_type].has_root)
-		{
-			if (m_data->m_prefix.empty() || traits[m_type].separator_after_prefix)
+		else if (traits[m_type].has_root) {
+			if (!m_data->m_prefix || traits[m_type].separator_after_prefix)
 				path += traits[m_type].separators[0];
 		}
 
-		if (traits[m_type].separatorEscape)
-		{
+		if (traits[m_type].separatorEscape) {
 			wxString tmp = segment;
 			EscapeSeparators(m_type, tmp);
 			path += tmp;
@@ -183,8 +166,8 @@ wxString CServerPath::GetPath() const
 			path += segment;
 	}
 
-	if (traits[m_type].prefixmode)
-		path += m_data->m_prefix;
+	if (traits[m_type].prefixmode && m_data->m_prefix)
+		path += *m_data->m_prefix;
 
 	if (traits[m_type].right_enclosure != 0)
 		path += traits[m_type].right_enclosure;
@@ -219,7 +202,7 @@ CServerPath CServerPath::GetParent() const
 	parent_data.m_segments.pop_back();
 
 	if (m_type == MVS)
-		parent_data.m_prefix = _T(".");
+		parent_data.m_prefix = CSparseOptional<wxString>(_T("."));
 
 	return parent;
 }
@@ -227,12 +210,12 @@ CServerPath CServerPath::GetParent() const
 wxString CServerPath::GetLastSegment() const
 {
 	if (!HasParent())
-		return _T("");
+		return wxString();
 
 	if (!m_data->m_segments.empty())
 		return m_data->m_segments.back();
 	else
-		return _T("");
+		return wxString();
 }
 
 // libc sprintf can be so slow at times...
@@ -255,52 +238,47 @@ wxChar* fast_sprint_number(wxChar* s, size_t n)
 	return s;
 }
 
-#ifdef wxUSE_UNICODE
 #define tstrcpy wcscpy
-#else
-#define tstrcpy strcpy
-#endif
 
 wxString CServerPath::GetSafePath() const
 {
 	if (m_bEmpty)
-		return _T("");
+		return wxString();
 
 	#define INTLENGTH 20 // 2^64 - 1
 
 	int len = 5 // Type and 2x' ' and terminating 0
 		+ INTLENGTH; // Max length of prefix
 
-	len += m_data->m_prefix.size();
-	for (tConstSegmentIter iter = m_data->m_segments.begin(); iter != m_data->m_segments.end(); ++iter)
-		len += iter->Length() + 2 + INTLENGTH;
+	len += m_data->m_prefix ? m_data->m_prefix->size() : 0;
+	for( auto const& segment : m_data->m_segments ) {
+		len += segment.size() + 2 + INTLENGTH;
+	}
 
 	wxString safepath;
-	wxChar* start = safepath.GetWriteBuf(len);
-	wxChar* t = start;
-
-	t = fast_sprint_number(t, m_type);
-	*(t++) = ' ';
-	t = fast_sprint_number(t, m_data->m_prefix.size());
-
-	if (!m_data->m_prefix.empty())
 	{
-		*(t++) = ' ';
-		tstrcpy(t, m_data->m_prefix);
-		t += m_data->m_prefix.size();
-	}
+		wxStringBuffer start(safepath, len);
+		wxChar* t = start;
 
-	for (tConstSegmentIter iter = m_data->m_segments.begin(); iter != m_data->m_segments.end(); ++iter)
-	{
+		t = fast_sprint_number(t, m_type);
 		*(t++) = ' ';
-		t = fast_sprint_number(t, iter->size());
-		*(t++) = ' ';
-		tstrcpy(t, *iter);
-		t += iter->size();
-	}
-	*t = 0;
+		t = fast_sprint_number(t, m_data->m_prefix ? m_data->m_prefix->size() : 0);
 
-	safepath.UngetWriteBuf( t - start );
+		if (m_data->m_prefix) {
+			*(t++) = ' ';
+			tstrcpy(t, *m_data->m_prefix);
+			t += m_data->m_prefix->size();
+		}
+
+		for( auto const& segment : m_data->m_segments ) {
+			*(t++) = ' ';
+			t = fast_sprint_number(t, segment.size());
+			*(t++) = ' ';
+			tstrcpy(t, segment);
+			t += segment.size();
+		}
+		*t = 0;
+	}
 	safepath.Shrink();
 
 	return safepath;
@@ -372,9 +350,11 @@ bool CServerPath::SetSafePath(const wxString& path, bool coalesce)
 	if (prefix_len)
 	{
 		*(p + prefix_len) = 0;
-		data.m_prefix = p;
-		if (coalesce)
-			::Coalesce(data.m_prefix);
+		if( *p ) {
+			data.m_prefix = CSparseOptional<wxString>(p);
+			if (coalesce)
+				::Coalesce(*data.m_prefix);
+		}
 
 		p += prefix_len + 1;
 	}
@@ -441,27 +421,33 @@ bool CServerPath::IsSubdirOf(const CServerPath &path, bool cmpNoCase) const
 	if (!HasParent())
 		return false;
 
-	if (traits[m_type].prefixmode != 1)
-	{
-		if (cmpNoCase && m_data->m_prefix.CmpNoCase(path.m_data->m_prefix))
-			return false;
+	if (traits[m_type].prefixmode != 1) {
+		if (cmpNoCase ) {
+			if( m_data->m_prefix && !path.m_data->m_prefix ) {
+				return false;
+			}
+			else if( !m_data->m_prefix && path.m_data->m_prefix ) {
+				return false;
+			}
+			else if( m_data->m_prefix && path.m_data->m_prefix && m_data->m_prefix->CmpNoCase(*path.m_data->m_prefix) ) {
+				return false;
+			}
+		}
 		if (!cmpNoCase && m_data->m_prefix != path.m_data->m_prefix)
 			return false;
 	}
 
 	// On MVS, dirs like 'FOO.BAR' without trailing dot cannot have
 	// subdirectories
-	if (traits[m_type].prefixmode == 1 && path.m_data->m_prefix == _T(""))
+	if (traits[m_type].prefixmode == 1 && !path.m_data->m_prefix)
 		return false;
 
 	tConstSegmentIter iter1 = m_data->m_segments.begin();
 	tConstSegmentIter iter2 = path.m_data->m_segments.begin();
-	while (iter1 != m_data->m_segments.end())
-	{
+	while (iter1 != m_data->m_segments.end()) {
 		if (iter2 == path.m_data->m_segments.end())
 			return true;
-		if (cmpNoCase)
-		{
+		if (cmpNoCase) {
 			if (iter1->CmpNoCase(*iter2))
 				return false;
 		}
@@ -494,9 +480,9 @@ bool CServerPath::ChangePath(wxString &subdir, bool isFile)
 	wxString dir = subdir;
 	wxString file;
 
-	if (dir == _T(""))
+	if (dir.empty())
 	{
-		if (IsEmpty() || isFile)
+		if (empty() || isFile)
 			return false;
 		else
 			return true;
@@ -517,7 +503,7 @@ bool CServerPath::ChangePath(wxString &subdir, bool isFile)
 
 				if (isFile)
 				{
-					if (IsEmpty())
+					if (empty())
 						return false;
 
 					file = dir;
@@ -539,7 +525,7 @@ bool CServerPath::ChangePath(wxString &subdir, bool isFile)
 				dir = dir.Left(pos2);
 
 				if (pos1)
-					data.m_prefix = dir.Left(pos1);
+					data.m_prefix = CSparseOptional<wxString>(dir.Left(pos1));
 				dir = dir.Mid(pos1 + 1);
 
 				data.m_segments.clear();
@@ -591,8 +577,7 @@ bool CServerPath::ChangePath(wxString &subdir, bool isFile)
 				c = dir.c_str()[++i];
 			dir.Remove(0, i);
 
-			while (dir != _T(""))
-			{
+			while (!dir.empty()) {
 				c = dir.Last();
 				if (c != FTP_MVS_DOUBLE_QUOTE)
 					break;
@@ -600,11 +585,10 @@ bool CServerPath::ChangePath(wxString &subdir, bool isFile)
 					dir.RemoveLast();
 			}
 		}
-		if (dir == _T(""))
+		if (dir.empty())
 			return false;
 
-		if (dir.c_str()[0] == traits[m_type].left_enclosure)
-		{
+		if (dir.c_str()[0] == traits[m_type].left_enclosure) {
 			if (dir.Last() != traits[m_type].right_enclosure)
 				return false;
 
@@ -618,8 +602,7 @@ bool CServerPath::ChangePath(wxString &subdir, bool isFile)
 		else if (m_bEmpty)
 			return false;
 
-		if (dir.Last() == ')')
-		{
+		if (dir.Last() == ')') {
 			// Partitioned dataset member
 			if (!isFile)
 				return false;
@@ -631,27 +614,25 @@ bool CServerPath::ChangePath(wxString &subdir, bool isFile)
 			file = dir.Mid(pos + 1);
 			dir = dir.Left(pos);
 
-			if (!m_bEmpty && data.m_prefix == _T("") && !dir.empty())
+			if (!m_bEmpty && !data.m_prefix && !dir.empty())
 				return false;
 
 			data.m_prefix.clear();
 		}
-		else
-		{
-			if (!m_bEmpty && data.m_prefix == _T(""))
+		else {
+			if (!m_bEmpty && !data.m_prefix)
 			{
 				if (dir.Find('.') != -1 || !isFile)
 					return false;
 			}
 
-			if (isFile)
-			{
+			if (isFile) {
 				if (!ExtractFile(dir, file))
 					return false;
-				data.m_prefix = _T(".");
+				data.m_prefix = CSparseOptional<wxString>(_T("."));
 			}
 			else if (dir.Last() == '.')
-				data.m_prefix = _T(".");
+				data.m_prefix = CSparseOptional<wxString>(_T("."));
 			else
 				data.m_prefix.clear();
 		}
@@ -676,7 +657,7 @@ bool CServerPath::ChangePath(wxString &subdir, bool isFile)
 		{
 			if (dir[0] != ':')
 			{
-				if (IsEmpty())
+				if (empty())
 					return false;
 			}
 			else
@@ -684,7 +665,7 @@ bool CServerPath::ChangePath(wxString &subdir, bool isFile)
 				int colon2;
 				if ((colon2 = dir.Mid(1).Find(':')) < 1)
 					return false;
-				data.m_prefix = dir.Left(colon2 + 2);
+				data.m_prefix = CSparseOptional<wxString>(dir.Left(colon2 + 2));
 				dir = dir.Mid(colon2 + 2);
 
 				data.m_segments.clear();
@@ -708,7 +689,7 @@ bool CServerPath::ChangePath(wxString &subdir, bool isFile)
 				return false;
 			if (dir.Left(2) == _T("//"))
 			{
-				data.m_prefix = traits[m_type].separators[0];
+				data.m_prefix = CSparseOptional<wxString>(traits[m_type].separators[0]);
 				dir = dir.Mid(1);
 			}
 
@@ -782,11 +763,14 @@ bool CServerPath::operator<(const CServerPath &op) const
 	else if (op.m_bEmpty)
 		return true;
 
-	const int cmp = m_data->m_prefix.Cmp(op.m_data->m_prefix);
-	if (cmp < 0)
-		return true;
-	if (cmp > 0)
-		return false;
+	if( m_data->m_prefix || op.m_data->m_prefix ) {
+		if( m_data->m_prefix < op.m_data->m_prefix ) {
+			return true;
+		}
+		else if( op.m_data->m_prefix < m_data->m_prefix ) {
+			return false;
+		}
+	}
 
 	if (m_type > op.m_type)
 		return false;
@@ -794,8 +778,7 @@ bool CServerPath::operator<(const CServerPath &op) const
 		return true;
 
 	tConstSegmentIter iter1, iter2;
-	for (iter1 = m_data->m_segments.begin(), iter2 = op.m_data->m_segments.begin(); iter1 != m_data->m_segments.end(); ++iter1, ++iter2)
-	{
+	for (iter1 = m_data->m_segments.begin(), iter2 = op.m_data->m_segments.begin(); iter1 != m_data->m_segments.end(); ++iter1, ++iter2) {
 		if (iter2 == op.m_data->m_segments.end())
 			return false;
 
@@ -814,13 +797,13 @@ wxString CServerPath::FormatFilename(const wxString &filename, bool omitPath /*=
 	if (m_bEmpty)
 		return filename;
 
-	if (filename == _T(""))
-		return _T("");
+	if (filename.empty())
+		return wxString();
 
 	if (m_bEmpty)
-		return _T("");
+		return wxString();
 
-	if (omitPath && (!traits[m_type].prefixmode || m_data->m_prefix == _T(".")))
+	if (omitPath && (!traits[m_type].prefixmode || (m_data->m_prefix && *m_data->m_prefix == _T("."))))
 		return filename;
 
 	wxString result = GetPath();
@@ -842,7 +825,7 @@ wxString CServerPath::FormatFilename(const wxString &filename, bool omitPath /*=
 			break;
 	}
 
-	if (traits[m_type].prefixmode == 1 && m_data->m_prefix.empty())
+	if (traits[m_type].prefixmode == 1 && !m_data->m_prefix)
 		result += _T("(") + filename + _T(")");
 	else
 		result += filename;
@@ -869,8 +852,7 @@ int CServerPath::CmpNoCase(const CServerPath &op) const
 
 	tConstSegmentIter iter = m_data->m_segments.begin();
 	tConstSegmentIter iter2 = op.m_data->m_segments.begin();
-	while (iter != m_data->m_segments.end())
-	{
+	while (iter != m_data->m_segments.end()) {
 		int res = iter++->CmpNoCase(*iter2++);
 		if (res)
 			return res;
@@ -881,7 +863,7 @@ int CServerPath::CmpNoCase(const CServerPath &op) const
 
 bool CServerPath::AddSegment(const wxString& segment)
 {
-	if (IsEmpty())
+	if (empty())
 		return false;
 
 	// TODO: Check for invalid characters
@@ -927,11 +909,10 @@ CServerPath CServerPath::GetCommonParent(const CServerPath& path) const
 
 	tConstSegmentIter last = m_data->m_segments.end();
 	tConstSegmentIter last2 = path.m_data->m_segments.end();
-	if (traits[m_type].prefixmode == 1)
-	{
-		if (m_data->m_prefix.empty())
+	if (traits[m_type].prefixmode == 1) {
+		if (!m_data->m_prefix)
 			--last;
-		if (path.m_data->m_prefix.empty())
+		if (!path.m_data->m_prefix)
 			--last2;
 		parentData.m_prefix = GetParent().m_data->m_prefix;
 	}
@@ -940,10 +921,8 @@ CServerPath CServerPath::GetCommonParent(const CServerPath& path) const
 
 	tConstSegmentIter iter = m_data->m_segments.begin();
 	tConstSegmentIter iter2 = path.m_data->m_segments.begin();
-	while (iter != last && iter2 != last2)
-	{
-		if (*iter != *iter2)
-		{
+	while (iter != last && iter2 != last2) {
+		if (*iter != *iter2) {
 			if (!traits[m_type].has_root && parentData.m_segments.empty())
 				return CServerPath();
 			else
@@ -1051,8 +1030,7 @@ bool CServerPath::ExtractFile(wxString& dir, wxString& file)
 
 void CServerPath::EscapeSeparators(ServerType type, wxString& subdir)
 {
-	if (traits[type].separatorEscape)
-	{
+	if (traits[type].separatorEscape) {
 		for (const wxChar* p = traits[type].separators; *p; ++p)
 			subdir.Replace((wxString)*p, (wxString)traits[type].separatorEscape + traits[type].separators[0]);
 	}
@@ -1061,7 +1039,10 @@ void CServerPath::EscapeSeparators(ServerType type, wxString& subdir)
 void CServerPath::Coalesce()
 {
 	CServerPathData& data = m_data.Get();
-	::Coalesce(data.m_prefix);
-	for (tSegmentIter iter = data.m_segments.begin(); iter != data.m_segments.end(); ++iter)
-		::Coalesce(*iter);
+	if( data.m_prefix ) {
+		::Coalesce(*data.m_prefix);
+	}
+	for (auto& segment : data.m_segments) {
+		::Coalesce(segment);
+	}
 }
