@@ -2,19 +2,18 @@
 #ifdef __WXMSW__
   // MinGW needs this for getaddrinfo
   #if defined(_WIN32_WINNT)
-    #if _WIN32_WINNT < 0x0501
+    #if _WIN32_WINNT < 0x0600
       #undef _WIN32_WINNT
-      #define _WIN32_WINNT 0x0501
+      #define _WIN32_WINNT 0x0600
     #endif
   #else
-    #define _WIN32_WINNT 0x0501
+    #define _WIN32_WINNT 0x0600
   #endif
   #include <winsock2.h>
   #include <ws2tcpip.h>
 #endif
 #include <filezilla.h>
 #include "socket.h"
-#include "threadex.h"
 #include <errno.h>
 #ifndef __WXMSW__
   #include <sys/types.h>
@@ -44,62 +43,6 @@ union sockaddr_u
 	struct sockaddr_in6 in6;
 };
 
-// --------------------------
-// Windows 2000 compatibility
-// --------------------------
-#ifdef __WXMSW__
-
-// Stupid Win2K has no getaddrinfo
-// Apparently it is too hard for the richest company in the
-// world to add this simple function with a service pack...
-
-extern "C"
-{
-	typedef int (WINAPI *t_getaddrinfo)(const char *nodename, const char *servname,
-						const struct addrinfo *hints, struct addrinfo **res);
-	typedef void (WINAPI *t_freeaddrinfo)(struct addrinfo* ai);
-	typedef int (WINAPI *t_getnameinfo)(const struct sockaddr* sa, socklen_t salen,
-						char* host, DWORD hostlen, char* serv, DWORD servlen, int flags);
-}
-
-static t_getaddrinfo p_getaddrinfo = 0;
-static t_freeaddrinfo p_freeaddrinfo = 0;
-static t_getnameinfo p_getnameinfo = 0;
-
-class CGetAddrinfoLoader
-{
-public:
-	CGetAddrinfoLoader()
-	{
-		HMODULE m_hDll = LoadLibrary(_T("ws2_32.dll"));
-		if (m_hDll)
-		{
-			p_getaddrinfo = (t_getaddrinfo)GetProcAddress(m_hDll, "getaddrinfo");
-			if (p_getaddrinfo)
-				p_freeaddrinfo = (t_freeaddrinfo)GetProcAddress(m_hDll, "freeaddrinfo");
-			p_getnameinfo = (t_getnameinfo)GetProcAddress(m_hDll, "getnameinfo");
-		}
-	}
-
-	~CGetAddrinfoLoader()
-	{
-		if (m_hDll)
-			FreeLibrary(m_hDll);
-	}
-protected:
-	HMODULE m_hDll;
-};
-static CGetAddrinfoLoader addrinfo_loader;
-
-#define getaddrinfo p_getaddrinfo
-#define freeaddrinfo p_freeaddrinfo
-#define getnameinfo p_getnameinfo
-
-#endif //__WXMSW__
-// ------------------------------
-// End Windows 2000 compatibility
-// ------------------------------
-
 #define WAIT_CONNECT 0x01
 #define WAIT_READ	 0x02
 #define WAIT_WRITE	 0x04
@@ -115,6 +58,10 @@ static int ConvertMSWErrorCode(int error)
 {
 	switch (error)
 	{
+	case WSAECONNREFUSED:
+		return ECONNREFUSED;
+	case WSAECONNABORTED:
+		return ECONNABORTED;
 	case WSAEINVAL:
 		return EAI_BADFLAGS;
 	case WSANO_RECOVERY:
@@ -141,6 +88,8 @@ static int ConvertMSWErrorCode(int error)
 		return EFAULT;
 	case WSAEACCES:
 		return EACCES;
+	case WSAETIMEDOUT:
+		return ETIMEDOUT;
 	default:
 		return error;
 	}
@@ -164,7 +113,7 @@ public:
 			int code = errors[i];
 #endif
 			if (CSocket::GetErrorDescription(code).Len() < 15)
-				wxMessageBox(CSocket::GetErrorDescription(code));
+				wxMessageBoxEx(CSocket::GetErrorDescription(code));
 		}
 
 	}
@@ -179,8 +128,9 @@ CSocketEventDispatcher::CSocketEventDispatcher()
 
 CSocketEventDispatcher::~CSocketEventDispatcher()
 {
-	for (std::list<CSocketEvent*>::iterator iter = m_pending_events.begin(); iter != m_pending_events.end(); ++iter)
-		delete *iter;
+	for (auto const& pendingEvent : m_pending_events ) {
+		delete pendingEvent;
+	}
 }
 
 CSocketEventDispatcher& CSocketEventDispatcher::Get()
@@ -207,45 +157,41 @@ void CSocketEventDispatcher::RemovePending(const CSocketEventHandler* pHandler)
 {
 	wxCriticalSectionLocker lock(m_sync);
 
-	std::list<CSocketEvent*> keep;
-	for (std::list<CSocketEvent*>::iterator iter = m_pending_events.begin(); iter != m_pending_events.end(); ++iter)
-	{
-		if ((*iter)->GetSocketEventHandler() != pHandler)
-		{
-			keep.push_back(*iter);
-			continue;
+	auto remaining = m_pending_events.begin();
+	for (auto iter = m_pending_events.begin(); iter != m_pending_events.end(); ++iter) {
+		if ((*iter)->GetSocketEventHandler() != pHandler) {
+			*remaining = *iter;
+			++remaining;
+		}
+		else {
+			delete *iter;
 		}
-
-		delete *iter;
 	}
-	m_pending_events.swap(keep);
+	m_pending_events.erase(remaining, m_pending_events.end());
 }
 
 void CSocketEventDispatcher::RemovePending(const CSocketEventSource* pSource)
 {
 	wxCriticalSectionLocker lock(m_sync);
 
-	std::list<CSocketEvent*> keep;
-	for (std::list<CSocketEvent*>::iterator iter = m_pending_events.begin(); iter != m_pending_events.end(); ++iter)
-	{
-		if ((*iter)->GetSocketEventSource() != pSource)
-		{
-			keep.push_back(*iter);
-			continue;
+	auto remaining = m_pending_events.begin();
+	for (auto iter = m_pending_events.begin(); iter != m_pending_events.end(); ++iter) {
+		if ((*iter)->GetSocketEventSource() != pSource) {
+			*remaining = *iter;
+			++remaining;
+		}
+		else {
+			delete *iter;
 		}
-
-		delete *iter;
 	}
-	m_pending_events.swap(keep);
+	m_pending_events.erase(remaining, m_pending_events.end());
 }
 
 void CSocketEventDispatcher::UpdatePending(const CSocketEventHandler* pOldHandler, const CSocketEventSource* pOldSource, CSocketEventHandler* pNewHandler, CSocketEventSource* pNewSource)
 {
 	wxCriticalSectionLocker lock(m_sync);
 
-	for (std::list<CSocketEvent*>::iterator iter = m_pending_events.begin(); iter != m_pending_events.end(); ++iter)
-	{
-		CSocketEvent* evt = *iter;
+	for (auto const& evt : m_pending_events ) {
 		if (evt->GetSocketEventSource() != pOldSource || evt->GetSocketEventHandler() != pOldHandler)
 			continue;
 
@@ -258,8 +204,7 @@ bool CSocketEventDispatcher::ProcessEvent(wxEvent& event)
 {
 	m_sync.Enter();
 	m_inside_loop = true;
-	while (!m_pending_events.empty())
-	{
+	while (!m_pending_events.empty()) {
 		CSocketEvent *evt = m_pending_events.front();
 		m_pending_events.pop_front();
 
@@ -305,7 +250,7 @@ CSocketEvent::~CSocketEvent()
 wxString CSocketEvent::GetData() const
 {
 	if (!m_data)
-		return wxEmptyString;
+		return wxString();
 
 	return m_data;
 }
@@ -320,12 +265,12 @@ CSocketEventSource::~CSocketEventSource()
 	CSocketEventDispatcher::Get().RemovePending(this);
 }
 
-class CSocketThread : protected wxThreadEx
+class CSocketThread final : protected wxThread
 {
 	friend class CSocket;
 public:
 	CSocketThread()
-		: wxThreadEx(wxTHREAD_JOINABLE), m_condition(m_sync)
+		: wxThread(wxTHREAD_JOINABLE), m_condition(m_sync)
 	{
 		m_pSocket = 0;
 		m_pHost = 0;
@@ -398,7 +343,7 @@ public:
 
 		// Connect method of CSocket ensures port is in range
 		m_pPort = new char[6];
-		sprintf(m_pPort, "%d", m_pSocket->m_port);
+		sprintf(m_pPort, "%u", m_pSocket->m_port);
 		m_pPort[5] = 0;
 
 		Start();
@@ -617,63 +562,6 @@ protected:
 		hints.ai_family = m_pSocket->m_family;
 		hints.ai_socktype = SOCK_STREAM;
 
-#ifdef __WXMSW__
-		if (!getaddrinfo)
-		{
-			// Win2K fallback
-			int port = atoi(pPort);
-
-			struct hostent* h = gethostbyname(pHost);
-
-			delete [] pHost;
-			delete [] pPort;
-
-			if (!Lock())
-			{
-				if (m_pSocket)
-					m_pSocket->m_state = CSocket::closed;
-				return false;
-			}
-
-			if (!h)
-			{
-				if (m_pSocket->m_pEvtHandler)
-				{
-					int res = ConvertMSWErrorCode(WSAGetLastError());
-					CSocketEvent *evt = new CSocketEvent(m_pSocket->GetEventHandler(), m_pSocket, CSocketEvent::connection, res);
-					CSocketEventDispatcher::Get().SendEvent(evt);
-				}
-				m_pSocket->m_state = CSocket::closed;
-				return false;
-			}
-
-			struct sockaddr_in addr_in = {0};
-			addr_in.sin_family = AF_INET;
-			addr_in.sin_addr = *((struct in_addr*)h->h_addr_list[0]);
-			addr_in.sin_port = htons(port);
-
-			struct addrinfo addr = {0};
-			addr.ai_family = AF_INET;
-			addr.ai_socktype = SOCK_STREAM;
-			addr.ai_addr = (struct sockaddr*)&addr_in;
-			addr.ai_addrlen = sizeof(struct sockaddr_in);
-
-			int res = TryConnectHost(&addr);
-			if (res == 1)
-				return true;
-
-			if (m_pSocket)
-			{
-				if (!res && m_pSocket->m_pEvtHandler)
-				{
-					CSocketEvent *evt = new CSocketEvent(m_pSocket->GetEventHandler(), m_pSocket, CSocketEvent::connection, ECONNABORTED);
-					CSocketEventDispatcher::Get().SendEvent(evt);
-				}
-				m_pSocket->m_state = CSocket::closed;
-			}
-			return false;
-		}
-#endif
 		int res = getaddrinfo(pHost, pPort, &hints, &addressList);
 
 		delete [] pHost;
@@ -957,12 +845,39 @@ protected:
 		}
 		if (m_triggered & WAIT_CLOSE)
 		{
-			CSocketEvent *evt = new CSocketEvent(m_pSocket->m_pEvtHandler, m_pSocket, CSocketEvent::close, m_triggered_errors[4]);
-			CSocketEventDispatcher::Get().SendEvent(evt);
+			SendCloseEvent();
 			m_triggered &= ~WAIT_CLOSE;
 		}
 	}
 
+	void SendCloseEvent()
+	{
+		if( !m_pSocket ) {
+			return;
+		}
+
+		CSocketEvent *evt = 0;
+#ifdef __WXMSW__
+		// MSDN says this:
+		//   FD_CLOSE being posted after all data is read from a socket.
+		//   An application should check for remaining data upon receipt
+		//   of FD_CLOSE to avoid any possibility of losing data.
+		// First half is actually plain wrong.
+		char buf;
+		if( !m_triggered_errors[4] && recv( m_pSocket->m_fd, &buf, 1, MSG_PEEK ) > 0) {
+			if( !(m_waiting & WAIT_READ) ) {
+				return;
+			}
+			evt = new CSocketEvent(m_pSocket->m_pEvtHandler, m_pSocket, CSocketEvent::read, 0);
+		}
+		else
+#endif
+		{
+			evt = new CSocketEvent(m_pSocket->m_pEvtHandler, m_pSocket, CSocketEvent::close, m_triggered_errors[4]);
+		}
+		CSocketEventDispatcher::Get().SendEvent(evt);
+	}
+
 	// Call only while locked
 	bool IdleLoop()
 	{
@@ -1085,7 +1000,6 @@ protected:
 
 CSocket::CSocket(CSocketEventHandler* pEvtHandler)
 	: m_pEvtHandler(pEvtHandler)
-	, m_synchronous_read_cb()
 {
 #ifdef ERRORCODETEST
 	CErrorCodeTest test;
@@ -1227,12 +1141,10 @@ void CSocket::SetEventHandler(CSocketEventHandler* pEvtHandler)
 			CSocketEventDispatcher::Get().UpdatePending(m_pEvtHandler, this, pEvtHandler, this);
 	}
 #ifdef __WXMSW__
-	if (pEvtHandler && !m_pEvtHandler && m_state == closing && m_pSocketThread)
-	{
+	if (pEvtHandler && !m_pEvtHandler && m_state == closing && m_pSocketThread) {
 		// After getting FD_CLOSE, no further events are recorded, so send
 		// it out to new handler manually
-		CSocketEvent *evt = new CSocketEvent(pEvtHandler, this, CSocketEvent::close, m_pSocketThread->m_triggered_errors[4]);
-		CSocketEventDispatcher::Get().SendEvent(evt);
+		m_pSocketThread->SendCloseEvent();
 	}
 #else
 	wxASSERT(!pEvtHandler || m_state != closing);
@@ -1271,7 +1183,7 @@ void CSocket::SetEventHandler(CSocketEventHandler* pEvtHandler)
 	}
 }
 
-#define ERRORDECL(c, desc) { c, _T(#c), wxTRANSLATE(desc) },
+#define ERRORDECL(c, desc) { c, _T(#c), desc },
 
 struct Error_table
 {
@@ -1282,72 +1194,73 @@ struct Error_table
 
 static struct Error_table error_table[] =
 {
-	ERRORDECL(EACCES, "Permission denied")
-	ERRORDECL(EADDRINUSE, "Local address in use")
-	ERRORDECL(EAFNOSUPPORT, "The specified address family is not supported")
-	ERRORDECL(EINPROGRESS, "Operation in progress")
-	ERRORDECL(EINVAL, "Invalid argument passed")
-	ERRORDECL(EMFILE, "Process file table overflow")
-	ERRORDECL(ENFILE, "System limit of open files exceeded")
-	ERRORDECL(ENOBUFS, "Out of memory")
-	ERRORDECL(ENOMEM, "Out of memory")
-	ERRORDECL(EPERM, "Permission denied")
-	ERRORDECL(EPROTONOSUPPORT, "Protocol not supported")
-	ERRORDECL(EAGAIN, "Resource temporarily unavailable")
-	ERRORDECL(EALREADY, "Operation already in progress")
-	ERRORDECL(EBADF, "Bad file descriptor")
-	ERRORDECL(ECONNREFUSED, "Connection refused by server")
-	ERRORDECL(EFAULT, "Socket address outside address space")
-	ERRORDECL(EINTR, "Interrupted by signal")
-	ERRORDECL(EISCONN, "Socket already connected")
-	ERRORDECL(ENETUNREACH, "Network unreachable")
-	ERRORDECL(ENOTSOCK, "File descriptior not a socket")
-	ERRORDECL(ETIMEDOUT, "Connection attempt timed out")
-	ERRORDECL(EHOSTUNREACH, "No route to host")
-	ERRORDECL(ENOTCONN, "Socket not connected")
-	ERRORDECL(ENETRESET, "Connection reset by network")
-	ERRORDECL(EOPNOTSUPP, "Operation not supported")
-	ERRORDECL(ESHUTDOWN, "Socket has been shut down")
-	ERRORDECL(EMSGSIZE, "Message too large")
-	ERRORDECL(ECONNABORTED, "Connection aborted")
-	ERRORDECL(ECONNRESET, "Connection reset by peer")
-	ERRORDECL(EPIPE, "Local endpoint has been closed")
+	ERRORDECL(EACCES, TRANSLATE_T("Permission denied"))
+	ERRORDECL(EADDRINUSE, TRANSLATE_T("Local address in use"))
+	ERRORDECL(EAFNOSUPPORT, TRANSLATE_T("The specified address family is not supported"))
+	ERRORDECL(EINPROGRESS, TRANSLATE_T("Operation in progress"))
+	ERRORDECL(EINVAL, TRANSLATE_T("Invalid argument passed"))
+	ERRORDECL(EMFILE, TRANSLATE_T("Process file table overflow"))
+	ERRORDECL(ENFILE, TRANSLATE_T("System limit of open files exceeded"))
+	ERRORDECL(ENOBUFS, TRANSLATE_T("Out of memory"))
+	ERRORDECL(ENOMEM, TRANSLATE_T("Out of memory"))
+	ERRORDECL(EPERM, TRANSLATE_T("Permission denied"))
+	ERRORDECL(EPROTONOSUPPORT, TRANSLATE_T("Protocol not supported"))
+	ERRORDECL(EAGAIN, TRANSLATE_T("Resource temporarily unavailable"))
+	ERRORDECL(EALREADY, TRANSLATE_T("Operation already in progress"))
+	ERRORDECL(EBADF, TRANSLATE_T("Bad file descriptor"))
+	ERRORDECL(ECONNREFUSED, TRANSLATE_T("Connection refused by server"))
+	ERRORDECL(EFAULT, TRANSLATE_T("Socket address outside address space"))
+	ERRORDECL(EINTR, TRANSLATE_T("Interrupted by signal"))
+	ERRORDECL(EISCONN, TRANSLATE_T("Socket already connected"))
+	ERRORDECL(ENETUNREACH, TRANSLATE_T("Network unreachable"))
+	ERRORDECL(ENOTSOCK, TRANSLATE_T("File descriptor not a socket"))
+	ERRORDECL(ETIMEDOUT, TRANSLATE_T("Connection attempt timed out"))
+	ERRORDECL(EHOSTUNREACH, TRANSLATE_T("No route to host"))
+	ERRORDECL(ENOTCONN, TRANSLATE_T("Socket not connected"))
+	ERRORDECL(ENETRESET, TRANSLATE_T("Connection reset by network"))
+	ERRORDECL(EOPNOTSUPP, TRANSLATE_T("Operation not supported"))
+	ERRORDECL(ESHUTDOWN, TRANSLATE_T("Socket has been shut down"))
+	ERRORDECL(EMSGSIZE, TRANSLATE_T("Message too large"))
+	ERRORDECL(ECONNABORTED, TRANSLATE_T("Connection aborted"))
+	ERRORDECL(ECONNRESET, TRANSLATE_T("Connection reset by peer"))
+	ERRORDECL(EPIPE, TRANSLATE_T("Local endpoint has been closed"))
 
 	// Getaddrinfo related
 #ifndef __WXMSW__
-	ERRORDECL(EAI_ADDRFAMILY, "Network host does not have any network addresses in the requested address family")
+	ERRORDECL(EAI_ADDRFAMILY, TRANSLATE_T("Network host does not have any network addresses in the requested address family"))
 #endif
-	ERRORDECL(EAI_AGAIN, "Temporary failure in name resolution")
-	ERRORDECL(EAI_BADFLAGS, "Invalid value for ai_flags")
+	ERRORDECL(EAI_AGAIN, TRANSLATE_T("Temporary failure in name resolution"))
+	ERRORDECL(EAI_BADFLAGS, TRANSLATE_T("Invalid value for ai_flags"))
 #ifdef EAI_BADHINTS
-	ERRORDECL(EAI_BADHINTS, "Invalid value for hints")
+	ERRORDECL(EAI_BADHINTS, TRANSLATE_T("Invalid value for hints"))
 #endif
-	ERRORDECL(EAI_FAIL, "Nonrecoverable failure in name resolution")
-	ERRORDECL(EAI_FAMILY, "The ai_family member is not supported")
-	ERRORDECL(EAI_MEMORY, "Memory allocation failure")
+	ERRORDECL(EAI_FAIL, TRANSLATE_T("Nonrecoverable failure in name resolution"))
+	ERRORDECL(EAI_FAMILY, TRANSLATE_T("The ai_family member is not supported"))
+	ERRORDECL(EAI_MEMORY, TRANSLATE_T("Memory allocation failure"))
 #ifdef EAI_NODATA
-	ERRORDECL(EAI_NODATA, "No address associated with nodename")
+	ERRORDECL(EAI_NODATA, TRANSLATE_T("No address associated with nodename"))
 #endif
-	ERRORDECL(EAI_NONAME, "Neither nodename nor servname provided, or not known")
+	ERRORDECL(EAI_NONAME, TRANSLATE_T("Neither nodename nor servname provided, or not known"))
 #ifdef EAI_OVERFLOW
-	ERRORDECL(EAI_OVERFLOW, "Argument buffer overflow")
+	ERRORDECL(EAI_OVERFLOW, TRANSLATE_T("Argument buffer overflow"))
 #endif
 #ifdef EAI_PROTOCOL
-	ERRORDECL(EAI_PROTOCOL, "Resolved protocol is unknown")
+	ERRORDECL(EAI_PROTOCOL, TRANSLATE_T("Resolved protocol is unknown"))
 #endif
-	ERRORDECL(EAI_SERVICE, "The servname parameter is not supported for ai_socktype")
-	ERRORDECL(EAI_SOCKTYPE, "The ai_socktype member is not supported")
+	ERRORDECL(EAI_SERVICE, TRANSLATE_T("The servname parameter is not supported for ai_socktype"))
+	ERRORDECL(EAI_SOCKTYPE, TRANSLATE_T("The ai_socktype member is not supported"))
 #ifndef __WXMSW__
-	ERRORDECL(EAI_SYSTEM, "Other system error")
+	ERRORDECL(EAI_SYSTEM, TRANSLATE_T("Other system error"))
 #endif
 
 	// Codes that have no POSIX equivalence
 #ifdef __WXMSW__
-	ERRORDECL(WSANOTINITIALISED, "Not initialized, need to call WSAStartup")
-	ERRORDECL(WSAENETDOWN, "System's network subsystem has failed")
-	ERRORDECL(WSAEPROTOTYPE, "Protocol not supported on given socket type")
-	ERRORDECL(WSAESOCKTNOSUPPORT, "Socket type not supported for address family")
-	ERRORDECL(WSAEADDRNOTAVAIL, "Cannot assign requested address")
+	ERRORDECL(WSANOTINITIALISED, TRANSLATE_T("Not initialized, need to call WSAStartup"))
+	ERRORDECL(WSAENETDOWN, TRANSLATE_T("System's network subsystem has failed"))
+	ERRORDECL(WSAEPROTOTYPE, TRANSLATE_T("Protocol not supported on given socket type"))
+	ERRORDECL(WSAESOCKTNOSUPPORT, TRANSLATE_T("Socket type not supported for address family"))
+	ERRORDECL(WSAEADDRNOTAVAIL, TRANSLATE_T("Cannot assign requested address"))
+	ERRORDECL(ERROR_NETNAME_DELETED, TRANSLATE_T("The specified network name is no longer available"))
 #endif
 	{ 0, 0, 0 }
 };
@@ -1446,7 +1359,7 @@ bool CSocket::Cleanup(bool force)
 		}
 		pThread->m_sync.Unlock();
 
-		pThread->Wait();
+		pThread->Wait(wxTHREAD_WAIT_BLOCK);
 		delete pThread;
 		waiting_socket_threads.erase(current);
 	}
@@ -1465,13 +1378,10 @@ int CSocket::Read(void* buffer, unsigned int size, int& error)
 #else
 		error = errno;
 #endif
-		if (error == EAGAIN)
-		{
-			if (m_pSocketThread)
-			{
+		if (error == EAGAIN) {
+			if (m_pSocketThread) {
 				m_pSocketThread->m_sync.Lock();
-				if (!(m_pSocketThread->m_waiting & WAIT_READ))
-				{
+				if (!(m_pSocketThread->m_waiting & WAIT_READ)) {
 					m_pSocketThread->m_waiting |= WAIT_READ;
 					m_pSocketThread->WakeupThread(true);
 				}
@@ -1560,27 +1470,9 @@ wxString CSocket::AddressToString(const struct sockaddr* addr, int addr_len, boo
 	char hostbuf[NI_MAXHOST];
 	char portbuf[NI_MAXSERV];
 
-#ifdef __WXMSW__
-	if (!getnameinfo)
-	{
-		// Win2K fallback
-		if (addr->sa_family != AF_INET)
-			return _T("");
-		char* s = inet_ntoa(((struct sockaddr_in*)addr)->sin_addr);
-		if (!s)
-			return _T("");
-
-		wxString host = wxString(s, wxConvLibc);
-		if (!with_port)
-			return host;
-
-		return host + wxString::Format(_T(":%d"), (int)ntohs(((struct sockaddr_in*)addr)->sin_port));
-	}
-#endif
-
 	int res = getnameinfo(addr, addr_len, hostbuf, NI_MAXHOST, portbuf, NI_MAXSERV, NI_NUMERICHOST | NI_NUMERICSERV);
 	if (res) // Should never fail
-		return _T("");
+		return wxString();
 
 	wxString host = wxString(hostbuf, wxConvLibc);
 	wxString port = wxString(portbuf, wxConvLibc);
@@ -1611,7 +1503,7 @@ wxString CSocket::GetLocalIP(bool strip_zone_index /*=false*/) const
 	socklen_t addr_len = sizeof(addr);
 	int res = getsockname(m_fd, (sockaddr*)&addr, &addr_len);
 	if (res)
-		return _T("");
+		return wxString();
 
 	return AddressToString((sockaddr *)&addr, addr_len, false, strip_zone_index);
 }
@@ -1622,7 +1514,7 @@ wxString CSocket::GetPeerIP(bool strip_zone_index /*=false*/) const
 	socklen_t addr_len = sizeof(addr);
 	int res = getpeername(m_fd, (sockaddr*)&addr, &addr_len);
 	if (res)
-		return _T("");
+		return wxString();
 
 	return AddressToString((sockaddr *)&addr, addr_len, false, strip_zone_index);
 }
@@ -1669,31 +1561,6 @@ int CSocket::Listen(enum address_family family, int port /*=0*/)
 		return EINVAL;
 	}
 
-#ifdef __WXMSW__
-	if (!getaddrinfo)
-	{
-		m_fd = socket(AF_INET, SOCK_STREAM, 0);
-		if (m_fd == -1)
-			return ConvertMSWErrorCode(WSAGetLastError());
-
-		SetNonblocking(m_fd);
-
-		struct sockaddr_in addr = {0};
-		addr.sin_family = AF_INET;
-		addr.sin_port = htons(port);
-
-		int res = bind(m_fd, (sockaddr *)&addr, sizeof(addr));
-		if (res)
-		{
-			res = WSAGetLastError();
-			closesocket(m_fd);
-			m_fd = -1;
-
-			return ConvertMSWErrorCode(res);
-		}
-	}
-	else
-#endif
 	{
 		struct addrinfo hints = {0};
 		hints.ai_family = m_family;
@@ -1973,3 +1840,8 @@ void CSocket::SetSynchronousReadCallback(CCallback* cb)
 	if (m_pSocketThread)
 		m_pSocketThread->m_sync.Unlock();
 }
+
+wxString CSocket::GetPeerHost() const
+{
+	return m_host;
+}
