@@ -6,12 +6,7 @@
 #include <gnutls/x509.h>
 #include <errno.h>
 
-#if GNUTLS_VERSION_NUMBER >= 0x030100
 char const ciphers[] = "SECURE256:+SECURE128:+ARCFOUR-128:-3DES-CBC:-MD5:+SIGN-ALL:-SIGN-RSA-MD5:+CTYPE-X509:-CTYPE-OPENPGP";
-#else
-// Versions before 3.1.0 cannot combine level keywords
-char const ciphers[] = "SECURE128:+ARCFOUR-128:-3DES-CBC:-MD5:+SIGN-ALL:-SIGN-RSA-MD5:+CTYPE-X509:-CTYPE-OPENPGP";
-#endif
 
 //#define TLSDEBUG 1
 #if TLSDEBUG
@@ -23,7 +18,7 @@ void log_func(int level, const char* msg)
 		return;
 	wxString s(msg, wxConvLocal);
 	s.Trim();
-	pLoggingControlSocket->LogMessage(Debug_Debug, _T("tls: %d %s"), level, s.c_str());
+	pLoggingControlSocket->LogMessage(MessageType::Debug_Debug, _T("tls: %d %s"), level, s);
 }
 #endif
 
@@ -141,9 +136,6 @@ bool CTlsSocket::InitSession()
 	gnutls_transport_set_push_function(m_session, PushFunction);
 	gnutls_transport_set_pull_function(m_session, PullFunction);
 	gnutls_transport_set_ptr(m_session, (gnutls_transport_ptr_t)this);
-#if GNUTLS_VERSION_NUMBER < 0x020c00
-	gnutls_transport_set_lowat(m_session, 0);
-#endif
 
 	return true;
 }
@@ -192,7 +184,7 @@ void CTlsSocket::UninitSession()
 }
 
 
-void CTlsSocket::LogError(int code, const wxString& function)
+void CTlsSocket::LogError(int code, const wxString& function, MessageType logLevel)
 {
 	if (code == GNUTLS_E_WARNING_ALERT_RECEIVED || code == GNUTLS_E_FATAL_ALERT_RECEIVED)
 		PrintAlert();
@@ -201,22 +193,18 @@ void CTlsSocket::LogError(int code, const wxString& function)
 
 	if (error)
 	{
-#if wxUSE_UNICODE
 		wxString str(error, wxConvLocal);
-#else
-		wxString str(error);
-#endif
-		if (function.IsEmpty())
-			m_pOwner->LogMessage(::Error, _T("GnuTLS error %d: %s"), code, str.c_str());
+		if (function.empty())
+			m_pOwner->LogMessage(logLevel, _T("GnuTLS error %d: %s"), code, str);
 		else
-			m_pOwner->LogMessage(::Error, _T("GnuTLS error %d in %s: %s"), code, function.c_str(), str.c_str());
+			m_pOwner->LogMessage(logLevel, _T("GnuTLS error %d in %s: %s"), code, function, str);
 	}
 	else
 	{
-		if (function.IsEmpty())
-			m_pOwner->LogMessage(::Error, _T("GnuTLS error %d"), code);
+		if (function.empty())
+			m_pOwner->LogMessage(logLevel, _T("GnuTLS error %d"), code);
 		else
-			m_pOwner->LogMessage(::Error, _T("GnuTLS error %d in %s"), code, function.c_str());
+			m_pOwner->LogMessage(logLevel, _T("GnuTLS error %d in %s"), code, function);
 	}
 }
 
@@ -226,15 +214,11 @@ void CTlsSocket::PrintAlert()
 	const char* alert = gnutls_alert_get_name(last_alert);
 	if (alert)
 	{
-#if wxUSE_UNICODE
 		wxString str(alert, wxConvLocal);
-#else
-		wxString str(alert);
-#endif
-		m_pOwner->LogMessage(::Debug_Warning, _T("GnuTLS alert %d: %s"), last_alert, str.c_str());
+		m_pOwner->LogMessage(MessageType::Debug_Warning, _T("GnuTLS alert %d: %s"), last_alert, str);
 	}
 	else
-		m_pOwner->LogMessage(::Debug_Warning, _T("GnuTLS alert %d"), last_alert);
+		m_pOwner->LogMessage(MessageType::Debug_Warning, _T("GnuTLS alert %d"), last_alert);
 }
 
 ssize_t CTlsSocket::PushFunction(gnutls_transport_ptr_t ptr, const void* data, size_t len)
@@ -250,7 +234,7 @@ ssize_t CTlsSocket::PullFunction(gnutls_transport_ptr_t ptr, void* data, size_t
 ssize_t CTlsSocket::PushFunction(const void* data, size_t len)
 {
 #if TLSDEBUG
-	m_pOwner->LogMessage(Debug_Debug, _T("CTlsSocket::PushFunction(%x, %d)"), data, len);
+	m_pOwner->LogMessage(MessageType::Debug_Debug, _T("CTlsSocket::PushFunction(%x, %d)"), data, len);
 #endif
 	if (!m_canWriteToSocket)
 	{
@@ -281,7 +265,7 @@ ssize_t CTlsSocket::PushFunction(const void* data, size_t len)
 	}
 
 #if TLSDEBUG
-	m_pOwner->LogMessage(Debug_Debug, _T("  returning %d"), written);
+	m_pOwner->LogMessage(MessageType::Debug_Debug, _T("  returning %d"), written);
 #endif
 
 	return written;
@@ -290,7 +274,7 @@ ssize_t CTlsSocket::PushFunction(const void* data, size_t len)
 ssize_t CTlsSocket::PullFunction(void* data, size_t len)
 {
 #if TLSDEBUG
-	m_pOwner->LogMessage(Debug_Debug, _T("CTlsSocket::PullFunction(%x, %d)"), data, len);
+	m_pOwner->LogMessage(MessageType::Debug_Debug, _T("CTlsSocket::PullFunction(%x, %d)"), data, len);
 #endif
 	if (!m_pSocketBackend)
 	{
@@ -335,7 +319,7 @@ ssize_t CTlsSocket::PullFunction(void* data, size_t len)
 		m_socket_eof = true;
 
 #if TLSDEBUG
-	m_pOwner->LogMessage(Debug_Debug, _T("  returning %d"), read);
+	m_pOwner->LogMessage(MessageType::Debug_Debug, _T("  returning %d"), read);
 #endif
 
 	return read;
@@ -364,7 +348,7 @@ void CTlsSocket::OnSocketEvent(CSocketEvent& event)
 			if (peeked >= 0)
 			{
 				if (peeked > 0)
-					m_pOwner->LogMessage(Debug_Verbose, _T("CTlsSocket::OnSocketEvent(): pending data, postponing close event"));
+					m_pOwner->LogMessage(MessageType::Debug_Verbose, _T("CTlsSocket::OnSocketEvent(): pending data, postponing close event"));
 				else
 				{
 					m_socket_eof = true;
@@ -376,7 +360,7 @@ void CTlsSocket::OnSocketEvent(CSocketEvent& event)
 					return;
 			}
 
-			m_pOwner->LogMessage(Debug_Info, _T("CTlsSocket::OnSocketEvent(): close event received"));
+			m_pOwner->LogMessage(MessageType::Debug_Info, _T("CTlsSocket::OnSocketEvent(): close event received"));
 
 			//Uninit();
 			CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close);
@@ -390,7 +374,7 @@ void CTlsSocket::OnSocketEvent(CSocketEvent& event)
 
 void CTlsSocket::OnRead()
 {
-	m_pOwner->LogMessage(Debug_Debug, _T("CTlsSocket::OnRead()"));
+	m_pOwner->LogMessage(MessageType::Debug_Debug, _T("CTlsSocket::OnRead()"));
 
 	m_canReadFromSocket = true;
 
@@ -414,7 +398,7 @@ void CTlsSocket::OnRead()
 
 void CTlsSocket::OnSend()
 {
-	m_pOwner->LogMessage(Debug_Debug, _T("CTlsSocket::OnSend()"));
+	m_pOwner->LogMessage(MessageType::Debug_Debug, _T("CTlsSocket::OnSend()"));
 
 	m_canWriteToSocket = true;
 
@@ -438,38 +422,24 @@ void CTlsSocket::OnSend()
 
 bool CTlsSocket::CopySessionData(const CTlsSocket* pPrimarySocket)
 {
-	size_t session_data_size = 0;
-
-	// Get buffer size
-	int res = gnutls_session_get_data(pPrimarySocket->m_session, 0, &session_data_size);
-	if (res && res != GNUTLS_E_SHORT_MEMORY_BUFFER )
-	{
-		m_pOwner->LogMessage(Debug_Warning, _T("gnutls_session_get_data on primary socket failed: %d"), res);
-		return true;
-	}
-
-	// Get session data
-	char *session_data = new char[session_data_size];
-	res = gnutls_session_get_data(pPrimarySocket->m_session, session_data, &session_data_size);
-	if (res)
-	{
-		delete [] session_data;
-		m_pOwner->LogMessage(Debug_Warning, _T("gnutls_session_get_data on primary socket failed: %d"), res);
+	gnutls_datum_t d;
+	int res = gnutls_session_get_data2(pPrimarySocket->m_session, &d);
+	if (res) {
+		m_pOwner->LogMessage(MessageType::Debug_Warning, _T("gnutls_session_get_data2 on primary socket failed: %d"), res);
 		return true;
 	}
 
 	// Set session data
-	res = gnutls_session_set_data(m_session, session_data, session_data_size );
-	delete [] session_data;
-	if (res)
-	{
-		m_pOwner->LogMessage(Debug_Info, _T("gnutls_session_set_data failed: %d. Going to reinitialize session."), res);
+	res = gnutls_session_set_data(m_session, d.data, d.size );
+	gnutls_free(d.data);
+	if (res) {
+		m_pOwner->LogMessage(MessageType::Debug_Info, _T("gnutls_session_set_data failed: %d. Going to reinitialize session."), res);
 		UninitSession();
 		if (!InitSession())
 			return false;
 	}
 	else
-		m_pOwner->LogMessage(Debug_Info, _T("Trying to resume existing TLS session."));
+		m_pOwner->LogMessage(MessageType::Debug_Info, _T("Trying to resume existing TLS session."));
 
 	return true;
 }
@@ -479,15 +449,16 @@ bool CTlsSocket::ResumedSession() const
 	return gnutls_session_is_resumed(m_session) != 0;
 }
 
-int CTlsSocket::Handshake(const CTlsSocket* pPrimarySocket /*=0*/, bool try_resume /*=false*/)
+int CTlsSocket::Handshake(const CTlsSocket* pPrimarySocket, bool try_resume)
 {
-	m_pOwner->LogMessage(Debug_Verbose, _T("CTlsSocket::Handshake()"));
+	m_pOwner->LogMessage(MessageType::Debug_Verbose, _T("CTlsSocket::Handshake()"));
 	wxASSERT(m_session);
 
 	m_tlsState = handshake;
 
-	if (pPrimarySocket)
-	{
+	wxString hostname;
+
+	if (pPrimarySocket) {
 		// Implicitly trust certificate of primary socket
 		unsigned int cert_list_size;
 		const gnutls_datum_t* const cert_list = gnutls_certificate_get_peers(pPrimarySocket->m_session, &cert_list_size);
@@ -504,6 +475,21 @@ int CTlsSocket::Handshake(const CTlsSocket* pPrimarySocket /*=0*/, bool try_resu
 			if (!CopySessionData(pPrimarySocket))
 				return FZ_REPLY_ERROR;
 		}
+
+		hostname = pPrimarySocket->m_pSocket->GetPeerHost();
+	}
+	else {
+		hostname = m_pSocket->GetPeerHost();
+	}
+
+	if( !hostname.empty() && !IsIpAddress(hostname) ) {
+		const wxWX2MBbuf utf8 = hostname.mb_str(wxConvUTF8);
+		if( utf8 ) {
+			int res = gnutls_server_name_set( m_session, GNUTLS_NAME_DNS, utf8, strlen(utf8) );
+			if( res ) {
+				LogError(res, _T("gnutls_server_name_set"), MessageType::Debug_Warning );
+			}
+		}
 	}
 
 	return ContinueHandshake();
@@ -511,7 +497,7 @@ int CTlsSocket::Handshake(const CTlsSocket* pPrimarySocket /*=0*/, bool try_resu
 
 int CTlsSocket::ContinueHandshake()
 {
-	m_pOwner->LogMessage(Debug_Verbose, _T("CTlsSocket::ContinueHandshake()"));
+	m_pOwner->LogMessage(MessageType::Debug_Verbose, _T("CTlsSocket::ContinueHandshake()"));
 	wxASSERT(m_session);
 	wxASSERT(m_tlsState == handshake);
 
@@ -525,17 +511,17 @@ int CTlsSocket::ContinueHandshake()
 	}
 	if (!res)
 	{
-		m_pOwner->LogMessage(Debug_Info, _T("TLS Handshake successful"));
+		m_pOwner->LogMessage(MessageType::Debug_Info, _T("TLS Handshake successful"));
 
 		if (ResumedSession())
-			m_pOwner->LogMessage(Debug_Info, _T("TLS Session resumed"));
+			m_pOwner->LogMessage(MessageType::Debug_Info, _T("TLS Session resumed"));
 
 		const wxString protocol = GetProtocolName();
 		const wxString keyExchange = GetKeyExchange();
 		const wxString cipherName = GetCipherName();
 		const wxString macName = GetMacName();
 
-		m_pOwner->LogMessage(Debug_Info, _T("Protocol: %s, Key exchange: %s, Cipher: %s, MAC: %s"), protocol.c_str(), keyExchange.c_str(), cipherName.c_str(), macName.c_str());
+		m_pOwner->LogMessage(MessageType::Debug_Info, _T("Protocol: %s, Key exchange: %s, Cipher: %s, MAC: %s"), protocol, keyExchange, cipherName, macName);
 
 		res = VerifyCertificate();
 		if (res != FZ_REPLY_OK)
@@ -742,8 +728,6 @@ void CTlsSocket::CheckResumeFailedReadWrite()
 		m_writeSkip += res;
 		m_lastWriteFailed = false;
 		m_canTriggerWrite = true;
-
-		wxASSERT(GNUTLS_E_INVALID_REQUEST == gnutls_record_send(m_session, 0, 0));
 	}
 	if (m_lastReadFailed)
 	{
@@ -779,7 +763,7 @@ void CTlsSocket::CheckResumeFailedReadWrite()
 
 void CTlsSocket::Failure(int code, int socket_error, const wxString& function)
 {
-	m_pOwner->LogMessage(::Debug_Debug, _T("CTlsSocket::Failure(%d, %d)"), code, socket_error);
+	m_pOwner->LogMessage(MessageType::Debug_Debug, _T("CTlsSocket::Failure(%d, %d)"), code, socket_error);
 	if (code)
 	{
 		LogError(code, function);
@@ -791,7 +775,7 @@ void CTlsSocket::Failure(int code, int socket_error, const wxString& function)
 #endif
 				)
 			{
-				m_pOwner->LogMessage(Status, _("Server did not properly shut down TLS connection"));
+				m_pOwner->LogMessage(MessageType::Status, _("Server did not properly shut down TLS connection"));
 			}
 		}
 	}
@@ -828,7 +812,7 @@ int CTlsSocket::Peek(void *buffer, unsigned int len, int& error)
 
 int CTlsSocket::Shutdown()
 {
-	m_pOwner->LogMessage(Debug_Verbose, _T("CTlsSocket::Shutdown()"));
+	m_pOwner->LogMessage(MessageType::Debug_Verbose, _T("CTlsSocket::Shutdown()"));
 
 	if (m_tlsState == closed)
 		return 0;
@@ -839,7 +823,7 @@ int CTlsSocket::Shutdown()
 	if (m_tlsState == handshake || m_tlsState == verifycert)
 	{
 		// Shutdown during handshake is not a good idea.
-		m_pOwner->LogMessage(Debug_Verbose, _T("Shutdown during handshake, postponing"));
+		m_pOwner->LogMessage(MessageType::Debug_Verbose, _T("Shutdown during handshake, postponing"));
 		m_shutdown_requested = true;
 		return EAGAIN;
 	}
@@ -867,7 +851,7 @@ int CTlsSocket::Shutdown()
 
 void CTlsSocket::ContinueShutdown()
 {
-	m_pOwner->LogMessage(Debug_Verbose, _T("CTlsSocket::ContinueShutdown()"));
+	m_pOwner->LogMessage(MessageType::Debug_Verbose, _T("CTlsSocket::ContinueShutdown()"));
 
 	int res = gnutls_bye(m_session, GNUTLS_SHUT_WR);
 	while ((res == GNUTLS_E_INTERRUPTED || res == GNUTLS_E_AGAIN) && m_canWriteToSocket)
@@ -890,7 +874,7 @@ void CTlsSocket::TrustCurrentCert(bool trusted)
 {
 	if (m_tlsState != verifycert)
 	{
-		m_pOwner->LogMessage(Debug_Warning, _T("TrustCurrentCert called at wrong time."));
+		m_pOwner->LogMessage(MessageType::Debug_Warning, _T("TrustCurrentCert called at wrong time."));
 		return;
 	}
 
@@ -913,7 +897,7 @@ void CTlsSocket::TrustCurrentCert(bool trusted)
 		return;
 	}
 
-	m_pOwner->LogMessage(::Error, _("Remote certificate not trusted."));
+	m_pOwner->LogMessage(MessageType::Error, _("Remote certificate not trusted."));
 	Failure(0, ECONNABORTED);
 }
 
@@ -934,17 +918,17 @@ static wxString bin2hex(const unsigned char* in, size_t size)
 bool CTlsSocket::ExtractCert(const void* in, CCertificate& out)
 {
 	const gnutls_datum_t* datum = reinterpret_cast<const gnutls_datum_t*>(in);
-	
+
 	gnutls_x509_crt_t cert;
 	if (gnutls_x509_crt_init(&cert))
 	{
-		m_pOwner->LogMessage(::Error, _("Could not initialize structure for peer certificates, gnutls_x509_crt_init failed"));
+		m_pOwner->LogMessage(MessageType::Error, _("Could not initialize structure for peer certificates, gnutls_x509_crt_init failed"));
 		return false;
 	}
 
 	if (gnutls_x509_crt_import(cert, datum, GNUTLS_X509_FMT_DER))
 	{
-		m_pOwner->LogMessage(::Error, _("Could not import peer certificates, gnutls_x509_crt_import failed"));
+		m_pOwner->LogMessage(MessageType::Error, _("Could not import peer certificates, gnutls_x509_crt_import failed"));
 		gnutls_x509_crt_deinit(cert);
 		return false;
 	}
@@ -1000,9 +984,8 @@ bool CTlsSocket::ExtractCert(const void* in, CCertificate& out)
 	}
 	else
 		LogError(res, _T("gnutls_x509_crt_get_dn"));
-	if (subject == _T(""))
-	{
-		m_pOwner->LogMessage(::Error, _("Could not get distinguished name of certificate subject, gnutls_x509_get_dn failed"));
+	if (subject.empty()) {
+		m_pOwner->LogMessage(MessageType::Error, _("Could not get distinguished name of certificate subject, gnutls_x509_get_dn failed"));
 		gnutls_x509_crt_deinit(cert);
 		return false;
 	}
@@ -1024,9 +1007,8 @@ bool CTlsSocket::ExtractCert(const void* in, CCertificate& out)
 	}
 	else
 		LogError(res, _T("gnutls_x509_crt_get_issuer_dn"));
-	if (issuer == _T(""))
-	{
-		m_pOwner->LogMessage(::Error, _("Could not get distinguished name of certificate issuer, gnutls_x509_get_issuer_dn failed"));
+	if (issuer.empty() ) {
+		m_pOwner->LogMessage(MessageType::Error, _("Could not get distinguished name of certificate issuer, gnutls_x509_get_issuer_dn failed"));
 		gnutls_x509_crt_deinit(cert);
 		return false;
 	}
@@ -1068,7 +1050,7 @@ int CTlsSocket::VerifyCertificate()
 {
 	if (m_tlsState != handshake)
 	{
-		m_pOwner->LogMessage(::Debug_Warning, _T("VerifyCertificate called at wrong time"));
+		m_pOwner->LogMessage(MessageType::Debug_Warning, _T("VerifyCertificate called at wrong time"));
 		return FZ_REPLY_ERROR;
 	}
 
@@ -1076,7 +1058,7 @@ int CTlsSocket::VerifyCertificate()
 
 	if (gnutls_certificate_type_get(m_session) != GNUTLS_CRT_X509)
 	{
-		m_pOwner->LogMessage(::Error, _("Unsupported certificate type"));
+		m_pOwner->LogMessage(MessageType::Error, _("Unsupported certificate type"));
 		Failure(0, ECONNABORTED);
 		return FZ_REPLY_ERROR;
 	}
@@ -1084,28 +1066,28 @@ int CTlsSocket::VerifyCertificate()
 	unsigned int status = 0;
 	if (gnutls_certificate_verify_peers2(m_session, &status) < 0)
 	{
-		m_pOwner->LogMessage(::Error, _("Failed to verify peer certificate"));
+		m_pOwner->LogMessage(MessageType::Error, _("Failed to verify peer certificate"));
 		Failure(0, ECONNABORTED);
 		return FZ_REPLY_ERROR;
 	}
 
 	if (status & GNUTLS_CERT_REVOKED)
 	{
-		m_pOwner->LogMessage(::Error, _("Beware! Certificate has been revoked"));
+		m_pOwner->LogMessage(MessageType::Error, _("Beware! Certificate has been revoked"));
 		Failure(0, ECONNABORTED);
 		return FZ_REPLY_ERROR;
 	}
 
 	if (status & GNUTLS_CERT_SIGNER_NOT_CA)
 	{
-		m_pOwner->LogMessage(::Error, _("Incomplete chain, top certificate is not self-signed certificate authority certificate"));
+		m_pOwner->LogMessage(MessageType::Error, _("Incomplete chain, top certificate is not self-signed certificate authority certificate"));
 		Failure(0, ECONNABORTED);
 		return FZ_REPLY_ERROR;
 	}
 
 	if (m_require_root_trust && status & GNUTLS_CERT_SIGNER_NOT_FOUND)
 	{
-		m_pOwner->LogMessage(::Error, _("Root certificate is not trusted"));
+		m_pOwner->LogMessage(MessageType::Error, _("Root certificate is not trusted"));
 		Failure(0, ECONNABORTED);
 		return FZ_REPLY_ERROR;
 	}
@@ -1114,7 +1096,7 @@ int CTlsSocket::VerifyCertificate()
 	const gnutls_datum_t* cert_list = gnutls_certificate_get_peers(m_session, &cert_list_size);
 	if (!cert_list || !cert_list_size)
 	{
-		m_pOwner->LogMessage(::Error, _("gnutls_certificate_get_peers returned no certificates"));
+		m_pOwner->LogMessage(MessageType::Error, _("gnutls_certificate_get_peers returned no certificates"));
 		Failure(0, ECONNABORTED);
 		return FZ_REPLY_ERROR;
 	}
@@ -1124,7 +1106,7 @@ int CTlsSocket::VerifyCertificate()
 		if (m_implicitTrustedCert.size != cert_list[0].size ||
 			memcmp(m_implicitTrustedCert.data, cert_list[0].data, cert_list[0].size))
 		{
-			m_pOwner->LogMessage(::Error, _("Primary connection and data connection certificates don't match."));
+			m_pOwner->LogMessage(MessageType::Error, _("Primary connection and data connection certificates don't match."));
 			Failure(0, ECONNABORTED);
 			return FZ_REPLY_ERROR;
 		}
@@ -1162,12 +1144,12 @@ int CTlsSocket::VerifyCertificate()
 
 	m_pOwner->SendAsyncRequest(pNotification);
 
-	m_pOwner->LogMessage(Status, _("Verifying certificate..."));
+	m_pOwner->LogMessage(MessageType::Status, _("Verifying certificate..."));
 
 	return FZ_REPLY_WOULDBLOCK;
 }
 
-void CTlsSocket::OnRateAvailable(enum CRateLimiter::rate_direction direction)
+void CTlsSocket::OnRateAvailable(enum CRateLimiter::rate_direction)
 {
 }
 
@@ -1216,7 +1198,7 @@ bool CTlsSocket::AddTrustedRootCertificate(const wxString& cert)
 	if (!m_initialized)
 		return false;
 
-	if (cert == _T(""))
+	if (cert.empty())
 		return false;
 
 	m_require_root_trust = true;
@@ -1232,24 +1214,20 @@ bool CTlsSocket::AddTrustedRootCertificate(const wxString& cert)
 
 wxString CTlsSocket::ListTlsCiphers(wxString priority)
 {
-	if (priority.IsEmpty())
+	if (priority.empty())
 		priority = wxString::FromUTF8(ciphers);
 
-	wxString list = wxString::Format(_T("Ciphers for %s:\n"), priority.c_str());
+	wxString list = wxString::Format(_T("Ciphers for %s:\n"), priority);
 
-#if GNUTLS_VERSION_NUMBER >= 0x030009
 	gnutls_priority_t pcache;
 	const char *err = 0;
 	int ret = gnutls_priority_init(&pcache, priority.mb_str(), &err);
-	if (ret < 0)
-	{
-		list += wxString::Format(_T("gnutls_priority_init failed with code %d: %s"), ret, wxString::FromUTF8(err ? err : "").c_str());
+	if (ret < 0) {
+		list += wxString::Format(_T("gnutls_priority_init failed with code %d: %s"), ret, err ? wxString::FromUTF8(err) : _T("Unknown error"));
 		return list;
 	}
-	else
-	{
-		for (size_t i = 0; ; i++)
-		{
+	else {
+		for (size_t i = 0; ; i++) {
 			unsigned int idx;
 			ret = gnutls_priority_get_cipher_suite_index(pcache, i, &idx);
 			if (ret == GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE)
@@ -1265,16 +1243,13 @@ wxString CTlsSocket::ListTlsCiphers(wxString priority)
 			{
 				list += wxString::Format(
 					_T("%-50s    0x%02x, 0x%02x    %s\n"),
-					wxString::FromUTF8(name).c_str(),
+					wxString::FromUTF8(name),
 					(unsigned char)id[0],
 					(unsigned char)id[1],
-					wxString::FromUTF8(gnutls_protocol_get_name(version)).c_str());
+					wxString::FromUTF8(gnutls_protocol_get_name(version)));
 			}
 		}
 	}
-#else
-	list += _T("Unknown\n");
-#endif
 
 	return list;
 }
