@@ -25,7 +25,7 @@ public:
 	bool Init();
 	void Uninit();
 
-	int Handshake(const CTlsSocket* pPrimarySocket = 0, bool try_resume = false);
+	int Handshake(const CTlsSocket* pPrimarySocket = 0, bool try_resume = 0);
 
 	virtual int Read(void *buffer, unsigned int size, int& error);
 	virtual int Peek(void *buffer, unsigned int size, int& error);
@@ -71,11 +71,11 @@ protected:
 
 	gnutls_certificate_credentials_t m_certCredentials;
 
-	void LogError(int code, const wxString& function);
+	void LogError(int code, const wxString& function, MessageType logLegel = MessageType::Error);
 	void PrintAlert();
 
 	// Failure logs the error, uninits the session and sends a close event
-	void Failure(int code, int socket_error, const wxString& function = _T(""));
+	void Failure(int code, int socket_error, const wxString& function = wxString());
 
 	static ssize_t PushFunction(gnutls_transport_ptr_t ptr, const void* data, size_t len);
 	static ssize_t PullFunction(gnutls_transport_ptr_t ptr, void* data, size_t len);
