@@ -5,24 +5,24 @@
 
 // Command IDs
 // -----------
-enum Command
+enum class Command
 {
-	cmd_none = 0,
-	cmd_connect,
-	cmd_disconnect,
-	cmd_cancel,
-	cmd_list,
-	cmd_transfer,
-	cmd_delete,
-	cmd_removedir,
-	cmd_mkdir,
-	cmd_rename,
-	cmd_chmod,
-	cmd_raw,
+	none = 0,
+	connect,
+	disconnect,
+	cancel,
+	list,
+	transfer,
+	del,
+	removedir,
+	mkdir,
+	rename,
+	chmod,
+	raw,
 
 	// Only used internally
-	cmd_cwd,
-	cmd_rawtransfer
+	cwd,
+	rawtransfer
 };
 
 // Reply codes
@@ -36,7 +36,7 @@ enum Command
 #define FZ_REPLY_SYNTAXERROR	(0x0010 | FZ_REPLY_ERROR)
 #define FZ_REPLY_NOTCONNECTED	(0x0020 | FZ_REPLY_ERROR)
 #define FZ_REPLY_DISCONNECTED	(0x0040)
-#define FZ_REPLY_INTERNALERROR	(0x0080 | FZ_REPLY_ERROR) // If you get this reply, the error description will be 
+#define FZ_REPLY_INTERNALERROR	(0x0080 | FZ_REPLY_ERROR) // If you get this reply, the error description will be
 														  // given by the last Debug_Warning log message. This
 														  // should not happen unless there is a bug in FileZilla 3.
 #define FZ_REPLY_BUSY			(0x0100 | FZ_REPLY_ERROR)
@@ -54,7 +54,7 @@ enum Command
 	class name : public CCommand \
 	{ \
 	public: \
-		virtual enum Command GetId() const { return id; } \
+		virtual Command GetId() const { return id; } \
 		virtual CCommand* Clone() const { return new name(*this); }
 
 // --------------- //
@@ -64,13 +64,13 @@ enum Command
 class CCommand
 {
 public:
-	CCommand() { }
-	virtual ~CCommand() { }
-	virtual enum Command GetId() const = 0;
+	CCommand() {}
+	virtual ~CCommand() {}
+	virtual Command GetId() const = 0;
 	virtual CCommand *Clone() const = 0;
 };
 
-DECLARE_COMMAND(CConnectCommand, cmd_connect)
+DECLARE_COMMAND(CConnectCommand, Command::connect)
 	CConnectCommand(const CServer &server, bool retry_conncting = true);
 
 	const CServer GetServer() const;
@@ -80,21 +80,21 @@ protected:
 	bool m_retry_connecting;
 };
 
-DECLARE_COMMAND(CDisconnectCommand, cmd_disconnect)
+DECLARE_COMMAND(CDisconnectCommand, Command::disconnect)
 };
 
-DECLARE_COMMAND(CCancelCommand, cmd_cancel)
+DECLARE_COMMAND(CCancelCommand, Command::cancel)
 };
 
 #define LIST_FLAG_REFRESH 1
 #define LIST_FLAG_AVOID 2
 #define LIST_FLAG_FALLBACK_CURRENT 4
 #define LIST_FLAG_LINK 8
-DECLARE_COMMAND(CListCommand, cmd_list)
+DECLARE_COMMAND(CListCommand, Command::list)
 	// Without a given directory, the current directory will be listed.
 	// Directories can either be given as absolute path or as
 	// pair of an absolute path and the very last path segments.
-	
+
 	// Set LIST_FLAG_REFRESH to get a directory listing even if a cache
 	// lookup can be made after finding out true remote directory.
 	//
@@ -109,8 +109,8 @@ DECLARE_COMMAND(CListCommand, cmd_list)
 	// no sane way to distinguish between symlinks to files and symlinks to
 	// directories.
 	CListCommand(int flags = 0);
-	CListCommand(CServerPath path, wxString subDir = _T(""), int flags = 0);
-	
+	CListCommand(CServerPath path, wxString subDir = wxString(), int flags = 0);
+
 	CServerPath GetPath() const;
 	wxString GetSubDir() const;
 
@@ -122,10 +122,15 @@ protected:
 	int m_flags;
 };
 
-DECLARE_COMMAND(CFileTransferCommand, cmd_transfer)
+DECLARE_COMMAND(CFileTransferCommand, Command::transfer)
 
-	struct t_transferSettings
+	class t_transferSettings
 	{
+	public:
+		t_transferSettings()
+			: binary(true)
+		{}
+
 		bool binary;
 	};
 
@@ -149,7 +154,7 @@ protected:
 	t_transferSettings m_transferSettings;
 };
 
-DECLARE_COMMAND(CRawCommand, cmd_raw)
+DECLARE_COMMAND(CRawCommand, Command::raw)
 	CRawCommand(const wxString &command);
 
 	wxString GetCommand() const;
@@ -158,9 +163,9 @@ protected:
 	wxString m_command;
 };
 
-DECLARE_COMMAND(CDeleteCommand, cmd_delete)
+DECLARE_COMMAND(CDeleteCommand, Command::del)
 	CDeleteCommand(const CServerPath& path, const std::list<wxString>& files);
-	
+
 	CServerPath GetPath() const { return m_path; }
 	const std::list<wxString>& GetFiles() const { return m_files; }
 
@@ -170,9 +175,9 @@ protected:
 	const std::list<wxString> m_files;
 };
 
-DECLARE_COMMAND(CRemoveDirCommand, cmd_removedir)
-    // Directories can either be given as absolute path or as
-    // pair of an absolute path and the very last path segments.
+DECLARE_COMMAND(CRemoveDirCommand, Command::removedir)
+	// Directories can either be given as absolute path or as
+	// pair of an absolute path and the very last path segments.
 	CRemoveDirCommand(const CServerPath& path, const wxString& subdDir);
 
 	CServerPath GetPath() const { return m_path; }
@@ -184,7 +189,7 @@ protected:
 	wxString m_subDir;
 };
 
-DECLARE_COMMAND(CMkdirCommand, cmd_mkdir)
+DECLARE_COMMAND(CMkdirCommand, Command::mkdir)
 	CMkdirCommand(const CServerPath& path);
 
 	CServerPath GetPath() const { return m_path; }
@@ -194,7 +199,7 @@ protected:
 	CServerPath m_path;
 };
 
-DECLARE_COMMAND(CRenameCommand, cmd_rename)
+DECLARE_COMMAND(CRenameCommand, Command::rename)
 	CRenameCommand(const CServerPath& fromPath, const wxString& fromFile,
 				   const CServerPath& toPath, const wxString& toFile);
 
@@ -210,10 +215,10 @@ protected:
 	wxString m_toFile;
 };
 
-DECLARE_COMMAND(CChmodCommand, cmd_chmod)
+DECLARE_COMMAND(CChmodCommand, Command::chmod)
 	// The permission string should be given in a format understandable by the server.
 	// Most likely it's the defaut octal representation used by the unix chmod command,
-    // i.e. chmod 755 foo.bar
+	// i.e. chmod 755 foo.bar
 	CChmodCommand(const CServerPath& path, const wxString& file, const wxString& permission);
 
 	CServerPath GetPath() const { return m_path; }
