@@ -1,6 +1,9 @@
 #ifndef __DIRECTORYLISTING_H__
 #define __DIRECTORYLISTING_H__
 
+#include "optional.h"
+#include "timeex.h"
+
 #include <map>
 
 class CDirentry
@@ -8,20 +11,14 @@ class CDirentry
 public:
 	wxString name;
 	wxLongLong size;
-	wxString permissions;
-	wxString ownerGroup;
+	CRefcountObject<wxString> permissions;
+	CRefcountObject<wxString> ownerGroup;
 
 	enum _flags
 	{
 		flag_dir = 1,
 		flag_link = 2,
-		flag_unsure = 4, // May be set on cached items if any changes were made to the file
-
-		flag_timestamp_date = 0x10,
-		flag_timestamp_time = 0x20,
-		flag_timestamp_seconds = 0x40,
-
-		flag_timestamp_mask = 0x70
+		flag_unsure = 4 // May be set on cached items if any changes were made to the file
 	};
 	int flags;
 
@@ -41,21 +38,20 @@ public:
 
 	inline bool has_date() const
 	{
-		return (flags & flag_timestamp_date) != 0;
+		return time.IsValid();;
 	}
 	inline bool has_time() const
 	{
-		return (flags & flag_timestamp_time) != 0;
+		return time.IsValid() && time.GetAccuracy() >= CDateTime::hours;
 	}
 	inline bool has_seconds() const
 	{
-		return (flags & flag_timestamp_seconds) != 0;
+		return time.IsValid() && time.GetAccuracy() >= CDateTime::seconds;
 	}
 
+	CSparseOptional<wxString> target; // Set to linktarget it link is true
 
-	wxString target; // Set to linktarget it link is true
-
-	wxDateTime time;
+	CDateTime time;
 
 	wxString dump() const;
 	bool operator==(const CDirentry &op) const;
@@ -66,6 +62,8 @@ public:
 class CDirectoryListing
 {
 public:
+	typedef CDirentry value_type;
+
 	CDirectoryListing();
 	CDirectoryListing(const CDirectoryListing& listing);
 
@@ -97,7 +95,13 @@ public:
 		unsure_dir_changed = 0x20,
 		unsure_dir_mask = 0x38,
 		unsure_unknown = 0x40,
-		unsure_invalid = 0x80 // Recommended action: Do a full refresh
+		unsure_invalid = 0x80, // Recommended action: Do a full refresh
+		unsure_mask = 0xff,
+
+		listing_failed = 0x100,
+		listing_has_dirs = 0x200,
+		listing_has_perms = 0x400,
+		listing_has_usergroup = 0x800
 	};
 	// Lowest bit indicates a file got added
 	// Next bit indicates a file got removed
@@ -107,14 +111,15 @@ public:
 	//
 	// These bits should help the user interface to choose an appropriate sorting
 	// algorithm for modified listings
-	int m_hasUnsureEntries;
-	bool m_failed;
-	bool m_hasDirs;
+	int m_flags;
 
-	bool m_has_perms;
-	bool m_has_usergroup;
+	int get_unsure_flags() const { return m_flags & unsure_mask; }
+	bool failed() const { return (m_flags & listing_failed) != 0; }
+	bool has_dirs() const { return (m_flags & listing_has_dirs) != 0; }
+	bool has_perms() const { return (m_flags & listing_has_perms) != 0; }
+	bool has_usergroup() const { return (m_flags & listing_has_usergroup) != 0; }
 
-	CTimeEx m_firstListTime;
+	CMonotonicTime m_firstListTime;
 
 	void Assign(const std::list<CDirentry> &entries);
 
