@@ -13,7 +13,10 @@
 #include <wx/tokenzr.h>
 #include "cmdline.h"
 #include "welcome_dialog.h"
+#include <msgbox.h>
+#include "local_filesys.h"
 
+#include <wx/xrc/xh_animatctrl.h>
 #include <wx/xrc/xh_bmpbt.h>
 #include <wx/xrc/xh_bttn.h>
 #include <wx/xrc/xh_chckb.h>
@@ -51,9 +54,7 @@
 #include <wx/stdpaths.h>
 #endif
 
-#ifdef __WXGTK__
 #include "locale_initializer.h"
-#endif
 
 #ifdef ENABLE_BINRELOC
 	#define BR_PTHREADS 0
@@ -65,9 +66,9 @@
 #endif
 
 #ifndef __WXGTK__
-IMPLEMENT_APP(CFileZillaApp);
+IMPLEMENT_APP(CFileZillaApp)
 #else
-IMPLEMENT_APP_NO_MAIN(CFileZillaApp);
+IMPLEMENT_APP_NO_MAIN(CFileZillaApp)
 #endif //__WXGTK__
 
 CFileZillaApp::CFileZillaApp()
@@ -93,7 +94,7 @@ bool IsServiceRunning(const wxString& serviceName)
 	SC_HANDLE hScm = OpenSCManager(0, 0, GENERIC_READ);
 	if (!hScm)
 	{
-		//wxMessageBox(_T("OpenSCManager failed"));
+		//wxMessageBoxEx(_T("OpenSCManager failed"));
 		return false;
 	}
 
@@ -101,7 +102,7 @@ bool IsServiceRunning(const wxString& serviceName)
 	if (!hService)
 	{
 		CloseServiceHandle(hScm);
-		//wxMessageBox(_T("OpenService failed"));
+		//wxMessageBoxEx(_T("OpenService failed"));
 		return false;
 	}
 
@@ -110,7 +111,7 @@ bool IsServiceRunning(const wxString& serviceName)
 	{
 		CloseServiceHandle(hService);
 		CloseServiceHandle(hScm);
-		//wxMessageBox(_T("ControlService failed"));
+		//wxMessageBoxEx(_T("ControlService failed"));
 		return false;
 	}
 
@@ -139,7 +140,7 @@ bool CheckForWin2003FirewallBug()
 	if (!key.Exists() || !key.Open(wxRegKey::Read))
 		return false;
 
-	long value;
+	long value = 0;
 	if (!key.HasValue(_T("EnableFirewall")) || !key.QueryValue(_T("EnableFirewall"), &value))
 		return false;
 
@@ -174,6 +175,65 @@ static void SetAppId()
 
 #endif //__WXMSW__
 
+void CFileZillaApp::InitLocale()
+{
+	wxString language = COptions::Get()->GetOption(OPTION_LANGUAGE);
+	const wxLanguageInfo* pInfo = wxLocale::FindLanguageInfo(language);
+	if (!language.empty())
+	{
+#ifdef __WXGTK__
+		if (CInitializer::error)
+		{
+			wxString error;
+
+			wxLocale *loc = wxGetLocale();
+			const wxLanguageInfo* currentInfo = loc ? loc->GetLanguageInfo(loc->GetLanguage()) : 0;
+			if (!loc || !currentInfo)
+			{
+				if (!pInfo)
+					error.Printf(_("Failed to set language to %s, using default system language."),
+						language);
+				else
+					error.Printf(_("Failed to set language to %s (%s), using default system language."),
+						pInfo->Description, language);
+			}
+			else
+			{
+				wxString currentName = currentInfo->CanonicalName;
+
+				if (!pInfo)
+					error.Printf(_("Failed to set language to %s, using default system language (%s, %s)."),
+						language, loc->GetLocale(),
+						currentName);
+				else
+					error.Printf(_("Failed to set language to %s (%s), using default system language (%s, %s)."),
+						pInfo->Description, language, loc->GetLocale(),
+						currentName);
+			}
+
+			error += _T("\n");
+			error += _("Please make sure the requested locale is installed on your system.");
+			wxMessageBoxEx(error, _("Failed to change language"), wxICON_EXCLAMATION);
+
+			COptions::Get()->SetOption(OPTION_LANGUAGE, _T(""));
+		}
+#else
+		if (!pInfo || !SetLocale(pInfo->Language)) {
+			for( language = GetFallbackLocale(language); !language.empty(); language = GetFallbackLocale(language) ) {
+				const wxLanguageInfo* fallbackInfo = wxLocale::FindLanguageInfo(language);
+				if( fallbackInfo && SetLocale(fallbackInfo->Language )) {
+					return;
+				}
+			}
+			if (pInfo && !pInfo->Description.empty())
+				wxMessageBoxEx(wxString::Format(_("Failed to set language to %s (%s), using default system language"), pInfo->Description, language), _("Failed to change language"), wxICON_EXCLAMATION);
+			else
+				wxMessageBoxEx(wxString::Format(_("Failed to set language to %s, using default system language"), language), _("Failed to change language"), wxICON_EXCLAMATION);
+		}
+#endif
+	}
+}
+
 bool CFileZillaApp::OnInit()
 {
 	AddStartupProfileRecord(_T("CFileZillaApp::OnInit()"));
@@ -199,13 +259,12 @@ bool CFileZillaApp::OnInit()
 	wxSystemOptions::SetOption(_T("no-maskblt"), 0);
 	wxSystemOptions::SetOption(_T("msw.window.no-clip-children"), 0);
 	wxSystemOptions::SetOption(_T("msw.font.no-proof-quality"), 0);
-#endif
-
-#ifdef __WXMSW__
 	wxSystemOptions::SetOption(_T("msw.remap"), 0);
+	wxSystemOptions::SetOption(_T("msw.staticbox.optimized-paint"), 0);
 #endif
 #ifdef __WXMAC__
 	wxSystemOptions::SetOption(_T("mac.listctrl.always_use_generic"), 1);
+	wxSystemOptions::SetOption(_T("mac.textcontrol-use-spell-checker"), 0);
 #endif
 
 	int cmdline_result = ProcessCommandLine();
@@ -224,61 +283,12 @@ bool CFileZillaApp::OnInit()
 
 	COptions::Init();
 
-	wxString language = COptions::Get()->GetOption(OPTION_LANGUAGE);
-	const wxLanguageInfo* pInfo = wxLocale::FindLanguageInfo(language);
-	if (language != _T(""))
-	{
-#ifdef __WXGTK__
-		if (CInitializer::error)
-		{
-			wxString error;
-
-			wxLocale *loc = wxGetLocale();
-			const wxLanguageInfo* currentInfo = loc ? loc->GetLanguageInfo(loc->GetLanguage()) : 0;
-			if (!loc || !currentInfo)
-			{
-				if (!pInfo)
-					error.Printf(_("Failed to set language to %s, using default system language."),
-						language.c_str());
-				else
-					error.Printf(_("Failed to set language to %s (%s), using default system language."),
-						pInfo->Description.c_str(), language.c_str());
-			}
-			else
-			{
-				wxString currentName = currentInfo->CanonicalName;
-
-				if (!pInfo)
-					error.Printf(_("Failed to set language to %s, using default system language (%s, %s)."),
-						language.c_str(), loc->GetLocale(),
-						currentName.c_str());
-				else
-					error.Printf(_("Failed to set language to %s (%s), using default system language (%s, %s)."),
-						pInfo->Description.c_str(), language.c_str(), loc->GetLocale(),
-						currentName.c_str());
-			}
-
-			error += _T("\n");
-			error += _("Please make sure the requested locale is installed on your system.");
-			wxMessageBox(error, _("Failed to change language"), wxICON_EXCLAMATION);
-
-			COptions::Get()->SetOption(OPTION_LANGUAGE, _T(""));
-		}
-#else
-		if (!pInfo || !SetLocale(pInfo->Language))
-		{
-			if (pInfo && pInfo->Description)
-				wxMessageBox(wxString::Format(_("Failed to set language to %s (%s), using default system language"), pInfo->Description.c_str(), language.c_str()), _("Failed to change language"), wxICON_EXCLAMATION);
-			else
-				wxMessageBox(wxString::Format(_("Failed to set language to %s, using default system language"), language.c_str()), _("Failed to change language"), wxICON_EXCLAMATION);
-		}
-#endif
-	}
+	InitLocale();
 
 #ifndef _DEBUG
 	const wxString& buildType = CBuildInfo::GetBuildType();
 	if (buildType == _T("nightly"))
-		wxMessageBox(_T("You are using a nightly development version of FileZilla 3, do not expect anything to work.\r\nPlease use the official releases instead.\r\n\r\n\
+		wxMessageBoxEx(_T("You are using a nightly development version of FileZilla 3, do not expect anything to work.\r\nPlease use the official releases instead.\r\n\r\n\
 Unless explicitly instructed otherwise,\r\n\
 DO NOT post bugreports,\r\n\
 DO NOT use it in production environments,\r\n\
@@ -307,7 +317,7 @@ See http://support.microsoft.com/kb/931130 for background information.\n\n\
 Unless you either disable Windows Firewall or the Application Layer Gateway service,\n\
 FileZilla will timeout on big transfers.\
 ");
-		wxMessageBox(error, _("Operating system problem detected"), wxICON_EXCLAMATION);
+		wxMessageBoxEx(error, _("Operating system problem detected"), wxICON_EXCLAMATION);
 	}
 #endif
 
@@ -355,12 +365,12 @@ int CFileZillaApp::OnExit()
 bool CFileZillaApp::FileExists(const wxString& file) const
 {
 	int pos = file.Find('*');
-	if (pos == -1)
+	if (pos < 0)
 		return wxFileExists(file);
 
 	wxASSERT(pos > 0);
 	wxASSERT(file[pos - 1] == '/');
-	wxASSERT(file[pos + 1] == '/');
+	wxASSERT(file.size() > static_cast<size_t>(pos + 1) && file[pos + 1] == '/');
 
 	wxLogNull nullLog;
 	wxDir dir(file.Left(pos));
@@ -380,7 +390,7 @@ bool CFileZillaApp::FileExists(const wxString& file) const
 	return false;
 }
 
-wxString CFileZillaApp::GetDataDir(wxString fileToFind) const
+CLocalPath CFileZillaApp::GetDataDir(wxString fileToFind) const
 {
 	/*
 	 * Finding the resources in all cases is a difficult task,
@@ -394,11 +404,11 @@ wxString CFileZillaApp::GetDataDir(wxString fileToFind) const
 	 */
 
 #ifdef __WXMAC__
-	wxString path = wxStandardPaths::Get().GetDataDir();
-	if (FileExists(path + fileToFind))
+	CLocalPath path(wxStandardPaths::Get().GetDataDir());
+	if (FileExists(path.GetPath() + fileToFind))
 		return path;
 
-	return _T("");
+	return CLocalPath();
 #else
 
 	wxPathList pathList;
@@ -418,7 +428,7 @@ wxString CFileZillaApp::GetDataDir(wxString fileToFind) const
 		wxString datadir(SELFPATH , *wxConvCurrent);
 		wxFileName fn(datadir);
 		datadir = fn.GetPath();
-		if (datadir != _T(""))
+		if (!datadir.empty())
 			pathList.Add(datadir);
 
 	}
@@ -426,7 +436,7 @@ wxString CFileZillaApp::GetDataDir(wxString fileToFind) const
 	if (path && *path)
 	{
 		wxString datadir(DATADIR, *wxConvCurrent);
-		if (datadir != _T(""))
+		if (!datadir.empty())
 			pathList.Add(datadir);
 	}
 #elif defined __WXMSW__
@@ -435,7 +445,7 @@ wxString CFileZillaApp::GetDataDir(wxString fileToFind) const
 	if (res > 0 && res < 1000)
 	{
 		wxFileName fn(path);
-		pathList.Add(fn.GetPath());
+		pathList.Add(fn.GetPath(wxPATH_GET_VOLUME | wxPATH_GET_SEPARATOR));
 	}
 #endif //ENABLE_BINRELOC and __WXMSW__ blocks
 
@@ -450,55 +460,48 @@ wxString CFileZillaApp::GetDataDir(wxString fileToFind) const
 
 	// For each path, check for the resources
 	wxPathList::const_iterator node;
-	for (node = pathList.begin(); node != pathList.end(); ++node)
-	{
-		wxString cur = *node;
+	for (node = pathList.begin(); node != pathList.end(); ++node) {
+		wxString cur = CLocalPath(*node).GetPath();
 		if (FileExists(cur + fileToFind))
-			return cur;
-		if (FileExists(cur + _T("/share/filezilla") + fileToFind))
-			return cur + _T("/share/filezilla");
-		if (FileExists(cur + _T("/filezilla") + fileToFind))
-			return cur + _T("/filezilla");
+			return CLocalPath(cur);
+		if (FileExists(cur + _T("share/filezilla/") + fileToFind))
+			return CLocalPath(cur + _T("/share/filezilla"));
+		if (FileExists(cur + _T("filezilla/") + fileToFind))
+			return CLocalPath(cur + _T("filezilla"));
 	}
 
-	for (node = pathList.begin(); node != pathList.end(); ++node)
-	{
-		wxString cur = *node;
-		if (FileExists(cur + _T("/..") + fileToFind))
-			return cur + _T("/..");
-		if (FileExists(cur + _T("/../share/filezilla") + fileToFind))
-			return cur + _T("/../share/filezilla");
+	for (node = pathList.begin(); node != pathList.end(); ++node) {
+		wxString cur = CLocalPath(*node).GetPath();
+		if (FileExists(cur + _T("../") + fileToFind))
+			return CLocalPath(cur + _T("/.."));
+		if (FileExists(cur + _T("../share/filezilla/") + fileToFind))
+			return CLocalPath(cur + _T("../share/filezilla"));
 	}
 
-	for (node = pathList.begin(); node != pathList.end(); ++node)
-	{
-		wxString cur = *node;
-		if (FileExists(cur + _T("/../../") + fileToFind))
-			return cur + _T("/../..");
+	for (node = pathList.begin(); node != pathList.end(); ++node) {
+		wxString cur = CLocalPath(*node).GetPath();
+		if (FileExists(cur + _T("../../") + fileToFind))
+			return CLocalPath(cur + _T("../.."));
 	}
 
-	return _T("");
+	return CLocalPath();
 #endif //__WXMAC__
 }
 
 bool CFileZillaApp::LoadResourceFiles()
 {
 	AddStartupProfileRecord(_T("CFileZillaApp::LoadResourceFiles"));
-	m_resourceDir = GetDataDir(_T("/resources/menus.xrc"));
+	m_resourceDir = GetDataDir(_T("resources/theme.xml"));
 
 	wxImage::AddHandler(new wxPNGHandler());
 
-	if (m_resourceDir == _T(""))
-	{
+	if (m_resourceDir.empty()) {
 		wxString msg = _("Could not find the resource files for FileZilla, closing FileZilla.\nYou can set the data directory of FileZilla using the '--datadir <custompath>' commandline option or by setting the FZ_DATADIR environment variable.");
-		wxMessageBox(msg, _("FileZilla Error"), wxOK | wxICON_ERROR);
+		wxMessageBoxEx(msg, _("FileZilla Error"), wxOK | wxICON_ERROR);
 		return false;
 	}
 
-	if (m_resourceDir[m_resourceDir.Length() - 1] != wxFileName::GetPathSeparator())
-		m_resourceDir += wxFileName::GetPathSeparator();
-
-	m_resourceDir += _T("resources/");
+	m_resourceDir.AddSegment(_T("resources"));
 
 	wxXmlResource *pResource = wxXmlResource::Get();
 
@@ -531,14 +534,15 @@ bool CFileZillaApp::LoadResourceFiles()
 	pResource->AddHandler(new wxStaticLineXmlHandler);
 	pResource->AddHandler(new wxScrolledWindowXmlHandler);
 	pResource->AddHandler(new wxHyperlinkCtrlXmlHandler);
+	pResource->AddHandler(new wxAnimationCtrlXmlHandler);
+	pResource->AddHandler(new wxStdDialogButtonSizerXmlHandler);
 
-	wxString resourceDir = m_resourceDir;
-#if wxUSE_FILESYSTEM
-	resourceDir.Replace(_T("%"), _T("%25"));
-	resourceDir.Replace(_T(":"), _T("%3A"));
-	resourceDir.Replace(_T("#"), _T("%23"));
-#endif
-	pResource->Load(resourceDir + _T("*.xrc"));
+	if (CLocalFileSystem::GetFileType(m_resourceDir.GetPath() + _T("xrc/resources.xrc")) == CLocalFileSystem::file) {
+		pResource->LoadFile(m_resourceDir.GetPath() + _T("xrc/resources.xrc"));
+	}
+	else {
+		pResource->Load(m_resourceDir.GetPath() + _T("xrc/*.xrc"));
+	}
 
 	return true;
 }
@@ -547,55 +551,46 @@ bool CFileZillaApp::InitDefaultsDir()
 {
 	AddStartupProfileRecord(_T("InitDefaultsDir"));
 #ifdef __WXGTK__
-	wxFileName fn = wxFileName(wxGetHomeDir(), _T("fzdefaults.xml"));
-	fn.AppendDir(_T(".filezilla"));
-	if (fn.FileExists())
-		m_defaultsDir = fn.GetPath();
-	else if (wxFileName::FileExists(_T("/etc/filezilla/fzdefaults.xml")))
-		m_defaultsDir = _T("/etc/filezilla");
-	else
-#endif
-	m_defaultsDir = GetDataDir(_T("/fzdefaults.xml"));
+	m_defaultsDir = COptions::GetUnadjustedSettingsDir();
+	if( m_defaultsDir.empty() || !wxFileName::FileExists(m_defaultsDir.GetPath() + _T("fzdefaults.xml"))) {
+		if (wxFileName::FileExists(_T("/etc/filezilla/fzdefaults.xml"))) {
+			m_defaultsDir.SetPath(_T("/etc/filezilla"));
+		}
+		else {
+			m_defaultsDir.clear();
+		}
+	}
 
-	return m_defaultsDir != _T("");
+#endif
+	if( m_defaultsDir.empty() ) {
+		m_defaultsDir = GetDataDir(_T("fzdefaults.xml"));
+	}
+
+	return !m_defaultsDir.empty();
 }
 
 bool CFileZillaApp::LoadLocales()
 {
 	AddStartupProfileRecord(_T("CFileZillaApp::LoadLocales"));
 #ifndef __WXMAC__
-	m_localesDir = GetDataDir(_T("/../locale/*/filezilla.mo"));
-	if (m_localesDir == _T(""))
-		m_localesDir = GetDataDir(_T("/../locale/*/LC_MESSAGES/filezilla.mo"));
-	if (m_localesDir != _T(""))
-	{
-		if (m_localesDir[m_localesDir.Length() - 1] != wxFileName::GetPathSeparator())
-			m_localesDir += wxFileName::GetPathSeparator();
-
-		m_localesDir += _T("../locale");
+	m_localesDir = GetDataDir(_T("../locale/*/filezilla.mo"));
+	if (m_localesDir.empty())
+		m_localesDir = GetDataDir(_T("../locale/*/LC_MESSAGES/filezilla.mo"));
+	if (!m_localesDir.empty()) {
+		m_localesDir.ChangePath( _T("../locale") );
 	}
-	else
-	{
-		m_localesDir = GetDataDir(_T("/locales/*/filezilla.mo"));
-		if (m_localesDir != _T(""))
-		{
-			if (m_localesDir[m_localesDir.Length() - 1] != wxFileName::GetPathSeparator())
-				m_localesDir += wxFileName::GetPathSeparator();
-
-			m_localesDir += _T("locales");
+	else {
+		m_localesDir = GetDataDir(_T("locales/*/filezilla.mo"));
+		if (!m_localesDir.empty()) {
+			m_localesDir.AddSegment(_T("locales"));
 		}
 	}
 #else
-	m_localesDir = wxStandardPaths::Get().GetDataDir() + _T("/locales");
+	m_localesDir.SetPath(wxStandardPaths::Get().GetDataDir() + _T("/locales"));
 #endif
 
-	if (m_localesDir != _T(""))
-	{
-		wxFileName fn(m_localesDir, _T(""));
-		fn.Normalize();
-		m_localesDir = fn.GetFullPath();
-
-		wxLocale::AddCatalogLookupPathPrefix(m_localesDir);
+	if (!m_localesDir.empty()) {
+		wxLocale::AddCatalogLookupPathPrefix(m_localesDir.GetPath());
 	}
 
 	SetLocale(wxLANGUAGE_DEFAULT);
@@ -645,38 +640,15 @@ int CFileZillaApp::GetCurrentLanguage() const
 wxString CFileZillaApp::GetCurrentLanguageCode() const
 {
 	if (!m_pLocale)
-		return _T("");
+		return wxString();
 
 	return m_pLocale->GetCanonicalName();
 }
 
 #if wxUSE_DEBUGREPORT && wxUSE_ON_FATAL_EXCEPTION
-
 void CFileZillaApp::OnFatalException()
 {
-	//GenerateReport(wxDebugReport::Context_Exception);
 }
-
-void CFileZillaApp::GenerateReport(wxDebugReport::Context ctx)
-{
-	/*
-	wxDebugReport report;
-
-	// add all standard files: currently this means just a minidump and an
-	// XML file with system info and stack trace
-	report.AddAll(ctx);
-
-	// calling Show() is not mandatory, but is more polite
-	if ( wxDebugReportPreviewStd().Show(report) )
-	{
-		if ( report.Process() )
-		{
-			// report successfully uploaded
-		}
-	}*/
-	//else: user cancelled the report
-}
-
 #endif
 
 void CFileZillaApp::DisplayEncodingWarning()
@@ -687,7 +659,7 @@ void CFileZillaApp::DisplayEncodingWarning()
 
 	displayedEncodingWarning = true;
 
-	wxMessageBox(_("A local filename could not be decoded.\nPlease make sure the LC_CTYPE (or LC_ALL) environment variable is set correctly.\nUnless you fix this problem, files might be missing in the file listings.\nNo further warning will be displayed this session."), _("Character encoding issue"), wxICON_EXCLAMATION);
+	wxMessageBoxEx(_("A local filename could not be decoded.\nPlease make sure the LC_CTYPE (or LC_ALL) environment variable is set correctly.\nUnless you fix this problem, files might be missing in the file listings.\nNo further warning will be displayed this session."), _("Character encoding issue"), wxICON_EXCLAMATION);
 }
 
 CWrapEngine* CFileZillaApp::GetWrapEngine()
@@ -708,7 +680,7 @@ void CFileZillaApp::CheckExistsFzsftp()
 	executable += _T("/fzsftp");
 	if (!wxFileName::FileExists(executable))
 	{
-		wxMessageBox(wxString::Format(_("%s could not be found. Without this component of FileZilla, SFTP will not work.\n\nPlease download FileZilla again. If this problem persists, please submit a bug report."), executable.c_str()),
+		wxMessageBoxEx(wxString::Format(_("%s could not be found. Without this component of FileZilla, SFTP will not work.\n\nPlease download FileZilla again. If this problem persists, please submit a bug report."), executable),
 			_("File not found"), wxICON_ERROR);
 		executable.clear();
 	}
@@ -761,7 +733,7 @@ void CFileZillaApp::CheckExistsFzsftp()
 		pathList.Add(cwd + _T("/putty"));
 
 		executable = pathList.FindAbsoluteValidPath(program);
-		if (executable != _T(""))
+		if (!executable.empty())
 			found = true;
 	}
 
@@ -792,7 +764,7 @@ void CFileZillaApp::CheckExistsFzsftp()
 		wxPathList pathList;
 		pathList.AddEnvList(_T("PATH"));
 		executable = pathList.FindAbsoluteValidPath(program);
-		if (executable != _T(""))
+		if (!executable.empty())
 			found = true;
 	}
 
@@ -802,8 +774,8 @@ void CFileZillaApp::CheckExistsFzsftp()
 
 	if (!found)
 	{
-		wxMessageBox(wxString::Format(_("%s could not be found. Without this component of FileZilla, SFTP will not work.\n\nPossible solutions:\n- Make sure %s is in a directory listed in your PATH environment variable.\n- Set the full path to %s in the FZ_FZSFTP environment variable."), program.c_str(), program.c_str(), program.c_str()),
-			_("File not found"), wxICON_ERROR);
+		wxMessageBoxEx(wxString::Format(_("%s could not be found. Without this component of FileZilla, SFTP will not work.\n\nPossible solutions:\n- Make sure %s is in a directory listed in your PATH environment variable.\n- Set the full path to %s in the FZ_FZSFTP environment variable."), program, program, program),
+			_("File not found"), wxICON_ERROR | wxOK);
 		executable.clear();
 	}
 #endif
@@ -816,7 +788,7 @@ void CFileZillaApp::CheckExistsFzsftp()
 }
 
 #ifdef __WXMSW__
-extern "C" BOOL CALLBACK EnumWindowCallback(HWND hwnd, LPARAM lParam)
+extern "C" BOOL CALLBACK EnumWindowCallback(HWND hwnd, LPARAM)
 {
 	HWND child = FindWindowEx(hwnd, 0, 0, _T("FileZilla process identificator 3919DB0A-082D-4560-8E2F-381A35969FB4"));
 	if (child)
@@ -846,7 +818,7 @@ int CFileZillaApp::ProcessCommandLine()
 
 		if (m_pCommandLine->HasSwitch(CCommandLine::version))
 		{
-			wxString out = wxString::Format(_T("FileZilla %s"), CBuildInfo::GetVersion().c_str());
+			wxString out = wxString::Format(_T("FileZilla %s"), CBuildInfo::GetVersion());
 			if (!CBuildInfo::GetBuildType().empty())
 				out += _T(" ") + CBuildInfo::GetBuildType() + _T(" build");
 			out += _T(", compiled on ") + CBuildInfo::GetBuildDateString();
@@ -929,5 +901,10 @@ void CFileZillaApp::ShowStartupProfile()
 		msg += _T("\n");
 	}
 
-	wxMessageBox(msg);
+	wxMessageBoxEx(msg);
+}
+
+wxString CFileZillaApp::GetSettingsFile(wxString const& name) const
+{
+	return COptions::Get()->GetOption(OPTION_DEFAULT_SETTINGSDIR) + name + _T(".xml");
 }
