@@ -6,6 +6,7 @@
 #include "queue.h"
 #include "filezillaapp.h"
 #include "filter.h"
+#include "file_utils.h"
 #include "inputdialog.h"
 #include <algorithm>
 #include "dndobjects.h"
@@ -61,6 +62,9 @@ public:
 			def == wxDragNone ||
 			def == wxDragCancel)
 			return def;
+		if( def == wxDragLink ) {
+			def = wxDragCopy;
+		}
 
 		if (m_pLocalListView->m_fileData.empty())
 			return wxDragError;
@@ -83,7 +87,7 @@ public:
 		}
 
 		CLocalPath dir = m_pLocalListView->m_pState->GetLocalDir();
-		if (subdir != _T(""))
+		if (!subdir.empty())
 		{
 			if (!dir.ChangePath(subdir))
 				return wxDragError;
@@ -101,13 +105,13 @@ public:
 		{
 			if (m_pRemoteDataObject->GetProcessId() != (int)wxGetProcessId())
 			{
-				wxMessageBox(_("Drag&drop between different instances of FileZilla has not been implemented yet."));
+				wxMessageBoxEx(_("Drag&drop between different instances of FileZilla has not been implemented yet."));
 				return wxDragNone;
 			}
 
 			if (!m_pLocalListView->m_pState->GetServer() || !m_pRemoteDataObject->GetServer().EqualsNoPass(*m_pLocalListView->m_pState->GetServer()))
 			{
-				wxMessageBox(_("Drag&drop between different servers has not been implemented yet."));
+				wxMessageBoxEx(_("Drag&drop between different servers has not been implemented yet."));
 				return wxDragNone;
 			}
 
@@ -200,7 +204,7 @@ public:
 		const wxString& subdir = DoDisplayDropHighlight(wxPoint(x, y));
 
 		CLocalPath dir = m_pLocalListView->m_pState->GetLocalDir();
-		if (subdir == _T(""))
+		if (subdir.empty())
 		{
 			const CDragDropManager* pDragDropManager = CDragDropManager::Get();
 			if (pDragDropManager && pDragDropManager->localParent == m_pLocalListView->m_dir)
@@ -247,6 +251,7 @@ BEGIN_EVENT_TABLE(CLocalListView, CFileListCtrl<CLocalFileData>)
 	EVT_MENU(XRCID("ID_UPLOAD"), CLocalListView::OnMenuUpload)
 	EVT_MENU(XRCID("ID_ADDTOQUEUE"), CLocalListView::OnMenuUpload)
 	EVT_MENU(XRCID("ID_MKDIR"), CLocalListView::OnMenuMkdir)
+	EVT_MENU(XRCID("ID_MKDIR_CHGDIR"), CLocalListView::OnMenuMkdirChgDir)
 	EVT_MENU(XRCID("ID_DELETE"), CLocalListView::OnMenuDelete)
 	EVT_MENU(XRCID("ID_RENAME"), CLocalListView::OnMenuRename)
 	EVT_KEY_DOWN(CLocalListView::OnKeyDown)
@@ -265,6 +270,7 @@ CLocalListView::CLocalListView(wxWindow* pParent, CState *pState, CQueueView *pQ
 	: CFileListCtrl<CLocalFileData>(pParent, pState, pQueue),
 	CStateEventHandler(pState)
 {
+	wxGetApp().AddStartupProfileRecord(_T("CLocalListView::CLocalListView"));
 	m_pState->RegisterHandler(this, STATECHANGE_LOCAL_DIR);
 	m_pState->RegisterHandler(this, STATECHANGE_APPLYFILTER);
 	m_pState->RegisterHandler(this, STATECHANGE_LOCAL_REFRESH_FILE);
@@ -310,6 +316,7 @@ bool CLocalListView::DisplayDir(wxString dirname)
 
 	wxString focused;
 	std::list<wxString> selectedNames;
+	bool ensureVisible = false;
 	if (m_dir != dirname)
 	{
 		ResetSearchPrefix();
@@ -318,7 +325,10 @@ bool CLocalListView::DisplayDir(wxString dirname)
 			ExitComparisonMode();
 
 		ClearSelection();
-		focused = _T("..");
+		focused = m_pState->GetPreviouslyVisitedLocalSubdir();
+		ensureVisible = !focused.empty();
+		if (focused.empty())
+			focused = _T("..");
 
 		if (GetItemCount())
 			EnsureVisible(0);
@@ -343,15 +353,12 @@ bool CLocalListView::DisplayDir(wxString dirname)
 	if (m_hasParent)
 	{
 		CLocalFileData data;
-		data.flags = normal;
 		data.dir = true;
-		data.icon = -2;
 		data.name = _T("..");
 #ifdef __WXMSW__
 		data.label = _T("..");
 #endif
 		data.size = -1;
-		data.hasTime = 0;
 		m_fileData.push_back(data);
 		m_indexMapping.push_back(0);
 	}
@@ -393,12 +400,10 @@ regular_dir:
 
 		int num = m_fileData.size();
 		CLocalFileData data;
-		data.flags = normal;
-		data.icon = -2;
 		bool wasLink;
-		while (local_filesystem.GetNextFile(data.name, wasLink, data.dir, &data.size, &data.lastModified, &data.attributes))
+		while (local_filesystem.GetNextFile(data.name, wasLink, data.dir, &data.size, &data.time, &data.attributes))
 		{
-			if (data.name.IsEmpty())
+			if (data.name.empty())
 			{
 				wxGetApp().DisplayEncodingWarning();
 				continue;
@@ -406,10 +411,9 @@ regular_dir:
 #ifdef __WXMSW__
 			data.label = data.name;
 #endif
-			data.hasTime = data.lastModified.IsValid();
 
 			m_fileData.push_back(data);
-			if (!filter.FilenameFiltered(data.name, dirname, data.dir, data.size, true, data.attributes, data.hasTime ? &data.lastModified : 0))
+			if (!filter.FilenameFiltered(data.name, dirname, data.dir, data.size, true, data.attributes, data.time))
 			{
 				if (data.dir)
 					totalDirCount++;
@@ -454,7 +458,7 @@ regular_dir:
 		RefreshComparison();
 	}
 
-	ReselectItems(selectedNames, focused);
+	ReselectItems(selectedNames, focused, ensureVisible);
 
 	RefreshListOnly();
 
@@ -483,7 +487,7 @@ int CLocalListView::OnGetItemImage(long item) const
 				path = m_dir + data->name;
 		}
 
-		icon = pThis->GetIconIndex(data->dir ? dir : file, path);
+		icon = pThis->GetIconIndex(data->dir ? iconType::dir : iconType::file, path);
 	}
 	return icon;
 }
@@ -541,8 +545,8 @@ void CLocalListView::OnItemActivated(wxListEvent &event)
 			wxString error;
 			if (!m_pState->SetLocalDir(data->name, &error))
 			{
-				if (error != _T(""))
-					wxMessageBox(error, _("Failed to change directory"), wxICON_INFORMATION);
+				if (!error.empty())
+					wxMessageBoxEx(error, _("Failed to change directory"), wxICON_INFORMATION);
 				else
 					wxBell();
 			}
@@ -554,7 +558,7 @@ void CLocalListView::OnItemActivated(wxListEvent &event)
 		return;
 	}
 
-	if (data->flags == fill)
+	if (data->comparison_flags == fill)
 	{
 		wxBell();
 		return;
@@ -584,7 +588,7 @@ void CLocalListView::OnItemActivated(wxListEvent &event)
 	}
 
 	CServerPath path = m_pState->GetRemotePath();
-	if (path.IsEmpty())
+	if (path.empty())
 	{
 		wxBell();
 		return;
@@ -621,8 +625,8 @@ void CLocalListView::OnMenuEnter(wxCommandEvent &event)
 	wxString error;
 	if (!m_pState->SetLocalDir(data->name, &error))
 	{
-		if (error != _T(""))
-			wxMessageBox(error, _("Failed to change directory"), wxICON_INFORMATION);
+		if (!error.empty())
+			wxMessageBoxEx(error, _("Failed to change directory"), wxICON_INFORMATION);
 		else
 			wxBell();
 	}
@@ -631,81 +635,31 @@ void CLocalListView::OnMenuEnter(wxCommandEvent &event)
 #ifdef __WXMSW__
 void CLocalListView::DisplayDrives()
 {
-	long drivesToHide = 0;
-	// Adhere to the NODRIVES group policy
-	wxRegKey key(_T("HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\Policies\\Explorer"));
-	if (key.Exists())
-	{
-		if (!key.HasValue(_T("NoDrives")) || !key.QueryValue(_T("NoDrives"), &drivesToHide))
-			drivesToHide = 0;
-	}
-
-	int len = GetLogicalDriveStrings(0, 0);
-	if (!len)
-		return;
-
-	wxChar* drives = new wxChar[len + 1];
-
-	if (!GetLogicalDriveStrings(len, drives))
-	{
-		delete [] drives;
-		return;
-	}
-
-	const wxChar* pDrive = drives;
-
 	int count = m_fileData.size();
-	int drive_count = 0;
-	while (*pDrive)
-	{
-		// Check if drive should be hidden by default
-		if (pDrive[0] != 0 && pDrive[1] == ':')
-		{
-			int bit = -1;
-			char letter = pDrive[0];
-			if (letter >= 'A' && letter <= 'Z')
-				bit = 1 << (letter - 'A');
-			if (letter >= 'a' && letter <= 'z')
-				bit = 1 << (letter - 'a');
 
-			if (bit != -1 && drivesToHide & bit)
-			{
-				pDrive += wxStrlen(pDrive) + 1;
-				continue;
-			}
-		}
-
-		drive_count++;
-
-		wxString path = pDrive;
-		if (path.Right(1) == _T("\\"))
-			path.Truncate(path.Length() - 1);
+	std::list<wxString> drives = CVolumeDescriptionEnumeratorThread::GetDrives();
+	for( std::list<wxString>::const_iterator it = drives.begin(); it != drives.end(); ++it ) {
+		wxString drive = *it;
+		if (drive.Right(1) == _T("\\"))
+			drive.RemoveLast();
 
 		CLocalFileData data;
-		data.flags = normal;
-		data.name = path;
+		data.name = drive;
 		data.label = data.name;
 		data.dir = true;
-		data.icon = -2;
 		data.size = -1;
-		data.hasTime = false;
 
 		m_fileData.push_back(data);
 		m_indexMapping.push_back(count);
-		pDrive += wxStrlen(pDrive) + 1;
 		count++;
 	}
 
-	delete [] drives;
-
 	if (m_pFilelistStatusBar)
-		m_pFilelistStatusBar->SetDirectoryContents(0, drive_count, 0, false, 0);
+		m_pFilelistStatusBar->SetDirectoryContents(0, drives.size(), 0, false, 0);
 
-	if (!m_pVolumeEnumeratorThread)
-	{
+	if (!m_pVolumeEnumeratorThread) {
 		m_pVolumeEnumeratorThread = new CVolumeDescriptionEnumeratorThread(this);
-		if (m_pVolumeEnumeratorThread->Failed())
-		{
+		if (m_pVolumeEnumeratorThread->Failed()) {
 			delete m_pVolumeEnumeratorThread;
 			m_pVolumeEnumeratorThread = 0;
 		}
@@ -745,15 +699,12 @@ void CLocalListView::DisplayShares(wxString computer)
 				continue;
 
 			CLocalFileData data;
-			data.flags = normal;
 			data.name = p->shi1_netname;
 #ifdef __WXMSW__
 			data.label = data.name;
 #endif
 			data.dir = true;
-			data.icon = -2;
 			data.size = -1;
-			data.hasTime = false;
 
 			m_fileData.push_back(data);
 			m_indexMapping.push_back(j++);
@@ -791,267 +742,32 @@ bool CLocalListView::IsItemValid(unsigned int item) const
 	return true;
 }
 
-// Helper classes for fast sorting using std::sort
-// -----------------------------------------------
-
-class CLocalListViewSort : public CListViewSort
-{
-public:
-	CLocalListViewSort(std::vector<CLocalFileData>& fileData, enum DirSortMode dirSortMode)
-		: m_fileData(fileData), m_dirSortMode(dirSortMode)
-	{
-	}
-
-	virtual ~CLocalListViewSort() {}
-
-	virtual bool operator()(int a, int b) const = 0;
-
-	#define CMP(f, data1, data2) \
-		{\
-			int res = f(data1, data2);\
-			if (res == -1)\
-				return true;\
-			else if (res == 1)\
-				return false;\
-		}
-
-	#define CMP_LESS(f, data1, data2) \
-		{\
-			int res = f(data1, data2);\
-			if (res == -1)\
-				return true;\
-			else\
-				return false;\
-		}
-
-	inline int CmpDir(const CLocalFileData &data1, const CLocalFileData &data2) const
-	{
-		switch (m_dirSortMode)
-		{
-		default:
-		case dirsort_ontop:
-			if (data1.dir)
-			{
-				if (!data2.dir)
-					return -1;
-				else
-					return 0;
-			}
-			else
-			{
-				if (data2.dir)
-					return 1;
-				else
-					return 0;
-			}
-		case dirsort_onbottom:
-			if (data1.dir)
-			{
-				if (!data2.dir)
-					return 1;
-				else
-					return 0;
-			}
-			else
-			{
-				if (data2.dir)
-					return -1;
-				else
-					return 0;
-			}
-		case dirsort_inline:
-			return 0;
-		}
-	}
-
-	inline int CmpName(const CLocalFileData &data1, const CLocalFileData &data2) const
-	{
-#ifdef __WXMSW__
-		return data1.name.CmpNoCase(data2.name);
-#else
-		return data1.name.Cmp(data2.name);
-#endif
-	}
-
-	inline int CmpSize(const CLocalFileData &data1, const CLocalFileData &data2) const
-	{
-		const wxLongLong diff = data1.size - data2.size;
-		if (diff < 0)
-			return -1;
-		else if (diff > 0)
-			return 1;
-		else
-			return 0;
-	}
-
-	inline int CmpType(const CLocalFileData &data1, const CLocalFileData &data2) const
-	{
-		return data1.fileType.CmpNoCase(data2.fileType);
-	}
-
-	inline int CmpTime(const CLocalFileData &data1, const CLocalFileData &data2) const
-	{
-		if (!data1.hasTime)
-		{
-			if (data2.hasTime)
-				return -1;
-			else
-				return 0;
-		}
-		else
-		{
-			if (!data2.hasTime)
-				return 1;
-
-			if (data1.lastModified < data2.lastModified)
-				return -1;
-			else if (data1.lastModified > data2.lastModified)
-				return 1;
-			else
-				return 0;
-		}
-	}
-
-protected:
-	std::vector<CLocalFileData>& m_fileData;
-
-	DirSortMode m_dirSortMode;
-};
-
-template<class T> class CReverseSort : public T
-{
-public:
-	CReverseSort(std::vector<CLocalFileData>& fileData, enum CLocalListViewSort::DirSortMode dirSortMode, CLocalListView* pListView)
-		: T(fileData, dirSortMode, pListView)
-	{
-	}
-
-	inline bool operator()(int a, int b) const
-	{
-		return T::operator()(b, a);
-	}
-};
-
-class CLocalListViewSortName : public CLocalListViewSort
-{
-public:
-	CLocalListViewSortName(std::vector<CLocalFileData>& fileData, enum DirSortMode dirSortMode, CLocalListView* pListView)
-		: CLocalListViewSort(fileData, dirSortMode)
-	{
-	}
-
-	bool operator()(int a, int b) const
-	{
-		const CLocalFileData &data1 = m_fileData[a];
-		const CLocalFileData &data2 = m_fileData[b];
-
-		CMP(CmpDir, data1, data2);
-
-		CMP_LESS(CmpName, data1, data2);
-	}
-};
-typedef CReverseSort<CLocalListViewSortName> CLocalListViewSortName_Reverse;
-
-class CLocalListViewSortSize : public CLocalListViewSort
-{
-public:
-	CLocalListViewSortSize(std::vector<CLocalFileData>& fileData, enum DirSortMode dirSortMode, CLocalListView* pListView)
-		: CLocalListViewSort(fileData, dirSortMode)
-	{
-	}
-
-	bool operator()(int a, int b) const
-	{
-		const CLocalFileData &data1 = m_fileData[a];
-		const CLocalFileData &data2 = m_fileData[b];
-
-		CMP(CmpDir, data1, data2);
-
-		CMP(CmpSize, data1, data2)
-
-		CMP_LESS(CmpName, data1, data2);
-	}
-};
-typedef CReverseSort<CLocalListViewSortSize> CLocalListViewSortSize_Reverse;
-
-class CLocalListViewSortType : public CLocalListViewSort
-{
-public:
-	CLocalListViewSortType(std::vector<CLocalFileData>& fileData, enum DirSortMode dirSortMode, CLocalListView* pListView)
-		: CLocalListViewSort(fileData, dirSortMode)
-	{
-		m_pListView = pListView;
-	}
-
-	bool operator()(int a, int b) const
-	{
-		CLocalFileData &data1 = m_fileData[a];
-		CLocalFileData &data2 = m_fileData[b];
-
-		if (data1.fileType.IsEmpty())
-			data1.fileType = m_pListView->GetType(data1.name, data1.dir, m_pListView->m_dir);
-		if (data2.fileType.IsEmpty())
-			data2.fileType = m_pListView->GetType(data2.name, data2.dir, m_pListView->m_dir);
-
-		CMP(CmpDir, data1, data2);
-
-		CMP(CmpType, data1, data2)
-
-		CMP_LESS(CmpName, data1, data2);
-	}
-
-protected:
-	CLocalListView* m_pListView;
-};
-typedef CReverseSort<CLocalListViewSortType> CLocalListViewSortType_Reverse;
-
-class CLocalListViewSortTime : public CLocalListViewSort
-{
-public:
-	CLocalListViewSortTime(std::vector<CLocalFileData>& fileData, enum DirSortMode dirSortMode, CLocalListView* pListView)
-		: CLocalListViewSort(fileData, dirSortMode)
-	{
-	}
-
-	bool operator()(int a, int b) const
-	{
-		const CLocalFileData &data1 = m_fileData[a];
-		const CLocalFileData &data2 = m_fileData[b];
-
-		CMP(CmpDir, data1, data2);
-
-		CMP(CmpTime, data1, data2)
-
-		CMP_LESS(CmpName, data1, data2);
-	}
-};
-typedef CReverseSort<CLocalListViewSortTime> CLocalListViewSortTime_Reverse;
-
 CFileListCtrl<CLocalFileData>::CSortComparisonObject CLocalListView::GetSortComparisonObject()
 {
-	CLocalListViewSort::DirSortMode dirSortMode = GetDirSortMode();
+	CFileListCtrlSortBase::DirSortMode dirSortMode = GetDirSortMode();
+	CFileListCtrlSortBase::NameSortMode nameSortMode = GetNameSortMode();
 
 	if (!m_sortDirection)
 	{
 		if (m_sortColumn == 1)
-			return CFileListCtrl<CLocalFileData>::CSortComparisonObject(new CLocalListViewSortSize(m_fileData, dirSortMode, this));
+			return CFileListCtrl<CLocalFileData>::CSortComparisonObject(new CFileListCtrlSortSize<std::vector<CLocalFileData>, CLocalFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 		else if (m_sortColumn == 2)
-			return CFileListCtrl<CLocalFileData>::CSortComparisonObject(new CLocalListViewSortType(m_fileData, dirSortMode, this));
+			return CFileListCtrl<CLocalFileData>::CSortComparisonObject(new CFileListCtrlSortType<std::vector<CLocalFileData>, CLocalFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 		else if (m_sortColumn == 3)
-			return CFileListCtrl<CLocalFileData>::CSortComparisonObject(new CLocalListViewSortTime(m_fileData, dirSortMode, this));
+			return CFileListCtrl<CLocalFileData>::CSortComparisonObject(new CFileListCtrlSortTime<std::vector<CLocalFileData>, CLocalFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 		else
-			return CFileListCtrl<CLocalFileData>::CSortComparisonObject(new CLocalListViewSortName(m_fileData, dirSortMode, this));
+			return CFileListCtrl<CLocalFileData>::CSortComparisonObject(new CFileListCtrlSortName<std::vector<CLocalFileData>, CLocalFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 	}
 	else
 	{
 		if (m_sortColumn == 1)
-			return CFileListCtrl<CLocalFileData>::CSortComparisonObject(new CLocalListViewSortSize_Reverse(m_fileData, dirSortMode, this));
+			return CFileListCtrl<CLocalFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortSize<std::vector<CLocalFileData>, CLocalFileData>, CLocalFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 		else if (m_sortColumn == 2)
-			return CFileListCtrl<CLocalFileData>::CSortComparisonObject(new CLocalListViewSortType_Reverse(m_fileData, dirSortMode, this));
+			return CFileListCtrl<CLocalFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortType<std::vector<CLocalFileData>, CLocalFileData>, CLocalFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 		else if (m_sortColumn == 3)
-			return CFileListCtrl<CLocalFileData>::CSortComparisonObject(new CLocalListViewSortTime_Reverse(m_fileData, dirSortMode, this));
+			return CFileListCtrl<CLocalFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortTime<std::vector<CLocalFileData>, CLocalFileData>, CLocalFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 		else
-			return CFileListCtrl<CLocalFileData>::CSortComparisonObject(new CLocalListViewSortName_Reverse(m_fileData, dirSortMode, this));
+			return CFileListCtrl<CLocalFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortName<std::vector<CLocalFileData>, CLocalFileData>, CLocalFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 	}
 }
 
@@ -1085,16 +801,13 @@ void CLocalListView::OnContextMenu(wxContextMenuEvent& event)
 		const CLocalFileData* const data = GetData(index);
 		if (!data || (!index && m_hasParent))
 		{
-			pMenu->Enable(XRCID("ID_UPLOAD"), false);
-			pMenu->Enable(XRCID("ID_ADDTOQUEUE"), false);
-			pMenu->Enable(XRCID("ID_DELETE"), false);
-			pMenu->Enable(XRCID("ID_RENAME"), false);
 			pMenu->Enable(XRCID("ID_OPEN"), false);
+			pMenu->Enable(XRCID("ID_RENAME"), false);
 			pMenu->Enable(XRCID("ID_EDIT"), false);
 		}
-		if (data->flags == fill)
+		if ((data && data->comparison_flags == fill) || (!index && m_hasParent))
 			fillCount++;
-		if (data->dir)
+		if (data && data->dir)
 			selectedDir = true;
 		index = GetNextItem(index, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 	}
@@ -1127,17 +840,6 @@ void CLocalListView::OnContextMenu(wxContextMenuEvent& event)
 
 void CLocalListView::OnMenuUpload(wxCommandEvent& event)
 {
-	long item = -1;
-	for (;;)
-	{
-		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
-		if (item == -1)
-			break;
-
-		if (!item && m_hasParent)
-			return;
-	}
-
 	const CServer* pServer = m_pState->GetServer();
 	if (!pServer)
 	{
@@ -1149,10 +851,12 @@ void CLocalListView::OnMenuUpload(wxCommandEvent& event)
 
 	bool queue_only = event.GetId() == XRCID("ID_ADDTOQUEUE");
 
-	item = -1;
+	long item = -1;
 	for (;;)
 	{
 		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
+		if (!item && m_hasParent)
+			continue;
 		if (item == -1)
 			break;
 
@@ -1160,11 +864,11 @@ void CLocalListView::OnMenuUpload(wxCommandEvent& event)
 		if (!data)
 			break;
 
-		if (data->flags == fill)
+		if (data->comparison_flags == fill)
 			continue;
 
 		CServerPath path = m_pState->GetRemotePath();
-		if (path.IsEmpty())
+		if (path.empty())
 		{
 			wxBell();
 			break;
@@ -1188,19 +892,49 @@ void CLocalListView::OnMenuUpload(wxCommandEvent& event)
 		m_pQueue->QueueFile_Finish(!queue_only);
 }
 
+// Create a new Directory
 void CLocalListView::OnMenuMkdir(wxCommandEvent& event)
+{
+	wxString newdir = MenuMkdir();
+	if (!newdir.empty()) {
+		m_pState->RefreshLocal();
+	}
+}
+
+// Create a new Directory and enter the new Directory
+void CLocalListView::OnMenuMkdirChgDir(wxCommandEvent&)
+{
+	wxString newdir = MenuMkdir();
+	if (newdir.empty()) {
+		return;
+	}
+
+	// OnMenuEnter
+	wxString error;
+	if (!m_pState->SetLocalDir(newdir, &error))
+	{
+		if (!error.empty())
+			wxMessageBoxEx(error, _("Failed to change directory"), wxICON_INFORMATION);
+		else
+			wxBell();
+	}
+}
+
+// Helper-Function to create a new Directory
+// Returns the name of the new directory
+wxString CLocalListView::MenuMkdir()
 {
 	CInputDialog dlg;
 	if (!dlg.Create(this, _("Create directory"), _("Please enter the name of the directory which should be created:")))
-		return;
+		return wxString();
 
 	if (dlg.ShowModal() != wxID_OK)
-		return;
+		return wxString();
 
-	if (dlg.GetValue() == _T(""))
+	if (dlg.GetValue().empty())
 	{
 		wxBell();
-		return;
+		return wxString();
 	}
 
 	wxFileName fn(dlg.GetValue(), _T(""));
@@ -1213,10 +947,15 @@ void CLocalListView::OnMenuMkdir(wxCommandEvent& event)
 		res = fn.Mkdir(fn.GetPath(), 0777, wxPATH_MKDIR_FULL);
 	}
 
-	if (!res)
+	if (!res) {
 		wxBell();
+		return wxString();
+	}
 
-	DisplayDir(m_dir);
+	// Return name of the New Directory
+	// return dlg.GetValue();
+	// return fn.GetDirs().Last();
+	return fn.GetPath();
 }
 
 void CLocalListView::OnMenuDelete(wxCommandEvent& event)
@@ -1236,7 +975,7 @@ void CLocalListView::OnMenuDelete(wxCommandEvent& event)
 		if (!data)
 			continue;
 
-		if (data->flags == fill)
+		if (data->comparison_flags == fill)
 			continue;
 
 		pathsToDelete.push_back(dir + data->name);
@@ -1263,7 +1002,7 @@ void CLocalListView::OnMenuRename(wxCommandEvent& event)
 	}
 
 	CLocalFileData *data = GetData(item);
-	if (!data || data->flags == fill)
+	if (!data || data->comparison_flags == fill)
 	{
 		wxBell();
 		return;
@@ -1321,77 +1060,26 @@ bool CLocalListView::OnBeginRename(const wxListEvent& event)
 		return false;
 
 	const CLocalFileData * const data = GetData(event.GetIndex());
-	if (!data || data->flags == fill)
+	if (!data || data->comparison_flags == fill)
 		return false;
 
 	return true;
 }
 
-bool Rename(wxWindow* parent, wxString dir, wxString from, wxString to)
-{
-	if (dir.Right(1) != _T("\\") && dir.Right(1) != _T("/"))
-		dir += wxFileName::GetPathSeparator();
-
-#ifdef __WXMSW__
-	to = to.Left(255);
-
-	if ((to.Find('/') != -1) ||
-		(to.Find('\\') != -1) ||
-		(to.Find(':') != -1) ||
-		(to.Find('*') != -1) ||
-		(to.Find('?') != -1) ||
-		(to.Find('"') != -1) ||
-		(to.Find('<') != -1) ||
-		(to.Find('>') != -1) ||
-		(to.Find('|') != -1))
-	{
-		wxMessageBox(_("Filenames may not contain any of the following characters: / \\ : * ? \" < > |"), _("Invalid filename"), wxICON_EXCLAMATION);
-		return false;
-	}
-
-	SHFILEOPSTRUCT op;
-	memset(&op, 0, sizeof(op));
-
-	from = dir + from + _T(" ");
-	from.SetChar(from.Length() - 1, '\0');
-	op.pFrom = from;
-	to = dir + to + _T(" ");
-	to.SetChar(to.Length()-1, '\0');
-	op.pTo = to;
-	op.hwnd = (HWND)parent->GetHandle();
-	op.wFunc = FO_RENAME;
-	op.fFlags = FOF_ALLOWUNDO;
-	return SHFileOperation(&op) == 0;
-#else
-	if ((to.Find('/') != -1) ||
-		(to.Find('*') != -1) ||
-		(to.Find('?') != -1) ||
-		(to.Find('<') != -1) ||
-		(to.Find('>') != -1) ||
-		(to.Find('|') != -1))
-	{
-		wxMessageBox(_("Filenames may not contain any of the following characters: / * ? < > |"), _("Invalid filename"), wxICON_EXCLAMATION);
-		return false;
-	}
-
-	return wxRename(dir + from, dir + to) == 0;
-#endif
-}
-
 bool CLocalListView::OnAcceptRename(const wxListEvent& event)
 {
 	const int index = event.GetIndex();
 	if (!index && m_hasParent)
 		return false;
 
-	if (event.GetLabel() == _T(""))
+	if (event.GetLabel().empty())
 		return false;
 
 	if (!m_pState->GetLocalDir().IsWriteable())
 		return false;
 
 	CLocalFileData *const data = GetData(event.GetIndex());
-	if (!data || data->flags == fill)
+	if (!data || data->comparison_flags == fill)
 		return false;
 
 	wxString newname = event.GetLabel();
@@ -1402,7 +1090,7 @@ bool CLocalListView::OnAcceptRename(const wxListEvent& event)
 	if (newname == data->name)
 		return false;
 
-	if (!Rename(this, m_dir, data->name, newname))
+	if (!RenameFile(this, m_dir, data->name, newname))
 		return false;
 
 	data->name = newname;
@@ -1443,10 +1131,9 @@ void CLocalListView::ApplyCurrentFilter()
 	for (unsigned int i = min; i < m_fileData.size(); i++)
 	{
 		const CLocalFileData& data = m_fileData[i];
-		if (data.flags == fill)
+		if (data.comparison_flags == fill)
 			continue;
-		if (filter.FilenameFiltered(data.name, m_dir, data.dir, data.size, true, data.attributes, data.hasTime ? &data.lastModified : 0))
-		{
+		if (filter.FilenameFiltered(data.name, m_dir, data.dir, data.size, true, data.attributes, data.time)) {
 			hidden++;
 			continue;
 		}
@@ -1499,7 +1186,7 @@ std::list<wxString> CLocalListView::RememberSelectedItems(wxString& focused)
 			if (item == -1)
 				break;
 			const CLocalFileData &data = m_fileData[m_indexMapping[item]];
-			if (data.flags != fill)
+			if (data.comparison_flags != fill)
 			{
 				if (data.dir)
 					selectedNames.push_back(_T("d") + data.name);
@@ -1514,7 +1201,7 @@ std::list<wxString> CLocalListView::RememberSelectedItems(wxString& focused)
 	if (item != -1)
 	{
 		const CLocalFileData &data = m_fileData[m_indexMapping[item]];
-		if (data.flags != fill)
+		if (data.comparison_flags != fill)
 			focused = data.name;
 
 		SetItemState(item, 0, wxLIST_STATE_FOCUSED);
@@ -1523,14 +1210,14 @@ std::list<wxString> CLocalListView::RememberSelectedItems(wxString& focused)
 	return selectedNames;
 }
 
-void CLocalListView::ReselectItems(const std::list<wxString>& selectedNames, wxString focused)
+void CLocalListView::ReselectItems(const std::list<wxString>& selectedNames, wxString focused, bool ensureVisible)
 {
 	// Reselect previous items if neccessary.
 	// Sorting direction did not change. We just have to scan through items once
 
 	if (selectedNames.empty())
 	{
-		if (focused == _T(""))
+		if (focused.empty())
 			return;
 		for (unsigned int i = 0; i < m_indexMapping.size(); i++)
 		{
@@ -1538,6 +1225,8 @@ void CLocalListView::ReselectItems(const std::list<wxString>& selectedNames, wxS
 			if (data.name == focused)
 			{
 				SetItemState(i, wxLIST_STATE_FOCUSED, wxLIST_STATE_FOCUSED);
+				if (ensureVisible)
+					EnsureVisible(i);
 				return;
 			}
 		}
@@ -1555,6 +1244,8 @@ void CLocalListView::ReselectItems(const std::list<wxString>& selectedNames, wxS
 			if (data.name == focused)
 			{
 				SetItemState(i, wxLIST_STATE_FOCUSED, wxLIST_STATE_FOCUSED);
+				if (ensureVisible)
+					EnsureVisible(i);
 				focused = _T("");
 			}
 			if (data.dir && *iter == (_T("d") + data.name))
@@ -1579,7 +1270,7 @@ void CLocalListView::ReselectItems(const std::list<wxString>& selectedNames, wxS
 		if (i == (int)m_indexMapping.size())
 			break;
 	}
-	if (focused != _T(""))
+	if (!focused.empty())
 	{
 		if (firstSelected != -1)
 			SetItemState(firstSelected, wxLIST_STATE_FOCUSED, wxLIST_STATE_FOCUSED);
@@ -1588,7 +1279,7 @@ void CLocalListView::ReselectItems(const std::list<wxString>& selectedNames, wxS
 	}
 }
 
-void CLocalListView::OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void* data2)
+void CLocalListView::OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void*)
 {
 	if (notification == STATECHANGE_LOCAL_DIR)
 		DisplayDir(m_pState->GetLocalDir().GetPath());
@@ -1631,7 +1322,7 @@ void CLocalListView::OnBeginDrag(wxListEvent& event)
 		if (!data)
 			continue;
 
-		if (data->flags == fill)
+		if (data->comparison_flags == fill)
 			continue;
 
 		const wxString name = m_dir + data->name;
@@ -1639,7 +1330,7 @@ void CLocalListView::OnBeginDrag(wxListEvent& event)
 		obj.AddFile(name);
 	}
 
-	if (obj.GetFilenames().IsEmpty())
+	if (obj.GetFilenames().empty())
 	{
 		pDragDropManager->Release();
 		return;
@@ -1668,28 +1359,25 @@ void CLocalListView::RefreshFile(const wxString& file)
 	CLocalFileData data;
 
 	bool wasLink;
-	enum CLocalFileSystem::local_fileType type = CLocalFileSystem::GetFileInfo(m_dir + file, wasLink, &data.size, &data.lastModified, &data.attributes);
+	enum CLocalFileSystem::local_fileType type = CLocalFileSystem::GetFileInfo(m_dir + file, wasLink, &data.size, &data.time, &data.attributes);
 	if (type == CLocalFileSystem::unknown)
 		return;
 
-	data.flags = normal;
-	data.icon = -2;
 	data.name = file;
 #ifdef __WXMSW__
 	data.label = file;
 #endif
 	data.dir = type == CLocalFileSystem::dir;
-	data.hasTime = data.lastModified.IsValid();
 
 	CFilterManager filter;
-	if (filter.FilenameFiltered(data.name, m_dir, data.dir, data.size, true, data.attributes, data.hasTime ? &data.lastModified : 0))
+	if (filter.FilenameFiltered(data.name, m_dir, data.dir, data.size, true, data.attributes, data.time))
 		return;
 
 	CancelLabelEdit();
 
 	// Look if file data already exists
 	unsigned int i = 0;
-	for (std::vector<CLocalFileData>::iterator iter = m_fileData.begin(); iter != m_fileData.end(); ++iter, ++i)
+	for (auto iter = m_fileData.begin(); iter != m_fileData.end(); ++iter, ++i)
 	{
 		const CLocalFileData& oldData = *iter;
 		if (oldData.name != file)
@@ -1822,7 +1510,7 @@ wxListItemAttr* CLocalListView::OnGetItemAttr(long item) const
 		return &pThis->m_dropHighlightAttribute;
 #endif
 
-	switch (data->flags)
+	switch (data->comparison_flags)
 	{
 	case different:
 		return &pThis->m_comparisonBackgrounds[0];
@@ -1833,8 +1521,6 @@ wxListItemAttr* CLocalListView::OnGetItemAttr(long item) const
 	default:
 		return 0;
 	}
-
-	return 0;
 }
 
 void CLocalListView::StartComparison()
@@ -1855,19 +1541,18 @@ void CLocalListView::StartComparison()
 	m_comparisonIndex = -1;
 
 	const CLocalFileData& last = m_fileData[m_fileData.size() - 1];
-	if (last.flags != fill)
+	if (last.comparison_flags != fill)
 	{
 		CLocalFileData data;
 		data.dir = false;
 		data.icon = -1;
-		data.hasTime = false;
 		data.size = -1;
-		data.flags = fill;
+		data.comparison_flags = fill;
 		m_fileData.push_back(data);
 	}
 }
 
-bool CLocalListView::GetNextFile(wxString& name, bool& dir, wxLongLong& size, wxDateTime& date, bool &hasTime)
+bool CLocalListView::GetNextFile(wxString& name, bool& dir, wxLongLong& size, CDateTime& date)
 {
 	if (++m_comparisonIndex >= (int)m_originalIndexMapping.size())
 		return false;
@@ -1881,8 +1566,7 @@ bool CLocalListView::GetNextFile(wxString& name, bool& dir, wxLongLong& size, wx
 	name = data.name;
 	dir = data.dir;
 	size = data.size;
-	date = data.lastModified;
-	hasTime = true;
+	date = data.time;
 
 	return true;
 }
@@ -1911,7 +1595,7 @@ wxString CLocalListView::GetItemText(int item, unsigned int column)
 {
 	CLocalFileData *data = GetData(item);
 	if (!data)
-		return _T("");
+		return wxString();
 
 	if (!column)
 	{
@@ -1924,31 +1608,27 @@ wxString CLocalListView::GetItemText(int item, unsigned int column)
 	else if (column == 1)
 	{
 		if (data->size < 0)
-			return _T("");
+			return wxString();
 		else
 			return CSizeFormat::Format(data->size);
 	}
 	else if (column == 2)
 	{
 		if (!item && m_hasParent)
-			return _T("");
+			return wxString();
 
-		if (data->flags == fill)
-			return _T("");
+		if (data->comparison_flags == fill)
+			return wxString();
 
-		if (data->fileType.IsEmpty())
+		if (data->fileType.empty())
 			data->fileType = GetType(data->name, data->dir, m_dir);
 
 		return data->fileType;
 	}
-	else if (column == 3)
-	{
-		if (!data->hasTime)
-			return _T("");
-
-		return CTimeFormat::FormatDateTime(data->lastModified);
+	else if (column == 3) {
+		return CTimeFormat::Format(data->time);
 	}
-	return _T("");
+	return wxString();
 }
 
 void CLocalListView::OnMenuEdit(wxCommandEvent& event)
@@ -1960,7 +1640,7 @@ void CLocalListView::OnMenuEdit(wxCommandEvent& event)
 	{
 		if (COptions::Get()->GetOptionVal(OPTION_EDIT_TRACK_LOCAL))
 		{
-			wxMessageBox(_("Cannot edit file, not connected to any server."), _("Editing failed"), wxICON_EXCLAMATION);
+			wxMessageBoxEx(_("Cannot edit file, not connected to any server."), _("Editing failed"), wxICON_EXCLAMATION);
 			return;
 		}
 	}
@@ -1969,9 +1649,9 @@ void CLocalListView::OnMenuEdit(wxCommandEvent& event)
 		server = *m_pState->GetServer();
 
 		path = m_pState->GetRemotePath();
-		if (path.IsEmpty())
+		if (path.empty())
 		{
-			wxMessageBox(_("Cannot edit file, remote path unknown."), _("Editing failed"), wxICON_EXCLAMATION);
+			wxMessageBoxEx(_("Cannot edit file, remote path unknown."), _("Editing failed"), wxICON_EXCLAMATION);
 			return;
 		}
 	}
@@ -1997,7 +1677,7 @@ void CLocalListView::OnMenuEdit(wxCommandEvent& event)
 			return;
 		}
 
-		if (data->flags == fill)
+		if (data->comparison_flags == fill)
 			continue;
 
 		selected_item_list.push_back(*data);
@@ -2032,24 +1712,24 @@ void CLocalListView::OnMenuEdit(wxCommandEvent& event)
 		if (cmd.empty())
 		{
 			CNewAssociationDialog dlg(this);
-			if (!dlg.Show(fn.GetFullName()))
+			if (!dlg.Run(fn.GetFullName()))
 				continue;
 			cmd = pEditHandler->CanOpen(CEditHandler::local, fn.GetFullPath(), dangerous, program_exists);
 			if (cmd.empty())
 			{
-				wxMessageBox(wxString::Format(_("The file '%s' could not be opened:\nNo program has been associated on your system with this file type."), fn.GetFullPath().c_str()), _("Opening failed"), wxICON_EXCLAMATION);
+				wxMessageBoxEx(wxString::Format(_("The file '%s' could not be opened:\nNo program has been associated on your system with this file type."), fn.GetFullPath()), _("Opening failed"), wxICON_EXCLAMATION);
 				continue;
 			}
 		}
 		if (!program_exists)
 		{
-			wxString msg = wxString::Format(_("The file '%s' cannot be opened:\nThe associated program (%s) could not be found.\nPlease check your filetype associations."), fn.GetFullPath().c_str(), cmd.c_str());
-			wxMessageBox(msg, _("Cannot edit file"), wxICON_EXCLAMATION);
+			wxString msg = wxString::Format(_("The file '%s' cannot be opened:\nThe associated program (%s) could not be found.\nPlease check your filetype associations."), fn.GetFullPath(), cmd);
+			wxMessageBoxEx(msg, _("Cannot edit file"), wxICON_EXCLAMATION);
 			continue;
 		}
 		if (dangerous)
 		{
-			int res = wxMessageBox(_("The selected file would be executed directly.\nThis can be dangerous and damage your system.\nDo you really want to continue?"), _("Dangerous filetype"), wxICON_EXCLAMATION | wxYES_NO);
+			int res = wxMessageBoxEx(_("The selected file would be executed directly.\nThis can be dangerous and damage your system.\nDo you really want to continue?"), _("Dangerous filetype"), wxICON_EXCLAMATION | wxYES_NO);
 			if (res != wxYES)
 			{
 				wxBell();
@@ -2062,11 +1742,11 @@ void CLocalListView::OnMenuEdit(wxCommandEvent& event)
 		{
 		case CEditHandler::upload:
 		case CEditHandler::upload_and_remove:
-			wxMessageBox(_("A file with that name is already being transferred."), _("Cannot view/edit selected file"), wxICON_EXCLAMATION);
+			wxMessageBoxEx(_("A file with that name is already being transferred."), _("Cannot view/edit selected file"), wxICON_EXCLAMATION);
 			continue;
 		case CEditHandler::edit:
 			{
-				int res = wxMessageBox(wxString::Format(_("A file with that name is already being edited. Do you want to reopen '%s'?"), fn.GetFullPath().c_str()), _("Selected file already being edited"), wxICON_QUESTION | wxYES_NO);
+				int res = wxMessageBoxEx(wxString::Format(_("A file with that name is already being edited. Do you want to reopen '%s'?"), fn.GetFullPath()), _("Selected file already being edited"), wxICON_QUESTION | wxYES_NO);
 				if (res != wxYES)
 				{
 					wxBell();
@@ -2082,7 +1762,7 @@ void CLocalListView::OnMenuEdit(wxCommandEvent& event)
 		wxString file = fn.GetFullPath();
 		if (!pEditHandler->AddFile(CEditHandler::local, file, path, server))
 		{
-			wxMessageBox(wxString::Format(_("The file '%s' could not be opened:\nThe associated command failed"), fn.GetFullPath().c_str()), _("Opening failed"), wxICON_EXCLAMATION);
+			wxMessageBoxEx(wxString::Format(_("The file '%s' could not be opened:\nThe associated command failed"), fn.GetFullPath()), _("Opening failed"), wxICON_EXCLAMATION);
 			continue;
 		}
 	}
@@ -2091,9 +1771,8 @@ void CLocalListView::OnMenuEdit(wxCommandEvent& event)
 void CLocalListView::OnMenuOpen(wxCommandEvent& event)
 {
 	long item = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
-	if (item == -1)
-	{
-		wxLaunchDefaultBrowser(CState::GetAsURL(m_dir));
+	if (item == -1) {
+		OpenInFileManager(m_dir);
 		return;
 	}
 
@@ -2112,7 +1791,7 @@ void CLocalListView::OnMenuOpen(wxCommandEvent& event)
 		if (!data)
 			continue;
 
-		if (data->flags == fill)
+		if (data->comparison_flags == fill)
 			continue;
 
 		selected_item_list.push_back(*data);
@@ -2147,42 +1826,41 @@ void CLocalListView::OnMenuOpen(wxCommandEvent& event)
 		if (data->dir)
 		{
 			CLocalPath path(m_dir);
-			if (!path.ChangePath(data->name))
-			{
+			if (!path.ChangePath(data->name)) {
 				wxBell();
 				continue;
 			}
 
-			wxLaunchDefaultBrowser(CState::GetAsURL(path.GetPath()));
+			OpenInFileManager(path.GetPath());
 			continue;
 		}
 
 		wxFileName fn(m_dir, data->name);
 
 		bool program_exists = false;
-		wxString cmd = pEditHandler->GetSystemOpenCommand(fn.GetFullPath(), program_exists);
-		if (cmd == _T(""))
+		wxString cmd = GetSystemOpenCommand(fn.GetFullPath(), program_exists);
+		if (cmd.empty())
 		{
 			int pos = data->name.Find('.') == -1;
 			if (pos == -1 || (pos == 0 && data->name.Mid(1).Find('.') == -1))
 				cmd = pEditHandler->GetOpenCommand(fn.GetFullPath(), program_exists);
 		}
-		if (cmd == _T(""))
+		if (cmd.empty())
 		{
-			wxMessageBox(wxString::Format(_("The file '%s' could not be opened:\nNo program has been associated on your system with this file type."), fn.GetFullPath().c_str()), _("Opening failed"), wxICON_EXCLAMATION);
+			wxMessageBoxEx(wxString::Format(_("The file '%s' could not be opened:\nNo program has been associated on your system with this file type."), fn.GetFullPath()), _("Opening failed"), wxICON_EXCLAMATION);
 			continue;
 		}
 		if (!program_exists)
 		{
-			wxString msg = wxString::Format(_("The file '%s' cannot be opened:\nThe associated program (%s) could not be found.\nPlease check your filetype associations."), fn.GetFullPath().c_str(), cmd.c_str());
-			wxMessageBox(msg, _("Cannot edit file"), wxICON_EXCLAMATION);
+			wxString msg = wxString::Format(_("The file '%s' cannot be opened:\nThe associated program (%s) could not be found.\nPlease check your filetype associations."), fn.GetFullPath(), cmd);
+			wxMessageBoxEx(msg, _("Cannot edit file"), wxICON_EXCLAMATION);
 			continue;
 		}
 
 		if (wxExecute(cmd))
 			continue;
 
-		wxMessageBox(wxString::Format(_("The file '%s' could not be opened:\nThe associated command failed"), fn.GetFullPath().c_str()), _("Opening failed"), wxICON_EXCLAMATION);
+		wxMessageBoxEx(wxString::Format(_("The file '%s' could not be opened:\nThe associated command failed"), fn.GetFullPath()), _("Opening failed"), wxICON_EXCLAMATION);
 	}
 }
 
@@ -2255,8 +1933,8 @@ void CLocalListView::OnNavigationEvent(bool forward)
 		wxString error;
 		if (!m_pState->SetLocalDir(_T(".."), &error))
 		{
-			if (error != _T(""))
-				wxMessageBox(error, _("Failed to change directory"), wxICON_INFORMATION);
+			if (!error.empty())
+				wxMessageBoxEx(error, _("Failed to change directory"), wxICON_INFORMATION);
 			else
 				wxBell();
 		}
