@@ -3,11 +3,13 @@
 #include "queue.h"
 #include "filezillaapp.h"
 #include "filter.h"
+#include "file_utils.h"
 #include <wx/dnd.h>
 #include "dndobjects.h"
 #include "inputdialog.h"
 #include "local_filesys.h"
 #include "dragdropmanager.h"
+#include "Options.h"
 
 #ifdef _DEBUG
 #define new DEBUG_NEW
@@ -56,7 +58,7 @@ public:
 
 #ifdef __WXMSW__
 		if (dir == _T("/"))
-			return _T("");
+			return wxString();
 #endif
 
 		return dir;
@@ -79,6 +81,9 @@ public:
 			def == wxDragNone ||
 			def == wxDragCancel)
 			return def;
+		if( def == wxDragLink ) {
+			def = wxDragCopy;
+		}
 
 		wxTreeItemId hit = GetHit(wxPoint(x, y));
 		if (!hit)
@@ -101,13 +106,13 @@ public:
 		{
 			if (m_pRemoteDataObject->GetProcessId() != (int)wxGetProcessId())
 			{
-				wxMessageBox(_("Drag&drop between different instances of FileZilla has not been implemented yet."));
+				wxMessageBoxEx(_("Drag&drop between different instances of FileZilla has not been implemented yet."));
 				return wxDragNone;
 			}
 
 			if (!m_pLocalTreeView->m_pState->GetServer() || !m_pRemoteDataObject->GetServer().EqualsNoPass(*m_pLocalTreeView->m_pState->GetServer()))
 			{
-				wxMessageBox(_("Drag&drop between different servers has not been implemented yet."));
+				wxMessageBoxEx(_("Drag&drop between different servers has not been implemented yet."));
 				return wxDragNone;
 			}
 
@@ -127,7 +132,7 @@ public:
 			return false;
 
 		const wxString dir = GetDirFromItem(hit);
-		if (dir == _T("") || !CLocalPath(dir).IsWriteable())
+		if (dir.empty() || !CLocalPath(dir).IsWriteable())
 			return false;
 
 		return true;
@@ -139,15 +144,15 @@ public:
 		if (!hit)
 		{
 			ClearDropHighlight();
-			return _T("");
+			return wxString();
 		}
 
 		wxString dir = GetDirFromItem(hit);
 
-		if (dir == _T(""))
+		if (dir.empty())
 		{
 			ClearDropHighlight();
-			return _T("");
+			return wxString();
 		}
 
 		const wxTreeItemId dropHighlight = m_pLocalTreeView->m_dropHighlight;
@@ -172,7 +177,7 @@ public:
 		}
 
 		const wxString& dir = DisplayDropHighlight(wxPoint(x, y));
-		if (dir == _T(""))
+		if (dir.empty())
 			return wxDragNone;
 
 		if (def == wxDragLink)
@@ -219,6 +224,7 @@ EVT_MENU(XRCID("ID_ADDTOQUEUE"), CLocalTreeView::OnMenuUpload)
 EVT_MENU(XRCID("ID_DELETE"), CLocalTreeView::OnMenuDelete)
 EVT_MENU(XRCID("ID_RENAME"), CLocalTreeView::OnMenuRename)
 EVT_MENU(XRCID("ID_MKDIR"), CLocalTreeView::OnMenuMkdir)
+EVT_MENU(XRCID("ID_MKDIR_CHGDIR"), CLocalTreeView::OnMenuMkdirChgDir)
 EVT_TREE_BEGIN_LABEL_EDIT(wxID_ANY, CLocalTreeView::OnBeginLabelEdit)
 EVT_TREE_END_LABEL_EDIT(wxID_ANY, CLocalTreeView::OnEndLabelEdit)
 EVT_CHAR(CLocalTreeView::OnChar)
@@ -241,16 +247,17 @@ CLocalTreeView::CLocalTreeView(wxWindow* parent, wxWindowID id, CState *pState,
 
 	SetImageList(GetSystemImageList());
 
+	UpdateSortMode();
 #ifdef __WXMSW__
 	m_pVolumeEnumeratorThread = 0;
 
 	CreateRoot();
 #else
 	wxTreeItemId root = AddRoot(_T("/"));
-	SetItemImage(root, GetIconIndex(dir), wxTreeItemIcon_Normal);
-	SetItemImage(root, GetIconIndex(opened_dir), wxTreeItemIcon_Selected);
-	SetItemImage(root, GetIconIndex(dir), wxTreeItemIcon_Expanded);
-	SetItemImage(root, GetIconIndex(opened_dir), wxTreeItemIcon_SelectedExpanded);
+	SetItemImage(root, GetIconIndex(iconType::dir), wxTreeItemIcon_Normal);
+	SetItemImage(root, GetIconIndex(iconType::opened_dir), wxTreeItemIcon_Selected);
+	SetItemImage(root, GetIconIndex(iconType::dir), wxTreeItemIcon_Expanded);
+	SetItemImage(root, GetIconIndex(iconType::opened_dir), wxTreeItemIcon_SelectedExpanded);
 
 	SetDir(_T("/"));
 #endif
@@ -269,7 +276,7 @@ void CLocalTreeView::SetDir(wxString localDir)
 {
 	if (m_currentDir == localDir)
 	{
-		Refresh();
+		RefreshListing();
 		return;
 	}
 
@@ -298,7 +305,7 @@ void CLocalTreeView::SetDir(wxString localDir)
 		return;
 	}
 
-	if (subDirs == _T(""))
+	if (subDirs.empty())
 	{
 		SafeSelectItem(parent);
 		return;
@@ -342,7 +349,7 @@ wxTreeItemId CLocalTreeView::GetNearestParent(wxString& localDir)
 		wxTreeItemId root = GetRootItem();
 #endif
 
-	while (localDir != _T(""))
+	while (!localDir.empty())
 	{
 		wxString subDir;
 		int pos = localDir.Find(separator);
@@ -388,70 +395,28 @@ wxTreeItemId CLocalTreeView::GetSubdir(wxTreeItemId parent, const wxString& subD
 bool CLocalTreeView::DisplayDrives(wxTreeItemId parent)
 {
 	wxGetApp().AddStartupProfileRecord(_T("CLocalTreeView::DisplayDrives"));
-	long drivesToHide = 0;
-	// Adhere to the NODRIVES group policy
-	wxRegKey key(_T("HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\Policies\\Explorer"));
-	if (key.Exists())
-	{
-		if (!key.HasValue(_T("NoDrives")) || !key.QueryValue(_T("NoDrives"), &drivesToHide))
-			drivesToHide = 0;
-	}
 
-	int len = GetLogicalDriveStrings(0, 0);
-	if (!len)
-		return false;
-
-	wxChar* drives = new wxChar[len + 1];
-
-	if (!GetLogicalDriveStrings(len, drives))
-	{
-		delete [] drives;
-		return false;
-	}
+	std::list<wxString> drives = CVolumeDescriptionEnumeratorThread::GetDrives();
 
 	m_pVolumeEnumeratorThread = new CVolumeDescriptionEnumeratorThread(this);
-	if (m_pVolumeEnumeratorThread->Failed())
-	{
+	if (m_pVolumeEnumeratorThread->Failed()) {
 		delete m_pVolumeEnumeratorThread;
 		m_pVolumeEnumeratorThread = 0;
 	}
 
-	const wxChar* pDrive = drives;
-	while (*pDrive)
-	{
-		// Check if drive should be hidden by default
-		if (pDrive[0] != 0 && pDrive[1] == ':')
-		{
-			int bit = 0;
-			char letter = pDrive[0];
-			if (letter >= 'A' && letter <= 'Z')
-				bit = 1 << (letter - 'A');
-			if (letter >= 'a' && letter <= 'z')
-				bit = 1 << (letter - 'a');
-
-			if (drivesToHide & bit)
-			{
-				pDrive += wxStrlen(pDrive) + 1;
-				continue;
-			}
-		}
-
-		wxString drive = pDrive;
+	for( std::list<wxString>::const_iterator it = drives.begin(); it != drives.end(); ++it ) {
+		wxString drive = *it;
 		if (drive.Right(1) == _T("\\"))
 			drive = drive.RemoveLast();
 
 		wxGetApp().AddStartupProfileRecord(wxString::Format(_T("CLocalTreeView::DisplayDrives adding drive %s"), drive.c_str()));
-		wxTreeItemId item = AppendItem(parent, drive, GetIconIndex(dir, pDrive));
+		wxTreeItemId item = AppendItem(parent, drive, GetIconIndex(iconType::dir, drive));
 		AppendItem(item, _T(""));
 		SortChildren(parent);
-
-		pDrive += wxStrlen(pDrive) + 1;
 	}
 
 	wxGetApp().AddStartupProfileRecord(_T("CLocalTreeView::DisplayDrives adding drives done"));
 
-	delete [] drives;
-
 	return true;
 }
 
@@ -465,18 +430,18 @@ void CLocalTreeView::DisplayDir(wxTreeItemId parent, const wxString& dirname, co
 		wxLogNull log;
 		if (!local_filesystem.BeginFindFiles(dirname, true))
 		{
-			if (knownSubdir != _T(""))
+			if (!knownSubdir.empty())
 			{
 				wxTreeItemId item = GetSubdir(parent, knownSubdir);
 				if (item != wxTreeItemId())
 					return;
 
 				const wxString fullName = dirname + knownSubdir;
-				item = AppendItem(parent, knownSubdir, GetIconIndex(::dir, fullName),
+				item = AppendItem(parent, knownSubdir, GetIconIndex(iconType::dir, fullName),
 #ifdef __WXMSW__
 						-1
 #else
-						GetIconIndex(opened_dir, fullName)
+						GetIconIndex(iconType::opened_dir, fullName)
 #endif
 					);
 				CheckSubdirStatus(item, fullName);
@@ -505,11 +470,11 @@ void CLocalTreeView::DisplayDir(wxTreeItemId parent, const wxString& dirname, co
 	int attributes;
 	bool is_dir;
 	const wxLongLong size(-1);
-	wxDateTime date;
+	CDateTime date;
 	while (local_filesystem.GetNextFile(file, wasLink, is_dir, 0, &date, &attributes))
 	{
 		wxASSERT(is_dir);
-		if (file == _T(""))
+		if (file.empty())
 		{
 			wxGetApp().DisplayEncodingWarning();
 			continue;
@@ -522,31 +487,31 @@ void CLocalTreeView::DisplayDir(wxTreeItemId parent, const wxString& dirname, co
 		if (file != knownSubdir)
 #endif
 		{
-			if (filter.FilenameFiltered(file, dirname, true, size, true, attributes, date.IsValid() ? &date : 0))
+			if (filter.FilenameFiltered(file, dirname, true, size, true, attributes, date))
 				continue;
 		}
 		else
 			matchedKnown = true;
 
-		wxTreeItemId item = AppendItem(parent, file, GetIconIndex(::dir, fullName),
+		wxTreeItemId item = AppendItem(parent, file, GetIconIndex(iconType::dir, fullName),
 #ifdef __WXMSW__
 				-1
 #else
-				GetIconIndex(opened_dir, fullName)
+				GetIconIndex(iconType::opened_dir, fullName)
 #endif
 			);
 
 		CheckSubdirStatus(item, fullName);
 	}
 
-	if (!matchedKnown && knownSubdir != _T(""))
+	if (!matchedKnown && !knownSubdir.empty())
 	{
 		const wxString fullName = dirname + knownSubdir;
-		wxTreeItemId item = AppendItem(parent, knownSubdir, GetIconIndex(::dir, fullName),
+		wxTreeItemId item = AppendItem(parent, knownSubdir, GetIconIndex(iconType::dir, fullName),
 #ifdef __WXMSW__
 				-1
 #else
-				GetIconIndex(opened_dir, fullName)
+				GetIconIndex(iconType::opened_dir, fullName)
 #endif
 			);
 
@@ -564,37 +529,37 @@ wxString CLocalTreeView::HasSubdir(const wxString& dirname)
 
 	CLocalFileSystem local_filesystem;
 	if (!local_filesystem.BeginFindFiles(dirname, true))
-		return _T("");
+		return wxString();
 
 	wxString file;
 	bool wasLink;
 	int attributes;
 	bool is_dir;
 	const wxLongLong size(-1);
-	wxDateTime date;
+	CDateTime date;
 	while (local_filesystem.GetNextFile(file, wasLink, is_dir, 0, &date, &attributes))
 	{
 		wxASSERT(is_dir);
-		if (file == _T(""))
+		if (file.empty())
 		{
 			wxGetApp().DisplayEncodingWarning();
 			continue;
 		}
 
-		if (filter.FilenameFiltered(file, dirname, true, size, true, attributes, date.IsValid() ? &date : 0))
+		if (filter.FilenameFiltered(file, dirname, true, size, true, attributes, date))
 			continue;
 
 		return file;
 	}
 
-	return _T("");
+	return wxString();
 }
 
 wxTreeItemId CLocalTreeView::MakeSubdirs(wxTreeItemId parent, wxString dirname, wxString subDir)
 {
 	const wxString& separator = wxFileName::GetPathSeparator();
 
-	while (subDir != _T(""))
+	while (!subDir.empty())
 	{
 		int pos = subDir.Find(separator);
 		wxString segment;
@@ -630,7 +595,7 @@ void CLocalTreeView::OnItemExpanding(wxTreeEvent& event)
 
 	wxTreeItemIdValue value;
 	wxTreeItemId child = GetFirstChild(item, value);
-	if (child && GetItemText(child) == _T(""))
+	if (child && GetItemText(child).empty())
 		DisplayDir(item, GetDirFromItem(item));
 }
 
@@ -648,7 +613,7 @@ wxString CLocalTreeView::GetDirFromItem(wxTreeItemId item)
 			{
 				if (SHGetFolderPath(0, CSIDL_DESKTOP, 0, SHGFP_TYPE_CURRENT, path) != S_OK)
 				{
-					wxMessageBox(_("Failed to get desktop path"));
+					wxMessageBoxEx(_("Failed to get desktop path"));
 					return _T("/");
 				}
 			}
@@ -662,7 +627,7 @@ wxString CLocalTreeView::GetDirFromItem(wxTreeItemId item)
 			wxChar path[MAX_PATH + 1];
 			if (SHGetFolderPath(0, CSIDL_PERSONAL, 0, SHGFP_TYPE_CURRENT, path) != S_OK)
 			{
-				wxMessageBox(_("Failed to get 'My Documents' path"));
+				wxMessageBoxEx(_("Failed to get 'My Documents' path"));
 				return _T("/");
 			}
 			dir = path;
@@ -700,16 +665,24 @@ struct t_dir
 	wxTreeItemId item;
 };
 
-static bool sortfunc(const wxString& a, const wxString& b)
+void CLocalTreeView::UpdateSortMode()
 {
-#ifdef __WXMSW__
-	return b.CmpNoCase(a) > 0;
-#else
-	return b.Cmp(a) > 0;
-#endif
+	switch (COptions::Get()->GetOptionVal(OPTION_FILELIST_NAMESORT))
+	{
+	case 0:
+	default:
+		m_nameSortMode = CFileListCtrlSortBase::namesort_caseinsensitive;
+		break;
+	case 1:
+		m_nameSortMode = CFileListCtrlSortBase::namesort_casesensitive;
+		break;
+	case 2:
+		m_nameSortMode = CFileListCtrlSortBase::namesort_natural;
+		break;
+	}
 }
 
-void CLocalTreeView::Refresh()
+void CLocalTreeView::RefreshListing()
 {
 	wxLogNull nullLog;
 
@@ -717,6 +690,8 @@ void CLocalTreeView::Refresh()
 
 	std::list<t_dir> dirsToCheck;
 
+	UpdateSortMode();
+
 #ifdef __WXMSW__
 	int prevErrorMode = SetErrorMode(SEM_FAILCRITICALERRORS);
 
@@ -774,34 +749,34 @@ void CLocalTreeView::Refresh()
 		}
 
 		// Step 2: Enumerate subdirectories on disk and sort them
-		std::list<wxString> dirs;
+		wxArrayString dirs;
 
 		wxString file;
 		const wxLongLong size(-1);
 		bool was_link;
 		bool is_dir;
 		int attributes;
-		wxDateTime date;
+		CDateTime date;
 		while (local_filesystem.GetNextFile(file, was_link, is_dir, 0, &date, &attributes))
 		{
-			if (file == _T(""))
+			if (file.empty())
 			{
 				wxGetApp().DisplayEncodingWarning();
 				continue;
 			}
 
-			if (filter.FilenameFiltered(file, dir.dir, true, size, true, attributes, date.IsValid() ? &date : 0))
+			if (filter.FilenameFiltered(file, dir.dir, true, size, true, attributes, date))
 				continue;
 
 			dirs.push_back(file);
 		}
-		dirs.sort(sortfunc);
+		dirs.Sort(CFileListCtrlSortBase::GetCmpFunction(m_nameSortMode));
 
 		bool inserted = false;
 
 		// Step 3: Merge list of subdirectories with subtree.
 		wxTreeItemId child = GetLastChild(dir.item);
-		std::list<wxString>::reverse_iterator iter = dirs.rbegin();
+		wxArrayString::reverse_iterator iter = dirs.rbegin();
 		while (child || iter != dirs.rend())
 		{
 			int cmp;
@@ -857,11 +832,11 @@ void CLocalTreeView::Refresh()
 			{
 				// New subdirectory, add treeitem
 				wxString fullname = dir.dir + *iter + separator;
-				wxTreeItemId newItem = AppendItem(dir.item, *iter, GetIconIndex(::dir, fullname),
+				wxTreeItemId newItem = AppendItem(dir.item, *iter, GetIconIndex(iconType::dir, fullname),
 #ifdef __WXMSW__
 						-1
 #else
-						GetIconIndex(opened_dir, fullname)
+						GetIconIndex(iconType::opened_dir, fullname)
 #endif
 					);
 
@@ -896,8 +871,8 @@ void CLocalTreeView::OnSelectionChanged(wxTreeEvent& event)
 	wxString error;
 	if (!m_pState->SetLocalDir(dir, &error))
 	{
-		if (error != _T(""))
-			wxMessageBox(error, _("Failed to change directory"), wxICON_INFORMATION);
+		if (!error.empty())
+			wxMessageBoxEx(error, _("Failed to change directory"), wxICON_INFORMATION);
 		else
 			wxBell();
 		m_setSelection = true;
@@ -910,11 +885,19 @@ int CLocalTreeView::OnCompareItems(const wxTreeItemId& item1, const wxTreeItemId
 {
 	wxString label1 = GetItemText(item1);
 	wxString label2 = GetItemText(item2);
-#ifdef __WXMSW__
-	return label1.CmpNoCase(label2);
-#else
-	return label1.Cmp(label2);
-#endif
+
+	switch (m_nameSortMode)
+	{
+	case CFileListCtrlSortBase::namesort_casesensitive:
+		return CFileListCtrlSortBase::CmpCase(label1, label2);
+
+	default:
+	case CFileListCtrlSortBase::namesort_caseinsensitive:
+		return CFileListCtrlSortBase::CmpNoCase(label1, label2);
+
+	case CFileListCtrlSortBase::namesort_natural:
+		return CFileListCtrlSortBase::CmpNatural(label1, label2);
+	}
 }
 
 void CLocalTreeView::OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void* data2)
@@ -924,7 +907,7 @@ void CLocalTreeView::OnStateChange(CState* pState, enum t_statechange_notificati
 	else
 	{
 		wxASSERT(notification == STATECHANGE_APPLYFILTER);
-		Refresh();
+		RefreshListing();
 	}
 }
 
@@ -1002,17 +985,17 @@ wxString CLocalTreeView::GetSpecialFolder(int folder, int &iconIndex, int &openI
 {
 	LPITEMIDLIST list;
 	if (SHGetSpecialFolderLocation((HWND)GetHandle(), folder, &list) != S_OK)
-		return _T("");
+		return wxString();
 
 	SHFILEINFO shFinfo;
 	if (!SHGetFileInfo((LPCTSTR)list, 0, &shFinfo, sizeof(shFinfo), SHGFI_PIDL | SHGFI_ICON | SHGFI_SMALLICON))
-		return _T("");
+		return wxString();
 
 	DestroyIcon(shFinfo.hIcon);
 	iconIndex = shFinfo.iIcon;
 
 	if (!SHGetFileInfo((LPCTSTR)list, 0, &shFinfo, sizeof(shFinfo), SHGFI_PIDL | SHGFI_ICON | SHGFI_SMALLICON | SHGFI_OPENICON | SHGFI_DISPLAYNAME))
-		return _T("");
+		return wxString();
 
 	DestroyIcon(shFinfo.hIcon);
 	openIconIndex = shFinfo.iIcon;
@@ -1037,7 +1020,7 @@ bool CLocalTreeView::CreateRoot()
 {
 	int iconIndex, openIconIndex;
 	wxString name = GetSpecialFolder(CSIDL_DESKTOP, iconIndex, openIconIndex);
-	if (name == _T(""))
+	if (name.empty())
 	{
 		name = _("Desktop");
 		iconIndex = openIconIndex = -1;
@@ -1046,7 +1029,7 @@ bool CLocalTreeView::CreateRoot()
 	m_desktop = AddRoot(name, iconIndex, openIconIndex);
 
 	name = GetSpecialFolder(CSIDL_PERSONAL, iconIndex, openIconIndex);
-	if (name == _T(""))
+	if (name.empty())
 	{
 		name = _("My Documents");
 		iconIndex = openIconIndex = -1;
@@ -1056,7 +1039,7 @@ bool CLocalTreeView::CreateRoot()
 
 
 	name = GetSpecialFolder(CSIDL_DRIVES, iconIndex, openIconIndex);
-	if (name == _T(""))
+	if (name.empty())
 	{
 		name = _("My Computer");
 		iconIndex = openIconIndex = -1;
@@ -1121,7 +1104,7 @@ void CLocalTreeView::OnContextMenu(wxTreeEvent& event)
 	const bool hasParent = path.HasParent();
 	const bool writeable = path.IsWriteable();
 
-	const bool remoteConnected = m_pState->IsRemoteConnected() && !m_pState->GetRemotePath().IsEmpty();
+	const bool remoteConnected = m_pState->IsRemoteConnected() && !m_pState->GetRemotePath().empty();
 
 	pMenu->Enable(XRCID("ID_UPLOAD"), hasParent && remoteConnected);
 	pMenu->Enable(XRCID("ID_ADDTOQUEUE"), hasParent && remoteConnected);
@@ -1148,7 +1131,7 @@ void CLocalTreeView::OnMenuUpload(wxCommandEvent& event)
 
 	const CServer server = *m_pState->GetServer();
 	CServerPath remotePath = m_pState->GetRemotePath();
-	if (remotePath.IsEmpty())
+	if (remotePath.empty())
 		return;
 
 	if (!remotePath.ChangePath(GetItemText(m_contextMenuItem)))
@@ -1157,10 +1140,41 @@ void CLocalTreeView::OnMenuUpload(wxCommandEvent& event)
 	m_pQueueView->QueueFolder(event.GetId() == XRCID("ID_ADDTOQUEUE"), false, path, remotePath, server);
 }
 
+// Create a new Directory
 void CLocalTreeView::OnMenuMkdir(wxCommandEvent& event)
 {
-	if (!m_contextMenuItem.IsOk())
+	wxString newdir = MenuMkdir();
+	if (!newdir.empty()) {
+		RefreshListing();
+		m_pState->RefreshLocal();
+	}
+}
+
+// Create a new Directory and enter the new Directory
+void CLocalTreeView::OnMenuMkdirChgDir(wxCommandEvent& event)
+{
+	wxString newdir = MenuMkdir();
+	if (newdir.empty()) {
 		return;
+	}
+
+	// OnMenuEnter
+	wxString error;
+	if (!m_pState->SetLocalDir(newdir, &error))
+	{
+		if (!error.empty())
+			wxMessageBoxEx(error, _("Failed to change directory"), wxICON_INFORMATION);
+		else
+			wxBell();
+	}
+}
+
+// Helper-Function to create a new Directory
+// Returns the name of the new directory
+wxString CLocalTreeView::MenuMkdir()
+{
+	if (!m_contextMenuItem.IsOk())
+		return wxString();
 
 	wxString path = GetDirFromItem(m_contextMenuItem);
 	if (path.Last() != wxFileName::GetPathSeparator())
@@ -1169,25 +1183,25 @@ void CLocalTreeView::OnMenuMkdir(wxCommandEvent& event)
 	if (!CLocalPath(path).IsWriteable())
 	{
 		wxBell();
-		return;
+		return wxString();
 	}
 
 	CInputDialog dlg;
 	if (!dlg.Create(this, _("Create directory"), _("Please enter the name of the directory which should be created:")))
-		return;
+		return wxString();
 
 	wxString newName = _("New directory");
 	dlg.SetValue(path + newName);
 	dlg.SelectText(path.Len(), path.Len() + newName.Len());
 
 	if (dlg.ShowModal() != wxID_OK)
-		return;
+		return wxString();
 
 	wxFileName fn(dlg.GetValue(), _T(""));
 	if (!fn.Normalize(wxPATH_NORM_ALL, path))
 	{
 		wxBell();
-		return;
+		return wxString();
 	}
 
 	bool res;
@@ -1196,11 +1210,12 @@ void CLocalTreeView::OnMenuMkdir(wxCommandEvent& event)
 		res = fn.Mkdir(fn.GetPath(), 0777, wxPATH_MKDIR_FULL);
 	}
 
-	if (!res)
+	if (!res) {
 		wxBell();
+		return wxString();
+	}
 
-	Refresh();
-	m_pState->RefreshLocal();
+	return fn.GetPath();
 }
 
 void CLocalTreeView::OnMenuRename(wxCommandEvent& event)
@@ -1249,7 +1264,7 @@ void CLocalTreeView::OnMenuDelete(wxCommandEvent& event)
 		if (GetItemParent(m_contextMenuItem) == GetSelection())
 			m_pState->RefreshLocal();
 		else
-			Refresh();
+			RefreshListing();
 		return;
 	}
 
@@ -1262,7 +1277,7 @@ void CLocalTreeView::OnMenuDelete(wxCommandEvent& event)
 		path = path.Left(pos);
 
 	m_pState->SetLocalDir(path);
-	Refresh();
+	RefreshListing();
 }
 
 void CLocalTreeView::OnBeginLabelEdit(wxTreeEvent& event)
@@ -1288,9 +1303,6 @@ void CLocalTreeView::OnBeginLabelEdit(wxTreeEvent& event)
 	}
 }
 
-// Defined in LocalListView.cpp
-extern bool Rename(wxWindow* pWnd, wxString dir, wxString from, wxString to);
-
 void CLocalTreeView::OnEndLabelEdit(wxTreeEvent& event)
 {
 	if (event.IsEditCancelled())
@@ -1330,7 +1342,7 @@ void CLocalTreeView::OnEndLabelEdit(wxTreeEvent& event)
 
 	const wxString& oldName = GetItemText(item);
 	const wxString& newName = event.GetLabel();
-	if (newName == _T(""))
+	if (newName.empty())
 	{
 		wxBell();
 		event.Veto();
@@ -1342,8 +1354,7 @@ void CLocalTreeView::OnEndLabelEdit(wxTreeEvent& event)
 	if (oldName == newName)
 		return;
 
-	if (!Rename(this, parent, oldName, newName))
-	{
+	if (!RenameFile(this, parent, oldName, newName)) {
 		event.Veto();
 		return;
 	}
@@ -1355,9 +1366,8 @@ void CLocalTreeView::OnEndLabelEdit(wxTreeEvent& event)
 	SetItemText(item, newName);
 
 	wxTreeItemId currentSel = GetSelection();
-	if (currentSel == wxTreeItemId())
-	{
-		Refresh();
+	if (currentSel == wxTreeItemId()) {
+		RefreshListing();
 		return;
 	}
 
@@ -1414,7 +1424,7 @@ bool CLocalTreeView::CheckSubdirStatus(wxTreeItemId& item, const wxString& path)
 		CFilterManager filter;
 
 		const int attributes = S_IRUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH;
-		if (!filter.FilenameFiltered(_T("localhost"), path, true, size, true, attributes, 0))
+		if (!filter.FilenameFiltered(_T("localhost"), path, true, size, true, attributes, CDateTime()))
 		{
 			if (!child)
 				AppendItem(item, _T(""));
@@ -1425,7 +1435,7 @@ bool CLocalTreeView::CheckSubdirStatus(wxTreeItemId& item, const wxString& path)
 
 	if (child)
 	{
-		if (GetItemText(child) != _T(""))
+		if (!GetItemText(child).empty())
 			return false;
 
 		CTreeItemData* pData = (CTreeItemData*)GetItemData(child);
@@ -1434,7 +1444,7 @@ bool CLocalTreeView::CheckSubdirStatus(wxTreeItemId& item, const wxString& path)
 			bool wasLink;
 			int attributes;
 			enum CLocalFileSystem::local_fileType type;
-			wxDateTime date;
+			CDateTime date;
 			if (path.Last() == CLocalFileSystem::path_separator)
 				type = CLocalFileSystem::GetFileInfo(path + pData->m_known_subdir, wasLink, 0, &date, &attributes);
 			else
@@ -1442,14 +1452,14 @@ bool CLocalTreeView::CheckSubdirStatus(wxTreeItemId& item, const wxString& path)
 			if (type == CLocalFileSystem::dir)
 			{
 				CFilterManager filter;
-				if (!filter.FilenameFiltered(pData->m_known_subdir, path, true, size, true, attributes, date.IsValid() ? &date : 0))
+				if (!filter.FilenameFiltered(pData->m_known_subdir, path, true, size, true, attributes, date))
 					return true;
 			}
 		}
 	}
 
 	wxString sub = HasSubdir(path);
-	if (sub != _T(""))
+	if (!sub.empty())
 	{
 		wxTreeItemId subItem = AppendItem(item, _T(""));
 		SetItemData(subItem, new CTreeItemData(sub));
@@ -1503,37 +1513,24 @@ void CLocalTreeView::OnDevicechange(WPARAM wParam, LPARAM lParam)
 	}
 }
 
-wxTreeItemId CLocalTreeView::AddDrive(wxChar drive)
+wxTreeItemId CLocalTreeView::AddDrive(wxChar letter)
 {
-	// Adhere to the NODRIVES group policy
-	long drivesToHide = 0;
-	wxRegKey key(_T("HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\Policies\\Explorer"));
-	if (key.Exists())
-	{
-		if (!key.HasValue(_T("NoDrives")) || !key.QueryValue(_T("NoDrives"), &drivesToHide))
-			drivesToHide = 0;
-	}
+	wxString drive = letter;
+	drive += _T(":");
 
-	int bit = 0;
-	if (drive >= 'A' && drive <= 'Z')
-		bit = 1 << (drive - 'A');
-	else if (drive >= 'a' && drive <= 'z')
-		bit = 1 << (drive - 'a');
-	if (drivesToHide & bit)
+	long drivesToHide = CVolumeDescriptionEnumeratorThread::GetDrivesToHide();
+	if( CVolumeDescriptionEnumeratorThread::IsHidden(drive, drivesToHide) ) {
 		return wxTreeItemId();
-
-	wxString driveName = drive;
-	driveName += _T(":");
+	}
 
 	// Get the label of the drive
 	wxChar volumeName[501];
 	int oldErrorMode = SetErrorMode(SEM_FAILCRITICALERRORS);
-	BOOL res = GetVolumeInformation(driveName + _T("\\"), volumeName, 500, 0, 0, 0, 0, 0);
+	BOOL res = GetVolumeInformation(drive + _T("\\"), volumeName, 500, 0, 0, 0, 0, 0);
 	SetErrorMode(oldErrorMode);
 
-	wxString itemLabel = driveName;
-	if (res && volumeName[0])
-	{
+	wxString itemLabel = drive;
+	if (res && volumeName[0]) {
 		itemLabel += _T(" (");
 		itemLabel += volumeName;
 		itemLabel += _T(")");
@@ -1543,7 +1540,7 @@ wxTreeItemId CLocalTreeView::AddDrive(wxChar drive)
 	wxTreeItemId driveItem = GetFirstChild(m_drives, value);
 	while (driveItem)
 	{
-		if (!GetItemText(driveItem).Left(2).CmpNoCase(driveName))
+		if (!GetItemText(driveItem).Left(2).CmpNoCase(drive))
 			break;
 
 		driveItem = GetNextSibling(driveItem);
@@ -1551,7 +1548,7 @@ wxTreeItemId CLocalTreeView::AddDrive(wxChar drive)
 	if (driveItem)
 	{
 		SetItemText(driveItem, itemLabel);
-		int icon = GetIconIndex(dir, driveName + _T("\\"));
+		int icon = GetIconIndex(iconType::dir, drive + _T("\\"));
 		SetItemImage(driveItem, icon, wxTreeItemIcon_Normal);
 		SetItemImage(driveItem, icon, wxTreeItemIcon_Selected);
 		SetItemImage(driveItem, icon, wxTreeItemIcon_Expanded);
@@ -1560,7 +1557,7 @@ wxTreeItemId CLocalTreeView::AddDrive(wxChar drive)
 		return driveItem;
 	}
 
-	wxTreeItemId item = AppendItem(m_drives, itemLabel, GetIconIndex(dir, driveName + _T("\\")));
+	wxTreeItemId item = AppendItem(m_drives, itemLabel, GetIconIndex(iconType::dir, drive + _T("\\")));
 	AppendItem(item, _T(""));
 	SortChildren(m_drives);
 
@@ -1596,7 +1593,7 @@ void CLocalTreeView::OnSelectionChanging(wxTreeEvent& event)
 
 	if (GetItemImage(item, wxTreeItemIcon_Selected) == -1)
 	{
-		int icon = GetIconIndex(opened_dir, GetDirFromItem(item));
+		int icon = GetIconIndex(iconType::opened_dir, GetDirFromItem(item));
 		SetItemImage(item, icon, wxTreeItemIcon_Selected);
 		SetItemImage(item, icon, wxTreeItemIcon_SelectedExpanded);
 	}
@@ -1604,14 +1601,14 @@ void CLocalTreeView::OnSelectionChanging(wxTreeEvent& event)
 
 #endif
 
-void CLocalTreeView::OnMenuOpen(wxCommandEvent& event)
+void CLocalTreeView::OnMenuOpen(wxCommandEvent&)
 {
 	if (!m_contextMenuItem.IsOk())
 		return;
 
 	wxString path = GetDirFromItem(m_contextMenuItem);
-	if (path == _T(""))
+	if (path.empty())
 		return;
 
-	CState::OpenInFileManager(path);
+	OpenInFileManager(path);
 }
