@@ -13,15 +13,13 @@ class CRemoteListViewDropTarget;
 class CRemoteListView : public CFileListCtrl<CGenericFileData>, CStateEventHandler
 {
 	friend class CRemoteListViewDropTarget;
-	friend class CRemoteListViewSortType;
-
 public:
 	CRemoteListView(wxWindow* pParent, CState* pState, CQueueView* pQueue);
 	virtual ~CRemoteListView();
 
 	virtual bool CanStartComparison(wxString* pError);
 	virtual void StartComparison();
-	virtual bool GetNextFile(wxString& name, bool &dir, wxLongLong &size, wxDateTime& date, bool& hasTime);
+	virtual bool GetNextFile(wxString& name, bool &dir, wxLongLong &size, CDateTime& date);
 	virtual void FinishComparison();
 	virtual void OnExitComparisonMode();
 
@@ -56,10 +54,10 @@ protected:
 
 	virtual void OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void* data2);
 	void ApplyCurrentFilter();
-	void SetDirectoryListing(const CSharedPointer<const CDirectoryListing> &pDirectoryListing, bool modified = false);
-	bool UpdateDirectoryListing(const CSharedPointer<const CDirectoryListing> &pDirectoryListing);
-	void UpdateDirectoryListing_Removed(const CSharedPointer<const CDirectoryListing> &pDirectoryListing);
-	void UpdateDirectoryListing_Added(const CSharedPointer<const CDirectoryListing> &pDirectoryListing);
+	void SetDirectoryListing(std::shared_ptr<CDirectoryListing> const& pDirectoryListing);
+	bool UpdateDirectoryListing(std::shared_ptr<CDirectoryListing> const& pDirectoryListing);
+	void UpdateDirectoryListing_Removed(std::shared_ptr<CDirectoryListing> const& pDirectoryListing);
+	void UpdateDirectoryListing_Added(std::shared_ptr<CDirectoryListing> const& pDirectoryListing);
 
 #ifdef __WXDEBUG__
 	void ValidateIndexMapping();
@@ -67,7 +65,7 @@ protected:
 
 	virtual void OnNavigationEvent(bool forward);
 
-	CSharedPointer <const CDirectoryListing> m_pDirectoryListing;
+	std::shared_ptr<CDirectoryListing> m_pDirectoryListing;
 
 	// Caller is responsible to check selection is valid!
 	void TransferSelectedFiles(const CLocalPath& local_parent, bool queueOnly);
@@ -86,7 +84,6 @@ protected:
 	virtual int GetOverlayIndex(int item);
 #endif
 
-	wxDropTarget* m_pDropTarget;
 	int m_dropTarget;
 
 	// Used to track state for resolving symlinks
@@ -100,21 +97,24 @@ protected:
 		CLocalPath local_path;
 	} *m_pLinkResolveState;
 
+	CServerPath MenuMkdir();
+
 	DECLARE_EVENT_TABLE()
 	void OnItemActivated(wxListEvent &event);
 	void OnContextMenu(wxContextMenuEvent& event);
 	void OnMenuDownload(wxCommandEvent& event);
-	void OnMenuMkdir(wxCommandEvent& event);
-	void OnMenuDelete(wxCommandEvent& event);
-	void OnMenuRename(wxCommandEvent& event);
+	void OnMenuMkdir(wxCommandEvent&);
+	void OnMenuMkdirChgDir(wxCommandEvent&);
+	void OnMenuDelete(wxCommandEvent&);
+	void OnMenuRename(wxCommandEvent&);
 	void OnKeyDown(wxKeyEvent& event);
 	void OnMenuChmod(wxCommandEvent& event);
 	void OnSize(wxSizeEvent& event);
-	void OnBeginDrag(wxListEvent& event);
-	void OnMenuEdit(wxCommandEvent& event);
-	void OnMenuEnter(wxCommandEvent& event);
-	void OnMenuGeturl(wxCommandEvent& event);
-	void OnMenuRefresh(wxCommandEvent& event);
+	void OnBeginDrag(wxListEvent&);
+	void OnMenuEdit(wxCommandEvent&);
+	void OnMenuEnter(wxCommandEvent&);
+	void OnMenuGeturl(wxCommandEvent&);
+	void OnMenuRefresh(wxCommandEvent&);
 	void OnMenuNewfile(wxCommandEvent& event);
 };
 
