@@ -11,6 +11,7 @@
 #include "queue.h"
 #include "QueueView.h"
 #include "themeprovider.h"
+#include "Options.h"
 
 #ifdef _DEBUG
 #define new DEBUG_NEW
@@ -69,7 +70,7 @@ public:
 			return wxDragNone;
 
 		CServerPath path = m_pRemoteTreeView->GetPathFromItem(hit);
-		if (path.IsEmpty())
+		if (path.empty())
 			return wxDragNone;
 
 		if (!GetData())
@@ -85,20 +86,20 @@ public:
 		{
 			if (m_pRemoteDataObject->GetProcessId() != (int)wxGetProcessId())
 			{
-				wxMessageBox(_("Drag&drop between different instances of FileZilla has not been implemented yet."));
+				wxMessageBoxEx(_("Drag&drop between different instances of FileZilla has not been implemented yet."));
 				return wxDragNone;
 			}
 
 			if (!m_pRemoteTreeView->m_pState->GetServer() || !m_pRemoteDataObject->GetServer().EqualsNoPass(*m_pRemoteTreeView->m_pState->GetServer()))
 			{
-				wxMessageBox(_("Drag&drop between different servers has not been implemented yet."));
+				wxMessageBoxEx(_("Drag&drop between different servers has not been implemented yet."));
 				return wxDragNone;
 			}
 
 			// Make sure path path is valid
 			if (path == m_pRemoteDataObject->GetServerPath())
 			{
-				wxMessageBox(_("Source and path of the drop operation are identical"));
+				wxMessageBoxEx(_("Source and path of the drop operation are identical"));
 				return wxDragNone;
 			}
 
@@ -114,7 +115,7 @@ public:
 						return wxDragNone;
 					else if (dir.IsParentOf(path, false))
 					{
-						wxMessageBox(_("A directory cannot be dragged into one of its subdirectories."));
+						wxMessageBoxEx(_("A directory cannot be dragged into one of its subdirectories."));
 						return wxDragNone;
 					}
 				}
@@ -143,7 +144,7 @@ public:
 			return false;
 
 		const CServerPath& path = m_pRemoteTreeView->GetPathFromItem(hit);
-		if (path.IsEmpty())
+		if (path.empty())
 			return false;
 
 		return true;
@@ -160,7 +161,7 @@ public:
 
 		const CServerPath& path = m_pRemoteTreeView->GetPathFromItem(hit);
 
-		if (path.IsEmpty())
+		if (path.empty())
 		{
 			ClearDropHighlight();
 			return CServerPath();
@@ -187,7 +188,7 @@ public:
 		}
 
 		const CServerPath& path = DisplayDropHighlight(wxPoint(x, y));
-		if (path.IsEmpty())
+		if (path.empty())
 			return wxDragNone;
 
 		if (def == wxDragLink)
@@ -232,6 +233,7 @@ EVT_MENU(XRCID("ID_RENAME"), CRemoteTreeView::OnMenuRename)
 EVT_TREE_BEGIN_LABEL_EDIT(wxID_ANY, CRemoteTreeView::OnBeginLabelEdit)
 EVT_TREE_END_LABEL_EDIT(wxID_ANY, CRemoteTreeView::OnEndLabelEdit)
 EVT_MENU(XRCID("ID_MKDIR"), CRemoteTreeView::OnMkdir)
+EVT_MENU(XRCID("ID_MKDIR_CHGDIR"), CRemoteTreeView::OnMenuMkdirChgDir)
 EVT_CHAR(CRemoteTreeView::OnChar)
 EVT_MENU(XRCID("ID_GETURL"), CRemoteTreeView::OnMenuGeturl)
 END_EVENT_TABLE()
@@ -267,7 +269,7 @@ CRemoteTreeView::~CRemoteTreeView()
 	delete m_pImageList;
 }
 
-void CRemoteTreeView::OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void* data2)
+void CRemoteTreeView::OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString&, const void*)
 {
 	if (notification == STATECHANGE_REMOTE_DIR)
 		SetDirectoryListing(pState->GetRemoteDir(), false);
@@ -277,10 +279,24 @@ void CRemoteTreeView::OnStateChange(CState* pState, enum t_statechange_notificat
 		ApplyFilters();
 }
 
-void CRemoteTreeView::SetDirectoryListing(const CSharedPointer<const CDirectoryListing> &pListing, bool modified)
+void CRemoteTreeView::SetDirectoryListing(std::shared_ptr<CDirectoryListing> const& pListing, bool modified)
 {
 	m_busy = true;
 
+	switch (COptions::Get()->GetOptionVal(OPTION_FILELIST_NAMESORT))
+	{
+	case 0:
+	default:
+		m_nameSortMode = CFileListCtrlSortBase::namesort_caseinsensitive;
+		break;
+	case 1:
+		m_nameSortMode = CFileListCtrlSortBase::namesort_casesensitive;
+		break;
+	case 2:
+		m_nameSortMode = CFileListCtrlSortBase::namesort_natural;
+		break;
+	}
+
 	if (!pListing)
 	{
 		m_ExpandAfterList = wxTreeItemId();
@@ -301,7 +317,7 @@ void CRemoteTreeView::SetDirectoryListing(const CSharedPointer<const CDirectoryL
 	}
 	Enable(true);
 
-	if (pListing->m_hasUnsureEntries && !(pListing->m_hasUnsureEntries & ~(CDirectoryListing::unsure_unknown | CDirectoryListing::unsure_file_mask)))
+	if (pListing->get_unsure_flags() && !(pListing->get_unsure_flags() & ~(CDirectoryListing::unsure_unknown | CDirectoryListing::unsure_file_mask)))
 	{
 		// Just files changed, does not affect directory tree
 		m_busy = false;
@@ -350,7 +366,7 @@ void CRemoteTreeView::SetDirectoryListing(const CSharedPointer<const CDirectoryL
 	SetItemImages(parent, false);
 
 #ifndef __WXMSW__
-	m_freezeCount--;
+	Thaw();
 #endif
 	if (!modified)
 		SafeSelectItem(parent);
@@ -370,7 +386,7 @@ wxTreeItemId CRemoteTreeView::MakeParent(CServerPath path, bool select)
 		pieces.push_front(path.GetLastSegment());
 		path = path.GetParent();
 	}
-	wxASSERT(path.GetPath() != _T(""));
+	wxASSERT(!path.GetPath().empty());
 	pieces.push_front(path.GetPath());
 
 	const wxTreeItemId root = GetRootItem();
@@ -383,7 +399,7 @@ wxTreeItemId CRemoteTreeView::MakeParent(CServerPath path, bool select)
 
 		wxTreeItemIdValue cookie;
 		wxTreeItemId child = GetFirstChild(parent, cookie);
-		if (child && GetItemText(child) == _T(""))
+		if (child && GetItemText(child).empty())
 		{
 			Delete(child);
 			child = wxTreeItemId();
@@ -457,12 +473,12 @@ wxBitmap CRemoteTreeView::CreateIcon(int index, const wxString& overlay /*=_T(""
 	GetSystemImageList()->Draw(index, dc, 0, 0, wxIMAGELIST_DRAW_TRANSPARENT);
 
 	// Load overlay
-	if (overlay != _T(""))
+	if (!overlay.empty())
 	{
 		wxImage unknownIcon = wxArtProvider::GetBitmap(overlay, wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall)).ConvertToImage();
 
 		// Convert mask into alpha channel
-		if (!unknownIcon.HasAlpha())
+		if (unknownIcon.IsOk() && !unknownIcon.HasAlpha())
 		{
 			wxASSERT(unknownIcon.HasMask());
 			unknownIcon.InitAlpha();
@@ -482,12 +498,12 @@ void CRemoteTreeView::CreateImageList()
 	m_pImageList = new wxImageList(s.x, s.y, true, 4);
 
 	// Normal directory
-	int index = GetIconIndex(dir, _T("{78013B9C-3532-4fe1-A418-5CD1955127CC}"), false);
+	int index = GetIconIndex(iconType::dir, _T("{78013B9C-3532-4fe1-A418-5CD1955127CC}"), false);
 	m_pImageList->Add(CreateIcon(index));
 	m_pImageList->Add(CreateIcon(index, _T("ART_UNKNOWN")));
 
 	// Opened directory
-	index = GetIconIndex(opened_dir, _T("{78013B9C-3532-4fe1-A418-5CD1955127CC}"), false);
+	index = GetIconIndex(iconType::opened_dir, _T("{78013B9C-3532-4fe1-A418-5CD1955127CC}"), false);
 	m_pImageList->Add(CreateIcon(index));
 	m_pImageList->Add(CreateIcon(index, _T("ART_UNKNOWN")));
 
@@ -496,7 +512,7 @@ void CRemoteTreeView::CreateImageList()
 
 bool CRemoteTreeView::HasSubdirs(const CDirectoryListing& listing, const CFilterManager& filter)
 {
-	if (!listing.m_hasDirs)
+	if (!listing.has_dirs())
 		return false;
 
 	if (!filter.HasActiveFilters())
@@ -508,7 +524,7 @@ bool CRemoteTreeView::HasSubdirs(const CDirectoryListing& listing, const CFilter
 		if (!listing[i].is_dir())
 			continue;
 
-		if (filter.FilenameFiltered(listing[i].name, path, true, -1, false, 0, listing[i].has_date() ? &listing[i].time : 0))
+		if (filter.FilenameFiltered(listing[i].name, path, true, -1, false, 0, listing[i].time))
 			continue;
 
 		return true;
@@ -529,7 +545,7 @@ void CRemoteTreeView::DisplayItem(wxTreeItemId parent, const CDirectoryListing&
 		if (!listing[i].is_dir())
 			continue;
 
-		if (filter.FilenameFiltered(listing[i].name, path, true, -1, false, 0, listing[i].has_date() ? &listing[i].time : 0))
+		if (filter.FilenameFiltered(listing[i].name, path, true, -1, false, 0, listing[i].time))
 			continue;
 
 		const wxString& name = listing[i].name;
@@ -555,24 +571,13 @@ void CRemoteTreeView::DisplayItem(wxTreeItemId parent, const CDirectoryListing&
 	SortChildren(parent);
 }
 
-static bool sortfunc(const wxString& a, const wxString& b)
-{
-	int cmp = a.CmpNoCase(b);
-
-	if (!cmp)
-		cmp = a.Cmp(b);
-
-	return cmp < 0;
-}
-
-
 void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing& listing, bool will_select_parent)
 {
 	SetItemImages(parent, false);
 
 	wxTreeItemIdValue cookie;
 	wxTreeItemId child = GetFirstChild(parent, cookie);
-	if (!child || GetItemText(child) == _T(""))
+	if (!child || GetItemText(child).empty())
 	{
 		DisplayItem(parent, listing);
 		return;
@@ -582,21 +587,21 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 
 	const wxString path = listing.path.GetPath();
 
-	std::list<wxString> dirs;
+	wxArrayString dirs;
 	for (unsigned int i = 0; i < listing.GetCount(); i++)
 	{
 		if (!listing[i].is_dir())
 			continue;
 
-		if (!filter.FilenameFiltered(listing[i].name, path, true, -1, false, 0, listing[i].has_date() ? &listing[i].time : 0))
+		if (!filter.FilenameFiltered(listing[i].name, path, true, -1, false, 0, listing[i].time))
 			dirs.push_back(listing[i].name);
 	}
 
-	dirs.sort(sortfunc);
+	dirs.Sort(CFileListCtrlSortBase::GetCmpFunction(m_nameSortMode));
 
 	bool inserted = false;
 	child = GetLastChild(parent);
-	std::list<wxString>::reverse_iterator iter = dirs.rbegin();
+	wxArrayString::reverse_iterator iter = dirs.rbegin();
 	while (child && iter != dirs.rend())
 	{
 		int cmp = GetItemText(child).CmpNoCase(*iter);
@@ -702,10 +707,18 @@ int CRemoteTreeView::OnCompareItems(const wxTreeItemId& item1, const wxTreeItemI
 	wxString label1 = GetItemText(item1);
 	wxString label2 = GetItemText(item2);
 
-	int cmp = label1.CmpNoCase(label2);
-	if (cmp)
-		return cmp;
-	return label1.Cmp(label2);
+	switch (m_nameSortMode)
+	{
+	case CFileListCtrlSortBase::namesort_casesensitive:
+		return CFileListCtrlSortBase::CmpCase(label1, label2);
+
+	default:
+	case CFileListCtrlSortBase::namesort_caseinsensitive:
+		return CFileListCtrlSortBase::CmpNoCase(label1, label2);
+
+	case CFileListCtrlSortBase::namesort_natural:
+		return CFileListCtrlSortBase::CmpNatural(label1, label2);
+	}
 }
 
 void CRemoteTreeView::OnItemExpanding(wxTreeEvent& event)
@@ -767,8 +780,8 @@ void CRemoteTreeView::OnSelectionChanged(wxTreeEvent& event)
 		return;
 
 	const CServerPath path = GetPathFromItem(item);
-	wxASSERT(!path.IsEmpty());
-	if (path.IsEmpty())
+	wxASSERT(!path.empty());
+	if (path.empty())
 		return;
 
 	m_pState->ChangeRemoteDir(path);
@@ -820,12 +833,12 @@ void CRemoteTreeView::OnBeginDrag(wxTreeEvent& event)
 		return;
 
 	const CServerPath& path = GetPathFromItem(item);
-	if (path.IsEmpty() || !path.HasParent())
+	if (path.empty() || !path.HasParent())
 		return;
 
 	const CServerPath& parent = path.GetParent();
 	const wxString& lastSegment = path.GetLastSegment();
-	if (lastSegment == _T(""))
+	if (lastSegment.empty())
 		return;
 
 	wxDataObjectComposite object;
@@ -847,7 +860,7 @@ void CRemoteTreeView::OnBeginDrag(wxTreeEvent& event)
 	{
 		const wxString& file = ext->GetDragDirectory();
 
-		wxASSERT(file != _T(""));
+		wxASSERT(!file.empty());
 
 		wxFileDataObject *pFileDataObject = new wxFileDataObject;
 		pFileDataObject->AddFile(file);
@@ -896,7 +909,7 @@ void CRemoteTreeView::OnBeginDrag(wxTreeEvent& event)
 			{
 				delete ext;
 				ext = 0;
-				wxMessageBox(_("Could not determine the target of the Drag&Drop operation.\nEither the shell extension is not installed properly or you didn't drop the files into an Explorer window."));
+				wxMessageBoxEx(_("Could not determine the target of the Drag&Drop operation.\nEither the shell extension is not installed properly or you didn't drop the files into an Explorer window."));
 				return;
 			}
 
@@ -938,14 +951,14 @@ void CRemoteTreeView::OnContextMenu(wxTreeEvent& event)
 		return;
 
 	const CServerPath& path = m_contextMenuItem ? GetPathFromItem(m_contextMenuItem) : CServerPath();
-	if (!m_pState->IsRemoteIdle() || path.IsEmpty())
+	if (!m_pState->IsRemoteIdle() || path.empty())
 	{
 		pMenu->Enable(XRCID("ID_DOWNLOAD"), false);
 		pMenu->Enable(XRCID("ID_ADDTOQUEUE"), false);
 		pMenu->Enable(XRCID("ID_MKDIR"), false);
+		pMenu->Enable(XRCID("ID_MKDIR_CHGDIR"), false);
 		pMenu->Enable(XRCID("ID_DELETE"), false);
 		pMenu->Enable(XRCID("ID_CHMOD"), false);
-		pMenu->Enable(XRCID("ID_MKDIR"), false);
 		pMenu->Enable(XRCID("ID_RENAME"), false);
 		pMenu->Enable(XRCID("ID_GETURL"), false);
 	}
@@ -962,7 +975,7 @@ void CRemoteTreeView::OnContextMenu(wxTreeEvent& event)
 	delete pMenu;
 }
 
-void CRemoteTreeView::OnMenuChmod(wxCommandEvent& event)
+void CRemoteTreeView::OnMenuChmod(wxCommandEvent&)
 {
 	if (!m_pState->IsRemoteIdle())
 		return;
@@ -971,7 +984,7 @@ void CRemoteTreeView::OnMenuChmod(wxCommandEvent& event)
 		return;
 
 	const CServerPath& path = GetPathFromItem(m_contextMenuItem);
-	if (path.IsEmpty())
+	if (path.empty())
 		return;
 
 	const bool hasParent = path.HasParent();
@@ -998,7 +1011,7 @@ void CRemoteTreeView::OnMenuChmod(wxCommandEvent& event)
 				if (listing[i].name != name)
 					continue;
 
-				pChmodDlg->ConvertPermissions(listing[i].permissions, permissions);
+				pChmodDlg->ConvertPermissions(*listing[i].permissions, permissions);
 			}
 		}
 	}
@@ -1090,7 +1103,7 @@ void CRemoteTreeView::OnMenuDownload(wxCommandEvent& event)
 		return;
 
 	const CServerPath& path = GetPathFromItem(m_contextMenuItem);
-	if (path.IsEmpty())
+	if (path.empty())
 		return;
 
 	const wxString& name = GetItemText(m_contextMenuItem);
@@ -1110,7 +1123,7 @@ void CRemoteTreeView::OnMenuDownload(wxCommandEvent& event)
 	pRecursiveOperation->StartRecursiveOperation(addOnly ? CRecursiveOperation::recursive_addtoqueue : CRecursiveOperation::recursive_download, path, filter.GetActiveFilters(false), true, currentPath);
 }
 
-void CRemoteTreeView::OnMenuDelete(wxCommandEvent& event)
+void CRemoteTreeView::OnMenuDelete(wxCommandEvent&)
 {
 	if (!m_pState->IsRemoteIdle())
 		return;
@@ -1119,10 +1132,10 @@ void CRemoteTreeView::OnMenuDelete(wxCommandEvent& event)
 		return;
 
 	const CServerPath& path = GetPathFromItem(m_contextMenuItem);
-	if (path.IsEmpty())
+	if (path.empty())
 		return;
 
-	if (wxMessageBox(_("Really delete all selected files and/or directories from the server?"), _("Confirmation needed"), wxICON_QUESTION | wxYES_NO, this) != wxYES)
+	if (wxMessageBoxEx(_("Really delete all selected files and/or directories from the server?"), _("Confirmation needed"), wxICON_QUESTION | wxYES_NO, this) != wxYES)
 		return;
 
 	const bool hasParent = path.HasParent();
@@ -1146,14 +1159,14 @@ void CRemoteTreeView::OnMenuDelete(wxCommandEvent& event)
 	const wxTreeItemId selected = GetSelection();
 	if (selected)
 		currentPath = GetPathFromItem(selected);
-	if (!currentPath.IsEmpty() && (path == currentPath || path.IsParentOf(currentPath, false)))
+	if (!currentPath.empty() && (path == currentPath || path.IsParentOf(currentPath, false)))
 		currentPath = startDir;
 
 	CFilterManager filter;
 	pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_delete, startDir, filter.GetActiveFilters(false), !hasParent, currentPath);
 }
 
-void CRemoteTreeView::OnMenuRename(wxCommandEvent& event)
+void CRemoteTreeView::OnMenuRename(wxCommandEvent&)
 {
 	if (!m_pState->IsRemoteIdle())
 		return;
@@ -1162,7 +1175,7 @@ void CRemoteTreeView::OnMenuRename(wxCommandEvent& event)
 		return;
 
 	const CServerPath& path = GetPathFromItem(m_contextMenuItem);
-	if (path.IsEmpty())
+	if (path.empty())
 		return;
 
 	if (!path.HasParent())
@@ -1180,7 +1193,7 @@ void CRemoteTreeView::OnBeginLabelEdit(wxTreeEvent& event)
 	}
 
 	const CServerPath& path = GetPathFromItem(event.GetItem());
-	if (path.IsEmpty())
+	if (path.empty())
 	{
 		event.Veto();
 		return;
@@ -1232,7 +1245,7 @@ void CRemoteTreeView::OnEndLabelEdit(wxTreeEvent& event)
 	const wxTreeItemId selected = GetSelection();
 	if (selected)
 		currentPath = GetPathFromItem(selected);
-	if (currentPath.IsEmpty())
+	if (currentPath.empty())
 		return;
 
 	if (currentPath == old_path || currentPath.IsSubdirOf(old_path, false))
@@ -1260,21 +1273,54 @@ void CRemoteTreeView::OnEndLabelEdit(wxTreeEvent& event)
 		m_pState->ChangeRemoteDir(currentPath);
 }
 
-void CRemoteTreeView::OnMkdir(wxCommandEvent& event)
+
+// Create a new Directory
+void CRemoteTreeView::OnMkdir(wxCommandEvent&)
+{
+	CServerPath newpath = MenuMkdir();
+
+	CServerPath listed;
+	if (newpath.HasParent())
+	{
+		listed = newpath.GetParent();
+		m_pState->ChangeRemoteDir(listed);
+	}
+
+	CServerPath currentPath;
+	const wxTreeItemId selected = GetSelection();
+	if (selected)
+		currentPath = GetPathFromItem(selected);
+	if (!currentPath.empty() && currentPath != listed)
+		m_pState->ChangeRemoteDir(currentPath);
+
+}
+
+// Create a new Directory and enter the new Directory
+void CRemoteTreeView::OnMenuMkdirChgDir(wxCommandEvent&)
+{
+	CServerPath newpath = MenuMkdir();
+	if (!newpath.empty()) {
+		m_pState->ChangeRemoteDir(newpath);
+	}
+}
+
+// Help-Function to create a new Directory
+// Returns the name of the new directory
+CServerPath CRemoteTreeView::MenuMkdir()
 {
 	if (!m_pState->IsRemoteIdle())
-		return;
+		return CServerPath();
 
 	if (!m_contextMenuItem)
-		return;
+		return CServerPath();
 
 	const CServerPath& path = GetPathFromItem(m_contextMenuItem);
-	if (path.IsEmpty())
-		return;
+	if (path.empty())
+		return CServerPath();
 
 	CInputDialog dlg;
 	if (!dlg.Create(this, _("Create directory"), _("Please enter the name of the directory which should be created:")))
-		return;
+		return CServerPath();
 
 	CServerPath newPath = path;
 
@@ -1294,36 +1340,25 @@ void CRemoteTreeView::OnMkdir(wxCommandEvent& event)
 	}
 
 	if (dlg.ShowModal() != wxID_OK)
-		return;
+		return CServerPath();
 
 	newPath = path;
 	if (!newPath.ChangePath(dlg.GetValue()))
 	{
 		wxBell();
-		return;
+		return CServerPath();
 	}
 
 	m_pState->m_pCommandQueue->ProcessCommand(new CMkdirCommand(newPath));
-	CServerPath listed;
-	if (newPath.HasParent())
-	{
-		listed = newPath.GetParent();
-		m_pState->ChangeRemoteDir(listed);
-	}
 
-	CServerPath currentPath;
-	const wxTreeItemId selected = GetSelection();
-	if (selected)
-		currentPath = GetPathFromItem(selected);
-	if (!currentPath.IsEmpty() && currentPath != listed)
-		m_pState->ChangeRemoteDir(currentPath);
+	return newPath;
 }
 
 bool CRemoteTreeView::ListExpand(wxTreeItemId item)
 {
 	const CServerPath path = GetPathFromItem(item);
-	wxASSERT(!path.IsEmpty());
-	if (path.IsEmpty())
+	wxASSERT(!path.empty());
+	if (path.empty())
 		return false;
 
 	CDirectoryListing listing;
@@ -1334,7 +1369,7 @@ bool CRemoteTreeView::ListExpand(wxTreeItemId item)
 		SetItemImages(item, true);
 
 		wxTreeItemId child = GetLastChild(item);
-		if (!child || GetItemText(child) == _T(""))
+		if (!child || GetItemText(child).empty())
 			return false;
 	}
 
@@ -1369,7 +1404,7 @@ void CRemoteTreeView::ApplyFilters()
 	for (wxTreeItemId child = GetFirstChild(root, cookie); child; child = GetNextSibling(child))
 	{
 		CServerPath path = GetPathFromItem(child);
-		if (path.IsEmpty())
+		if (path.empty())
 			continue;
 
 		struct _parents dir;
@@ -1392,10 +1427,10 @@ void CRemoteTreeView::ApplyFilters()
 			for (wxTreeItemId child = GetFirstChild(parent.item, cookie); child; child = GetNextSibling(child))
 			{
 				CServerPath path = GetPathFromItem(child);
-				if (path.IsEmpty())
+				if (path.empty())
 					continue;
 
-				if (filter.FilenameFiltered(GetItemText(child), path.GetPath(), true, -1, false, 0, 0))
+				if (filter.FilenameFiltered(GetItemText(child), path.GetPath(), true, -1, false, 0, CDateTime()))
 				{
 					wxTreeItemId sel = GetSelection();
 					while (sel && sel != child)
@@ -1419,7 +1454,7 @@ void CRemoteTreeView::ApplyFilters()
 		for (wxTreeItemId child = GetFirstChild(parent.item, cookie); child; child = GetNextSibling(child))
 		{
 			CServerPath path = GetPathFromItem(child);
-			if (path.IsEmpty())
+			if (path.empty())
 				continue;
 
 			struct _parents dir;
@@ -1430,13 +1465,13 @@ void CRemoteTreeView::ApplyFilters()
 	}
 }
 
-void CRemoteTreeView::OnMenuGeturl(wxCommandEvent& event)
+void CRemoteTreeView::OnMenuGeturl(wxCommandEvent&)
 {
 	if (!m_contextMenuItem)
 		return;
 
 	const CServerPath& path = GetPathFromItem(m_contextMenuItem);
-	if (path.IsEmpty())
+	if (path.empty())
 	{
 		wxBell();
 		return;
@@ -1451,7 +1486,7 @@ void CRemoteTreeView::OnMenuGeturl(wxCommandEvent& event)
 
 	if (!wxTheClipboard->Open())
 	{
-		wxMessageBox(_("Could not open clipboard"), _("Could not copy URLs"), wxICON_EXCLAMATION);
+		wxMessageBoxEx(_("Could not open clipboard"), _("Could not copy URLs"), wxICON_EXCLAMATION);
 		return;
 	}
 
