@@ -1,6 +1,10 @@
 #ifndef __STATUSVIEW_H__
 #define __STATUSVIEW_H__
 
+#ifdef __WXMSW__
+#include "richedit.h"
+#endif
+
 class CFastTextCtrl;
 class CStatusView : public wxWindow
 {
@@ -9,7 +13,7 @@ public:
 	virtual ~CStatusView();
 
 	void AddToLog(CLogmsgNotification *pNotification);
-	void AddToLog(enum MessageType messagetype, const wxString& message, const wxDateTime& time);
+	void AddToLog(MessageType messagetype, const wxString& message, const wxDateTime& time);
 
 	void InitDefAttr();
 
@@ -23,12 +27,11 @@ protected:
 	wxString m_Content;
 	CFastTextCtrl *m_pTextCtrl;
 
-	void OnSize(wxSizeEvent &event);
-
 	DECLARE_EVENT_TABLE()
-	void OnContextMenu(wxContextMenuEvent& event);
-	void OnClear(wxCommandEvent& event);
-	void OnCopy(wxCommandEvent& event);
+	void OnSize(wxSizeEvent &);
+	void OnContextMenu(wxContextMenuEvent&);
+	void OnClear(wxCommandEvent& );
+	void OnCopy(wxCommandEvent& );
 
 	std::list<int> m_lineLengths;
 
@@ -37,7 +40,10 @@ protected:
 		wxString prefix;
 		int len;
 		wxTextAttr attr;
-	} m_attributeCache[MessageTypeCount];
+#ifdef __WXMSW__
+		CHARFORMAT2 cf;
+#endif
+	} m_attributeCache[static_cast<int>(MessageType::count)];
 
 	bool m_rtl;
 
@@ -56,10 +62,6 @@ protected:
 	bool m_showTimestamps;
 	wxDateTime m_lastTime;
 	wxString m_lastTimeString;
-
-#ifdef __WXMAC__
-	int m_insertionPoint;
-#endif
 };
 
 #endif
