@@ -190,7 +190,7 @@ bool CAsyncRequestQueue::ProcessNextRequest()
 					else
 					{
 						// For the notifications already in the request queue, we have to set the queue action directly
-						for (std::list<t_queueEntry>::iterator iter = ++m_requestList.begin(); iter != m_requestList.end(); ++iter)
+						for (auto iter = ++m_requestList.begin(); iter != m_requestList.end(); ++iter)
 						{
 							if (pNotification->GetRequestID() != reqId_fileexists)
 								continue;
@@ -201,16 +201,16 @@ bool CAsyncRequestQueue::ProcessNextRequest()
 								p->overwriteAction = CFileExistsNotification::OverwriteAction(action);
 						}
 
-						enum TransferDirection direction;
+						TransferDirection direction;
 						if (directionOnly)
 						{
 							if (pNotification->download)
-								direction = download;
+								direction = TransferDirection::download;
 							else
-								direction = upload;
+								direction = TransferDirection::upload;
 						}
 						else
-							direction = both;
+							direction = TransferDirection::both;
 
 						if (m_pQueueView)
 							m_pQueueView->SetDefaultFileExistsAction(action, direction);
@@ -243,14 +243,14 @@ bool CAsyncRequestQueue::ProcessNextRequest()
 				wxString defaultName;
 				if (pNotification->download)
 				{
-					msg.Printf(_("The file %s already exists.\nPlease enter a new name:"), pNotification->localFile.c_str());
+					msg.Printf(_("The file %s already exists.\nPlease enter a new name:"), pNotification->localFile);
 					wxFileName fn = pNotification->localFile;
 					defaultName = fn.GetFullName();
 				}
 				else
 				{
 					wxString fullName = pNotification->remotePath.GetPath() + pNotification->remoteFile;
-					msg.Printf(_("The file %s already exists.\nPlease enter a new name:"), fullName.c_str());
+					msg.Printf(_("The file %s already exists.\nPlease enter a new name:"), fullName);
 					defaultName = pNotification->remoteFile;
 				}
 				wxTextEntryDialog dlg(m_pMainFrame, msg, _("Rename file"), defaultName);
@@ -261,7 +261,7 @@ bool CAsyncRequestQueue::ProcessNextRequest()
 					int res = dlg.ShowModal();
 					if (res == wxID_OK)
 					{
-						if (dlg.GetValue() == _T(""))
+						if (dlg.GetValue().empty())
 							continue; // Disallow empty names
 						if (dlg.GetValue() == defaultName)
 						{
@@ -374,7 +374,7 @@ void CAsyncRequestQueue::ClearPending(const CFileZillaEngine *pEngine)
 
 	// Remove older requests coming from the same engine, but never the first
 	// entry in the list as that one displays a dialog at this moment.
-	for (std::list<t_queueEntry>::iterator iter = ++m_requestList.begin(); iter != m_requestList.end(); ++iter)
+	for (auto iter = ++m_requestList.begin(); iter != m_requestList.end(); ++iter)
 	{
 		if (iter->pEngine == pEngine)
 		{
@@ -439,7 +439,7 @@ bool CAsyncRequestQueue::CheckWindowState()
 {
 	m_timer.Stop();
 	wxMouseState mouseState = wxGetMouseState();
-	if (mouseState.LeftDown() || mouseState.MiddleDown() || mouseState.RightDown())
+	if (mouseState.LeftIsDown() || mouseState.MiddleIsDown() || mouseState.RightIsDown())
 	{
 		m_timer.Start(1000, true);
 		return false;
