@@ -16,8 +16,10 @@
 #include "view.h"
 #include "viewheader.h"
 
-DECLARE_EVENT_TYPE(fzEVT_TAB_CLOSING_DEFERRED, -1);
-DEFINE_EVENT_TYPE(fzEVT_TAB_CLOSING_DEFERRED);
+#include <wx/wupdlock.h>
+
+DECLARE_EVENT_TYPE(fzEVT_TAB_CLOSING_DEFERRED, -1)
+DEFINE_EVENT_TYPE(fzEVT_TAB_CLOSING_DEFERRED)
 
 BEGIN_EVENT_TABLE(CContextControl, wxSplitterWindow)
 EVT_MENU(XRCID("ID_TABCONTEXT_REFRESH"), CContextControl::OnTabRefresh)
@@ -51,7 +53,11 @@ void CContextControl::Create(wxWindow *parent)
 void CContextControl::CreateTab()
 {
 	wxGetApp().AddStartupProfileRecord(_T("CContextControl::CreateTab"));
-	Freeze();
+#ifndef __WXMAC__
+	// Some reparenting is being done when creating tabs. Reparenting of frozen windows isn't working
+	// on OS X.
+	wxWindowUpdateLocker lock(this);
+#endif
 
 	CState* pState = 0;
 
@@ -76,7 +82,7 @@ void CContextControl::CreateTab()
 		pState = CContextManager::Get()->CreateState(m_pMainFrame);
 		if (!pState->CreateEngine())
 		{
-			wxMessageBox(_("Failed to initialize FTP engine"));
+			wxMessageBoxEx(_("Failed to initialize FTP engine"));
 		}
 	}
 
@@ -101,8 +107,6 @@ void CContextControl::CreateTab()
 
 	if (m_tabs)
 		m_tabs->SetSelection(m_tabs->GetPageCount() - 1);
-
-	Thaw();
 }
 
 void CContextControl::CreateContextControls(CState* pState)
@@ -209,9 +213,9 @@ void CContextControl::CreateContextControls(CState* pState)
 			context_controls.pViewSplitter->SplitVertically(context_controls.pLocalSplitter, context_controls.pRemoteSplitter);
 	}
 
-	context_controls.pLocalViewHeader = new CLocalViewHeader(context_controls.pLocalSplitter, pState);
 	if (COptions::Get()->GetOptionVal(OPTION_SHOW_TREE_LOCAL))
 	{
+		context_controls.pLocalViewHeader = new CLocalViewHeader(context_controls.pLocalTreeViewPanel, pState);
 		context_controls.pLocalTreeViewPanel->SetHeader(context_controls.pLocalViewHeader);
 		if (layout == 3 && swap)
 			context_controls.pLocalSplitter->SplitVertically(context_controls.pLocalListViewPanel, context_controls.pLocalTreeViewPanel);
@@ -223,13 +227,14 @@ void CContextControl::CreateContextControls(CState* pState)
 	else
 	{
 		context_controls.pLocalTreeViewPanel->Hide();
+		context_controls.pLocalViewHeader = new CLocalViewHeader(context_controls.pLocalListViewPanel, pState);
 		context_controls.pLocalListViewPanel->SetHeader(context_controls.pLocalViewHeader);
 		context_controls.pLocalSplitter->Initialize(context_controls.pLocalListViewPanel);
 	}
 
-	context_controls.pRemoteViewHeader = new CRemoteViewHeader(context_controls.pRemoteSplitter, pState);
 	if (COptions::Get()->GetOptionVal(OPTION_SHOW_TREE_REMOTE))
 	{
+		context_controls.pRemoteViewHeader = new CRemoteViewHeader(context_controls.pRemoteTreeViewPanel, pState);
 		context_controls.pRemoteTreeViewPanel->SetHeader(context_controls.pRemoteViewHeader);
 		if (layout == 3 && !swap)
 			context_controls.pRemoteSplitter->SplitVertically(context_controls.pRemoteListViewPanel, context_controls.pRemoteTreeViewPanel);
@@ -241,6 +246,7 @@ void CContextControl::CreateContextControls(CState* pState)
 	else
 	{
 		context_controls.pRemoteTreeViewPanel->Hide();
+		context_controls.pRemoteViewHeader = new CRemoteViewHeader(context_controls.pRemoteListViewPanel, pState);
 		context_controls.pRemoteListViewPanel->SetHeader(context_controls.pRemoteViewHeader);
 		context_controls.pRemoteSplitter->Initialize(context_controls.pRemoteListViewPanel);
 	}
@@ -277,7 +283,7 @@ void CContextControl::CreateContextControls(CState* pState)
 	else
 	{
 		context_controls.tab_index = 0;
-		context_controls.site_bookmarks = new CContextControl::_context_controls::_site_bookmarks;
+		context_controls.site_bookmarks = std::make_shared<CContextControl::_context_controls::_site_bookmarks>();
 
 		context_controls.site_bookmarks->path = COptions::Get()->GetOption(OPTION_LAST_CONNECTED_SITE);
 		CSiteManager::GetBookmarks(context_controls.site_bookmarks->path,
@@ -289,7 +295,7 @@ void CContextControl::CreateContextControls(CState* pState)
 	m_context_controls.push_back(context_controls);
 }
 
-void CContextControl::OnTabRefresh(wxCommandEvent& event)
+void CContextControl::OnTabRefresh(wxCommandEvent&)
 {
 	if (m_right_clicked_tab == -1)
 		return;
@@ -345,16 +351,18 @@ bool CContextControl::CloseTab(int tab)
 
 	if (!pState->m_pCommandQueue->Idle())
 	{
-		if (wxMessageBox(_("Cannot close tab while busy.\nCancel current operation and close tab?"), _T("FileZilla"), wxYES_NO | wxICON_QUESTION) != wxYES)
+		if (wxMessageBoxEx(_("Cannot close tab while busy.\nCancel current operation and close tab?"), _T("FileZilla"), wxYES_NO | wxICON_QUESTION) != wxYES)
 			return false;
-
-		Freeze();
-
-		pState->m_pCommandQueue->Cancel();
-		pState->GetRecursiveOperationHandler()->StopRecursiveOperation();
 	}
-	else
-		Freeze();
+
+#ifndef __WXMAC__
+	// Some reparenting is being done when closing tabs. Reparenting of frozen windows isn't working
+	// on OS X.
+	wxWindowUpdateLocker lock(this);
+#endif
+
+	pState->m_pCommandQueue->Cancel();
+	pState->GetRecursiveOperationHandler()->StopRecursiveOperation();
 
 	pState->GetComparisonManager()->SetListings(0, 0);
 
@@ -384,7 +392,7 @@ bool CContextControl::CloseTab(int tab)
 		m_tabs = 0;
 
 		m_context_controls[i].tab_index = -1;
-		m_context_controls[i].site_bookmarks.clear();
+		m_context_controls[i].site_bookmarks.reset();
 
 		CContextManager::Get()->SetCurrentContext(m_context_controls[j].pState);
 
@@ -412,18 +420,16 @@ bool CContextControl::CloseTab(int tab)
 				m_context_controls[j].tab_index--;
 		}
 		m_context_controls[i].tab_index = -1;
-		m_context_controls[i].site_bookmarks.clear();
+		m_context_controls[i].site_bookmarks.reset();
 		m_tabs->DeletePage(tab);
 	}
 
 	pState->Disconnect();
 
-	Thaw();
-
 	return true;
 }
 
-void CContextControl::OnTabBgDoubleclick(wxAuiNotebookEvent& event)
+void CContextControl::OnTabBgDoubleclick(wxAuiNotebookEvent&)
 {
 	CreateTab();
 }
@@ -485,7 +491,7 @@ void CContextControl::OnTabClosing_Deferred(wxCommandEvent& event)
 }
 
 
-void CContextControl::OnTabChanged(wxAuiNotebookEvent& event)
+void CContextControl::OnTabChanged(wxAuiNotebookEvent&)
 {
 	int i = m_tabs->GetSelection();
 	if (i < 0 || i >= (int)m_context_controls.size())
@@ -561,7 +567,7 @@ void CContextControl::AdvanceTab(bool forward)
 	m_tabs->AdvanceTab(forward);
 }
 
-void CContextControl::OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void* data2)
+void CContextControl::OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString&, const void*)
 {
 	if (notification == STATECHANGE_CHANGEDCONTEXT)
 	{
@@ -591,7 +597,7 @@ void CContextControl::OnStateChange(CState* pState, enum t_statechange_notificat
 	}
 }
 
-void CContextControl::OnTabContextNew(wxCommandEvent& event)
+void CContextControl::OnTabContextNew(wxCommandEvent&)
 {
 	CreateTab();
 }
