@@ -8,8 +8,8 @@ class wxDialogEx : public wxDialog, public CWrapEngine
 public:
 	bool Load(wxWindow *pParent, const wxString& name);
 
-	bool SetLabel(int id, const wxString& label, unsigned long maxLength = 0);
-	wxString GetLabel(int id);
+	bool SetChildLabel(int id, const wxString& label, unsigned long maxLength = 0);
+	wxString GetChildLabel(int id);
 
 	virtual int ShowModal();
 
@@ -17,7 +17,9 @@ public:
 
 	static int ShownDialogs() { return m_shown_dialogs; }
 
+	static bool CanShowPopupDialog();
 protected:
+	virtual void InitDialog();
 
 	DECLARE_EVENT_TABLE()
 	virtual void OnChar(wxKeyEvent& event);
