@@ -2,15 +2,12 @@
 #include "edithandler.h"
 #include "dialogex.h"
 #include "filezillaapp.h"
+#include "file_utils.h"
 #include "queue.h"
 #include "Options.h"
 #include "window_state_manager.h"
 #include "local_filesys.h"
 
-// Defined in optionspage_edit.cpp
-bool UnquoteCommand(wxString& command, wxString& arguments, bool is_dde = false);
-bool ProgramExists(const wxString& editor);
-
 class CChangedFileDialog : public wxDialogEx
 {
 	DECLARE_EVENT_TABLE()
@@ -99,34 +96,40 @@ void CEditHandler::RemoveTemporaryFiles(const wxString& temp)
 				f.l_start = 0;
 				f.l_len = 1;
 				f.l_pid = getpid();
-				if (fcntl(fd, F_SETLK, &f))
-				{
+				if (fcntl(fd, F_SETLK, &f)) {
 					// In use by other process
+					close(fd);
 					continue;
 				}
 				close(fd);
 			}
 #endif
-			wxRemoveFile(lockfile);
+			{
+				wxLogNull log;
+				wxRemoveFile(lockfile);
+			}
 			if (wxFileName::FileExists(lockfile))
 				continue;
 		}
 
 		{
+			wxLogNull log;
 			wxString file2;
 			wxDir dir2(temp + file);
 			bool res;
-			for ((res = dir2.GetFirst(&file2, _T(""), wxDIR_FILES)); res; res = dir2.GetNext(&file2));
+			for ((res = dir2.GetFirst(&file2, _T(""), wxDIR_FILES)); res; res = dir2.GetNext(&file2)) {
 				wxRemoveFile(temp + file + sep + file2);
+			}
 		}
 
+		wxLogNull log;
 		wxRmdir(temp + file + sep);
 	} while (dir.GetNext(&file));
 }
 
 wxString CEditHandler::GetLocalDirectory()
 {
-	if (m_localDir != _T(""))
+	if (!m_localDir.empty())
 		return m_localDir;
 
 	wxFileName tmpdir(wxFileName::GetTempDir(), _T(""));
@@ -134,8 +137,8 @@ wxString CEditHandler::GetLocalDirectory()
 	// which will cause problems when calculating maximum allowed file
 	// length
 	wxString dir = tmpdir.GetLongPath();
-	if (dir == _T("") || !wxFileName::DirExists(dir))
-		return _T("");
+	if (dir.empty() || !wxFileName::DirExists(dir))
+		return wxString();
 
 	if (dir.Last() != wxFileName::GetPathSeparator())
 		dir += wxFileName::GetPathSeparator();
@@ -154,7 +157,7 @@ wxString CEditHandler::GetLocalDirectory()
 			continue;
 
 		if (!wxMkdir(newDir, 0700))
-			return _T("");
+			return wxString();
 
 		m_localDir = newDir + wxFileName::GetPathSeparator();
 		break;
@@ -193,7 +196,7 @@ void CEditHandler::Release()
 	if (m_busyTimer.IsRunning())
 		m_busyTimer.Stop();
 
-	if (m_localDir != _T(""))
+	if (!m_localDir.empty())
 	{
 #ifdef __WXMSW__
 		if (m_lockfile_handle != INVALID_HANDLE_VALUE)
@@ -205,7 +208,10 @@ void CEditHandler::Release()
 			close(m_lockfile_descriptor);
 #endif
 
-		wxRemoveFile(m_localDir + _T("empty_file_yq744zm"));
+		{
+			wxLogNull log;
+			wxRemoveFile(m_localDir + _T("empty_file_yq744zm"));
+		}
 
 		RemoveAll(true);
 		wxRmdir(m_localDir);
@@ -378,7 +384,7 @@ bool CEditHandler::RemoveAll(bool force)
 	m_fileDataList[remote].swap(keep);
 	keep.clear();
 
-	for (std::list<t_fileData>::iterator iter = m_fileDataList[local].begin(); iter != m_fileDataList[local].end(); ++iter)
+	for (auto iter = m_fileDataList[local].begin(); iter != m_fileDataList[local].end(); ++iter)
 	{
 		if (force)
 			continue;
@@ -403,7 +409,7 @@ bool CEditHandler::RemoveAll(enum fileState state, const CServer* pServer /*=0*/
 
 	std::list<t_fileData> keep;
 
-	for (std::list<t_fileData>::iterator iter = m_fileDataList[remote].begin(); iter != m_fileDataList[remote].end(); ++iter)
+	for (auto iter = m_fileDataList[remote].begin(); iter != m_fileDataList[remote].end(); ++iter)
 	{
 		if (iter->state != state)
 		{
@@ -498,7 +504,7 @@ std::list<CEditHandler::t_fileData>::const_iterator CEditHandler::GetFile(const
 
 void CEditHandler::FinishTransfer(bool successful, const wxString& fileName)
 {
-	std::list<t_fileData>::iterator iter = GetFile(fileName);
+	auto iter = GetFile(fileName);
 	if (iter == m_fileDataList[local].end())
 		return;
 
@@ -524,7 +530,7 @@ void CEditHandler::FinishTransfer(bool successful, const wxString& fileName)
 
 void CEditHandler::FinishTransfer(bool successful, const wxString& fileName, const CServerPath& remotePath, const CServer& server)
 {
-	std::list<t_fileData>::iterator iter = GetFile(fileName, remotePath, server);
+	auto iter = GetFile(fileName, remotePath, server);
 	if (iter == m_fileDataList[remote].end())
 		return;
 
@@ -575,7 +581,7 @@ void CEditHandler::FinishTransfer(bool successful, const wxString& fileName, con
 
 bool CEditHandler::StartEditing(const wxString& file)
 {
-	std::list<t_fileData>::iterator iter = GetFile(file);
+	auto iter = GetFile(file);
 	if (iter == m_fileDataList[local].end())
 		return false;
 
@@ -584,7 +590,7 @@ bool CEditHandler::StartEditing(const wxString& file)
 
 bool CEditHandler::StartEditing(const wxString& file, const CServerPath& remotePath, const CServer& server)
 {
-	std::list<t_fileData>::iterator iter = GetFile(file, remotePath, server);
+	auto iter = GetFile(file, remotePath, server);
 	if (iter == m_fileDataList[remote].end())
 		return false;
 
@@ -596,14 +602,12 @@ bool CEditHandler::StartEditing(enum CEditHandler::fileType type, t_fileData& da
 	wxASSERT(type != none);
 	wxASSERT(data.state == edit);
 
-	wxFileName fn(data.file);
-
-	if (!fn.FileExists())
+	bool is_link;
+	if (CLocalFileSystem::GetFileInfo(data.file, is_link, 0, &data.modificationTime, 0) != CLocalFileSystem::file)
 		return false;
-	data.modificationTime = fn.GetModificationTime();
 
 	bool program_exists = false;
-	wxString cmd = GetOpenCommand(fn.GetFullPath(), program_exists);
+	wxString cmd = GetOpenCommand(data.file, program_exists);
 	if (cmd.empty() || !program_exists)
 		return false;
 
@@ -631,44 +635,34 @@ void CEditHandler::CheckForModifications(bool emitEvent)
 	for (int i = 0; i < 2; i++)
 	{
 checkmodifications_loopbegin:
-		for (std::list<t_fileData>::iterator iter = m_fileDataList[i].begin(); iter != m_fileDataList[i].end(); ++iter)
+		for (auto iter = m_fileDataList[i].begin(); iter != m_fileDataList[i].end(); ++iter)
 		{
 			if (iter->state != edit)
 				continue;
 
-			wxFileName fn(iter->file);
-			if (!fn.FileExists())
-			{
+			CDateTime mtime;
+			bool is_link;
+			if (CLocalFileSystem::GetFileInfo(iter->file, is_link, 0, &mtime, 0) != CLocalFileSystem::file) {
 				m_fileDataList[i].erase(iter);
 
 				// Evil goto. Imo the next C++ standard needs a comefrom keyword.
 				goto checkmodifications_loopbegin;
 			}
 
-			wxDateTime mtime;
-
-			{
-				wxLogNull log; // If GetModificationTime fails wx spams error messages
-				mtime = fn.GetModificationTime();
-			}
-
 			if (!mtime.IsValid())
 				continue;
 
-			if (iter->modificationTime.IsValid() && iter->modificationTime == mtime)
+			if (iter->modificationTime.IsValid() && !iter->modificationTime.Compare(mtime))
 				continue;
 
 			// File has changed, ask user what to do
 
 			m_busyTimer.Stop();
-			wxMouseState mouseState = wxGetMouseState();
-			if (mouseState.LeftDown() || mouseState.MiddleDown() || mouseState.RightDown())
-			{
+			if( !wxDialogEx::CanShowPopupDialog() ) {
 				m_busyTimer.Start(1000, true);
 				insideCheckForModifications = false;
 				return;
 			}
-
 			wxTopLevelWindow* pTopWindow = (wxTopLevelWindow*)wxTheApp->GetTopWindow();
 			if (pTopWindow && pTopWindow->IsIconized())
 			{
@@ -691,7 +685,7 @@ checkmodifications_loopbegin:
 			{
 				if (i == remote)
 				{
-					if (!fn.FileExists() || wxRemoveFile(fn.GetFullPath()))
+					if (CLocalFileSystem::GetFileInfo(iter->file, is_link, 0, &mtime, 0) != CLocalFileSystem::file || wxRemoveFile(iter->file))
 					{
 						m_fileDataList[i].erase(iter);
 						goto checkmodifications_loopbegin;
@@ -704,7 +698,7 @@ checkmodifications_loopbegin:
 					goto checkmodifications_loopbegin;
 				}
 			}
-			else if (!fn.FileExists())
+			else if (CLocalFileSystem::GetFileInfo(iter->file, is_link, 0, &mtime, 0) != CLocalFileSystem::file)
 			{
 				m_fileDataList[i].erase(iter);
 				goto checkmodifications_loopbegin;
@@ -729,7 +723,7 @@ int CEditHandler::DisplayChangeNotification(CEditHandler::fileType type, std::li
 	else
 		XRCCTRL(dlg, "ID_DESC_UPLOAD_REMOTE", wxStaticText)->Hide();
 
-	dlg.SetLabel(XRCID("ID_FILENAME"), iter->name);
+	dlg.SetChildLabel(XRCID("ID_FILENAME"), iter->name);
 
 	if (type == local)
 	{
@@ -749,10 +743,10 @@ int CEditHandler::DisplayChangeNotification(CEditHandler::fileType type, std::li
 			XRCCTRL(dlg, "ID_OPENEDAS", wxStaticText)->Hide();
 		}
 		else
-			dlg.SetLabel(XRCID("ID_OPENEDAS"), file);
+			dlg.SetChildLabel(XRCID("ID_OPENEDAS"), file);
 	}
-	dlg.SetLabel(XRCID("ID_SERVER"), iter->server.FormatServer());
-	dlg.SetLabel(XRCID("ID_REMOTEPATH"), iter->remotePath.GetPath());
+	dlg.SetChildLabel(XRCID("ID_SERVER"), iter->server.FormatServer());
+	dlg.SetChildLabel(XRCID("ID_REMOTEPATH"), iter->remotePath.GetPath());
 
 	dlg.GetSizer()->Fit(&dlg);
 
@@ -777,8 +771,8 @@ bool CEditHandler::UploadFile(const wxString& file, bool unedit)
 
 bool CEditHandler::UploadFile(enum fileType type, std::list<t_fileData>::iterator iter, bool unedit)
 {
-	wxASSERT(type != none);
-
+	wxCHECK(type != none, false);
+		
 	if (iter == m_fileDataList[type].end())
 		return false;
 
@@ -789,7 +783,7 @@ bool CEditHandler::UploadFile(enum fileType type, std::list<t_fileData>::iterato
 	iter->state = unedit ? upload_and_remove : upload;
 
 	wxLongLong size;
-	wxDateTime mtime;
+	CDateTime mtime;
 
 	bool is_link;
 	if (CLocalFileSystem::GetFileInfo(iter->file, is_link, &size, &mtime, 0) != CLocalFileSystem::file)
@@ -799,7 +793,7 @@ bool CEditHandler::UploadFile(enum fileType type, std::list<t_fileData>::iterato
 	}
 
 	if (!mtime.IsValid())
-		mtime = wxDateTime::Now();
+		mtime = CDateTime::Now();
 
 	iter->modificationTime = mtime;
 
@@ -813,7 +807,7 @@ bool CEditHandler::UploadFile(enum fileType type, std::list<t_fileData>::iterato
 		return false;
 	}
 
-	m_pQueue->QueueFile(false, false, file, iter->name, localPath, iter->remotePath, iter->server, wxLongLong(size.GetHi(), size.GetLo()), type, priority_high);
+	m_pQueue->QueueFile(false, false, file, iter->name, localPath, iter->remotePath, iter->server, wxLongLong(size.GetHi(), size.GetLo()), type, QueuePriority::high);
 	m_pQueue->QueueFile_Finish(true);
 
 	return true;
@@ -874,20 +868,20 @@ wxString CEditHandler::GetOpenCommand(const wxString& file, bool& program_exists
 	if (!COptions::Get()->GetOptionVal(OPTION_EDIT_ALWAYSDEFAULT))
 	{
 		const wxString command = GetCustomOpenCommand(file, program_exists);
-		if (command != _T(""))
+		if (!command.empty())
 			return command;
 
 		if (COptions::Get()->GetOptionVal(OPTION_EDIT_INHERITASSOCIATIONS))
 		{
 			const wxString command = GetSystemOpenCommand(file, program_exists);
-			if (command != _T(""))
+			if (!command.empty())
 				return command;
 		}
 	}
 
 	wxString command = COptions::Get()->GetOption(OPTION_EDIT_DEFAULTEDITOR);
-	if (command[0] == '0')
-		return _T(""); // None set
+	if (command.empty() || command[0] == '0')
+		return wxString(); // None set
 	else if (command[0] == '1')
 	{
 		// Text editor
@@ -903,12 +897,12 @@ wxString CEditHandler::GetOpenCommand(const wxString& file, bool& program_exists
 		command = command.Mid(1);
 
 	if (command.empty())
-		return _T("");
+		return wxString();
 
 	wxString args;
 	wxString editor = command;
 	if (!UnquoteCommand(editor, args))
-		return _T("");
+		return wxString();
 
 	if (!ProgramExists(editor))
 	{
@@ -920,126 +914,12 @@ wxString CEditHandler::GetOpenCommand(const wxString& file, bool& program_exists
 	return command + _T(" \"") + file + _T("\"");
 }
 
-
-static bool PathExpand(wxString& cmd)
-{
-#ifndef __WXMSW__
-	if (cmd[0] == '/')
-		return true;
-#else
-	if (cmd[0] == '\\')
-		// UNC or root of current working dir, whatever that is
-		return true;
-	if (cmd.Len() > 2 && cmd[1] == ':')
-		// Absolute path
-		return true;
-#endif
-
-	// Need to search for program in $PATH
-	wxString path;
-	if (!wxGetEnv(_T("PATH"), &path))
-		return false;
-
-	wxString full_cmd;
-	bool found = wxFindFileInPath(&full_cmd, path, cmd);
-#ifdef __WXMSW__
-	if (!found && cmd.Right(4).Lower() != _T(".exe"))
-	{
-		cmd += _T(".exe");
-		found = wxFindFileInPath(&full_cmd, path, cmd);
-	}
-#endif
-
-	if (!found)
-		return false;
-
-	cmd = full_cmd;
-	return true;
-}
-
-wxString CEditHandler::GetSystemOpenCommand(wxString file, bool &program_exists)
-{
-	wxFileName fn(file);
-
-	const wxString& ext = fn.GetExt();
-	if (ext == _T(""))
-		return _T("");
-
-	for (;;)
-	{
-		wxFileType* pType = wxTheMimeTypesManager->GetFileTypeFromExtension(ext);
-		if (!pType)
-			return _T("");
-
-		wxString cmd;
-		if (!pType->GetOpenCommand(&cmd, wxFileType::MessageParameters(file)))
-		{
-			delete pType;
-			return _T("");
-		}
-		delete pType;
-
-		if (cmd.empty())
-			return wxEmptyString;
-
-		program_exists = false;
-
-		wxString editor;
-		bool is_dde = false;
-#ifdef __WXMSW__
-		if (cmd.Left(7) == _T("WX_DDE#"))
-		{
-			// See wxWidget's wxExecute in src/msw/utilsexc.cpp
-			// WX_DDE#<command>#DDE_SERVER#DDE_TOPIC#DDE_COMMAND
-			editor = cmd.Mid(7);
-			int pos = editor.Find('#');
-			if (pos < 1)
-				return cmd;
-			editor = editor.Left(pos);
-			is_dde = true;
-		}
-		else
-#endif
-		{
-			editor = cmd;
-		}
-
-		wxString args;
-		if (!UnquoteCommand(editor, args, is_dde) || editor.empty())
-			return cmd;
-
-		if (!PathExpand(editor))
-			return cmd;
-
-		if (ProgramExists(editor))
-			program_exists = true;
-
-#ifdef __WXGTK__
-		int pos = args.Find(file);
-		if (pos != -1 && file.Find(' ') != -1 && file[0] != '\'' && file[0] != '"')
-		{
-			// Might need to quote filename, wxWidgets doesn't do it
-			if ((!pos || (args[pos - 1] != '\'' && args[pos - 1] != '"')) &&
-				args[pos + file.Length()] != '\'' && args[pos + file.Length()] != '"')
-			{
-				// Filename in command arguments isn't quoted. Repeat with quoted filename
-				file = _T("\"") + file + _T("\"");
-				continue;
-			}
-		}
-#endif
-		return cmd;
-	}
-
-	return wxEmptyString;
-}
-
 wxString CEditHandler::GetCustomOpenCommand(const wxString& file, bool& program_exists)
 {
 	wxFileName fn(file);
 
 	wxString ext = fn.GetExt();
-	if (ext == _T(""))
+	if (ext.empty())
 	{
 		if (fn.GetFullName()[0] == '.')
 			ext = _T(".");
@@ -1055,7 +935,7 @@ wxString CEditHandler::GetCustomOpenCommand(const wxString& file, bool& program_
 		wxString assoc = associations.Left(pos);
 		associations = associations.Mid(pos + 1);
 
-		if (assoc == _T(""))
+		if (assoc.empty())
 			continue;
 
 		wxString command;
@@ -1069,10 +949,10 @@ wxString CEditHandler::GetCustomOpenCommand(const wxString& file, bool& program_
 
 		wxString args;
 		if (!UnquoteCommand(prog, args))
-			return _T("");
+			return wxString();
 
-		if (prog == _T(""))
-			return _T("");
+		if (prog.empty())
+			return wxString();
 
 		if (!ProgramExists(prog))
 		{
@@ -1084,7 +964,7 @@ wxString CEditHandler::GetCustomOpenCommand(const wxString& file, bool& program_
 		return command + _T(" \"") + fn.GetFullPath() + _T("\"");
 	}
 
-	return _T("");
+	return wxString();
 }
 
 void CEditHandler::OnChangedFileEvent(wxCommandEvent& event)
@@ -1104,8 +984,8 @@ wxString CEditHandler::GetTemporaryFile(wxString name)
 	if (max != -1)
 	{
 		name = TruncateFilename(m_localDir, name, max);
-		if (name == _T(""))
-			return _T("");
+		if (name.empty())
+			return wxString();
 	}
 
 	wxString file = m_localDir + name;
@@ -1124,8 +1004,8 @@ wxString CEditHandler::GetTemporaryFile(wxString name)
 			cutoff *= 10;
 			max--;
 			name = TruncateFilename(m_localDir, name, max);
-			if (name == _T(""))
-				return _T("");
+			if (name.empty())
+				return wxString();
 		}
 
 		int pos = name.Find('.', true);
@@ -1138,7 +1018,7 @@ wxString CEditHandler::GetTemporaryFile(wxString name)
 			return file;
 	}
 
-	return _T("");
+	return wxString();
 }
 
 wxString CEditHandler::TruncateFilename(const wxString path, const wxString& name, int max)
@@ -1154,7 +1034,7 @@ wxString CEditHandler::TruncateFilename(const wxString path, const wxString& nam
 			if (pathlen + extlen >= max)
 			{
 				// Cannot truncate extension
-				return _T("");
+				return wxString();
 			}
 
 			return name.Left(max - pathlen - extlen) + name.Mid(pos);
@@ -1224,7 +1104,7 @@ int CEditHandlerStatusDialog::ShowModal()
 
 	if (!pEditHandler->GetFileCount(CEditHandler::none, CEditHandler::unknown))
 	{
-		wxMessageBox(_("No files are currently being edited."), _("Cannot show dialog"), wxICON_INFORMATION, m_pParent);
+		wxMessageBoxEx(_("No files are currently being edited."), _("Cannot show dialog"), wxICON_INFORMATION, m_pParent);
 		return wxID_CANCEL;
 	}
 
@@ -1558,7 +1438,7 @@ CNewAssociationDialog::CNewAssociationDialog(wxWindow *parent)
 {
 }
 
-bool CNewAssociationDialog::Show(const wxString &file)
+bool CNewAssociationDialog::Run(const wxString &file)
 {
 	if (!Load(m_pParent, _T("ID_EDIT_NOPROGRAM")))
 		return true;
@@ -1572,10 +1452,10 @@ bool CNewAssociationDialog::Show(const wxString &file)
 		m_ext.clear();
 
 	wxStaticText *pDesc = XRCCTRL(*this, "ID_DESC", wxStaticText);
-	pDesc->SetLabel(wxString::Format(pDesc->GetLabel(), m_ext.c_str()));
+	pDesc->SetLabel(wxString::Format(pDesc->GetLabel(), m_ext));
 
 	bool program_exists = false;
-	wxString cmd = CEditHandler::Get()->GetSystemOpenCommand(_T("foo.txt"), program_exists);
+	wxString cmd = GetSystemOpenCommand(_T("foo.txt"), program_exists);
 	if (!program_exists)
 		cmd.clear();
 	if (!cmd.empty())
@@ -1598,7 +1478,7 @@ bool CNewAssociationDialog::Show(const wxString &file)
 	else
 	{
 		XRCCTRL(*this, "ID_EDITOR_DESC_NONE", wxStaticText)->Hide();
-		pDesc->SetLabel(wxString::Format(pDesc->GetLabel(), cmd.c_str()));
+		pDesc->SetLabel(wxString::Format(pDesc->GetLabel(), cmd));
 	}
 
 	SetCtrlState();
@@ -1643,12 +1523,12 @@ void CNewAssociationDialog::OnOK(wxCommandEvent& event)
 		wxString args;
 		if (!UnquoteCommand(editor, args) || editor.empty())
 		{
-			wxMessageBox(_("You need to enter a properly quoted command."), _("Cannot set file association"), wxICON_EXCLAMATION);
+			wxMessageBoxEx(_("You need to enter a properly quoted command."), _("Cannot set file association"), wxICON_EXCLAMATION);
 			return;
 		}
 		if (!ProgramExists(editor))
 		{
-			wxMessageBox(_("Selected editor does not exist."), _("Cannot set file association"), wxICON_EXCLAMATION, this);
+			wxMessageBoxEx(_("Selected editor does not exist."), _("Cannot set file association"), wxICON_EXCLAMATION, this);
 			return;
 		}
 
@@ -1672,7 +1552,7 @@ void CNewAssociationDialog::OnOK(wxCommandEvent& event)
 		else
 		{
 			bool program_exists = false;
-			wxString cmd = CEditHandler::Get()->GetSystemOpenCommand(_T("foo.txt"), program_exists);
+			wxString cmd = GetSystemOpenCommand(_T("foo.txt"), program_exists);
 			if (!program_exists)
 				cmd.clear();
 			if (!cmd.empty())
@@ -1687,7 +1567,7 @@ void CNewAssociationDialog::OnOK(wxCommandEvent& event)
 #endif
 				)
 			{
-				wxMessageBox(_("The default editor for text files could not be found."), _("Cannot set file association"), wxICON_EXCLAMATION, this);
+				wxMessageBoxEx(_("The default editor for text files could not be found."), _("Cannot set file association"), wxICON_EXCLAMATION, this);
 				return;
 			}
 			if (cmd.Find(' ') != -1)
@@ -1721,13 +1601,13 @@ void CNewAssociationDialog::OnBrowseEditor(wxCommandEvent& event)
 		return;
 
 	wxString editor = dlg.GetPath();
-	if (editor == _T(""))
+	if (editor.empty())
 		return;
 
 	if (!ProgramExists(editor))
 	{
 		XRCCTRL(*this, "ID_EDITOR", wxWindow)->SetFocus();
-		wxMessageBox(_("Selected editor does not exist."), _("File not found"), wxICON_EXCLAMATION, this);
+		wxMessageBoxEx(_("Selected editor does not exist."), _("File not found"), wxICON_EXCLAMATION, this);
 		return;
 	}
 
