@@ -8,7 +8,7 @@
 #include "conditionaldialog.h"
 #include <algorithm>
 #include "filelist_statusbar.h"
-#ifdef __WXGTK__
+#if defined(__WXGTK__) && !defined(__WXGTK3__)
 #include <gtk/gtk.h>
 #endif
 
@@ -116,7 +116,7 @@ template<class CFileData> bool CFileListCtrl<CFileData>::MSWOnNotify(int idCtrl,
 
 				const int index = m_indexMapping[item];
 				const CFileData& data = m_fileData[index];
-				if (data.flags == fill)
+				if (data.comparison_flags == fill)
 					continue;
 
 				if (ItemIsDir(index))
@@ -137,7 +137,7 @@ template<class CFileData> bool CFileListCtrl<CFileData>::MSWOnNotify(int idCtrl,
 
 				const int index = m_indexMapping[i];
 				const CFileData& data = m_fileData[index];
-				if (data.flags == fill)
+				if (data.comparison_flags == fill)
 					continue;
 
 				if (ItemIsDir(index))
@@ -201,7 +201,7 @@ template<class CFileData> bool CFileListCtrl<CFileData>::MSWOnNotify(int idCtrl,
 }
 #endif
 
-#ifdef __WXGTK__
+#if defined(__WXGTK__) && !defined(__WXGTK3__)
 // Need to call a member function of a C++ template class
 // from a C function.
 // Sadly template functions with C linkage aren't possible,
@@ -242,7 +242,7 @@ static gboolean gtk_button_release_event(GtkWidget*, void *gdk_event, CGtkEventC
 }
 #endif
 
-template<class CFileData> CFileListCtrl<CFileData>::CFileListCtrl(wxWindow* pParent, CState* pState, CQueueView* pQueue, bool border /*=false*/)
+template<class CFileData> CFileListCtrl<CFileData>::CFileListCtrl(wxWindow* pParent, CState*, CQueueView* pQueue, bool border /*=false*/)
 : wxListCtrlEx(pParent, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxLC_VIRTUAL | wxLC_REPORT | wxLC_EDIT_LABELS | (border ? wxBORDER_SUNKEN : wxNO_BORDER)),
 	CComparableListing(this)
 {
@@ -272,11 +272,11 @@ template<class CFileData> CFileListCtrl<CFileData>::CFileListCtrl(wxWindow* pPar
 	m_focusItem = -1;
 #endif
 
-#ifdef __WXGTK__
-	m_gtkEventCallbackProxy = new CGtkEventCallbackProxy<CFileData>(this);
+#if defined __WXGTK__ && !defined(__WXGTK3__)
+	m_gtkEventCallbackProxy.reset(new CGtkEventCallbackProxy<CFileData>(this));
 
 	GtkWidget* widget = GetMainWindow()->GetConnectWidget();
-	g_signal_connect(widget, "button_release_event", G_CALLBACK(gtk_button_release_event), m_gtkEventCallbackProxy.Value());
+	g_signal_connect(widget, "button_release_event", G_CALLBACK(gtk_button_release_event), m_gtkEventCallbackProxy.get());
 #endif
 
 	m_genericTypes[genericTypes::file] = _("File");
@@ -408,31 +408,53 @@ template<class CFileData> void CFileListCtrl<CFileData>::SortList_UpdateSelectio
 	}
 }
 
-template<class CFileData> CListViewSort::DirSortMode CFileListCtrl<CFileData>::GetDirSortMode()
+template<class CFileData> CFileListCtrlSortBase::DirSortMode CFileListCtrl<CFileData>::GetDirSortMode()
 {
 	const int dirSortOption = COptions::Get()->GetOptionVal(OPTION_FILELIST_DIRSORT);
 
-	enum CListViewSort::DirSortMode dirSortMode;
+	enum CFileListCtrlSortBase::DirSortMode dirSortMode;
 	switch (dirSortOption)
 	{
 	case 0:
 	default:
-		dirSortMode = CListViewSort::dirsort_ontop;
+		dirSortMode = CFileListCtrlSortBase::dirsort_ontop;
 		break;
 	case 1:
 		if (m_sortDirection)
-			dirSortMode = CListViewSort::dirsort_onbottom;
+			dirSortMode = CFileListCtrlSortBase::dirsort_onbottom;
 		else
-			dirSortMode = CListViewSort::dirsort_ontop;
+			dirSortMode = CFileListCtrlSortBase::dirsort_ontop;
 		break;
 	case 2:
-		dirSortMode = CListViewSort::dirsort_inline;
+		dirSortMode = CFileListCtrlSortBase::dirsort_inline;
 		break;
 	}
 
 	return dirSortMode;
 }
 
+template<class CFileData> CFileListCtrlSortBase::NameSortMode CFileListCtrl<CFileData>::GetNameSortMode()
+{
+	const int nameSortOption = COptions::Get()->GetOptionVal(OPTION_FILELIST_NAMESORT);
+
+	enum CFileListCtrlSortBase::NameSortMode nameSortMode;
+	switch (nameSortOption)
+	{
+	case 0:
+	default:
+		nameSortMode = CFileListCtrlSortBase::namesort_caseinsensitive;
+		break;
+	case 1:
+		nameSortMode = CFileListCtrlSortBase::namesort_casesensitive;
+		break;
+	case 2:
+		nameSortMode = CFileListCtrlSortBase::namesort_natural;
+		break;
+	}
+
+	return nameSortMode;
+}
+
 template<class CFileData> void CFileListCtrl<CFileData>::OnColumnClicked(wxListEvent &event)
 {
 	int col = m_pVisibleColumnMapping[event.GetColumn()];
@@ -464,7 +486,9 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnColumnClicked(wxListE
 	RefreshListOnly(false);
 }
 
-static wxString GetExt(const wxString& file)
+#ifdef __WXMSW__
+namespace {
+wxString GetExt(const wxString& file)
 {
 	wxString ret;
 
@@ -474,6 +498,8 @@ static wxString GetExt(const wxString& file)
 
 	return ret;
 }
+}
+#endif
 
 template<class CFileData> wxString CFileListCtrl<CFileData>::GetType(wxString name, bool dir, const wxString& path /*=_T("")*/)
 {
@@ -486,7 +512,7 @@ template<class CFileData> wxString CFileListCtrl<CFileData>::GetType(wxString na
 
 	wxString type;
 	int flags = SHGFI_TYPENAME;
-	if (path.IsEmpty())
+	if (path.empty())
 		flags |= SHGFI_USEFILEATTRIBUTES;
 	else if (path == _T("\\"))
 		name += _T("\\");
@@ -503,7 +529,7 @@ template<class CFileData> wxString CFileListCtrl<CFileData>::GetType(wxString na
 	{
 		if (!*shFinfo.szTypeName)
 		{
-			if (!ext.IsEmpty())
+			if (!ext.empty())
 			{
 				type = ext;
 				type.MakeUpper();
@@ -516,13 +542,13 @@ template<class CFileData> wxString CFileListCtrl<CFileData>::GetType(wxString na
 		else
 		{
 			type = shFinfo.szTypeName;
-			if (!dir && !ext.IsEmpty())
+			if (!dir && !ext.empty())
 				m_fileTypeMap[ext.MakeLower()] = type;
 		}
 	}
 	else
 	{
-		if (!ext.IsEmpty())
+		if (!ext.empty())
 		{
 			type = ext;
 			type.MakeUpper();
@@ -534,6 +560,8 @@ template<class CFileData> wxString CFileListCtrl<CFileData>::GetType(wxString na
 	}
 	return type;
 #else
+	(void)path;
+
 	if (dir)
 		return m_genericTypes[genericTypes::directory];
 
@@ -558,7 +586,7 @@ template<class CFileData> wxString CFileListCtrl<CFileData>::GetType(wxString na
 	}
 
 	wxString desc;
-	if (pType->GetDescription(&desc) && desc != _T(""))
+	if (pType->GetDescription(&desc) && !desc.empty())
 	{
 		delete pType;
 		m_fileTypeMap[ext] = desc;
@@ -600,7 +628,7 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnExitComparisonMode()
 	m_indexMapping.swap(m_originalIndexMapping);
 
 	for (unsigned int i = 0; i < m_fileData.size() - 1; i++)
-		m_fileData[i].flags = normal;
+		m_fileData[i].comparison_flags = normal;
 
 	SetItemCount(m_indexMapping.size());
 
@@ -618,7 +646,7 @@ template<class CFileData> void CFileListCtrl<CFileData>::CompareAddFile(t_fileEn
 	}
 
 	int index = m_originalIndexMapping[m_comparisonIndex];
-	m_fileData[index].flags = flags;
+	m_fileData[index].comparison_flags = flags;
 
 	m_indexMapping.push_back(index);
 }
@@ -635,7 +663,7 @@ template<class CFileData> void CFileListCtrl<CFileData>::ComparisonRememberSelec
 	{
 		SetItemState(focus, 0, wxLIST_STATE_FOCUSED);
 		int index = m_indexMapping[focus];
-		if (m_fileData[index].flags == fill)
+		if (m_fileData[index].comparison_flags == fill)
 			focus = -1;
 		else
 			focus = index;
@@ -651,7 +679,7 @@ template<class CFileData> void CFileListCtrl<CFileData>::ComparisonRememberSelec
 		while ((item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1)
 		{
 			int index = m_indexMapping[item];
-			if (m_fileData[index].flags == fill)
+			if (m_fileData[index].comparison_flags == fill)
 				continue;
 			m_comparisonSelections.push_back(index);
 		}
@@ -702,7 +730,7 @@ template<class CFileData> void CFileListCtrl<CFileData>::ComparisonRestoreSelect
 	}
 }
 
-template<class CFileData> void CFileListCtrl<CFileData>::OnColumnRightClicked(wxListEvent& event)
+template<class CFileData> void CFileListCtrl<CFileData>::OnColumnRightClicked(wxListEvent&)
 {
 	ShowColumnEditor();
 }
@@ -710,7 +738,12 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnColumnRightClicked(wx
 template<class CFileData> void CFileListCtrl<CFileData>::InitSort(int optionID)
 {
 	wxString sortInfo = COptions::Get()->GetOption(optionID);
-	m_sortDirection = sortInfo[0] - '0';
+	if( !sortInfo.empty() ) {
+		m_sortDirection = sortInfo[0] - '0';
+	}
+	else {
+		m_sortDirection = 0;
+	}
 	if (m_sortDirection < 0 || m_sortDirection > 1)
 		m_sortDirection = 0;
 
@@ -753,7 +786,7 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnItemSelected(wxListEv
 
 	const int index = m_indexMapping[item];
 	const CFileData& data = m_fileData[index];
-	if (data.flags == fill)
+	if (data.comparison_flags == fill)
 		return;
 
 	if (ItemIsDir(index))
@@ -789,7 +822,7 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnItemDeselected(wxList
 
 	const int index = m_indexMapping[item];
 	const CFileData& data = m_fileData[index];
-	if (data.flags == fill)
+	if (data.comparison_flags == fill)
 		return;
 
 	if (ItemIsDir(index))
@@ -890,7 +923,7 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnProcessFocusChange(wx
 
 		const int index = m_indexMapping[i];
 		const CFileData& data = m_fileData[index];
-		if (data.flags == fill)
+		if (data.comparison_flags == fill)
 			continue;
 
 		if (selected)
@@ -920,7 +953,7 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnLeftDown(wxMouseEvent
 	AddPendingEvent(evt);
 }
 
-template<class CFileData> void CFileListCtrl<CFileData>::OnProcessMouseEvent(wxCommandEvent& event)
+template<class CFileData> void CFileListCtrl<CFileData>::OnProcessMouseEvent(wxCommandEvent&)
 {
 	if (m_pending_focus_processing)
 		return;
@@ -977,7 +1010,7 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnKeyDown(wxKeyEvent& e
 		for (unsigned int i = m_hasParent ? 1 : 0; i < m_indexMapping.size(); i++)
 		{
 			const CFileData& data = m_fileData[m_indexMapping[i]];
-			if (data.flags != fill)
+			if (data.comparison_flags != fill)
 				SetSelection(i, true);
 			else
 				SetSelection(i, false);
