@@ -1,14 +1,16 @@
 #include <filezilla.h>
 #include "filter.h"
 #include "filteredit.h"
-#include "ipcmutex.h"
 #include "filezillaapp.h"
-#include "xmlfunctions.h"
-#include <wx/regex.h>
+#include "inputdialog.h"
+#include "ipcmutex.h"
+#include "local_filesys.h"
 #include "Mainfrm.h"
 #include "Options.h"
-#include "inputdialog.h"
 #include "state.h"
+#include "xmlfunctions.h"
+
+#include <wx/regex.h>
 
 bool CFilterManager::m_loaded = false;
 std::vector<CFilter> CFilterManager::m_globalFilters;
@@ -201,13 +203,11 @@ void CFilterDialog::SaveFilters()
 {
 	CInterProcessMutex mutex(MUTEX_FILTERS);
 
-	wxFileName file(COptions::Get()->GetOption(OPTION_DEFAULT_SETTINGSDIR), _T("filters.xml"));
-	CXmlFile xml(file);
+	CXmlFile xml(wxGetApp().GetSettingsFile(_T("filters")));
 	TiXmlElement* pDocument = xml.Load();
-	if (!pDocument)
-	{
+	if (!pDocument) {
 		wxString msg = xml.GetError() + _T("\n\n") + _("Any changes made to the filters could not be saved.");
-		wxMessageBox(msg, _("Error loading xml file"), wxICON_ERROR);
+		wxMessageBoxEx(msg, _("Error loading xml file"), wxICON_ERROR);
 
 		return;
 	}
@@ -256,7 +256,7 @@ void CFilterDialog::SaveFilters()
 		}
 	}
 
-	xml.Save();
+	xml.Save(true);
 
 	m_filters_disabled = false;
 }
@@ -307,7 +307,7 @@ void CFilterDialog::OnFilterSelect(wxCommandEvent& event)
 	if (localOnly && event.GetEventObject() != pLocal)
 	{
 		pRemote->Check(item, false);
-		wxMessageBox(_("Selected filter only works for local files."), _("Cannot select filter"), wxICON_INFORMATION);
+		wxMessageBoxEx(_("Selected filter only works for local files."), _("Cannot select filter"), wxICON_INFORMATION);
 		return;
 	}
 
@@ -345,9 +345,9 @@ void CFilterDialog::OnSaveAs(wxCommandEvent& event)
 		return;
 
 	wxString name = dlg.GetValue();
-	if (name == _T(""))
+	if (name.empty())
 	{
-		wxMessageBox(_("No name for the filterset given."), _("Cannot save filterset"), wxICON_INFORMATION);
+		wxMessageBoxEx(_("No name for the filterset given."), _("Cannot save filterset"), wxICON_INFORMATION);
 		return;
 	}
 	wxChoice* pChoice = XRCCTRL(*this, "ID_SETS", wxChoice);
@@ -362,7 +362,7 @@ void CFilterDialog::OnSaveAs(wxCommandEvent& event)
 	int pos = pChoice->FindString(name);
 	if (pos != wxNOT_FOUND)
 	{
-		if (wxMessageBox(_("Given filterset name already exists, overwrite filter set?"), _("Filter set already exists"), wxICON_QUESTION | wxYES_NO) != wxYES)
+		if (wxMessageBoxEx(_("Given filterset name already exists, overwrite filter set?"), _("Filter set already exists"), wxICON_QUESTION | wxYES_NO) != wxYES)
 			return;
 	}
 
@@ -394,13 +394,13 @@ void CFilterDialog::OnRename(wxCommandEvent& event)
 
 	if (!old_pos)
 	{
-		wxMessageBox(_("This filter set cannot be renamed."));
+		wxMessageBoxEx(_("This filter set cannot be renamed."));
 		return;
 	}
 
 	CInputDialog dlg;
 
-	wxString msg = wxString::Format(_("Please enter a new name for the filter set \"%s\""), pChoice->GetStringSelection().c_str());
+	wxString msg = wxString::Format(_("Please enter a new name for the filter set \"%s\""), pChoice->GetStringSelection());
 
 	dlg.Create(this, _("Enter new name for filterset"), msg);
 	if (dlg.ShowModal() != wxID_OK)
@@ -414,16 +414,16 @@ void CFilterDialog::OnRename(wxCommandEvent& event)
 		return;
 	}
 
-	if (name == _T(""))
+	if (name.empty())
 	{
-		wxMessageBox(_("No name for the filterset given."), _("Cannot save filterset"), wxICON_INFORMATION);
+		wxMessageBoxEx(_("No name for the filterset given."), _("Cannot save filterset"), wxICON_INFORMATION);
 		return;
 	}
 
 	int pos = pChoice->FindString(name);
 	if (pos != wxNOT_FOUND)
 	{
-		if (wxMessageBox(_("Given filterset name already exists, overwrite filter set?"), _("Filter set already exists"), wxICON_QUESTION | wxYES_NO) != wxYES)
+		if (wxMessageBoxEx(_("Given filterset name already exists, overwrite filter set?"), _("Filter set already exists"), wxICON_QUESTION | wxYES_NO) != wxYES)
 			return;
 	}
 
@@ -459,7 +459,7 @@ void CFilterDialog::OnDeleteSet(wxCommandEvent& event)
 
 	if (!pos)
 	{
-		wxMessageBox(_("This filter set cannot be removed."));
+		wxMessageBoxEx(_("This filter set cannot be removed."));
 		return;
 	}
 
@@ -604,7 +604,7 @@ bool CFilterManager::HasSameLocalAndRemoteFilters() const
 	return true;
 }
 
-bool CFilterManager::FilenameFiltered(const wxString& name, const wxString& path, bool dir, wxLongLong size, bool local, int attributes,const wxDateTime* date) const
+bool CFilterManager::FilenameFiltered(const wxString& name, const wxString& path, bool dir, wxLongLong size, bool local, int attributes, CDateTime const& date) const
 {
 	if (m_filters_disabled)
 		return false;
@@ -633,43 +633,38 @@ bool CFilterManager::FilenameFiltered(const wxString& name, const wxString& path
 	return false;
 }
 
-bool CFilterManager::FilenameFiltered(const std::list<CFilter> &filters, const wxString& name, const wxString& path, bool dir, wxLongLong size, bool local, int attributes, const wxDateTime* date) const
+bool CFilterManager::FilenameFiltered(const std::list<CFilter> &filters, const wxString& name, const wxString& path, bool dir, wxLongLong size, bool local, int attributes, CDateTime const& date) const
 {
-	for (std::list<CFilter>::const_iterator iter = filters.begin(); iter != filters.end(); ++iter)
-	{
-		if (FilenameFilteredByFilter(*iter, name, path, dir, size, attributes, date))
+	for( auto const& filter : filters ) {
+		if (FilenameFilteredByFilter(filter, name, path, dir, size, attributes, date))
 			return true;
 	}
 
 	return false;
 }
 
-static bool StringMatch(const wxString& subject, const wxString& filter, int condition, bool matchCase, const CSharedPointer<const wxRegEx>& pRegEx)
+static bool StringMatch(const wxString& subject, const wxString& filter, int condition, bool matchCase, std::shared_ptr<const wxRegEx> const& pRegEx)
 {
 	bool match = false;
 
 	switch (condition)
 	{
 	case 0:
-		if (matchCase)
-		{
+		if (matchCase) {
 			if (subject.Contains(filter))
 				match = true;
 		}
-		else
-		{
+		else {
 			if (subject.Lower().Contains(filter.Lower()))
 				match = true;
 		}
 		break;
 	case 1:
-		if (matchCase)
-		{
+		if (matchCase) {
 			if (subject == filter)
 				match = true;
 		}
-		else
-		{
+		else {
 			if (!subject.CmpNoCase(filter))
 				match = true;
 		}
@@ -677,13 +672,11 @@ static bool StringMatch(const wxString& subject, const wxString& filter, int con
 	case 2:
 		{
 			const wxString& left = subject.Left(filter.Len());
-			if (matchCase)
-			{
+			if (matchCase) {
 				if (left == filter)
 					match = true;
 			}
-			else
-			{
+			else {
 				if (!left.CmpNoCase(filter))
 					match = true;
 			}
@@ -692,13 +685,11 @@ static bool StringMatch(const wxString& subject, const wxString& filter, int con
 	case 3:
 		{
 			const wxString& right = subject.Right(filter.Len());
-			if (matchCase)
-			{
+			if (matchCase) {
 				if (right == filter)
 					match = true;
 			}
-			else
-			{
+			else {
 				if (!right.CmpNoCase(filter))
 					match = true;
 			}
@@ -710,13 +701,11 @@ static bool StringMatch(const wxString& subject, const wxString& filter, int con
 			match = true;
 		break;
 	case 5:
-		if (matchCase)
-		{
+		if (matchCase) {
 			if (!subject.Contains(filter))
 				match = true;
 		}
-		else
-		{
+		else {
 			if (!subject.Lower().Contains(filter.Lower()))
 				match = true;
 		}
@@ -726,7 +715,7 @@ static bool StringMatch(const wxString& subject, const wxString& filter, int con
 	return match;
 }
 
-bool CFilterManager::FilenameFilteredByFilter(const CFilter& filter, const wxString& name, const wxString& path, bool dir, wxLongLong size, int attributes, const wxDateTime* date)
+bool CFilterManager::FilenameFilteredByFilter(const CFilter& filter, const wxString& name, const wxString& path, bool dir, wxLongLong size, int attributes, CDateTime const& date)
 {
 	if (dir && !filter.filterDirs)
 		return false;
@@ -853,27 +842,23 @@ bool CFilterManager::FilenameFilteredByFilter(const CFilter& filter, const wxStr
 #endif //__WXMSW__
 			break;
 		case filter_date:
-			if (!date)
-				break;
-
-			switch (condition.condition)
-			{
-			case 0:
-				// Before
-				match = date->GetDateOnly() < condition.date;
-				break;
-			case 1:
-				// Equals
-				match = date->GetDateOnly() == condition.date;
-				break;
-			case 2:
-				// Not equals
-				match = date->GetDateOnly() != condition.date;
-				break;
-			case 3:
-				// After
-				match = date->GetDateOnly() > condition.date;
-				break;
+			if (date.IsValid()) {
+				int cmp = date.Compare( condition.date );
+				switch (condition.condition)
+				{
+				case 0: // Before
+					match = cmp < 0;
+					break;
+				case 1: // Equals
+					match = cmp == 0;
+					break;
+				case 2: // Not equals
+					match = cmp != 0;
+					break;
+				case 3: // After
+					match = cmp > 0;
+					break;
+				}
 			}
 			break;
 		default:
@@ -902,20 +887,18 @@ bool CFilterManager::FilenameFilteredByFilter(const CFilter& filter, const wxStr
 
 bool CFilterManager::CompileRegexes(CFilter& filter)
 {
-	for (std::vector<CFilterCondition>::iterator iter = filter.filters.begin(); iter != filter.filters.end(); ++iter)
+	for (auto iter = filter.filters.begin(); iter != filter.filters.end(); ++iter)
 	{
 		CFilterCondition& condition = *iter;
-		if ((condition.type == filter_name || condition.type == filter_path) && condition.condition == 4)
-		{
-			condition.pRegEx = new wxRegEx(condition.strValue);
-			if (!condition.pRegEx->IsValid())
-			{
-				condition.pRegEx.clear();
+		if ((condition.type == filter_name || condition.type == filter_path) && condition.condition == 4) {
+			condition.pRegEx = std::make_shared<wxRegEx>(condition.strValue);
+			if (!condition.pRegEx->IsValid()) {
+				condition.pRegEx.reset();
 				return false;
 			}
 		}
 		else
-			condition.pRegEx.clear();
+			condition.pRegEx.reset();
 	}
 
 	return true;
@@ -984,7 +967,7 @@ bool CFilterManager::LoadFilter(TiXmlElement* pElement, CFilter& filter)
 		}
 		condition.strValue = GetTextElement(pCondition, "Value");
 		condition.matchCase = filter.matchCase;
-		if (condition.strValue == _T(""))
+		if (condition.strValue.empty())
 			continue;
 
 		// TODO: 64bit filesize
@@ -1003,8 +986,10 @@ bool CFilterManager::LoadFilter(TiXmlElement* pElement, CFilter& filter)
 		}
 		else if (condition.type == filter_date)
 		{
-			if (!condition.date.ParseFormat(condition.strValue, _T("%Y-%m-%d")) || !condition.date.IsValid())
+			wxDateTime t;
+			if (!t.ParseFormat(condition.strValue, _T("%Y-%m-%d")) || !t.IsValid())
 				continue;
+			condition.date = CDateTime(t, CDateTime::days);
 		}
 
 		filter.filters.push_back(condition);
@@ -1022,16 +1007,16 @@ void CFilterManager::LoadFilters()
 
 	CInterProcessMutex mutex(MUTEX_FILTERS);
 
-	wxFileName file(COptions::Get()->GetOption(OPTION_DEFAULT_SETTINGSDIR), _T("filters.xml"));
-	if (!file.FileExists() || file.GetSize() == 0)
-		file = wxFileName(wxGetApp().GetResourceDir(), _T("defaultfilters.xml"));
+	wxString file(wxGetApp().GetSettingsFile(_T("filters")));
+	if (CLocalFileSystem::GetSize(file) < 1) {
+		file = wxGetApp().GetResourceDir().GetPath() + _T("defaultfilters.xml");
+	}
 
 	CXmlFile xml(file);
 	TiXmlElement* pDocument = xml.Load();
-	if (!pDocument)
-	{
+	if (!pDocument) {
 		wxString msg = xml.GetError() + _T("\n\n") + _("Any changes made to the filters will not be saved.");
-		wxMessageBox(msg, _("Error loading xml file"), wxICON_ERROR);
+		wxMessageBoxEx(msg, _("Error loading xml file"), wxICON_ERROR);
 
 		return;
 	}
@@ -1048,7 +1033,7 @@ void CFilterManager::LoadFilters()
 
 		bool loaded = LoadFilter(pFilter, filter);
 
-		if (loaded && filter.name != _T("") && !filter.filters.empty())
+		if (loaded && !filter.name.empty() && !filter.filters.empty())
 			m_globalFilters.push_back(filter);
 
 		pFilter = pFilter->NextSiblingElement("Filter");
@@ -1077,7 +1062,7 @@ void CFilterManager::LoadFilters()
 		if (!m_globalFilterSets.empty())
 		{
 			set.name = GetTextElement(pSet, "Name");
-			if (set.name == _T(""))
+			if (set.name.empty())
 				continue;
 		}
 
