@@ -4,6 +4,8 @@
 #include "dialogex.h"
 #include <wx/regex.h>
 
+#include <memory>
+
 enum t_filterType
 {
 	filter_name = 0x01,
@@ -32,9 +34,9 @@ public:
 
 	wxString strValue; // All other types
 	wxLongLong value; // If type is size
-	wxDateTime date; // If type is date
+	CDateTime date; // If type is date
 	bool matchCase;
-	CSharedPointer<const wxRegEx> pRegEx;
+	std::shared_ptr<wxRegEx> pRegEx;
 };
 
 class CFilter
@@ -77,9 +79,9 @@ public:
 	CFilterManager();
 
 	// Note: Under non-windows, attributes are permissions
-	bool FilenameFiltered(const wxString& name, const wxString& path, bool dir, wxLongLong size, bool local, int attributes, const wxDateTime* date) const;
-	bool FilenameFiltered(const std::list<CFilter> &filters, const wxString& name, const wxString& path, bool dir, wxLongLong size, bool local, int attributes, const wxDateTime* date) const;
-	static bool FilenameFilteredByFilter(const CFilter& filter, const wxString& name, const wxString& path, bool dir, wxLongLong size, int attributes, const wxDateTime* date);
+	bool FilenameFiltered(const wxString& name, const wxString& path, bool dir, wxLongLong size, bool local, int attributes, CDateTime const& date) const;
+	bool FilenameFiltered(const std::list<CFilter> &filters, const wxString& name, const wxString& path, bool dir, wxLongLong size, bool local, int attributes, CDateTime const& date) const;
+	static bool FilenameFilteredByFilter(const CFilter& filter, const wxString& name, const wxString& path, bool dir, wxLongLong size, int attributes, CDateTime const& date);
 	static bool HasActiveFilters(bool ignore_disabled = false);
 
 	bool HasSameLocalAndRemoteFilters() const;
@@ -125,7 +127,7 @@ protected:
 
 	void SetCtrlState();
 
-	DECLARE_EVENT_TABLE();
+	DECLARE_EVENT_TABLE()
 	void OnOK(wxCommandEvent& event);
 	void OnCancel(wxCommandEvent& event);
 	void OnApply(wxCommandEvent& event);
