@@ -71,13 +71,9 @@ wxListCtrlEx::~wxListCtrlEx()
 }
 
 #ifndef __WXMSW__
-wxScrolledWindow* wxListCtrlEx::GetMainWindow() const
+wxWindow* wxListCtrlEx::GetMainWindow() const
 {
-#ifdef __WXMAC__
-	return (wxScrolledWindow*)m_genericImpl->m_mainWin;
-#else
-	return (wxScrolledWindow*)m_mainWin;
-#endif
+	return reinterpret_cast<wxWindow*>(m_mainWin);
 }
 #endif
 
@@ -85,7 +81,7 @@ void wxListCtrlEx::OnPostScroll()
 {
 }
 
-void wxListCtrlEx::OnPostScrollEvent(wxCommandEvent& event)
+void wxListCtrlEx::OnPostScrollEvent(wxCommandEvent&)
 {
 	OnPostScroll();
 }
@@ -121,7 +117,6 @@ void wxListCtrlEx::OnSelectionChanged(wxListEvent& event)
 
 void wxListCtrlEx::ScrollTopItem(int item)
 {
-#ifdef __WXMSW__
 	const int current = GetTopItem();
 
 	int delta = item - current;
@@ -133,10 +128,6 @@ void wxListCtrlEx::ScrollTopItem(int item)
 
 	delta *= rect.GetHeight();
 	ScrollList(0, delta);
-#else
-	GetMainWindow()->Scroll(0, item);
-	EnsureVisible(item);
-#endif
 }
 
 
@@ -174,7 +165,7 @@ void wxListCtrlEx::HandlePrefixSearch(wxChar character)
 		if (text.Length() >= m_prefixSearch_prefix.Length() && !m_prefixSearch_prefix.CmpNoCase(text.Left(m_prefixSearch_prefix.Length())))
 			beep = true;
 	}
-	else if (m_prefixSearch_prefix == _T(""))
+	else if (m_prefixSearch_prefix.empty())
 		beep = true;
 
 	int start = item;
@@ -183,8 +174,7 @@ void wxListCtrlEx::HandlePrefixSearch(wxChar character)
 
 	int newPos = FindItemWithPrefix(newPrefix, start);
 
-	if (newPos == -1 && m_prefixSearch_prefix[0] == character && !m_prefixSearch_prefix[1] && item != -1 && beep)
-	{
+	if (newPos == -1 && (m_prefixSearch_prefix.Len() == 1 && m_prefixSearch_prefix[0] == character) && item != -1 && beep) {
 		// Search the next item that starts with the same letter
 		newPrefix = m_prefixSearch_prefix;
 		newPos = FindItemWithPrefix(newPrefix, item + 1);
@@ -285,7 +275,7 @@ void wxListCtrlEx::OnKeyDown(wxKeyEvent& event)
 		return;
 	}
 
-#if defined(__WXMSW__) && wxUSE_UNICODE
+#if defined(__WXMSW__)
 
 	if (code >= 300 && code != WXK_NUMPAD_DECIMAL)
 	{
@@ -321,16 +311,14 @@ void wxListCtrlEx::OnKeyDown(wxKeyEvent& event)
 #else
 	if (code > 32 && code < 300 && !event.HasModifiers())
 	{
-#if wxUSE_UNICODE
 		int unicodeKey = event.GetUnicodeKey();
 		if (unicodeKey)
 			code = unicodeKey;
-#endif
 		HandlePrefixSearch(code);
 	}
 	else
 		event.Skip();
-#endif //defined(__WXMSW__) && wxUSE_UNICODE
+#endif //defined(__WXMSW__)
 }
 
 // Declared const due to design error in wxWidgets.
@@ -758,7 +746,7 @@ protected:
 		{
 			wxCheckListBox* pListBox = XRCCTRL(*this, "ID_ACTIVE", wxCheckListBox);
 			pListBox->Check(0);
-			wxMessageBox(_("The filename column can neither be hidden nor moved."), _("Column properties"));
+			wxMessageBoxEx(_("The filename column can neither be hidden nor moved."), _("Column properties"));
 		}
 	}
 };
@@ -808,7 +796,7 @@ void wxListCtrlEx::ShowColumnEditor()
 		if (!isChecked && !col)
 		{
 			isChecked = true;
-			wxMessageBox(_("The filename column cannot be hidden."));
+			wxMessageBoxEx(_("The filename column cannot be hidden."));
 		}
 		MoveColumn(col, i);
 		if (m_columnInfo[col].shown != isChecked)
@@ -872,9 +860,9 @@ void wxListCtrlEx::InitHeaderSortImageList()
 
 	wxBitmap bmp;
 
-	bmp.LoadFile(wxGetApp().GetResourceDir() + _T("up.png"), wxBITMAP_TYPE_PNG);
+	bmp.LoadFile(wxGetApp().GetResourceDir().GetPath() + _T("up.png"), wxBITMAP_TYPE_PNG);
 	m_pHeaderImageList->Add(bmp);
-	bmp.LoadFile(wxGetApp().GetResourceDir() + _T("down.png"), wxBITMAP_TYPE_PNG);
+	bmp.LoadFile(wxGetApp().GetResourceDir().GetPath() + _T("down.png"), wxBITMAP_TYPE_PNG);
 	m_pHeaderImageList->Add(bmp);
 
 	HWND hWnd = (HWND)GetHandle();
@@ -1190,7 +1178,7 @@ CListCtrlDropTarget::CListCtrlDropTarget(wxListCtrlEx* pListCtrl)
 }
 
 
-bool CListCtrlDropTarget::OnDrop(wxCoord x, wxCoord y)
+bool CListCtrlDropTarget::OnDrop(wxCoord, wxCoord)
 {
 	m_timer.Stop();
 	return true;
