@@ -17,13 +17,10 @@ CRecursiveOperation::CNewDir::CNewDir()
 
 CRecursiveOperation::CRecursiveOperation(CState* pState)
 	: CStateEventHandler(pState),
-	  m_operationMode(recursive_none), m_pState(pState)
+	  m_operationMode(recursive_none)
 {
 	pState->RegisterHandler(this, STATECHANGE_REMOTE_DIR, false);
 	pState->RegisterHandler(this, STATECHANGE_REMOTE_LINKNOTDIR, false);
-
-	m_pChmodDlg = 0;
-	m_pQueue = 0;
 }
 
 CRecursiveOperation::~CRecursiveOperation()
@@ -35,7 +32,7 @@ CRecursiveOperation::~CRecursiveOperation()
 	}
 }
 
-void CRecursiveOperation::OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void* data2)
+void CRecursiveOperation::OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString&, const void* data2)
 {
 	if (notification == STATECHANGE_REMOTE_LINKNOTDIR)
 	{
@@ -46,7 +43,7 @@ void CRecursiveOperation::OnStateChange(CState* pState, enum t_statechange_notif
 	{
 		wxASSERT(pState);
 		wxASSERT(notification == STATECHANGE_REMOTE_DIR);
-		ProcessDirectoryListing(pState->GetRemoteDir().Value());
+		ProcessDirectoryListing(pState->GetRemoteDir().get());
 	}
 }
 
@@ -54,7 +51,7 @@ void CRecursiveOperation::StartRecursiveOperation(enum OperationMode mode, const
 {
 	wxCHECK_RET(m_operationMode == recursive_none, _T("StartRecursiveOperation called with m_operationMode != recursive_none"));
 	wxCHECK_RET(m_pState->IsRemoteConnected(), _T("StartRecursiveOperation while disconnected"));
-	wxCHECK_RET(!startDir.IsEmpty(), _T("Empty startDir in StartRecursiveOperation"));
+	wxCHECK_RET(!startDir.empty(), _T("Empty startDir in StartRecursiveOperation"));
 
 	if (mode == recursive_chmod && !m_pChmodDlg)
 		return;
@@ -73,7 +70,7 @@ void CRecursiveOperation::StartRecursiveOperation(enum OperationMode mode, const
 
 	m_startDir = startDir;
 
-	if (finalDir.IsEmpty())
+	if (finalDir.empty())
 		m_finalDir = startDir;
 	else
 		m_finalDir = finalDir;
@@ -132,7 +129,7 @@ bool CRecursiveOperation::NextOperation()
 
 bool CRecursiveOperation::BelowRecursionRoot(const CServerPath& path, CNewDir &dir)
 {
-	if (!dir.start_dir.IsEmpty())
+	if (!dir.start_dir.empty())
 	{
 		if (path.IsSubdirOf(dir.start_dir, false))
 			return true;
@@ -171,7 +168,7 @@ void CRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing* pDire
 	if (m_operationMode == recursive_none)
 		return;
 
-	if (pDirectoryListing->m_failed)
+	if (pDirectoryListing->failed())
 	{
 		// Ignore this.
 		// It will get handled by the failed command in ListingFailed
@@ -195,7 +192,7 @@ void CRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing* pDire
 		return;
 	}
 
-	if (m_operationMode == recursive_delete && dir.doVisit && dir.subdir != _T(""))
+	if (m_operationMode == recursive_delete && dir.doVisit && !dir.subdir.empty())
 	{
 		// After recursing into directory to delete its contents, delete directory itself
 		// Gets handled in NextOperation
@@ -237,7 +234,7 @@ void CRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing* pDire
 	CFilterManager filter;
 
 	// Is operation restricted to a single child?
-	bool restrict = !dir.restrict.IsEmpty();
+	bool restrict = !dir.restrict.empty();
 
 	std::list<wxString> filesToDelete;
 
@@ -254,7 +251,7 @@ void CRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing* pDire
 			if (entry.name != dir.restrict)
 				continue;
 		}
-		else if (filter.FilenameFiltered(m_filters, entry.name, path, entry.is_dir(), entry.size, false, 0, entry.has_date() ? &entry.time : 0))
+		else if (filter.FilenameFiltered(m_filters, entry.name, path, entry.is_dir(), entry.size, false, 0, entry.time))
 			continue;
 
 		if (entry.is_dir() && (!entry.is_link() || m_operationMode != recursive_delete))
@@ -321,7 +318,7 @@ void CRecursiveOperation::ProcessDirectoryListing(const CDirectoryListing* pDire
 				(entry.is_dir() && applyType == 2))
 			{
 				char permissions[9];
-				bool res = m_pChmodDlg->ConvertPermissions(entry.permissions, permissions);
+				bool res = m_pChmodDlg->ConvertPermissions(*entry.permissions, permissions);
 				wxString newPerms = m_pChmodDlg->GetPermissions(res ? permissions : 0, entry.is_dir());
 				m_pState->m_pCommandQueue->ProcessCommand(new CChmodCommand(pDirectoryListing->path, entry.name, newPerms));
 			}
