@@ -13,12 +13,10 @@
 #include "timeformatting.h"
 #include "window_state_manager.h"
 
-class CSearchFileData : public CGenericFileData
+class CSearchFileData : public CGenericFileData, public CDirentry
 {
 public:
 	CServerPath path;
-
-	CDirentry entry;
 };
 
 class CSearchDialogFileList : public CFileListCtrl<CSearchFileData>
@@ -45,315 +43,17 @@ protected:
 #endif
 
 private:
-	virtual bool CanStartComparison(wxString* pError) { return false; }
+	virtual bool CanStartComparison(wxString*) { return false; }
 	virtual void StartComparison() {}
-	virtual bool GetNextFile(wxString& name, bool &dir, wxLongLong &size, wxDateTime& date, bool &hasTime) { return false; }
-	virtual void CompareAddFile(CComparableListing::t_fileEntryFlags flags) {}
+	virtual bool GetNextFile(wxString&, bool &, wxLongLong &, CDateTime&) { return false; }
+	virtual void CompareAddFile(CComparableListing::t_fileEntryFlags) {}
 	virtual void FinishComparison() {}
-	virtual void ScrollTopItem(int item) {}
+	virtual void ScrollTopItem(int) {}
 	virtual void OnExitComparisonMode() {}
 
 	int m_dirIcon;
 };
 
-// Helper classes for fast sorting using std::sort
-// -----------------------------------------------
-
-class CSearchDialogFileList;
-class CSearchSort : public CListViewSort
-{
-public:
-	CSearchSort(CSearchDialogFileList* pListCtrl, std::vector<CSearchFileData> &fileData, enum DirSortMode dirSortMode)
-		: m_fileData(fileData), m_dirSortMode(dirSortMode), m_pListCtrl(pListCtrl)
-	{
-	}
-
-	#define CMP(f, data1, data2) \
-		{\
-			int res = f(data1, data2);\
-			if (res == -1)\
-				return true;\
-			else if (res == 1)\
-				return false;\
-		}
-
-	#define CMP_LESS(f, data1, data2) \
-		{\
-			int res = f(data1, data2);\
-			if (res == -1)\
-				return true;\
-			else\
-				return false;\
-		}
-
-	inline int CmpDir(const CDirentry &data1, const CDirentry &data2) const
-	{
-		switch (m_dirSortMode)
-		{
-		default:
-		case dirsort_ontop:
-			if (data1.is_dir())
-			{
-				if (!data2.is_dir())
-					return -1;
-				else
-					return 0;
-			}
-			else
-			{
-				if (data2.is_dir())
-					return 1;
-				else
-					return 0;
-			}
-		case dirsort_onbottom:
-			if (data1.is_dir())
-			{
-				if (!data2.is_dir())
-					return 1;
-				else
-					return 0;
-			}
-			else
-			{
-				if (data2.is_dir())
-					return -1;
-				else
-					return 0;
-			}
-		case dirsort_inline:
-			return 0;
-		}
-	}
-
-	inline int CmpName(const CDirentry &data1, const CDirentry &data2) const
-	{
-#ifdef __WXMSW__
-		return data1.name.CmpNoCase(data2.name);
-#else
-		return data1.name.Cmp(data2.name);
-#endif
-	}
-
-	inline int CmpSize(const CDirentry &data1, const CDirentry &data2) const
-	{
-		const wxLongLong diff = data1.size - data2.size;
-		if (diff < 0)
-			return -1;
-		else if (diff > 0)
-			return 1;
-		else
-			return 0;
-	}
-
-	inline int CmpStringNoCase(const wxString &data1, const wxString &data2) const
-	{
-		return data1.CmpNoCase(data2);
-	}
-
-	inline int CmpTime(const CDirentry &data1, const CDirentry &data2) const
-	{
-		if (!data1.has_date())
-		{
-			if (data2.has_date())
-				return -1;
-			else
-				return 0;
-		}
-		else
-		{
-			if (!data2.has_date())
-				return 1;
-
-			if (data1.time < data2.time)
-				return -1;
-			else if (data1.time > data2.time)
-				return 1;
-			else
-				return 0;
-		}
-	}
-
-protected:
-	std::vector<CSearchFileData> &m_fileData;
-
-	const enum DirSortMode m_dirSortMode;
-
-	CSearchDialogFileList* m_pListCtrl;
-};
-
-template<class T> class CReverseSort : public T
-{
-public:
-	CReverseSort(CSearchDialogFileList* pListCtrl, std::vector<CSearchFileData>& fileData, enum CSearchSort::DirSortMode dirSortMode)
-		: T(pListCtrl, fileData, dirSortMode)
-	{
-	}
-
-	inline bool operator()(int a, int b) const
-	{
-		return T::operator()(b, a);
-	}
-};
-
-class CSearchSortName : public CSearchSort
-{
-public:
-	CSearchSortName(CSearchDialogFileList* pListCtrl, std::vector<CSearchFileData>& fileData, enum CSearchSort::DirSortMode dirSortMode)
-		: CSearchSort(pListCtrl, fileData, dirSortMode)
-	{
-	}
-
-	bool operator()(int a, int b) const
-	{
-		const CDirentry& data1 = m_fileData[a].entry;
-		const CDirentry& data2 = m_fileData[b].entry;
-
-		CMP(CmpDir, data1, data2);
-
-		CMP_LESS(CmpName, data1, data2);
-	}
-};
-typedef CReverseSort<CSearchSortName> CSearchSortName_Reverse;
-
-class CSearchSortPath : public CSearchSort
-{
-public:
-	CSearchSortPath(CSearchDialogFileList* pListCtrl, std::vector<CSearchFileData>& fileData, enum CSearchSort::DirSortMode dirSortMode)
-		: CSearchSort(pListCtrl, fileData, dirSortMode)
-	{
-	}
-
-	bool operator()(int a, int b) const
-	{
-		const CDirentry& data1 = m_fileData[a].entry;
-		const CDirentry& data2 = m_fileData[b].entry;
-
-		if (m_fileData[a].path < m_fileData[b].path)
-			return true;
-		if (m_fileData[a].path != m_fileData[b].path)
-			return false;
-
-		CMP_LESS(CmpName, data1, data2);
-	}
-};
-typedef CReverseSort<CSearchSortPath> CSearchSortPath_Reverse;
-
-class CSearchSortSize : public CSearchSort
-{
-public:
-	CSearchSortSize(CSearchDialogFileList* pListCtrl, std::vector<CSearchFileData>& fileData, enum CSearchSort::DirSortMode dirSortMode)
-		: CSearchSort(pListCtrl, fileData, dirSortMode)
-	{
-	}
-
-	bool operator()(int a, int b) const
-	{
-		const CDirentry& data1 = m_fileData[a].entry;
-		const CDirentry& data2 = m_fileData[b].entry;
-
-		CMP(CmpDir, data1, data2);
-
-		CMP(CmpSize, data1, data2);
-
-		CMP_LESS(CmpName, data1, data2);
-	}
-};
-typedef CReverseSort<CSearchSortSize> CSearchSortSize_Reverse;
-
-class CSearchSortType : public CSearchSort
-{
-public:
-	CSearchSortType(CSearchDialogFileList* pListCtrl, std::vector<CSearchFileData>& fileData, enum CSearchSort::DirSortMode dirSortMode)
-		: CSearchSort(pListCtrl, fileData, dirSortMode)
-	{
-	}
-
-	bool operator()(int a, int b) const
-	{
-		CSearchFileData &data1 = m_fileData[a];
-		CSearchFileData &data2 = m_fileData[b];
-
-		CMP(CmpDir, data1.entry, data2.entry);
-
-		if (data1.fileType.IsEmpty())
-			data1.fileType = m_pListCtrl->GetType(data1.entry.name, data1.entry.is_dir());
-		if (data2.fileType.IsEmpty())
-			data2.fileType = m_pListCtrl->GetType(data2.entry.name, data2.entry.is_dir());
-
-		CMP(CmpStringNoCase, data1.fileType, data2.fileType);
-
-		CMP_LESS(CmpName, data1.entry, data2.entry);
-	}
-};
-typedef CReverseSort<CSearchSortType> CSearchSortType_Reverse;
-
-class CSearchSortTime : public CSearchSort
-{
-public:
-	CSearchSortTime(CSearchDialogFileList* pListCtrl, std::vector<CSearchFileData>& fileData, enum CSearchSort::DirSortMode dirSortMode)
-		: CSearchSort(pListCtrl, fileData, dirSortMode)
-	{
-	}
-
-	bool operator()(int a, int b) const
-	{
-		const CDirentry& data1 = m_fileData[a].entry;
-		const CDirentry& data2 = m_fileData[b].entry;
-
-		CMP(CmpDir, data1, data2);
-
-		CMP(CmpTime, data1, data2);
-
-		CMP_LESS(CmpName, data1, data2);
-	}
-};
-typedef CReverseSort<CSearchSortTime> CSearchSortTime_Reverse;
-
-class CSearchSortPermissions : public CSearchSort
-{
-public:
-	CSearchSortPermissions(CSearchDialogFileList* pListCtrl, std::vector<CSearchFileData>& fileData, enum CSearchSort::DirSortMode dirSortMode)
-		: CSearchSort(pListCtrl, fileData, dirSortMode)
-	{
-	}
-
-	bool operator()(int a, int b) const
-	{
-		const CDirentry& data1 = m_fileData[a].entry;
-		const CDirentry& data2 = m_fileData[b].entry;
-
-		CMP(CmpDir, data1, data2);
-
-		CMP(CmpStringNoCase, data1.permissions, data2.permissions);
-
-		CMP_LESS(CmpName, data1, data2);
-	}
-};
-typedef CReverseSort<CSearchSortPermissions> CSearchSortPermissions_Reverse;
-
-class CSearchSortOwnerGroup : public CSearchSort
-{
-public:
-	CSearchSortOwnerGroup(CSearchDialogFileList* pListCtrl, std::vector<CSearchFileData>& fileData, enum CSearchSort::DirSortMode dirSortMode)
-		: CSearchSort(pListCtrl, fileData, dirSortMode)
-	{
-	}
-
-	bool operator()(int a, int b) const
-	{
-		const CDirentry& data1 = m_fileData[a].entry;
-		const CDirentry& data2 = m_fileData[b].entry;
-
-		CMP(CmpDir, data1, data2);
-
-		CMP(CmpStringNoCase, data1.ownerGroup, data2.ownerGroup);
-
-		CMP_LESS(CmpName, data1, data2);
-	}
-};
-typedef CReverseSort<CSearchSortOwnerGroup> CSearchSortOwnerGroup_Reverse;
-
 // Search dialog file list
 // -----------------------
 
@@ -368,7 +68,7 @@ CSearchDialogFileList::CSearchDialogFileList(CSearchDialog* pParent, CState* pSt
 
 	SetImageList(GetSystemImageList(), wxIMAGE_LIST_SMALL);
 
-	m_dirIcon = GetIconIndex(dir);
+	m_dirIcon = GetIconIndex(iconType::dir);
 
 	InitSort(OPTION_SEARCH_SORTORDER);
 
@@ -388,61 +88,62 @@ CSearchDialogFileList::CSearchDialogFileList(CSearchDialog* pParent, CState* pSt
 
 bool CSearchDialogFileList::ItemIsDir(int index) const
 {
-	return m_fileData[index].entry.is_dir();
+	return m_fileData[index].is_dir();
 }
 
 wxLongLong CSearchDialogFileList::ItemGetSize(int index) const
 {
-	return m_fileData[index].entry.size;
+	return m_fileData[index].size;
 }
 
 CFileListCtrl<CSearchFileData>::CSortComparisonObject CSearchDialogFileList::GetSortComparisonObject()
 {
-	CSearchSort::DirSortMode dirSortMode = GetDirSortMode();
+	CFileListCtrlSortBase::DirSortMode dirSortMode = GetDirSortMode();
+	CFileListCtrlSortBase::NameSortMode nameSortMode = GetNameSortMode();
 
 	if (!m_sortDirection)
 	{
 		if (m_sortColumn == 1)
-			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CSearchSortPath(this, m_fileData, dirSortMode));
+			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CFileListCtrlSortPath<std::vector<CSearchFileData>, CSearchFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 		else if (m_sortColumn == 2)
-			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CSearchSortSize(this, m_fileData, dirSortMode));
+			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CFileListCtrlSortSize<std::vector<CSearchFileData>, CSearchFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 		else if (m_sortColumn == 3)
-			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CSearchSortType(this, m_fileData, dirSortMode));
+			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CFileListCtrlSortType<std::vector<CSearchFileData>, CSearchFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 		else if (m_sortColumn == 4)
-			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CSearchSortTime(this, m_fileData, dirSortMode));
+			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CFileListCtrlSortTime<std::vector<CSearchFileData>, CSearchFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 		else if (m_sortColumn == 5)
-			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CSearchSortPermissions(this, m_fileData, dirSortMode));
+			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CFileListCtrlSortPermissions<std::vector<CSearchFileData>, CSearchFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 		else if (m_sortColumn == 6)
-			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CSearchSortOwnerGroup(this, m_fileData, dirSortMode));
+			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CFileListCtrlSortOwnerGroup<std::vector<CSearchFileData>, CSearchFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 		else
-			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CSearchSortName(this, m_fileData, dirSortMode));
+			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CFileListCtrlSortName<std::vector<CSearchFileData>, CSearchFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 	}
 	else
 	{
 		if (m_sortColumn == 1)
-			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CSearchSortPath_Reverse(this, m_fileData, dirSortMode));
+			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortPath<std::vector<CSearchFileData>, CSearchFileData>, CSearchFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 		else if (m_sortColumn == 2)
-			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CSearchSortSize_Reverse(this, m_fileData, dirSortMode));
+			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortSize<std::vector<CSearchFileData>, CSearchFileData>, CSearchFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 		else if (m_sortColumn == 3)
-			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CSearchSortType_Reverse(this, m_fileData, dirSortMode));
+			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortType<std::vector<CSearchFileData>, CSearchFileData>, CSearchFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 		else if (m_sortColumn == 4)
-			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CSearchSortTime_Reverse(this, m_fileData, dirSortMode));
+			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortTime<std::vector<CSearchFileData>, CSearchFileData>, CSearchFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 		else if (m_sortColumn == 5)
-			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CSearchSortPermissions_Reverse(this, m_fileData, dirSortMode));
+			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortPermissions<std::vector<CSearchFileData>, CSearchFileData>, CSearchFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 		else if (m_sortColumn == 6)
-			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CSearchSortOwnerGroup_Reverse(this, m_fileData, dirSortMode));
+			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortOwnerGroup<std::vector<CSearchFileData>, CSearchFileData>, CSearchFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 		else
-			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CSearchSortName_Reverse(this, m_fileData, dirSortMode));
+			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortName<std::vector<CSearchFileData>, CSearchFileData>, CSearchFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 	}
 }
 
 wxString CSearchDialogFileList::GetItemText(int item, unsigned int column)
 {
 	if (item < 0 || item >= (int)m_indexMapping.size())
-		return _T("");
+		return wxString();
 	int index = m_indexMapping[item];
 
-	const CDirentry& entry = m_fileData[index].entry;
+	const CDirentry& entry = m_fileData[index];
 	if (!column)
 		return entry.name;
 	else if (column == 1)
@@ -450,14 +151,14 @@ wxString CSearchDialogFileList::GetItemText(int item, unsigned int column)
 	else if (column == 2)
 	{
 		if (entry.is_dir() || entry.size < 0)
-			return _T("");
+			return wxString();
 		else
 			return CSizeFormat::Format(entry.size);
 	}
 	else if (column == 3)
 	{
 		CSearchFileData& data = m_fileData[index];
-		if (data.fileType.IsEmpty())
+		if (data.fileType.empty())
 		{
 			if (data.path.GetType() == VMS)
 				data.fileType = GetType(StripVMSRevision(entry.name), entry.is_dir());
@@ -468,20 +169,12 @@ wxString CSearchDialogFileList::GetItemText(int item, unsigned int column)
 		return data.fileType;
 	}
 	else if (column == 4)
-	{
-		if (!entry.has_date())
-			return _T("");
-
-		if (entry.has_time())
-			return CTimeFormat::FormatDateTime(entry.time);
-		else
-			return CTimeFormat::FormatDate(entry.time);
-	}
+		return CTimeFormat::Format(entry.time);
 	else if (column == 5)
-		return entry.permissions;
+		return *entry.permissions;
 	else if (column == 6)
-		return entry.ownerGroup;
-	return _T("");
+		return *entry.ownerGroup;
+	return wxString();
 }
 
 int CSearchDialogFileList::OnGetItemImage(long item) const
@@ -496,7 +189,7 @@ int CSearchDialogFileList::OnGetItemImage(long item) const
 	if (icon != -2)
 		return icon;
 
-	icon = pThis->GetIconIndex(file, pThis->m_fileData[index].entry.name, false);
+	icon = pThis->GetIconIndex(iconType::file, pThis->m_fileData[index].name, false);
 	return icon;
 }
 
@@ -551,7 +244,7 @@ bool CSearchDialog::Load()
 	m_results->SetFilelistStatusBar(pStatusBar);
 
 	const CServerPath path = m_pState->GetRemotePath();
-	if (!path.IsEmpty())
+	if (!path.empty())
 		XRCCTRL(*this, "ID_PATH", wxTextCtrl)->ChangeValue(path.GetPath());
 
 	SetCtrlState();
@@ -594,12 +287,12 @@ void CSearchDialog::Run()
 			m_pState->m_pCommandQueue->Cancel();
 			m_pState->GetRecursiveOperationHandler()->StopRecursiveOperation();
 		}
-		if (!m_original_dir.IsEmpty())
+		if (!m_original_dir.empty())
 			m_pState->ChangeRemoteDir(m_original_dir);
 	}
 	else
 	{
-		if (m_pState->IsRemoteIdle() && !m_original_dir.IsEmpty())
+		if (m_pState->IsRemoteIdle() && !m_original_dir.empty())
 			m_pState->ChangeRemoteDir(m_original_dir);
 	}
 }
@@ -618,9 +311,9 @@ void CSearchDialog::OnStateChange(CState* pState, enum t_statechange_notificatio
 
 void CSearchDialog::ProcessDirectoryListing()
 {
-	CSharedPointer<const CDirectoryListing> listing = m_pState->GetRemoteDir();
+	std::shared_ptr<CDirectoryListing> listing = m_pState->GetRemoteDir();
 
-	if (!listing || listing->m_failed)
+	if (!listing || listing->failed())
 		return;
 
 	// Do not process same directory multiple times
@@ -637,12 +330,11 @@ void CSearchDialog::ProcessDirectoryListing()
 	{
 		const CDirentry& entry = (*listing)[i];
 
-		if (m_search_filter.filters.size() && !CFilterManager::FilenameFilteredByFilter(m_search_filter, entry.name, listing->path.GetPath(), entry.is_dir(), entry.size, 0, entry.has_date() ? &entry.time : 0))
+		if (!m_search_filter.filters.empty() && !CFilterManager::FilenameFilteredByFilter(m_search_filter, entry.name, listing->path.GetPath(), entry.is_dir(), entry.size, 0, entry.time))
 			continue;
 
 		CSearchFileData data;
-		data.flags = CComparableListing::normal;
-		data.entry = entry;
+		static_cast<CDirentry&>(data) = entry;
 		data.path = listing->path;
 		data.icon = entry.is_dir() ? m_results->m_dirIcon : -2;
 		m_results->m_fileData.push_back(data);
@@ -674,13 +366,13 @@ void CSearchDialog::OnSearch(wxCommandEvent& event)
 	const CServer* pServer = m_pState->GetServer();
 	if (!pServer)
 	{
-		wxMessageBox(_("Connection to server lost."), _("Remote file search"), wxICON_EXCLAMATION);
+		wxMessageBoxEx(_("Connection to server lost."), _("Remote file search"), wxICON_EXCLAMATION);
 		return;
 	}
 	path.SetType(pServer->GetType());
-	if (!path.SetPath(XRCCTRL(*this, "ID_PATH", wxTextCtrl)->GetValue()) || path.IsEmpty())
+	if (!path.SetPath(XRCCTRL(*this, "ID_PATH", wxTextCtrl)->GetValue()) || path.empty())
 	{
-		wxMessageBox(_("Need to enter valid remote path"), _("Remote file search"), wxICON_EXCLAMATION);
+		wxMessageBoxEx(_("Need to enter valid remote path"), _("Remote file search"), wxICON_EXCLAMATION);
 		return;
 	}
 
@@ -690,13 +382,13 @@ void CSearchDialog::OnSearch(wxCommandEvent& event)
 	wxString error;
 	if (!ValidateFilter(error, true))
 	{
-		wxMessageBox(wxString::Format(_("Invalid search conditions: %s"), error.c_str()), _("Remote file search"), wxICON_EXCLAMATION);
+		wxMessageBoxEx(wxString::Format(_("Invalid search conditions: %s"), error), _("Remote file search"), wxICON_EXCLAMATION);
 		return;
 	}
 	m_search_filter = GetFilter();
 	if (!CFilterManager::CompileRegexes(m_search_filter))
 	{
-		wxMessageBox(_("Invalid regular expression in search conditions."), _("Remote file search"), wxICON_EXCLAMATION);
+		wxMessageBoxEx(_("Invalid regular expression in search conditions."), _("Remote file search"), wxICON_EXCLAMATION);
 		return;
 	}
 	m_search_filter.matchCase = XRCCTRL(*this, "ID_CASE", wxCheckBox)->GetValue();
@@ -775,7 +467,7 @@ public:
 		{
 			wxString files = wxString::Format(wxPLURAL("%d file", "%d files", count_files), count_files);
 			wxString dirs = wxString::Format(wxPLURAL("%d directory with its contents", "%d directories with their contents", count_dirs), count_dirs);
-			desc.Printf(_("Selected %s and %s for transfer."), files.c_str(), dirs.c_str());
+			desc.Printf(_("Selected %s and %s for transfer."), files, dirs);
 		}
 		XRCCTRL(*this, "ID_DESC", wxStaticText)->SetLabel(desc);
 
@@ -815,13 +507,13 @@ void CSearchDownloadDialog::OnOK(wxCommandEvent& event)
 	CLocalPath path(pText->GetValue());
 	if (path.empty())
 	{
-		wxMessageBox(_("You have to enter a local directory."), _("Download search results"), wxICON_EXCLAMATION);
+		wxMessageBoxEx(_("You have to enter a local directory."), _("Download search results"), wxICON_EXCLAMATION);
 		return;
 	}
 
 	if (!path.IsWriteable())
 	{
-		wxMessageBox(_("You have to enter a writable local directory."), _("Download search results"), wxICON_EXCLAMATION);
+		wxMessageBoxEx(_("You have to enter a writable local directory."), _("Download search results"), wxICON_EXCLAMATION);
 		return;
 	}
 
@@ -837,11 +529,11 @@ void CSearchDialog::ProcessSelection(std::list<int> &selected_files, std::list<C
 			continue;
 		int index = m_results->m_indexMapping[sel];
 
-		if (m_results->m_fileData[index].entry.is_dir())
+		if (m_results->m_fileData[index].is_dir())
 		{
 			CServerPath path = m_results->m_fileData[index].path;
-			path.ChangePath(m_results->m_fileData[index].entry.name);
-			if (path.IsEmpty())
+			path.ChangePath(m_results->m_fileData[index].name);
+			if (path.empty())
 				continue;
 
 			bool replaced = false;
@@ -909,7 +601,7 @@ void CSearchDialog::ProcessSelection(std::list<int> &selected_files, std::list<C
 	// covered by any of those directories.
 }
 
-void CSearchDialog::OnDownload(wxCommandEvent& event)
+void CSearchDialog::OnDownload(wxCommandEvent&)
 {
 	if (!m_pState->IsRemoteIdle())
 		return;
@@ -924,7 +616,7 @@ void CSearchDialog::OnDownload(wxCommandEvent& event)
 
 	if (selected_dirs.size() > 1)
 	{
-		wxMessageBox(_("Downloading multiple unrelated directories is not yet supported"), _("Downloading search results"), wxICON_EXCLAMATION);
+		wxMessageBoxEx(_("Downloading multiple unrelated directories is not yet supported"), _("Downloading search results"), wxICON_EXCLAMATION);
 		return;
 	}
 
@@ -954,7 +646,7 @@ void CSearchDialog::OnDownload(wxCommandEvent& event)
 
 	for (std::list<int>::const_iterator iter = selected_files.begin(); iter != selected_files.end(); ++iter)
 	{
-		const CDirentry& entry = m_results->m_fileData[*iter].entry;
+		const CDirentry& entry = m_results->m_fileData[*iter];
 
 		CLocalPath target_path = path;
 		if (!flatten)
@@ -1000,7 +692,7 @@ void CSearchDialog::OnDownload(wxCommandEvent& event)
 	}
 }
 
-void CSearchDialog::OnDelete(wxCommandEvent& event)
+void CSearchDialog::OnDelete(wxCommandEvent&)
 {
 	if (!m_pState->IsRemoteIdle())
 		return;
@@ -1015,7 +707,7 @@ void CSearchDialog::OnDelete(wxCommandEvent& event)
 
 	if (selected_dirs.size() > 1)
 	{
-		wxMessageBox(_("Deleting multiple unrelated directories is not yet supported"), _("Deleting directories"), wxICON_EXCLAMATION);
+		wxMessageBoxEx(_("Deleting multiple unrelated directories is not yet supported"), _("Deleting directories"), wxICON_EXCLAMATION);
 		return;
 	}
 
@@ -1028,15 +720,15 @@ void CSearchDialog::OnDelete(wxCommandEvent& event)
 	{
 		wxString files = wxString::Format(wxPLURAL("%d file", "%d files", selected_files.size()), selected_files.size());
 		wxString dirs = wxString::Format(wxPLURAL("%d directory with its contents", "%d directories with their contents", selected_dirs.size()), selected_dirs.size());
-		question.Printf(_("Really delete %s and %s from the server?"), files.c_str(), dirs.c_str());
+		question.Printf(_("Really delete %s and %s from the server?"), files, dirs);
 	}
 
-	if (wxMessageBox(question, _("Confirm deletion"), wxICON_QUESTION | wxYES_NO) != wxYES)
+	if (wxMessageBoxEx(question, _("Confirm deletion"), wxICON_QUESTION | wxYES_NO) != wxYES)
 		return;
 
 	for (std::list<int>::const_iterator iter = selected_files.begin(); iter != selected_files.end(); ++iter)
 	{
-		const CDirentry& entry = m_results->m_fileData[*iter].entry;
+		const CDirentry& entry = m_results->m_fileData[*iter];
 		std::list<wxString> files_to_delete;
 		files_to_delete.push_back(entry.name);
 		m_pState->m_pCommandQueue->ProcessCommand(new CDeleteCommand(m_results->m_fileData[*iter].path, files_to_delete));
@@ -1076,7 +768,7 @@ int CSearchDialogFileList::GetOverlayIndex(int item)
 		return -1;
 	int index = m_indexMapping[item];
 
-	if (m_fileData[index].entry.is_link())
+	if (m_fileData[index].is_link())
 		return GetLinkOverlayIndex();
 
 	return 0;
@@ -1087,12 +779,10 @@ void CSearchDialog::LoadConditions()
 {
 	CInterProcessMutex mutex(MUTEX_SEARCHCONDITIONS);
 
-	CXmlFile file;
-
-	TiXmlElement* pDocument = file.Load(_T("search"));
-	if (!pDocument)
-	{
-		wxMessageBox(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
+	CXmlFile file(wxGetApp().GetSettingsFile(_T("search")));
+	TiXmlElement* pDocument = file.Load();
+	if (!pDocument) {
+		wxMessageBoxEx(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
 		return;
 	}
 
@@ -1107,13 +797,11 @@ void CSearchDialog::LoadConditions()
 void CSearchDialog::SaveConditions()
 {
 	CInterProcessMutex mutex(MUTEX_SEARCHCONDITIONS);
-
-	CXmlFile file;
-
-	TiXmlElement* pDocument = file.Load(_T("search"));
-	if (!pDocument)
-	{
-		wxMessageBox(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
+	
+	CXmlFile file(wxGetApp().GetSettingsFile(_T("search")));
+	TiXmlElement* pDocument = file.Load();
+	if (!pDocument) {
+		wxMessageBoxEx(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
 		return;
 	}
 
@@ -1124,5 +812,5 @@ void CSearchDialog::SaveConditions()
 
 	CFilterDialog::SaveFilter(pFilter, m_search_filter);
 
-	file.Save();
+	file.Save(true);
 }
