@@ -1,4 +1,4 @@
-#include <filezilla.h>
+	#include <filezilla.h>
 #include "sitemanager.h"
 
 #include "filezillaapp.h"
@@ -6,7 +6,7 @@
 #include "Options.h"
 #include "xmlfunctions.h"
 
-std::map<int, struct CSiteManager::_menu_data> CSiteManager::m_idMap;
+std::map<int, CSiteManagerItemData_Site*> CSiteManager::m_idMap;
 
 bool CSiteManager::Load(TiXmlElement *pElement, CSiteManagerXmlHandler* pHandler)
 {
@@ -55,13 +55,13 @@ bool CSiteManager::Load(TiXmlElement *pElement, CSiteManagerXmlHandler* pHandler
 					if (remoteDir)
 						data->m_remoteDir.SetSafePath(ConvLocal(remoteDir->Value()));
 
-					if (data->m_localDir.empty() && data->m_remoteDir.IsEmpty())
+					if (data->m_localDir.empty() && data->m_remoteDir.empty())
 					{
 						delete data;
 						continue;
 					}
 
-					if (!data->m_localDir.empty() && !data->m_remoteDir.IsEmpty())
+					if (!data->m_localDir.empty() && !data->m_remoteDir.empty())
 						data->m_sync = GetTextElementBool(pBookmark, "SyncBrowsing", false);
 
 					pHandler->AddBookmark(name, data);
@@ -100,7 +100,7 @@ CSiteManagerItemData_Site* CSiteManager::ReadServerElement(TiXmlElement *pElemen
 	if (remoteDir)
 		data->m_remoteDir.SetSafePath(ConvLocal(remoteDir->Value()));
 
-	if (!data->m_localDir.empty() && !data->m_remoteDir.IsEmpty())
+	if (!data->m_localDir.empty() && !data->m_remoteDir.empty())
 		data->m_sync = GetTextElementBool(pElement, "SyncBrowsing", false);
 
 	return data;
@@ -109,7 +109,7 @@ CSiteManagerItemData_Site* CSiteManager::ReadServerElement(TiXmlElement *pElemen
 class CSiteManagerXmlHandler_Menu : public CSiteManagerXmlHandler
 {
 public:
-	CSiteManagerXmlHandler_Menu(wxMenu* pMenu, std::map<int, struct CSiteManager::_menu_data> *idMap, bool predefined)
+	CSiteManagerXmlHandler_Menu(wxMenu* pMenu, std::map<int, CSiteManagerItemData_Site*> *idMap, bool predefined)
 		: m_pMenu(pMenu), m_idMap(idMap)
 	{
 		m_added_site = false;
@@ -131,10 +131,10 @@ public:
 
 			// Use same sorting as site tree in site manager
 #ifdef __WXMSW__
-			if (pItem->GetLabel().CmpNoCase(name) > 0)
+			if (pItem->GetItemLabelText().CmpNoCase(name) > 0)
 				break;
 #else
-			if (pItem->GetLabel() > name)
+			if (pItem->GetItemLabelText() > name)
 				break;
 #endif
 		}
@@ -147,11 +147,7 @@ public:
 		m_parents.push_back(m_pMenu);
 		m_childNames.push_back(name);
 		m_paths.push_back(path);
-
-		wxString str(name);
-		str.Replace(_T("\\"), _T("\\\\"));
-		str.Replace(_T("/"), _T("\\/"));
-		path += _T("/") + str;
+		path += _T("/") + CSiteManager::EscapeSegment(name);
 
 		m_pMenu = new wxMenu;
 
@@ -165,14 +161,9 @@ public:
 		newName.Replace(_T("&"), _T("&&"));
 		wxMenuItem* pItem = m_pMenu->Insert(i, wxID_ANY, newName);
 
-		struct CSiteManager::_menu_data menu_data;
-		menu_data.data = data;
-		menu_data.path = data->m_server.GetName();
-		menu_data.path.Replace(_T("\\"), _T("\\\\"));
-		menu_data.path.Replace(_T("/"), _T("\\/"));
-		menu_data.path = path + _T("/") + menu_data.path;
+		data->m_path = path + _T("/") + CSiteManager::EscapeSegment(data->m_server.GetName());
 
-		(*m_idMap)[pItem->GetId()] = menu_data;
+		(*m_idMap)[pItem->GetId()] = data;
 
 		m_added_site = true;
 
@@ -222,7 +213,7 @@ public:
 protected:
 	wxMenu* m_pMenu;
 
-	std::map<int, struct CSiteManager::_menu_data> *m_idMap;
+	std::map<int, CSiteManagerItemData_Site*> *m_idMap;
 
 	std::list<wxMenu*> m_parents;
 	std::list<wxString> m_childNames;
@@ -243,11 +234,11 @@ wxMenu* CSiteManager::GetSitesMenu()
 
 	wxMenu* predefinedSites = GetSitesMenu_Predefined(m_idMap);
 
-	CXmlFile file(_T("sitemanager"));
+	CXmlFile file(wxGetApp().GetSettingsFile(_T("sitemanager")));
 	TiXmlElement* pDocument = file.Load();
 	if (!pDocument)
 	{
-		wxMessageBox(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
+		wxMessageBoxEx(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
 
 		if (!predefinedSites)
 			return predefinedSites;
@@ -303,19 +294,19 @@ wxMenu* CSiteManager::GetSitesMenu()
 
 void CSiteManager::ClearIdMap()
 {
-	for (std::map<int, struct _menu_data>::iterator iter = m_idMap.begin(); iter != m_idMap.end(); ++iter)
-		delete iter->second.data;
+	for (auto iter = m_idMap.begin(); iter != m_idMap.end(); ++iter)
+		delete iter->second;
 
 	m_idMap.clear();
 }
 
-wxMenu* CSiteManager::GetSitesMenu_Predefined(std::map<int, struct _menu_data> &idMap)
+wxMenu* CSiteManager::GetSitesMenu_Predefined(std::map<int, CSiteManagerItemData_Site*> &idMap)
 {
-	const wxString& defaultsDir = wxGetApp().GetDefaultsDir();
-	if (defaultsDir == _T(""))
+	CLocalPath const defaultsDir = wxGetApp().GetDefaultsDir();
+	if (defaultsDir.empty())
 		return 0;
 
-	wxFileName name(defaultsDir, _T("fzdefaults.xml"));
+	wxString const name(defaultsDir.GetPath() + _T("fzdefaults.xml"));
 	CXmlFile file(name);
 
 	TiXmlElement* pDocument = file.Load();
@@ -344,16 +335,15 @@ wxMenu* CSiteManager::GetSitesMenu_Predefined(std::map<int, struct _menu_data> &
 	return pMenu;
 }
 
-CSiteManagerItemData_Site* CSiteManager::GetSiteById(int id, wxString& path)
+CSiteManagerItemData_Site* CSiteManager::GetSiteById(int id)
 {
-	std::map<int, struct _menu_data>::iterator iter = m_idMap.find(id);
+	auto iter = m_idMap.find(id);
 
 	CSiteManagerItemData_Site *pData;
 	if (iter != m_idMap.end())
 	{
-		pData = iter->second.data;
-		iter->second.data = 0;
-		path = iter->second.path;
+		pData = iter->second;
+		iter->second = 0;
 	}
 	else
 		pData = 0;
@@ -394,7 +384,7 @@ bool CSiteManager::UnescapeSitePath(wxString path, std::list<wxString>& result)
 			}
 			else
 			{
-				if (!name.IsEmpty())
+				if (!name.empty())
 					result.push_back(name);
 				name.clear();
 			}
@@ -405,18 +395,34 @@ bool CSiteManager::UnescapeSitePath(wxString path, std::list<wxString>& result)
 	}
 	if (lastBackslash)
 		return false;
-	if (name != _T(""))
+	if (!name.empty())
 		result.push_back(name);
 
 	return !result.empty();
 }
 
+wxString CSiteManager::EscapeSegment(wxString segment)
+{
+	segment.Replace(_T("\\"), _T("\\\\"));
+	segment.Replace(_T("/"), _T("\\/"));
+	return segment;
+}
+
+wxString CSiteManager::BuildPath(wxChar root, std::list<wxString> const& segments)
+{
+	wxString ret = root;
+	for (std::list<wxString>::const_iterator it = segments.begin(); it != segments.end(); ++it)
+		ret += _T("/") + EscapeSegment(*it);
+
+	return ret;
+}
+
 CSiteManagerItemData_Site* CSiteManager::GetSiteByPath(wxString sitePath)
 {
-	wxChar c = sitePath[0];
+	wxChar c = sitePath.empty() ? 0 : sitePath[0];
 	if (c != '0' && c != '1')
 	{
-		wxMessageBox(_("Site path has to begin with 0 or 1."), _("Invalid site path"));
+		wxMessageBoxEx(_("Site path has to begin with 0 or 1."), _("Invalid site path"));
 		return 0;
 	}
 
@@ -427,47 +433,41 @@ CSiteManagerItemData_Site* CSiteManager::GetSiteByPath(wxString sitePath)
 	CInterProcessMutex mutex(MUTEX_SITEMANAGER);
 
 	CXmlFile file;
-	TiXmlElement* pDocument = 0;
-
-	if (c == '0')
-		pDocument = file.Load(_T("sitemanager"));
-	else
-	{
-		const wxString& defaultsDir = wxGetApp().GetDefaultsDir();
-		if (defaultsDir == _T(""))
-		{
-			wxMessageBox(_("Site does not exist."), _("Invalid site path"));
+	if (c == '0') {
+		file.SetFileName(wxGetApp().GetSettingsFile(_T("sitemanager")));
+	}
+	else {
+		CLocalPath const defaultsDir = wxGetApp().GetDefaultsDir();
+		if (defaultsDir.empty()) {
+			wxMessageBoxEx(_("Site does not exist."), _("Invalid site path"));
 			return 0;
 		}
-		wxFileName name(defaultsDir, _T("fzdefaults.xml"));
-		pDocument = file.Load(name);
+		file.SetFileName(defaultsDir.GetPath() + _T("fzdefaults.xml"));
 	}
 
-	if (!pDocument)
-	{
-		wxMessageBox(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
-
+	TiXmlElement* pDocument = file.Load();
+	if (!pDocument) {
+		wxMessageBoxEx(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
 		return 0;
 	}
 
 	TiXmlElement* pElement = pDocument->FirstChildElement("Servers");
-	if (!pElement)
-	{
-		wxMessageBox(_("Site does not exist."), _("Invalid site path"));
+	if (!pElement) {
+		wxMessageBoxEx(_("Site does not exist."), _("Invalid site path"));
 		return 0;
 	}
 
 	std::list<wxString> segments;
-	if (!UnescapeSitePath(sitePath, segments))
+	if (!UnescapeSitePath(sitePath, segments) || segments.empty())
 	{
-		wxMessageBox(_("Site path is malformed."), _("Invalid site path"));
+		wxMessageBoxEx(_("Site path is malformed."), _("Invalid site path"));
 		return 0;
 	}
 
 	TiXmlElement* pChild = GetElementByPath(pElement, segments);
 	if (!pChild)
 	{
-		wxMessageBox(_("Site does not exist."), _("Invalid site path"));
+		wxMessageBoxEx(_("Site does not exist."), _("Invalid site path"));
 		return 0;
 	}
 
@@ -476,6 +476,7 @@ CSiteManagerItemData_Site* CSiteManager::GetSiteByPath(wxString sitePath)
 	{
 		pBookmark = pChild;
 		pChild = pChild->Parent()->ToElement();
+		segments.pop_back();
 	}
 	else
 		pBookmark = 0;
@@ -484,7 +485,7 @@ CSiteManagerItemData_Site* CSiteManager::GetSiteByPath(wxString sitePath)
 
 	if (!data)
 	{
-		wxMessageBox(_("Could not read server item."), _("Invalid site path"));
+		wxMessageBoxEx(_("Could not read server item."), _("Invalid site path"));
 		return 0;
 	}
 
@@ -500,7 +501,7 @@ CSiteManagerItemData_Site* CSiteManager::GetSiteByPath(wxString sitePath)
 		TiXmlText* remoteDir = handle.FirstChildElement("RemoteDir").FirstChild().Text();
 		if (remoteDir)
 			remotePath.SetSafePath(ConvLocal(remoteDir->Value()));
-		if (!localPath.empty() && !remotePath.IsEmpty())
+		if (!localPath.empty() && !remotePath.empty())
 		{
 			data->m_sync = GetTextElementBool(pBookmark, "SyncBrowsing", false);
 		}
@@ -511,11 +512,15 @@ CSiteManagerItemData_Site* CSiteManager::GetSiteByPath(wxString sitePath)
 		data->m_remoteDir = remotePath;
 	}
 
+	data->m_path = BuildPath( c, segments );
+
 	return data;
 }
 
 bool CSiteManager::GetBookmarks(wxString sitePath, std::list<wxString> &bookmarks)
 {
+	if (sitePath.empty())
+		return false;
 	wxChar c = sitePath[0];
 	if (c != '0' && c != '1')
 		return false;
@@ -527,22 +532,21 @@ bool CSiteManager::GetBookmarks(wxString sitePath, std::list<wxString> &bookmark
 	CInterProcessMutex mutex(MUTEX_SITEMANAGER);
 
 	CXmlFile file;
-	TiXmlElement* pDocument = 0;
-
-	if (c == '0')
-		pDocument = file.Load(_T("sitemanager"));
-	else
-	{
-		const wxString& defaultsDir = wxGetApp().GetDefaultsDir();
-		if (defaultsDir == _T(""))
-			return false;
-		pDocument = file.Load(wxFileName(defaultsDir, _T("fzdefaults.xml")));
+	if (c == '0') {
+		file.SetFileName(wxGetApp().GetSettingsFile(_T("sitemanager")));
 	}
-
-	if (!pDocument)
-	{
-		wxMessageBox(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
-
+	else {
+		CLocalPath const defaultsDir = wxGetApp().GetDefaultsDir();
+		if (defaultsDir.empty()) {
+			wxMessageBoxEx(_("Site does not exist."), _("Invalid site path"));
+			return 0;
+		}
+		file.SetFileName(defaultsDir.GetPath() + _T("fzdefaults.xml"));
+	}
+	
+	TiXmlElement* pDocument = file.Load();
+	if (!pDocument) {
+		wxMessageBoxEx(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
 		return false;
 	}
 
@@ -551,13 +555,16 @@ bool CSiteManager::GetBookmarks(wxString sitePath, std::list<wxString> &bookmark
 		return false;
 
 	std::list<wxString> segments;
-	if (!UnescapeSitePath(sitePath, segments))
-	{
-		wxMessageBox(_("Site path is malformed."), _("Invalid site path"));
+	if (!UnescapeSitePath(sitePath, segments)) {
+		wxMessageBoxEx(_("Site path is malformed."), _("Invalid site path"));
 		return 0;
 	}
 
 	TiXmlElement* pChild = GetElementByPath(pElement, segments);
+
+	if (pChild && !strcmp(pChild->Value(), "Bookmark"))
+		pChild = pChild->Parent()->ToElement();
+
 	if (!pChild || strcmp(pChild->Value(), "Server"))
 		return 0;
 
@@ -579,7 +586,7 @@ bool CSiteManager::GetBookmarks(wxString sitePath, std::list<wxString> &bookmark
 		if (remoteDir)
 			remotePath.SetSafePath(ConvLocal(remoteDir->Value()));
 
-		if (localPath.empty() && remotePath.IsEmpty())
+		if (localPath.empty() && remotePath.empty())
 			continue;
 
 		bookmarks.push_back(name);
@@ -594,20 +601,18 @@ wxString CSiteManager::AddServer(CServer server)
 	// to the same file or one is reading while the other one writes.
 	CInterProcessMutex mutex(MUTEX_SITEMANAGER);
 
-	CXmlFile file;
-	TiXmlElement* pDocument = file.Load(_T("sitemanager"));
-
-	if (!pDocument)
-	{
+	CXmlFile file(wxGetApp().GetSettingsFile(_T("sitemanager")));
+	TiXmlElement* pDocument = file.Load();
+	if (!pDocument) {
 		wxString msg = file.GetError() + _T("\n") + _("The server could not be added.");
-		wxMessageBox(msg, _("Error loading xml file"), wxICON_ERROR);
+		wxMessageBoxEx(msg, _("Error loading xml file"), wxICON_ERROR);
 
-		return _T("");
+		return wxString();
 	}
 
 	TiXmlElement* pElement = pDocument->FirstChildElement("Servers");
 	if (!pElement)
-		return _T("");
+		return wxString();
 
 	std::list<wxString> names;
 	for (TiXmlElement* pChild = pElement->FirstChildElement("Server"); pChild; pChild = pChild->NextSiblingElement("Server"))
@@ -648,29 +653,23 @@ wxString CSiteManager::AddServer(CServer server)
 		delete [] utf8;
 	}
 
-	wxString error;
-	if (!file.Save(&error))
-	{
+	if (!file.Save(false)) {
 		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2)
-			return _T("");
+			return wxString();
 
-		wxString msg = wxString::Format(_("Could not write \"%s\", any changes to the Site Manager could not be saved: %s"), file.GetFileName().GetFullPath().c_str(), error.c_str());
-		wxMessageBox(msg, _("Error writing xml file"), wxICON_ERROR);
-		return _T("");
+		wxString msg = wxString::Format(_("Could not write \"%s\", any changes to the Site Manager could not be saved: %s"), file.GetFileName(), file.GetError());
+		wxMessageBoxEx(msg, _("Error writing xml file"), wxICON_ERROR);
+		return wxString();
 	}
 
-	name.Replace(_T("\\"), _T("\\\\"));
-	name.Replace(_T("/"), _T("\\/"));
-
-	return _T("0/") + name;
+	return _T("0/") + EscapeSegment(name);
 }
 
-TiXmlElement* CSiteManager::GetElementByPath(TiXmlElement* pNode, std::list<wxString> &segments)
+TiXmlElement* CSiteManager::GetElementByPath(TiXmlElement* pNode, std::list<wxString> const& segments)
 {
-	while (!segments.empty())
+	for (std::list<wxString>::const_iterator it = segments.begin(); it != segments.end(); ++it)
 	{
-		const wxString segment = segments.front();
-		segments.pop_front();
+		const wxString & segment = *it;
 
 		TiXmlElement* pChild;
 		for (pChild = pNode->FirstChildElement(); pChild; pChild = pChild->NextSiblingElement())
@@ -700,10 +699,11 @@ TiXmlElement* CSiteManager::GetElementByPath(TiXmlElement* pNode, std::list<wxSt
 
 bool CSiteManager::AddBookmark(wxString sitePath, const wxString& name, const wxString &local_dir, const CServerPath &remote_dir, bool sync)
 {
-	if (local_dir.empty() && remote_dir.IsEmpty())
+	if (local_dir.empty() && remote_dir.empty())
 		return false;
 
-	if (sitePath[0] != '0')
+	wxChar c = sitePath.empty() ? 0 : sitePath[0];
+	if (c != '0')
 		return false;
 
 	sitePath = sitePath.Mid(1);
@@ -712,13 +712,11 @@ bool CSiteManager::AddBookmark(wxString sitePath, const wxString& name, const wx
 	// to the same file or one is reading while the other one writes.
 	CInterProcessMutex mutex(MUTEX_SITEMANAGER);
 
-	CXmlFile file;
-	TiXmlElement* pDocument = file.Load(_T("sitemanager"));
-
-	if (!pDocument)
-	{
+	CXmlFile file(wxGetApp().GetSettingsFile(_T("sitemanager")));
+	TiXmlElement* pDocument = file.Load();
+	if (!pDocument) {
 		wxString msg = file.GetError() + _T("\n") + _("The bookmark could not be added.");
-		wxMessageBox(msg, _("Error loading xml file"), wxICON_ERROR);
+		wxMessageBoxEx(msg, _("Error loading xml file"), wxICON_ERROR);
 
 		return false;
 	}
@@ -730,14 +728,14 @@ bool CSiteManager::AddBookmark(wxString sitePath, const wxString& name, const wx
 	std::list<wxString> segments;
 	if (!UnescapeSitePath(sitePath, segments))
 	{
-		wxMessageBox(_("Site path is malformed."), _("Invalid site path"));
+		wxMessageBoxEx(_("Site path is malformed."), _("Invalid site path"));
 		return 0;
 	}
 
 	TiXmlElement* pChild = GetElementByPath(pElement, segments);
 	if (!pChild || strcmp(pChild->Value(), "Server"))
 	{
-		wxMessageBox(_("Site does not exist."), _("Invalid site path"));
+		wxMessageBoxEx(_("Site does not exist."), _("Invalid site path"));
 		return 0;
 	}
 
@@ -754,7 +752,7 @@ bool CSiteManager::AddBookmark(wxString sitePath, const wxString& name, const wx
 
 		if (name == old_name)
 		{
-			wxMessageBox(_("Name of bookmark already exists."), _("New bookmark"), wxICON_EXCLAMATION);
+			wxMessageBoxEx(_("Name of bookmark already exists."), _("New bookmark"), wxICON_EXCLAMATION);
 			return false;
 		}
 		if (name < old_name && !pInsertBefore)
@@ -768,19 +766,17 @@ bool CSiteManager::AddBookmark(wxString sitePath, const wxString& name, const wx
 	AddTextElement(pBookmark, "Name", name);
 	if (!local_dir.empty())
 		AddTextElement(pBookmark, "LocalDir", local_dir);
-	if (!remote_dir.IsEmpty())
+	if (!remote_dir.empty())
 		AddTextElement(pBookmark, "RemoteDir", remote_dir.GetSafePath());
 	if (sync)
 		AddTextElementRaw(pBookmark, "SyncBrowsing", "1");
 
-	wxString error;
-	if (!file.Save(&error))
-	{
+	if (!file.Save(false)) {
 		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2)
 			return true;
 
-		wxString msg = wxString::Format(_("Could not write \"%s\", the selected sites could not be exported: %s"), file.GetFileName().GetFullPath().c_str(), error.c_str());
-		wxMessageBox(msg, _("Error writing xml file"), wxICON_ERROR);
+		wxString msg = wxString::Format(_("Could not write \"%s\", the selected sites could not be exported: %s"), file.GetFileName(), file.GetError());
+		wxMessageBoxEx(msg, _("Error writing xml file"), wxICON_ERROR);
 	}
 
 	return true;
@@ -788,7 +784,8 @@ bool CSiteManager::AddBookmark(wxString sitePath, const wxString& name, const wx
 
 bool CSiteManager::ClearBookmarks(wxString sitePath)
 {
-	if (sitePath[0] != '0')
+	wxChar const c = sitePath.empty() ? 0 : sitePath[0];
+	if (c != '0')
 		return false;
 
 	sitePath = sitePath.Mid(1);
@@ -797,13 +794,11 @@ bool CSiteManager::ClearBookmarks(wxString sitePath)
 	// to the same file or one is reading while the other one writes.
 	CInterProcessMutex mutex(MUTEX_SITEMANAGER);
 
-	CXmlFile file;
-	TiXmlElement* pDocument = file.Load(_T("sitemanager"));
-
-	if (!pDocument)
-	{
+	CXmlFile file(wxGetApp().GetSettingsFile(_T("sitemanager")));
+	TiXmlElement *pDocument = file.Load();
+	if (!pDocument) {
 		wxString msg = file.GetError() + _T("\n") + _("The bookmarks could not be cleared.");
-		wxMessageBox(msg, _("Error loading xml file"), wxICON_ERROR);
+		wxMessageBoxEx(msg, _("Error loading xml file"), wxICON_ERROR);
 
 		return false;
 	}
@@ -813,34 +808,29 @@ bool CSiteManager::ClearBookmarks(wxString sitePath)
 		return false;
 
 	std::list<wxString> segments;
-	if (!UnescapeSitePath(sitePath, segments))
-	{
-		wxMessageBox(_("Site path is malformed."), _("Invalid site path"));
+	if (!UnescapeSitePath(sitePath, segments)) {
+		wxMessageBoxEx(_("Site path is malformed."), _("Invalid site path"));
 		return 0;
 	}
 
 	TiXmlElement* pChild = GetElementByPath(pElement, segments);
-	if (!pChild || strcmp(pChild->Value(), "Server"))
-	{
-		wxMessageBox(_("Site does not exist."), _("Invalid site path"));
+	if (!pChild || strcmp(pChild->Value(), "Server")) {
+		wxMessageBoxEx(_("Site does not exist."), _("Invalid site path"));
 		return 0;
 	}
 
 	TiXmlElement *pBookmark = pChild->FirstChildElement("Bookmark");
-	while (pBookmark)
-	{
+	while (pBookmark) {
 		pChild->RemoveChild(pBookmark);
 		pBookmark = pChild->FirstChildElement("Bookmark");
 	}
 
-	wxString error;
-	if (!file.Save(&error))
-	{
+	if (!file.Save(false)) {
 		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2)
 			return true;
 
-		wxString msg = wxString::Format(_("Could not write \"%s\", the selected sites could not be exported: %s"), file.GetFileName().GetFullPath().c_str(), error.c_str());
-		wxMessageBox(msg, _("Error writing xml file"), wxICON_ERROR);
+		wxString msg = wxString::Format(_("Could not write \"%s\", the selected sites could not be exported: %s"), file.GetFileName(), file.GetError());
+		wxMessageBoxEx(msg, _("Error writing xml file"), wxICON_ERROR);
 	}
 
 	return true;
