@@ -10,6 +10,7 @@
 #include "wrapengine.h"
 #include "xmlfunctions.h"
 
+#include <wx/dcclient.h>
 #include <wx/dnd.h>
 
 #ifdef __WXMSW__
@@ -58,7 +59,7 @@ public:
 
 	virtual bool GetDataHere(void *buf) const { memset(buf, 0, 1); return true; }
 
-	virtual bool SetData(size_t len, const void *buf) { return true; }
+	virtual bool SetData(size_t, const void *) { return true; }
 };
 
 class CSiteManagerDropTarget : public wxDropTarget
@@ -312,7 +313,7 @@ bool CSiteManagerDialog::Create(wxWindow* parent, std::vector<_connected_site> *
 	m_pSiteManagerMutex = new CInterProcessMutex(MUTEX_SITEMANAGERGLOBAL, false);
 	if (m_pSiteManagerMutex->TryLock() == 0)
 	{
-		int answer = wxMessageBox(_("The Site Manager is opened in another instance of FileZilla 3.\nDo you want to continue? Any changes made in the Site Manager won't be saved then."),
+		int answer = wxMessageBoxEx(_("The Site Manager is opened in another instance of FileZilla 3.\nDo you want to continue? Any changes made in the Site Manager won't be saved then."),
 								  _("Site Manager already open"), wxYES_NO | wxICON_QUESTION);
 		if (answer != wxYES)
 			return false;
@@ -365,7 +366,7 @@ bool CSiteManagerDialog::Create(wxWindow* parent, std::vector<_connected_site> *
 #ifdef __WXMAC__
 		width += 20; // Guessed
 #else
-		width += 10;
+		width += 20;
 #endif
 	}
 
@@ -517,7 +518,7 @@ void CSiteManagerDialog::CreateControls(wxWindow* parent)
 	pEncryption->SetSelection(0);
 }
 
-void CSiteManagerDialog::OnOK(wxCommandEvent& event)
+void CSiteManagerDialog::OnOK(wxCommandEvent&)
 {
 	if (!Verify())
 		return;
@@ -531,12 +532,12 @@ void CSiteManagerDialog::OnOK(wxCommandEvent& event)
 	EndModal(wxID_OK);
 }
 
-void CSiteManagerDialog::OnCancel(wxCommandEvent& event)
+void CSiteManagerDialog::OnCancel(wxCommandEvent&)
 {
 	EndModal(wxID_CANCEL);
 }
 
-void CSiteManagerDialog::OnConnect(wxCommandEvent& event)
+void CSiteManagerDialog::OnConnect(wxCommandEvent&)
 {
 	wxTreeCtrl *pTree = XRCCTRL(*this, "ID_SITETREE", wxTreeCtrl);
 	if (!pTree)
@@ -723,12 +724,12 @@ bool CSiteManagerDialog::Load()
 	pTree->SetItemImage(treeId, 1, wxTreeItemIcon_Expanded);
 	pTree->SetItemImage(treeId, 1, wxTreeItemIcon_SelectedExpanded);
 
-	CXmlFile file(_T("sitemanager"));
+	CXmlFile file(wxGetApp().GetSettingsFile(_T("sitemanager")));
 	TiXmlElement* pDocument = file.Load();
 	if (!pDocument)
 	{
 		wxString msg = file.GetError() + _T("\n") + _("Any changes made in the Site Manager will not be saved unless you repair the file.");
-		wxMessageBox(msg, _("Error loading xml file"), wxICON_ERROR);
+		wxMessageBoxEx(msg, _("Error loading xml file"), wxICON_ERROR);
 
 		return false;
 	}
@@ -738,7 +739,7 @@ bool CSiteManagerDialog::Load()
 		return true;
 
 	wxString lastSelection = COptions::Get()->GetOption(OPTION_SITEMANAGER_LASTSELECTED);
-	if (lastSelection[0] == '0')
+	if (!lastSelection.empty() && lastSelection[0] == '0')
 	{
 		if (lastSelection == _T("0"))
 			pTree->SelectItem(treeId);
@@ -776,21 +777,18 @@ bool CSiteManagerDialog::Save(TiXmlElement *pElement /*=0*/, wxTreeItemId treeId
 		// to the same file or one is reading while the other one writes.
 		CInterProcessMutex mutex(MUTEX_SITEMANAGER);
 
-		wxFileName file(COptions::Get()->GetOption(OPTION_DEFAULT_SETTINGSDIR), _T("sitemanager.xml"));
-		CXmlFile xml(file);
+		CXmlFile xml(wxGetApp().GetSettingsFile(_T("sitemanager")));
 
 		TiXmlElement* pDocument = xml.Load();
-		if (!pDocument)
-		{
+		if (!pDocument) {
 			wxString msg = xml.GetError() + _T("\n") + _("Any changes made in the Site Manager could not be saved.");
-			wxMessageBox(msg, _("Error loading xml file"), wxICON_ERROR);
+			wxMessageBoxEx(msg, _("Error loading xml file"), wxICON_ERROR);
 
 			return false;
 		}
 
 		TiXmlElement *pServers = pDocument->FirstChildElement("Servers");
-		while (pServers)
-		{
+		while (pServers) {
 			pDocument->RemoveChild(pServers);
 			pServers = pDocument->FirstChildElement("Servers");
 		}
@@ -801,13 +799,11 @@ bool CSiteManagerDialog::Save(TiXmlElement *pElement /*=0*/, wxTreeItemId treeId
 
 		bool res = Save(pElement, m_ownSites);
 
-		wxString error;
-		if (!xml.Save(&error))
-		{
+		if (!xml.Save(false)) {
 			if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2)
 				return res;
-			wxString msg = wxString::Format(_("Could not write \"%s\", any changes to the Site Manager could not be saved: %s"), file.GetFullPath().c_str(), error.c_str());
-			wxMessageBox(msg, _("Error writing xml file"), wxICON_ERROR);
+			wxString msg = wxString::Format(_("Could not write \"%s\", any changes to the Site Manager could not be saved: %s"), xml.GetFileName(), xml.GetError());
+			wxMessageBoxEx(msg, _("Error writing xml file"), wxICON_ERROR);
 		}
 
 		return res;
@@ -939,10 +935,10 @@ bool CSiteManagerDialog::Verify()
 	if (data->m_type == CSiteManagerItemData::SITE)
 	{
 		const wxString& host = XRCCTRL(*this, "ID_HOST", wxTextCtrl)->GetValue();
-		if (host == _T(""))
+		if (host.empty())
 		{
 			XRCCTRL(*this, "ID_HOST", wxTextCtrl)->SetFocus();
-			wxMessageBox(_("You have to enter a hostname."), _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this);
+			wxMessageBoxEx(_("You have to enter a hostname."), _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this);
 			return false;
 		}
 
@@ -953,7 +949,7 @@ bool CSiteManagerDialog::Verify()
 			logon_type == ACCOUNT)
 		{
 			XRCCTRL(*this, "ID_LOGONTYPE", wxChoice)->SetFocus();
-			wxMessageBox(_("'Account' logontype not supported by selected protocol"), _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this);
+			wxMessageBoxEx(_("'Account' logontype not supported by selected protocol"), _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this);
 			return false;
 		}
 
@@ -972,7 +968,7 @@ bool CSiteManagerDialog::Verify()
 			XRCCTRL(*this, "ID_LOGONTYPE", wxChoice)->SetStringSelection(CServer::GetNameFromLogonType(ASK));
 			XRCCTRL(*this, "ID_PASS", wxTextCtrl)->SetValue(_T(""));
 			logon_type = ASK;
-			wxMessageBox(msg, _("Site Manager - Cannot remember password"), wxICON_INFORMATION, this);
+			wxMessageBoxEx(msg, _("Site Manager - Cannot remember password"), wxICON_INFORMATION, this);
 		}
 
 		// Set selected type
@@ -988,7 +984,7 @@ bool CSiteManagerDialog::Verify()
 		if (!server.ParseUrl(host, port, _T(""), _T(""), error, path))
 		{
 			XRCCTRL(*this, "ID_HOST", wxTextCtrl)->SetFocus();
-			wxMessageBox(error, _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this);
+			wxMessageBoxEx(error, _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this);
 			return false;
 		}
 
@@ -999,10 +995,10 @@ bool CSiteManagerDialog::Verify()
 
 		if (XRCCTRL(*this, "ID_CHARSET_CUSTOM", wxRadioButton)->GetValue())
 		{
-			if (XRCCTRL(*this, "ID_ENCODING", wxTextCtrl)->GetValue() == _T(""))
+			if (XRCCTRL(*this, "ID_ENCODING", wxTextCtrl)->GetValue().empty())
 			{
 				XRCCTRL(*this, "ID_ENCODING", wxTextCtrl)->SetFocus();
-				wxMessageBox(_("Need to specify a character encoding"), _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this);
+				wxMessageBoxEx(_("Need to specify a character encoding"), _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this);
 				return false;
 			}
 		}
@@ -1012,15 +1008,15 @@ bool CSiteManagerDialog::Verify()
 		if (logon_type != ANONYMOUS &&
 			logon_type != ASK &&
 			logon_type != INTERACTIVE &&
-			user == _T(""))
+			user.empty())
 		{
 			XRCCTRL(*this, "ID_USER", wxTextCtrl)->SetFocus();
-			wxMessageBox(_("You have to specify a user name"), _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this);
+			wxMessageBoxEx(_("You have to specify a user name"), _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this);
 			return false;
 		}
 
 		// The way TinyXML handles blanks, we can't use username of only spaces
-		if (user != _T(""))
+		if (!user.empty())
 		{
 			bool space_only = true;
 			for (unsigned int i = 0; i < user.Len(); ++i)
@@ -1034,22 +1030,22 @@ bool CSiteManagerDialog::Verify()
 			if (space_only)
 			{
 				XRCCTRL(*this, "ID_USER", wxTextCtrl)->SetFocus();
-				wxMessageBox(_("Username cannot be a series of spaces"), _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this);
+				wxMessageBoxEx(_("Username cannot be a series of spaces"), _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this);
 				return false;
 			}
 		}
 
 		// Require account for account logon type
 		if (logon_type == ACCOUNT &&
-			XRCCTRL(*this, "ID_ACCOUNT", wxTextCtrl)->GetValue() == _T(""))
+			XRCCTRL(*this, "ID_ACCOUNT", wxTextCtrl)->GetValue().empty())
 		{
 			XRCCTRL(*this, "ID_ACCOUNT", wxTextCtrl)->SetFocus();
-			wxMessageBox(_("You have to enter an account name"), _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this);
+			wxMessageBoxEx(_("You have to enter an account name"), _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this);
 			return false;
 		}
 
 		const wxString remotePathRaw = XRCCTRL(*this, "ID_REMOTEDIR", wxTextCtrl)->GetValue();
-		if (remotePathRaw != _T(""))
+		if (!remotePathRaw.empty())
 		{
 			const wxString serverType = XRCCTRL(*this, "ID_SERVERTYPE", wxChoice)->GetStringSelection();
 
@@ -1058,7 +1054,7 @@ bool CSiteManagerDialog::Verify()
 			if (!remotePath.SetPath(remotePathRaw))
 			{
 				XRCCTRL(*this, "ID_REMOTEDIR", wxTextCtrl)->SetFocus();
-				wxMessageBox(_("Default remote path cannot be parsed. Make sure it is a valid absolute path for the selected server type."), _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this);
+				wxMessageBoxEx(_("Default remote path cannot be parsed. Make sure it is a valid absolute path for the selected server type."), _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this);
 				return false;
 			}
 		}
@@ -1069,7 +1065,7 @@ bool CSiteManagerDialog::Verify()
 			if (remotePathRaw.empty() || localPath.empty())
 			{
 				XRCCTRL(*this, "ID_SYNC", wxCheckBox)->SetFocus();
-				wxMessageBox(_("You need to enter both a local and a remote path to enable synchronized browsing for this site."), _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this);
+				wxMessageBoxEx(_("You need to enter both a local and a remote path to enable synchronized browsing for this site."), _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this);
 				return false;
 			}
 		}
@@ -1082,7 +1078,7 @@ bool CSiteManagerDialog::Verify()
 			return false;
 
 		const wxString remotePathRaw = XRCCTRL(*this, "ID_BOOKMARK_REMOTEDIR", wxTextCtrl)->GetValue();
-		if (remotePathRaw != _T(""))
+		if (!remotePathRaw.empty())
 		{
 			CServerPath remotePath;
 			remotePath.SetType(pServer->m_server.GetType());
@@ -1091,10 +1087,10 @@ bool CSiteManagerDialog::Verify()
 				XRCCTRL(*this, "ID_BOOKMARK_REMOTEDIR", wxTextCtrl)->SetFocus();
 				wxString msg;
 				if (pServer->m_server.GetType() != DEFAULT)
-					msg = wxString::Format(_("Remote path cannot be parsed. Make sure it is a valid absolute path and is supported by the servertype (%s) selected on the parent site."), CServer::GetNameFromServerType(pServer->m_server.GetType()).c_str());
+					msg = wxString::Format(_("Remote path cannot be parsed. Make sure it is a valid absolute path and is supported by the servertype (%s) selected on the parent site."), CServer::GetNameFromServerType(pServer->m_server.GetType()));
 				else
 					msg = _("Remote path cannot be parsed. Make sure it is a valid absolute path.");
-				wxMessageBox(msg, _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this);
+				wxMessageBoxEx(msg, _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this);
 				return false;
 			}
 		}
@@ -1104,7 +1100,7 @@ bool CSiteManagerDialog::Verify()
 		if (remotePathRaw.empty() && localPath.empty())
 		{
 			XRCCTRL(*this, "ID_BOOKMARK_LOCALDIR", wxTextCtrl)->SetFocus();
-			wxMessageBox(_("You need to enter at least one path, empty bookmarks are not supported."), _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this);
+			wxMessageBoxEx(_("You need to enter at least one path, empty bookmarks are not supported."), _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this);
 			return false;
 		}
 
@@ -1113,7 +1109,7 @@ bool CSiteManagerDialog::Verify()
 			if (remotePathRaw.empty() || localPath.empty())
 			{
 				XRCCTRL(*this, "ID_BOOKMARK_SYNC", wxCheckBox)->SetFocus();
-				wxMessageBox(_("You need to enter both a local and a remote path to enable synchronized browsing for this bookmark."), _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this);
+				wxMessageBoxEx(_("You need to enter both a local and a remote path to enable synchronized browsing for this bookmark."), _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this);
 				return false;
 			}
 		}
@@ -1187,7 +1183,7 @@ void CSiteManagerDialog::OnEndLabelEdit(wxTreeEvent& event)
 			continue;
 		if (!name.CmpNoCase(pTree->GetItemText(child)))
 		{
-			wxMessageBox(_("Name already exists"), _("Cannot rename entry"), wxICON_EXCLAMATION, this);
+			wxMessageBoxEx(_("Name already exists"), _("Cannot rename entry"), wxICON_EXCLAMATION, this);
 			event.Veto();
 			return;
 		}
@@ -1344,7 +1340,7 @@ bool CSiteManagerDialog::UpdateServer(CSiteManagerItemData_Site &server, const w
 	XRCCTRL(*this, "ID_PORT", wxTextCtrl)->GetValue().ToULong(&port);
 	wxString host = XRCCTRL(*this, "ID_HOST", wxTextCtrl)->GetValue();
 	// SetHost does not accept URL syntax
-	if (host[0] == '[')
+	if (!host.empty() && host[0] == '[')
 	{
 		host.RemoveLast();
 		host = host.Mid(1);
@@ -1438,9 +1434,9 @@ bool CSiteManagerDialog::GetServer(CSiteManagerItemData_Site& data)
 		data = *pSiteData;
 		if (!pData->m_localDir.empty())
 			data.m_localDir = pData->m_localDir;
-		if (!pData->m_remoteDir.IsEmpty())
+		if (!pData->m_remoteDir.empty())
 			data.m_remoteDir = pData->m_remoteDir;
-		if (data.m_localDir.empty() || data.m_remoteDir.IsEmpty())
+		if (data.m_localDir.empty() || data.m_remoteDir.empty())
 			data.m_sync = false;
 		else
 			data.m_sync = pData->m_sync;
@@ -1448,6 +1444,8 @@ bool CSiteManagerDialog::GetServer(CSiteManagerItemData_Site& data)
 	else
 		data = *(CSiteManagerItemData_Site *)pData;
 
+	data.m_path = GetSitePath(item);
+
 	return true;
 }
 
@@ -1745,7 +1743,7 @@ void CSiteManagerDialog::OnCopySite(wxCommandEvent& event)
 		parent = pTree->GetItemParent(item);
 
 	const wxString name = pTree->GetItemText(item);
-	wxString newName = wxString::Format(_("Copy of %s"), name.c_str());
+	wxString newName = wxString::Format(_("Copy of %s"), name);
 	int index = 2;
 	for (;;)
 	{
@@ -1768,7 +1766,7 @@ void CSiteManagerDialog::OnCopySite(wxCommandEvent& event)
 		if (!found)
 			break;
 
-		newName = wxString::Format(_("Copy (%d) of %s"), ++index, name.c_str());
+		newName = wxString::Format(_("Copy (%d) of %s"), ++index, name);
 	}
 
 	wxTreeItemId newItem;
@@ -1800,12 +1798,11 @@ void CSiteManagerDialog::OnCopySite(wxCommandEvent& event)
 
 bool CSiteManagerDialog::LoadDefaultSites()
 {
-	const wxString& defaultsDir = wxGetApp().GetDefaultsDir();
-	if (defaultsDir == _T(""))
+	CLocalPath const defaultsDir = wxGetApp().GetDefaultsDir();
+	if (defaultsDir.empty())
 		return false;
 
-	wxFileName name(defaultsDir, _T("fzdefaults.xml"));
-	CXmlFile file(name);
+	CXmlFile file(defaultsDir.GetPath() + _T("fzdefaults.xml"));
 
 	TiXmlElement* pDocument = file.Load();
 	if (!pDocument)
@@ -1828,7 +1825,7 @@ bool CSiteManagerDialog::LoadDefaultSites()
 	pTree->SetItemImage(m_predefinedSites, 1, wxTreeItemIcon_SelectedExpanded);
 
 	wxString lastSelection = COptions::Get()->GetOption(OPTION_SITEMANAGER_LASTSELECTED);
-	if (lastSelection[0] == '1')
+	if (!lastSelection.empty() && lastSelection[0] == '1')
 	{
 		if (lastSelection == _T("1"))
 			pTree->SelectItem(m_predefinedSites);
@@ -1868,6 +1865,7 @@ void CSiteManagerDialog::OnBeginDrag(wxTreeEvent& event)
 		event.Veto();
 		return;
 	}
+	UpdateItem();
 
 	wxTreeCtrl *pTree = XRCCTRL(*this, "ID_SITETREE", wxTreeCtrl);
 	if (!pTree)
@@ -1901,6 +1899,8 @@ void CSiteManagerDialog::OnBeginDrag(wxTreeEvent& event)
 	source.DoDragDrop(predefined ? wxDrag_CopyOnly : wxDrag_DefaultMove);
 
 	m_dropSource = wxTreeItemId();
+
+	SetCtrlState();
 }
 
 struct itempair
@@ -1957,7 +1957,7 @@ bool CSiteManagerDialog::MoveItems(wxTreeItemId source, wxTreeItemId target, boo
 
 		if (!sourceName.CmpNoCase(childName))
 		{
-			wxMessageBox(_("An item with the same name as the dragged item already exists at the target location."), _("Failed to copy or move sites"), wxICON_INFORMATION);
+			wxMessageBoxEx(_("An item with the same name as the dragged item already exists at the target location."), _("Failed to copy or move sites"), wxICON_INFORMATION);
 			return false;
 		}
 
@@ -2030,7 +2030,7 @@ bool CSiteManagerDialog::MoveItems(wxTreeItemId source, wxTreeItemId target, boo
 		pTree->Delete(source);
 	}
 
-	for (std::list<wxTreeItemId>::iterator iter = expand.begin(); iter != expand.end(); ++iter)
+	for (auto iter = expand.begin(); iter != expand.end(); ++iter)
 		pTree->Expand(*iter);
 
 	pTree->Expand(target);
@@ -2133,32 +2133,7 @@ void CSiteManagerDialog::AddNewBookmark(wxTreeItemId parent)
 
 void CSiteManagerDialog::RememberLastSelected()
 {
-	wxTreeCtrl *pTree = XRCCTRL(*this, "ID_SITETREE", wxTreeCtrl);
-	if (!pTree)
-		return;
-
-	wxTreeItemId item = pTree->GetSelection();
-	if (!item.IsOk())
-		return;
-
-	wxString path;
-	while (item != pTree->GetRootItem() && item != m_ownSites && item != m_predefinedSites)
-	{
-		wxString name = pTree->GetItemText(item);
-		name.Replace(_T("\\"), _T("\\\\"));
-		name.Replace(_T("/"), _T("\\/"));
-
-		path = name + _T("/") + path;
-
-		item = pTree->GetItemParent(item);
-	}
-
-	if (item == m_predefinedSites)
-		path = _T("1") + path;
-	else
-		path = _T("0") + path;
-
-	COptions::Get()->SetOption(OPTION_SITEMANAGER_LASTSELECTED, path);
+	COptions::Get()->SetOption(OPTION_SITEMANAGER_LASTSELECTED, GetSitePath(false));
 }
 
 void CSiteManagerDialog::OnContextMenu(wxTreeEvent& event)
@@ -2173,7 +2148,7 @@ void CSiteManagerDialog::OnContextMenu(wxTreeEvent& event)
 	delete pMenu;
 }
 
-void CSiteManagerDialog::OnExportSelected(wxCommandEvent& event)
+void CSiteManagerDialog::OnExportSelected(wxCommandEvent&)
 {
 	wxFileDialog dlg(this, _("Select file for exported sites"), _T(""),
 					_T("sites.xml"), _T("XML files (*.xml)|*.xml"),
@@ -2182,23 +2157,20 @@ void CSiteManagerDialog::OnExportSelected(wxCommandEvent& event)
 	if (dlg.ShowModal() != wxID_OK)
 		return;
 
-	wxFileName fn(dlg.GetPath());
-	CXmlFile xml(fn);
+	CXmlFile xml(dlg.GetPath());
 
 	TiXmlElement* exportRoot = xml.CreateEmpty();
 
 	TiXmlElement* pServers = exportRoot->LinkEndChild(new TiXmlElement("Servers"))->ToElement();
 	SaveChild(pServers, m_contextMenuItem);
 
-	wxString error;
-	if (!xml.Save(&error))
-	{
-		wxString msg = wxString::Format(_("Could not write \"%s\", the selected sites could not be exported: %s"), fn.GetFullPath().c_str(), error.c_str());
-		wxMessageBox(msg, _("Error writing xml file"), wxICON_ERROR);
+	if (!xml.Save(false)) {
+		wxString msg = wxString::Format(_("Could not write \"%s\", the selected sites could not be exported: %s"), xml.GetFileName(), xml.GetError());
+		wxMessageBoxEx(msg, _("Error writing xml file"), wxICON_ERROR);
 	}
 }
 
-void CSiteManagerDialog::OnBookmarkBrowse(wxCommandEvent& event)
+void CSiteManagerDialog::OnBookmarkBrowse(wxCommandEvent&)
 {
 	wxTreeCtrl *pTree = XRCCTRL(*this, "ID_SITETREE", wxTreeCtrl);
 	if (!pTree)
@@ -2219,7 +2191,7 @@ void CSiteManagerDialog::OnBookmarkBrowse(wxCommandEvent& event)
 	XRCCTRL(*this, "ID_BOOKMARK_LOCALDIR", wxTextCtrl)->SetValue(dlg.GetPath());
 }
 
-void CSiteManagerDialog::OnNewBookmark(wxCommandEvent& event)
+void CSiteManagerDialog::OnNewBookmark(wxCommandEvent&)
 {
 	wxTreeCtrl *pTree = XRCCTRL(*this, "ID_SITETREE", wxTreeCtrl);
 	if (!pTree)
@@ -2241,51 +2213,45 @@ void CSiteManagerDialog::OnNewBookmark(wxCommandEvent& event)
 	AddNewBookmark(item);
 }
 
-wxString CSiteManagerDialog::GetSitePath(wxTreeItemId item)
+wxString CSiteManagerDialog::GetSitePath(wxTreeItemId item, bool stripBookmark)
 {
 	wxTreeCtrl *pTree = XRCCTRL(*this, "ID_SITETREE", wxTreeCtrl);
 	if (!pTree)
-		return _T("");
+		return wxString();
 
 	CSiteManagerItemData* pData = reinterpret_cast<CSiteManagerItemData* >(pTree->GetItemData(item));
 	if (!pData)
-		return _T("");
+		return wxString();
 
-	if (pData->m_type == CSiteManagerItemData::BOOKMARK)
+	if (stripBookmark && pData->m_type == CSiteManagerItemData::BOOKMARK)
 		item = pTree->GetItemParent(item);
 
 	wxString path;
 	while (item)
 	{
 		if (item == m_predefinedSites)
-			return _T("1/") + path;
+			return _T("1") + path;
 		else if (item == m_ownSites)
-			return _T("0/") + path;
-		wxString segment = pTree->GetItemText(item);
-		segment.Replace(_T("\\"), _T("\\\\"));
-		segment.Replace(_T("/"), _T("\\/"));
-		if (path != _T(""))
-			path = segment + _T("/") + path;
-		else
-			path = segment;
+			return _T("0") + path;
+		path = _T("/") + CSiteManager::EscapeSegment(pTree->GetItemText(item)) + path;
 
 		item = pTree->GetItemParent(item);
 	}
 
-	return _T("0/") + path;
+	return _T("0") + path;
 }
 
-wxString CSiteManagerDialog::GetSitePath()
+wxString CSiteManagerDialog::GetSitePath(bool stripBookmark)
 {
 	wxTreeCtrl *pTree = XRCCTRL(*this, "ID_SITETREE", wxTreeCtrl);
 	if (!pTree)
-		return _T("");
+		return wxString();
 
 	wxTreeItemId item = pTree->GetSelection();
 	if (!item.IsOk())
-		return _T("");
+		return wxString();
 
-	return GetSitePath(item);
+	return GetSitePath(item, stripBookmark);
 }
 
 void CSiteManagerDialog::SetProtocol(ServerProtocol protocol)
@@ -2345,6 +2311,4 @@ ServerProtocol CSiteManagerDialog::GetProtocol() const
 /*	case 3:
 		return INSECURE_FTP;*/
 	}
-
-	return FTP;
 }
