@@ -10,11 +10,7 @@
 	#ifndef SHGetIconOverlayIndex
 		extern "C" int WINAPI SHGetIconOverlayIndexW(LPCWSTR pszIconPath, int iIconIndex);
 		extern "C" int WINAPI SHGetIconOverlayIndexA(LPCSTR pszIconPath, int iIconIndex);
-		#if wxUSE_UNICODE
-			#define SHGetIconOverlayIndex SHGetIconOverlayIndexW
-		#else
-			#define SHGetIconOverlayIndex SHGetIconOverlayIndexA
-		#endif
+		#define SHGetIconOverlayIndex SHGetIconOverlayIndexW
 	#endif
 #endif
 #ifndef __WXMSW__
@@ -166,22 +162,22 @@ wxBitmap PrepareIcon(wxIcon icon, wxSize size)
 }
 #endif
 
-int CSystemImageList::GetIconIndex(enum filetype type, const wxString& fileName /*=_T("")*/, bool physical /*=true*/, bool symlink /*=false*/)
+int CSystemImageList::GetIconIndex(iconType type, const wxString& fileName /*=_T("")*/, bool physical /*=true*/, bool symlink /*=false*/)
 {
 	if (!m_pImageList)
 		return -1;
 
 #ifdef __WXMSW__
-	if (fileName == _T(""))
+	if (fileName.empty())
 		physical = false;
 
 	SHFILEINFO shFinfo;
 	memset(&shFinfo, 0, sizeof(SHFILEINFO));
-	if (SHGetFileInfo(fileName != _T("") ? fileName : _T("{B97D3074-1830-4b4a-9D8A-17A38B074052}"),
-		(type != file) ? FILE_ATTRIBUTE_DIRECTORY : FILE_ATTRIBUTE_NORMAL,
+	if (SHGetFileInfo(!fileName.empty() ? fileName : _T("{B97D3074-1830-4b4a-9D8A-17A38B074052}"),
+		(type != iconType::file) ? FILE_ATTRIBUTE_DIRECTORY : FILE_ATTRIBUTE_NORMAL,
 		&shFinfo,
 		sizeof(SHFILEINFO),
-		SHGFI_ICON | ((type == opened_dir) ? SHGFI_OPENICON : 0) | ((physical) ? 0 : SHGFI_USEFILEATTRIBUTES) ) )
+		SHGFI_ICON | ((type == iconType::opened_dir) ? SHGFI_OPENICON : 0) | ((physical) ? 0 : SHGFI_USEFILEATTRIBUTES) ) )
 	{
 		int icon = shFinfo.iIcon;
 		// we only need the index from the system image list
@@ -192,30 +188,30 @@ int CSystemImageList::GetIconIndex(enum filetype type, const wxString& fileName
 	int icon;
 	switch (type)
 	{
-	case file:
+	case iconType::file:
 	default:
 		icon = symlink ? 3 : 0;
 		break;
-	case dir:
+	case iconType::dir:
 		return symlink ? 4 : 1;
-	case opened_dir:
+	case iconType::opened_dir:
 		return symlink ? 5 : 2;
 	}
 
 	wxFileName fn(fileName);
 	wxString ext = fn.GetExt();
-	if (ext == _T(""))
+	if (ext.empty())
 		return icon;
 
 	if (symlink)
 	{
-		std::map<wxString, int>::iterator cacheIter = m_iconCache.find(ext);
+		auto cacheIter = m_iconCache.find(ext);
 		if (cacheIter != m_iconCache.end())
 			return cacheIter->second;
 	}
 	else
 	{
-		std::map<wxString, int>::iterator cacheIter = m_iconSymlinkCache.find(ext);
+		auto cacheIter = m_iconSymlinkCache.find(ext);
 		if (cacheIter != m_iconSymlinkCache.end())
 			return cacheIter->second;
 	}
