@@ -1,66 +1,68 @@
 #include <filezilla.h>
+#include "filezillaapp.h"
 #include "verifycertdialog.h"
+#include <wx/statbox.h>
 #include <wx/tokenzr.h>
 #include "dialogex.h"
 #include "ipcmutex.h"
 #include "Options.h"
 
+CVerifyCertDialog::CVerifyCertDialog()
+	: m_xmlFile(wxGetApp().GetSettingsFile(_T("trustedcerts")))
+{
+}
+
 CVerifyCertDialog::~CVerifyCertDialog()
 {
-	for (std::list<t_certData>::iterator iter = m_trustedCerts.begin(); iter != m_trustedCerts.end(); ++iter)
+	for (auto iter = m_trustedCerts.begin(); iter != m_trustedCerts.end(); ++iter)
 		delete [] iter->data;
-	for (std::list<t_certData>::iterator iter = m_sessionTrustedCerts.begin(); iter != m_sessionTrustedCerts.end(); ++iter)
+	for (auto iter = m_sessionTrustedCerts.begin(); iter != m_sessionTrustedCerts.end(); ++iter)
 		delete [] iter->data;
 }
 
 bool CVerifyCertDialog::DisplayCert(wxDialogEx* pDlg, const CCertificate& cert)
 {
 	bool warning = false;
-	if (cert.GetActivationTime().IsValid())
-	{
-		if (cert.GetActivationTime() > wxDateTime::Now())
-		{
-			pDlg->SetLabel(XRCID("ID_ACTIVATION_TIME"), wxString::Format(_("%s - Not yet valid!"), cert.GetActivationTime().FormatDate().c_str()));
+	if (cert.GetActivationTime().IsValid()) {
+		if (cert.GetActivationTime() > wxDateTime::Now()) {
+			pDlg->SetChildLabel(XRCID("ID_ACTIVATION_TIME"), wxString::Format(_("%s - Not yet valid!"), cert.GetActivationTime().FormatDate()));
 			warning = true;
 		}
 		else
-			pDlg->SetLabel(XRCID("ID_ACTIVATION_TIME"), cert.GetActivationTime().FormatDate());
+			pDlg->SetChildLabel(XRCID("ID_ACTIVATION_TIME"), cert.GetActivationTime().FormatDate());
+	}
+	else {
+		warning = true;
+		pDlg->SetChildLabel(XRCID("ID_ACTIVATION_TIME"), _("Invalid date"));
+	}
+
+	if (cert.GetExpirationTime().IsValid()) {
+		if (cert.GetExpirationTime() < wxDateTime::Now()) {
+			pDlg->SetChildLabel(XRCID("ID_EXPIRATION_TIME"), wxString::Format(_("%s - Certificate expired!"), cert.GetExpirationTime().FormatDate()));
+			warning = true;
+		}
+		else
+			pDlg->SetChildLabel(XRCID("ID_EXPIRATION_TIME"), cert.GetExpirationTime().FormatDate());
 	}
 	else
 	{
 		warning = true;
-		pDlg->SetLabel(XRCID("ID_ACTIVATION_TIME"), _("Invalid date"));
+		pDlg->SetChildLabel(XRCID("ID_EXPIRATION_TIME"), _("Invalid date"));
 	}
 
-	if (cert.GetExpirationTime().IsValid())
-	{
-		if (cert.GetExpirationTime() < wxDateTime::Now())
-		{
-			pDlg->SetLabel(XRCID("ID_EXPIRATION_TIME"), wxString::Format(_("%s - Certificate expired!"), cert.GetExpirationTime().FormatDate().c_str()));
-			warning = true;
-		}
-		else
-			pDlg->SetLabel(XRCID("ID_EXPIRATION_TIME"), cert.GetExpirationTime().FormatDate());
-	}
+	if (!cert.GetSerial().empty())
+		pDlg->SetChildLabel(XRCID("ID_SERIAL"), cert.GetSerial());
 	else
-	{
-		warning = true;
-		pDlg->SetLabel(XRCID("ID_EXPIRATION_TIME"), _("Invalid date"));
-	}
+		pDlg->SetChildLabel(XRCID("ID_SERIAL"), _("None"));
 
-	if (cert.GetSerial() != _T(""))
-		pDlg->SetLabel(XRCID("ID_SERIAL"), cert.GetSerial());
-	else
-		pDlg->SetLabel(XRCID("ID_SERIAL"), _("None"));
+	pDlg->SetChildLabel(XRCID("ID_PKALGO"), wxString::Format(_("%s with %d bits"), cert.GetPkAlgoName(), cert.GetPkAlgoBits()));
+	pDlg->SetChildLabel(XRCID("ID_SIGNALGO"), cert.GetSignatureAlgorithm());
 
-	pDlg->SetLabel(XRCID("ID_PKALGO"), wxString::Format(_("%s with %d bits"), cert.GetPkAlgoName().c_str(), cert.GetPkAlgoBits()));
-	pDlg->SetLabel(XRCID("ID_SIGNALGO"), cert.GetSignatureAlgorithm());
+	pDlg->SetChildLabel(XRCID("ID_FINGERPRINT_MD5"), cert.GetFingerPrintMD5());
+	pDlg->SetChildLabel(XRCID("ID_FINGERPRINT_SHA1"), cert.GetFingerPrintSHA1());
 
-	pDlg->SetLabel(XRCID("ID_FINGERPRINT_MD5"), cert.GetFingerPrintMD5());
-	pDlg->SetLabel(XRCID("ID_FINGERPRINT_SHA1"), cert.GetFingerPrintSHA1());
-
-	ParseDN(pDlg, cert.GetSubject(), m_pSubjectSizer);
-	ParseDN(pDlg, cert.GetIssuer(), m_pIssuerSizer);
+	ParseDN(XRCCTRL(*pDlg, "ID_SUBJECT_BOX", wxStaticBox), cert.GetSubject(), m_pSubjectSizer);
+	ParseDN(XRCCTRL(*pDlg, "ID_ISSUER_BOX", wxStaticBox), cert.GetIssuer(), m_pIssuerSizer);
 
 	return warning;
 }
@@ -100,13 +102,13 @@ void CVerifyCertDialog::ShowVerificationDialog(CCertificateNotification* pNotifi
 		pChoice->Connect(wxEVT_COMMAND_CHOICE_SELECTED, wxCommandEventHandler(CVerifyCertDialog::OnCertificateChoice), 0, this);
 	}
 
-	m_pDlg->SetLabel(XRCID("ID_HOST"), wxString::Format(_T("%s:%d"), pNotification->GetHost().c_str(), pNotification->GetPort()));
+	m_pDlg->SetChildLabel(XRCID("ID_HOST"), wxString::Format(_T("%s:%d"), pNotification->GetHost(), pNotification->GetPort()));
 
 	m_pSubjectSizer = XRCCTRL(*m_pDlg, "ID_SUBJECT_DUMMY", wxStaticText)->GetContainingSizer();
-	XRCCTRL(*m_pDlg, "ID_SUBJECT_DUMMY", wxStaticText)->Destroy();
+	m_pSubjectSizer->Clear(true);
 
 	m_pIssuerSizer = XRCCTRL(*m_pDlg, "ID_ISSUER_DUMMY", wxStaticText)->GetContainingSizer();
-	XRCCTRL(*m_pDlg, "ID_ISSUER_DUMMY", wxStaticText)->Destroy();
+	m_pIssuerSizer->Clear(true);
 
 	wxSize minSize(0, 0);
 	for (unsigned int i = 0; i < m_certificates.size(); ++i)
@@ -120,10 +122,10 @@ void CVerifyCertDialog::ShowVerificationDialog(CCertificateNotification* pNotifi
 
 	bool warning = DisplayCert(m_pDlg, m_certificates[0]);
 
-	m_pDlg->SetLabel(XRCID("ID_PROTOCOL"), pNotification->GetProtocol());
-	m_pDlg->SetLabel(XRCID("ID_KEYEXCHANGE"), pNotification->GetKeyExchange());
-	m_pDlg->SetLabel(XRCID("ID_CIPHER"), pNotification->GetSessionCipher());
-	m_pDlg->SetLabel(XRCID("ID_MAC"), pNotification->GetSessionMac());
+	m_pDlg->SetChildLabel(XRCID("ID_PROTOCOL"), pNotification->GetProtocol());
+	m_pDlg->SetChildLabel(XRCID("ID_KEYEXCHANGE"), pNotification->GetKeyExchange());
+	m_pDlg->SetChildLabel(XRCID("ID_CIPHER"), pNotification->GetSessionCipher());
+	m_pDlg->SetChildLabel(XRCID("ID_MAC"), pNotification->GetSessionMac());
 
 	if (warning)
 	{
@@ -166,7 +168,7 @@ void CVerifyCertDialog::ShowVerificationDialog(CCertificateNotification* pNotifi
 	m_pDlg = 0;
 }
 
-void CVerifyCertDialog::ParseDN(wxDialog* pDlg, const wxString& dn, wxSizer* pSizer)
+void CVerifyCertDialog::ParseDN(wxWindow* parent, const wxString& dn, wxSizer* pSizer)
 {
 	pSizer->Clear(true);
 
@@ -176,22 +178,22 @@ void CVerifyCertDialog::ParseDN(wxDialog* pDlg, const wxString& dn, wxSizer* pSi
 	while (tokens.HasMoreTokens())
 		tokenlist.push_back(tokens.GetNextToken());
 
-	ParseDN_by_prefix(pDlg, tokenlist, _T("CN"), _("Common name:"), pSizer);
-	ParseDN_by_prefix(pDlg, tokenlist, _T("O"), _("Organization:"), pSizer);
-	ParseDN_by_prefix(pDlg, tokenlist, _T("2.5.4.15"), _("Business category:"), pSizer, true);
-	ParseDN_by_prefix(pDlg, tokenlist, _T("OU"), _("Unit:"), pSizer);
-	ParseDN_by_prefix(pDlg, tokenlist, _T("T"), _("Title:"), pSizer);
-	ParseDN_by_prefix(pDlg, tokenlist, _T("C"), _("Country:"), pSizer);
-	ParseDN_by_prefix(pDlg, tokenlist, _T("ST"), _("State or province:"), pSizer);
-	ParseDN_by_prefix(pDlg, tokenlist, _T("L"), _("Locality:"), pSizer);
-	ParseDN_by_prefix(pDlg, tokenlist, _T("2.5.4.17"), _("Postal code:"), pSizer, true);
-	ParseDN_by_prefix(pDlg, tokenlist, _T("postalCode"), _("Postal code:"), pSizer, true);
-	ParseDN_by_prefix(pDlg, tokenlist, _T("STREET"), _("Street:"), pSizer);
-	ParseDN_by_prefix(pDlg, tokenlist, _T("EMAIL"), _("E-Mail:"), pSizer);
-	ParseDN_by_prefix(pDlg, tokenlist, _T("serialNumber"), _("Serial number:"), pSizer);
-	ParseDN_by_prefix(pDlg, tokenlist, _T("1.3.6.1.4.1.311.60.2.1.3"), _("Jurisdiction country:"), pSizer, true);
-	ParseDN_by_prefix(pDlg, tokenlist, _T("1.3.6.1.4.1.311.60.2.1.2"), _("Jurisdiction state or province:"), pSizer, true);
-	ParseDN_by_prefix(pDlg, tokenlist, _T("1.3.6.1.4.1.311.60.2.1.1"), _("Jurisdiction locality:"), pSizer, true);
+	ParseDN_by_prefix(parent, tokenlist, _T("CN"), _("Common name:"), pSizer);
+	ParseDN_by_prefix(parent, tokenlist, _T("O"), _("Organization:"), pSizer);
+	ParseDN_by_prefix(parent, tokenlist, _T("2.5.4.15"), _("Business category:"), pSizer, true);
+	ParseDN_by_prefix(parent, tokenlist, _T("OU"), _("Unit:"), pSizer);
+	ParseDN_by_prefix(parent, tokenlist, _T("T"), _("Title:"), pSizer);
+	ParseDN_by_prefix(parent, tokenlist, _T("C"), _("Country:"), pSizer);
+	ParseDN_by_prefix(parent, tokenlist, _T("ST"), _("State or province:"), pSizer);
+	ParseDN_by_prefix(parent, tokenlist, _T("L"), _("Locality:"), pSizer);
+	ParseDN_by_prefix(parent, tokenlist, _T("2.5.4.17"), _("Postal code:"), pSizer, true);
+	ParseDN_by_prefix(parent, tokenlist, _T("postalCode"), _("Postal code:"), pSizer, true);
+	ParseDN_by_prefix(parent, tokenlist, _T("STREET"), _("Street:"), pSizer);
+	ParseDN_by_prefix(parent, tokenlist, _T("EMAIL"), _("E-Mail:"), pSizer);
+	ParseDN_by_prefix(parent, tokenlist, _T("serialNumber"), _("Serial number:"), pSizer);
+	ParseDN_by_prefix(parent, tokenlist, _T("1.3.6.1.4.1.311.60.2.1.3"), _("Jurisdiction country:"), pSizer, true);
+	ParseDN_by_prefix(parent, tokenlist, _T("1.3.6.1.4.1.311.60.2.1.2"), _("Jurisdiction state or province:"), pSizer, true);
+	ParseDN_by_prefix(parent, tokenlist, _T("1.3.6.1.4.1.311.60.2.1.1"), _("Jurisdiction locality:"), pSizer, true);
 
 	if (!tokenlist.empty())
 	{
@@ -199,12 +201,12 @@ void CVerifyCertDialog::ParseDN(wxDialog* pDlg, const wxString& dn, wxSizer* pSi
 		for (std::list<wxString>::const_iterator iter = ++tokenlist.begin(); iter != tokenlist.end(); ++iter)
 			value += _T(",") + *iter;
 
-		pSizer->Add(new wxStaticText(pDlg, wxID_ANY, _("Other:")));
-		pSizer->Add(new wxStaticText(pDlg, wxID_ANY, value));
+		pSizer->Add(new wxStaticText(parent, wxID_ANY, _("Other:")));
+		pSizer->Add(new wxStaticText(parent, wxID_ANY, value));
 	}
 }
 
-void CVerifyCertDialog::ParseDN_by_prefix(wxDialog* pDlg, std::list<wxString>& tokens, wxString prefix, const wxString& name, wxSizer* pSizer, bool decode /*=false*/)
+void CVerifyCertDialog::ParseDN_by_prefix(wxWindow* parent, std::list<wxString>& tokens, wxString prefix, const wxString& name, wxSizer* pSizer, bool decode /*=false*/)
 {
 	prefix += _T("=");
 	int len = prefix.Length();
@@ -213,7 +215,7 @@ void CVerifyCertDialog::ParseDN_by_prefix(wxDialog* pDlg, std::list<wxString>& t
 
 	bool append = false;
 
-	std::list<wxString>::iterator iter = tokens.begin();
+	auto iter = tokens.begin();
 	while (iter != tokens.end())
 	{
 		if (!append)
@@ -224,7 +226,7 @@ void CVerifyCertDialog::ParseDN_by_prefix(wxDialog* pDlg, std::list<wxString>& t
 				continue;
 			}
 
-			if (value != _T(""))
+			if (!value.empty())
 				value += _T("\n");
 		}
 		else
@@ -242,17 +244,17 @@ void CVerifyCertDialog::ParseDN_by_prefix(wxDialog* pDlg, std::list<wxString>& t
 			len = 0;
 		}
 
-		std::list<wxString>::iterator remove = iter++;
+		auto remove = iter++;
 		tokens.erase(remove);
 	}
 
 	if (decode)
 		value = DecodeValue(value);
 
-	if (value != _T(""))
+	if (!value.empty())
 	{
-		pSizer->Add(new wxStaticText(pDlg, wxID_ANY, name));
-		pSizer->Add(new wxStaticText(pDlg, wxID_ANY, value));
+		pSizer->Add(new wxStaticText(parent, wxID_ANY, name));
+		pSizer->Add(new wxStaticText(parent, wxID_ANY, value));
 	}
 }
 
@@ -269,44 +271,39 @@ bool CVerifyCertDialog::IsTrusted(CCertificateNotification* pNotification)
 	return IsTrusted(pNotification->GetHost(), pNotification->GetPort(), data, len, false);
 }
 
-bool CVerifyCertDialog::IsTrusted(const wxString& host, int port, const unsigned char* data, unsigned int len, bool permanentOnly)
+bool CVerifyCertDialog::DoIsTrusted(const wxString& host, int port, const unsigned char* data, unsigned int len, std::list<CVerifyCertDialog::t_certData> const& trustedCerts)
 {
-	for (std::list<t_certData>::const_iterator iter = m_trustedCerts.begin(); iter != m_trustedCerts.end(); ++iter)
-	{
-		if (host != iter->host)
-			continue;
-
-		if (port != iter->port)
-			continue;
-
-		if (iter->len != len)
-			continue;
-
-		if (!memcmp(iter->data, data, len))
-			return true;
+	if( !data || !len ) {
+		return false;
 	}
 
-	if (permanentOnly)
-		return false;
-
-	for (std::list<t_certData>::const_iterator iter = m_sessionTrustedCerts.begin(); iter != m_sessionTrustedCerts.end(); ++iter)
-	{
-		if (host != iter->host)
+	for ( auto const& cert : trustedCerts ) {
+		if (host != cert.host)
 			continue;
 
-		if (port != iter->port)
+		if (port != cert.port)
 			continue;
 
-		if (iter->len != len)
+		if (cert.len != len)
 			continue;
 
-		if (!memcmp(iter->data, data, len))
+		if (!memcmp(cert.data, data, len))
 			return true;
 	}
 
 	return false;
 }
 
+bool CVerifyCertDialog::IsTrusted(const wxString& host, int port, const unsigned char* data, unsigned int len, bool permanentOnly)
+{
+	bool trusted = DoIsTrusted(host, port, data, len, m_trustedCerts);
+	if( !trusted && !permanentOnly ) {
+		trusted = DoIsTrusted(host, port, data, len, m_sessionTrustedCerts);
+	}
+
+	return trusted;
+}
+
 wxString CVerifyCertDialog::ConvertHexToString(const unsigned char* data, unsigned int len)
 {
 	wxString str;
@@ -333,11 +330,15 @@ wxString CVerifyCertDialog::ConvertHexToString(const unsigned char* data, unsign
 
 unsigned char* CVerifyCertDialog::ConvertStringToHex(const wxString& str, unsigned int &len)
 {
-	len = str.Length() / 2;
+	if (str.size() % 2) {
+		return 0;
+	}
+
+	len = str.size() / 2;
 	unsigned char* data = new unsigned char[len];
 
 	unsigned int j = 0;
-	for (unsigned int i = 0; i < str.Length(); i++, j++)
+	for (unsigned int i = 0; i < str.size(); i++, j++)
 	{
 		wxChar high = str[i++];
 		wxChar low = str[i];
@@ -368,19 +369,15 @@ unsigned char* CVerifyCertDialog::ConvertStringToHex(const wxString& str, unsign
 	return data;
 }
 
-void CVerifyCertDialog::LoadTrustedCerts(bool close /*=true*/)
+void CVerifyCertDialog::LoadTrustedCerts()
 {
 	CReentrantInterProcessMutexLocker mutex(MUTEX_TRUSTEDCERTS);
-	if (!m_xmlFile.HasFileName() || m_xmlFile.Modified())
-		m_xmlFile.Load(_T("trustedcerts"));
-	else
+	if (!m_xmlFile.Modified()) {
 		return;
+	}
 
-	TiXmlElement* pElement = m_xmlFile.GetElement();
-	if (!pElement)
-	{
-		if (close)
-			m_xmlFile.Close();
+	TiXmlElement* pElement = m_xmlFile.Load();
+	if (!pElement) {
 		return;
 	}
 
@@ -392,14 +389,13 @@ void CVerifyCertDialog::LoadTrustedCerts(bool close /*=true*/)
 	bool modified = false;
 
 	TiXmlElement* pCert = pElement->FirstChildElement("Certificate");
-	while (pCert)
-	{
+	while (pCert) {
 		wxString value = GetTextElement(pCert, "Data");
 
 		TiXmlElement* pRemove = 0;
 
 		t_certData data;
-		if (value == _T("") || !(data.data = ConvertStringToHex(value, data.len)))
+		if (value.empty() || !(data.data = ConvertStringToHex(value, data.len)))
 			pRemove = pCert;
 
 		data.host = GetTextElement(pCert, "Host");
@@ -433,10 +429,7 @@ void CVerifyCertDialog::LoadTrustedCerts(bool close /*=true*/)
 	}
 
 	if (modified)
-		m_xmlFile.Save();
-
-	if (close)
-		m_xmlFile.Close();
+		m_xmlFile.Save(false);
 }
 
 void CVerifyCertDialog::SetPermanentlyTrusted(const CCertificateNotification* const pNotification)
@@ -446,11 +439,9 @@ void CVerifyCertDialog::SetPermanentlyTrusted(const CCertificateNotification* co
 	const unsigned char* const data = certificate.GetRawData(len);
 
 	CReentrantInterProcessMutexLocker mutex(MUTEX_TRUSTEDCERTS);
-	LoadTrustedCerts(false);
+	LoadTrustedCerts();
 
-	if (IsTrusted(pNotification->GetHost(), pNotification->GetPort(), data, len, true))
-	{
-		m_xmlFile.Close();
+	if (IsTrusted(pNotification->GetHost(), pNotification->GetPort(), data, len, true))	{
 		return;
 	}
 
@@ -462,19 +453,12 @@ void CVerifyCertDialog::SetPermanentlyTrusted(const CCertificateNotification* co
 	memcpy(cert.data, data, len);
 	m_trustedCerts.push_back(cert);
 
-	if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2)
+	if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2) {
 		return;
-
-	TiXmlElement* pElement = m_xmlFile.GetElement();
-	if (!pElement)
-	{
-		m_xmlFile.Load();
-		pElement = m_xmlFile.GetElement();
 	}
 
-	if (!pElement)
-	{
-		m_xmlFile.Close();
+	TiXmlElement* pElement = m_xmlFile.GetElement();
+	if (!pElement) {
 		return;
 	}
 
@@ -495,8 +479,7 @@ void CVerifyCertDialog::SetPermanentlyTrusted(const CCertificateNotification* co
 	AddTextElement(pCert, "Host", pNotification->GetHost());
 	AddTextElement(pCert, "Port", pNotification->GetPort());
 
-	m_xmlFile.Save();
-	m_xmlFile.Close();
+	m_xmlFile.Save(true);
 }
 
 wxString CVerifyCertDialog::DecodeValue(const wxString& value)
@@ -505,16 +488,14 @@ wxString CVerifyCertDialog::DecodeValue(const wxString& value)
 	// #xxxx466F6F626172 -> Foobar
 	// First two encoded bytes are ignored, some weird type information I don't care about
 	// Only accepts ASCII for now.
-	if (value[0] != '#')
+	if (value.empty() || value[0] != '#')
 		return value;
 
 	unsigned int len = value.Len();
-	//if (!(len % 2))
-	//	return value;
 
 	wxString out;
 
-	for (unsigned int i = 5; i < len; i += 2)
+	for (unsigned int i = 5; i + 1 < len; i += 2)
 	{
 		wxChar c = value[i];
 		wxChar d = value[i + 1];
@@ -538,8 +519,6 @@ wxString CVerifyCertDialog::DecodeValue(const wxString& value)
 			//return value;
 
 		c = c * 16 + d;
-		//if (c > 127)
-		//	return value;
 		if (c > 127 || c < 32)
 			continue;
 		out += c;
