@@ -22,7 +22,7 @@ bool CWelcomeDialog::Run(wxWindow* parent, bool force /*=false*/, bool delay /*=
 			return true;
 		}
 
-		if (greetingVersion != _T("") &&
+		if (!greetingVersion.empty() &&
 			CBuildInfo::ConvertToVersionNumber(ownVersion) <= CBuildInfo::ConvertToVersionNumber(greetingVersion))
 		{
 			// Been there done that
@@ -32,12 +32,16 @@ bool CWelcomeDialog::Run(wxWindow* parent, bool force /*=false*/, bool delay /*=
 		}
 		COptions::Get()->SetOption(OPTION_GREETINGVERSION, ownVersion);
 
-		if (greetingVersion.IsEmpty() && !COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE))
+		if (greetingVersion.empty() && !COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE))
 			COptions::Get()->SetOption(OPTION_PROMPTPASSWORDSAVE, 1);
 	}
 
-	if (!wxXmlResource::Get()->LoadDialog(this, parent, _T("ID_WELCOME")))
+	if (!wxXmlResource::Get()->LoadDialog(this, parent, _T("ID_WELCOME"))) {
+		if( delay ) {
+			delete this;
+		}
 		return false;
+	}
 
 	XRCCTRL(*this, "ID_FZVERSION", wxStaticText)->SetLabel(_T("FileZilla ") + CBuildInfo::GetVersion());
 
@@ -46,10 +50,10 @@ bool CWelcomeDialog::Run(wxWindow* parent, bool force /*=false*/, bool delay /*=
 	wxHyperlinkCtrl* pNews = XRCCTRL(*this, "ID_LINK_NEWS", wxHyperlinkCtrl);
 	pNews->SetURL(wxString::Format(url, _T("news")) + _T("&oldversion=") + greetingVersion);
 
-	if (greetingVersion != _T(""))
+	if (!greetingVersion.empty())
 	{
 		wxHyperlinkCtrl* pNews = XRCCTRL(*this, "ID_LINK_NEWS", wxHyperlinkCtrl);
-		pNews->SetLabel(wxString::Format(_("New features and improvements in %s"), CBuildInfo::GetVersion().c_str()));
+		pNews->SetLabel(wxString::Format(_("New features and improvements in %s"), CBuildInfo::GetVersion()));
 	}
 	else
 	{
@@ -77,7 +81,7 @@ bool CWelcomeDialog::Run(wxWindow* parent, bool force /*=false*/, bool delay /*=
 	return true;
 }
 
-void CWelcomeDialog::OnTimer(wxTimerEvent& event)
+void CWelcomeDialog::OnTimer(wxTimerEvent&)
 {
 	ShowModal();
 	Destroy();
