@@ -9,15 +9,26 @@
 #include <wx/statbox.h>
 #include <wx/wizard.h>
 
+#ifdef __WXGTK3__
+#include <gtk/gtk.h>
+#endif
+
+#include <algorithm>
+
 bool CWrapEngine::m_use_cache = true;
 
 #define WRAPDEBUG 0
-#if wxUSE_UNICODE
+
+#if WRAPDEBUG
+#define WRAPASSERT(x) wxASSERT(x)
+#else
+#define WRAPASSERT(x)
+#endif
+
 // Chinese equivalents to ".", "," and ":"
 static const wxChar noWrapChars_Chinese[] = { '.', ',', ':', 0x3002, 0xFF0C, 0xFF1A, 0};
 // Remark: Chinese (Taiwan) uses ascii punctuation marks though, but those
 // don't have to be added, as only characters >= 128 will be wrapped.
-#endif
 
 bool CWrapEngine::CanWrapBefore(const wxChar& c)
 {
@@ -39,11 +50,11 @@ bool CWrapEngine::CanWrapBefore(const wxChar& c)
 
 bool CWrapEngine::WrapTextChinese(wxWindow* parent, wxString &text, unsigned long maxLength)
 {
-	wxASSERT(text.Find(_T("  ")) == -1);
-	wxASSERT(text.Find(_T(" \n")) == -1);
-	wxASSERT(text.Find(_T("\n ")) == -1);
-	wxASSERT(text.Last() != ' ');
-	wxASSERT(text.Last() != '\n');
+	WRAPASSERT(text.Find(_T("  ")) == -1);
+	WRAPASSERT(text.Find(_T(" \n")) == -1);
+	WRAPASSERT(text.Find(_T("\n ")) == -1);
+	WRAPASSERT(text.Last() != ' ');
+	WRAPASSERT(text.Last() != '\n');
 
 	// See comment at start of WrapText function what this function does
 	wxString wrappedText;
@@ -88,7 +99,7 @@ bool CWrapEngine::WrapTextChinese(wxWindow* parent, wxString &text, unsigned lon
 			else
 				lineLength += iter->second;
 
-			wxASSERT(*p != '\r');
+			WRAPASSERT(*p != '\r');
 			if (*p == '\n')
 			{
 				// Wrap on newline
@@ -147,7 +158,7 @@ bool CWrapEngine::WrapTextChinese(wxWindow* parent, wxString &text, unsigned lon
 		}
 	}
 
-#ifdef __WXDEBUG__
+#if WRAPDEBUG
 	wxString temp = wrappedText;
 	wxASSERT(temp.Find(_T("  ")) == -1);
 	wxASSERT(temp.Find(_T(" \n")) == -1);
@@ -155,7 +166,7 @@ bool CWrapEngine::WrapTextChinese(wxWindow* parent, wxString &text, unsigned lon
 	wxASSERT(temp.Last() != ' ');
 	wxASSERT(temp.Last() != '\n');
 	temp.Replace(_T("&"), _T(""));
-	while (temp != _T(""))
+	while (!temp.empty())
 	{
 		wxString piece;
 		int pos = temp.Find(_T("\n"));
@@ -204,14 +215,14 @@ bool CWrapEngine::WrapText(wxWindow* parent, wxString& text, unsigned long maxLe
 	if (!m_font.IsOk())
 		m_font = parent->GetFont();
 
-#ifdef __WXDEBUG__
+#if WRAPDEBUG
 	const wxString original = text;
 #endif
 
 	if (m_wrapOnEveryChar)
 	{
 		bool res = WrapTextChinese(parent, text, maxLength);
-#ifdef __WXDEBUG__
+#if WRAPDEBUG
 		wxString unwrapped = UnwrapText(text);
 		wxASSERT(original == unwrapped);
 #endif
@@ -222,8 +233,9 @@ bool CWrapEngine::WrapText(wxWindow* parent, wxString& text, unsigned long maxLe
 
 	int width = 0, height = 0;
 
-	int spaceWidth = 0;
-	parent->GetTextExtent(_T(" "), &spaceWidth, &height, 0, 0, &m_font);
+	if (m_spaceWidth == -1) {
+		parent->GetTextExtent(_T(" "), &m_spaceWidth, &height, 0, 0, &m_font);
+	}
 
 	int strLen = text.Length();
 	int wrapAfter = -1;
@@ -232,34 +244,30 @@ bool CWrapEngine::WrapText(wxWindow* parent, wxString& text, unsigned long maxLe
 
 	bool url = false;
 	bool containsURL = false;
-	for (int i = 0; i <= strLen; i++)
-	{
-		if ((text[i] == ':' && text[i + 1] == '/' && text[i + 2] == '/') || // absolute
-			(text[i] == '/' && (!i || text[i - 1] == ' '))) // relative
+	for (int i = 0; i <= strLen; i++) {
+		if ((i < strLen - 2 && text[i] == ':' && text[i + 1] == '/' && text[i + 2] == '/') || // absolute
+			(i < strLen && text[i] == '/' && (!i || text[i - 1] == ' '))) // relative
 		{
 			url = true;
 			containsURL = true;
 		}
-		if (text[i] != ' ' && text[i] != 0)
-		{
+		if (i < strLen && text[i] != ' ') {
 			// If url, wrap on slashes and ampersands, but not first slash of something://
 			if (!url ||
-				 ((text[i] != '/' || text[i + 1] == '/') && (text[i] != '&' || text[i + 1] == '&') && text[i] != '?'))
+				 ((i < strLen - 1 && (text[i] != '/' || text[i + 1] == '/')) && (i < strLen - 1 && (text[i] != '&' || text[i + 1] == '&')) && text[i] != '?'))
 			continue;
 		}
 
 		wxString segment;
-		if (wrapAfter == -1)
-		{
-			if (text[i] == '/' || text[i] == '?' || text[i] == '&')
+		if (wrapAfter == -1) {
+			if (i < strLen && (text[i] == '/' || text[i] == '?' || text[i] == '&'))
 				segment = text.Mid(start, i - start + 1);
 			else
 				segment = text.Mid(start, i - start);
 			wrapAfter = i;
 		}
-		else
-		{
-			if (text[i] == '/' || text[i] == '?' || text[i] == '&')
+		else {
+			if (i < strLen && (text[i] == '/' || text[i] == '?' || text[i] == '&'))
 				segment = text.Mid(wrapAfter + 1, i - wrapAfter);
 			else
 				segment = text.Mid(wrapAfter + 1, i - wrapAfter - 1);
@@ -268,22 +276,19 @@ bool CWrapEngine::WrapText(wxWindow* parent, wxString& text, unsigned long maxLe
 		segment = wxStripMenuCodes(segment);
 		parent->GetTextExtent(segment, &width, &height, 0, 0, &m_font);
 
-		if (lineLength + spaceWidth + width > maxLength)
-		{
+		if (lineLength + m_spaceWidth + width > maxLength) {
 			// Cannot be appended to current line without overflow, so start a new line
-			if (wrappedText != _T(""))
+			if (!wrappedText.empty())
 				wrappedText += _T("\n");
 			wrappedText += text.Mid(start, wrapAfter - start);
-			if (text[wrapAfter] != ' ' && text[wrapAfter] != '\0')
+			if (wrapAfter < strLen && text[wrapAfter] != ' ' && text[wrapAfter] != '\0')
 				wrappedText += text[wrapAfter];
 
-			if (width + spaceWidth >= (int)maxLength)
-			{
+			if (width + m_spaceWidth >= (int)maxLength) {
 				// Current segment too big to even fit into a line just by itself
 
-				if( i != wrapAfter )
-				{
-					if (wrappedText != _T(""))
+				if( i != wrapAfter ) {
+					if (!wrappedText.empty())
 						wrappedText += _T("\n");
 					wrappedText += text.Mid(wrapAfter + 1, i - wrapAfter - 1);
 				}
@@ -292,49 +297,46 @@ bool CWrapEngine::WrapText(wxWindow* parent, wxString& text, unsigned long maxLe
 				wrapAfter = -1;
 				lineLength = 0;
 			}
-			else
-			{
+			else {
 				start = wrapAfter + 1;
 				wrapAfter = i;
 				lineLength = width;
 			}
 		}
-		else if (lineLength + spaceWidth + width + spaceWidth >= maxLength)
-		{
-			if (wrappedText != _T(""))
+		else if (lineLength + m_spaceWidth + width + m_spaceWidth >= maxLength) {
+			if (!wrappedText.empty())
 				wrappedText += _T("\n");
 			wrappedText += text.Mid(start, i - start);
-			if (text[i] != ' ' && text[i] != '\0')
+			if (i < strLen && text[i] != ' ' && text[i] != '\0')
 				wrappedText += text[i];
 			start = i + 1;
 			wrapAfter = -1;
 			lineLength = 0;
 		}
-		else
-		{
+		else {
 			if (lineLength)
-				lineLength += spaceWidth;
+				lineLength += m_spaceWidth;
 			lineLength += width;
 			wrapAfter = i;
 		}
 
-		if (text[i] == ' ')
+		if (i < strLen && text[i] == ' ')
 			url = false;
 	}
-	if (start < strLen)
-	{
-		if (wrappedText != _T(""))
+	if (start < strLen) {
+		if (!wrappedText.empty())
 			wrappedText += _T("\n");
 		wrappedText += text.Mid(start);
 	}
 
 	text = wrappedText;
 
-#ifdef __WXDEBUG__
+#if WRAPDEBUG
 		wxString unwrapped = UnwrapText(text);
 		wxASSERT(original == unwrapped || containsURL);
+#else
+	(void)containsURL;
 #endif
-
 	return true;
 }
 
@@ -363,7 +365,7 @@ int CWrapEngine::WrapRecursive(wxWindow* wnd, wxSizer* sizer, int max)
 
 #if WRAPDEBUG >= 3
 	static int level = 1;
-	plvl printf("Enter with max = %d\n", max);
+	plvl printf("Enter with max = %d, current = %d, sizer is %s\n", max, wnd ? wnd->GetRect().GetWidth() : -1, static_cast<char const*>(wxString(sizer->GetClassInfo()->GetClassName())));
 #endif
 
 	if (max <= 0)
@@ -376,8 +378,7 @@ int CWrapEngine::WrapRecursive(wxWindow* wnd, wxSizer* sizer, int max)
 
 	int result = 0;
 
-	for (unsigned int i = 0; i < sizer->GetChildren().GetCount(); i++)
-	{
+	for (unsigned int i = 0; i < sizer->GetChildren().GetCount(); ++i) {
 		wxSizerItem* item = sizer->GetItem(i);
 		if (!item || !item->IsShown())
 			continue;
@@ -401,11 +402,9 @@ int CWrapEngine::WrapRecursive(wxWindow* wnd, wxSizer* sizer, int max)
 
 		wxWindow* window;
 		wxSizer* subSizer = 0;
-		if ((window = item->GetWindow()))
-		{
+		if ((window = item->GetWindow())) {
 			wxStaticText* text = wxDynamicCast(window, wxStaticText);
-			if (text)
-			{
+			if (text) {
 #ifdef __WXMAC__
 				const int offset = 3;
 #else
@@ -423,8 +422,14 @@ int CWrapEngine::WrapRecursive(wxWindow* wnd, wxSizer* sizer, int max)
 					return result | wrap_failed;
 				}
 				text->SetLabel(str);
-
 				result |= wrap_didwrap;
+
+#ifdef __WXGTK3__
+				gtk_widget_set_size_request(text->GetHandle(), -1, -1);
+				GtkRequisition req;
+				gtk_widget_get_preferred_size(text->GetHandle(), 0, &req);
+				text->CacheBestSize(wxSize(req.width, req.height));
+#endif
 				continue;
 			}
 
@@ -459,7 +464,8 @@ int CWrapEngine::WrapRecursive(wxWindow* wnd, wxSizer* sizer, int max)
 			if (wxDynamicCast(window, wxCheckBox) || wxDynamicCast(window, wxRadioButton) || wxDynamicCast(window, wxChoice))
 			{
 #if WRAPDEBUG >= 3
-				plvl printf("Leave: WrapRecursive on unshrinkable controls failed\n");
+				plvl printf("Leave: WrapRecursive on unshrinkable control failed: %s\n",
+					static_cast<char const*>(wxString(window->GetClassInfo()->GetClassName())));
 #endif
 				result |= wrap_failed;
 				return result;
@@ -470,6 +476,7 @@ int CWrapEngine::WrapRecursive(wxWindow* wnd, wxSizer* sizer, int max)
 		else if ((subSizer = item->GetSizer()))
 		{
 			int subBorder = 0;
+			wxWindow* subWnd = wnd;
 
 			// Add border of static box sizer
 			wxStaticBoxSizer* sboxSizer;
@@ -477,13 +484,14 @@ int CWrapEngine::WrapRecursive(wxWindow* wnd, wxSizer* sizer, int max)
 			{
 				int top, other;
 				sboxSizer->GetStaticBox()->GetBordersForSizer(&top, &other);
-				subBorder += other;
+				subBorder += other * 2;
+				subWnd = sboxSizer->GetStaticBox();
 			}
 
 #if WRAPDEBUG >= 3
 			level++;
 #endif
-			result |= WrapRecursive(0, subSizer, max - rborder - subBorder);
+			result |= WrapRecursive(subWnd, subSizer, max - lborder - rborder - subBorder);
 #if WRAPDEBUG >= 3
 			level--;
 #endif
@@ -497,10 +505,23 @@ int CWrapEngine::WrapRecursive(wxWindow* wnd, wxSizer* sizer, int max)
 		}
 	}
 
-#if WRAPDEBUG >= 3
-	plvl printf("Leave: Success\n");
+	wxStaticBoxSizer* sboxSizer = wxDynamicCast(sizer, wxStaticBoxSizer);
+	if( sboxSizer ) {
+#ifdef __WXGTK3__
+		gtk_widget_set_size_request(sboxSizer->GetStaticBox()->GetHandle(), -1, -1);
+		GtkRequisition req;
+		gtk_widget_get_preferred_size(sboxSizer->GetStaticBox()->GetHandle(), 0, &req);
+		sboxSizer->GetStaticBox()->CacheBestSize(wxSize(req.width, req.height));
+#elif defined(__WXMAC__) || defined(__WXGTK__)
+		sboxSizer->GetStaticBox()->CacheBestSize(wxSize(0, 0));
+#else
+		sboxSizer->GetStaticBox()->CacheBestSize(wxDefaultSize);
 #endif
+	}
 
+#if WRAPDEBUG >= 3
+	plvl printf("Leave: Success, new min: %d\n", sizer->CalcMin().x);
+#endif
 
 	return result;
 }
@@ -539,36 +560,34 @@ void CWrapEngine::UnwrapRecursive_Wrapped(const std::list<int> &wrapped, std::ve
 
 bool CWrapEngine::WrapRecursive(std::vector<wxWindow*>& windows, double ratio, const char* name /*=""*/, wxSize canvas /*=wxSize()*/, wxSize minRequestedSize /*wxSize()*/)
 {
+#ifdef __WXMAC__
+	const int offset = 6;
+#elif defined(__WXGTK__)
+	const int offset = 0;
+#else
+	const int offset = 0;
+#endif
+
 	int maxWidth = GetWidthFromCache(name);
-	if (maxWidth)
-	{
-		for (std::vector<wxWindow*>::iterator iter = windows.begin(); iter != windows.end(); ++iter)
-		{
+	if (maxWidth) {
+		for (auto iter = windows.begin(); iter != windows.end(); ++iter) {
 			wxSizer* pSizer = (*iter)->GetSizer();
 			if (!pSizer)
 				continue;
 
 			pSizer->Layout();
 
-#ifdef __WXMAC__
-			const int offset = 6;
-#elif defined(__WXGTK__)
-			const int offset = 0;
-#else
-			const int offset = 0;
-#endif
-
-#ifdef __WXDEBUG__
+#if WRAPDEBUG
 			int res =
 #endif
 			WrapRecursive(*iter, pSizer, maxWidth - offset);
-			wxASSERT(!(res & wrap_failed));
+			WRAPASSERT(!(res & wrap_failed));
 			pSizer->Layout();
 			pSizer->Fit(*iter);
-#ifdef __WXDEBUG__
+#if WRAPDEBUG
 			wxSize size = pSizer->GetMinSize();
 #endif
-			wxASSERT(size.x <= maxWidth);
+			WRAPASSERT(size.x <= maxWidth);
 		}
 		return true;
 	}
@@ -577,7 +596,7 @@ bool CWrapEngine::WrapRecursive(std::vector<wxWindow*>& windows, double ratio, c
 
 	wxSize size = minRequestedSize;
 
-	for (std::vector<wxWindow*>::iterator iter = windows.begin(); iter != windows.end(); ++iter)
+	for (auto iter = windows.begin(); iter != windows.end(); ++iter)
 	{
 		wxSizer* pSizer = (*iter)->GetSizer();
 		if (!pSizer)
@@ -617,17 +636,11 @@ bool CWrapEngine::WrapRecursive(std::vector<wxWindow*>& windows, double ratio, c
 		std::list<int> didwrap;
 
 		wxSize size = minRequestedSize;
-		for (std::vector<wxWindow*>::iterator iter = windows.begin(); iter != windows.end(); ++iter)
+		int res = 0;
+		for (auto iter = windows.begin(); iter != windows.end(); ++iter)
 		{
 			wxSizer* pSizer = (*iter)->GetSizer();
-#ifdef __WXMAC__
-			const int offset = 6;
-#elif defined(__WXGTK__)
-			const int offset = 0;
-#else
-			const int offset = 0;
-#endif
-			int res = WrapRecursive(*iter, pSizer, desiredWidth - offset);
+			res = WrapRecursive(*iter, pSizer, desiredWidth - offset);
 			if (res & wrap_didwrap)
 				pSizer->Layout();
 			didwrap.push_back(res);
@@ -640,7 +653,7 @@ bool CWrapEngine::WrapRecursive(std::vector<wxWindow*>& windows, double ratio, c
 		}
 
 #if WRAPDEBUG > 0
-		printf("Current: % 4d % 4d   desiredWidth: %d, min: %d, max: %d\n", size.GetWidth(), size.GetHeight(), desiredWidth, min, max);
+		printf("After current wrapping: size=%dx%d  desiredWidth=%d  min=%d  max=%d  res=%d\n", size.GetWidth(), size.GetHeight(), desiredWidth, min, max, res);
 #endif
 		if (size.GetWidth() > desiredWidth)
 		{
@@ -663,7 +676,7 @@ bool CWrapEngine::WrapRecursive(std::vector<wxWindow*>& windows, double ratio, c
 
 		double newRatio = ((double)(size.GetWidth() + canvas.x) / (size.GetHeight() + canvas.y));
 #if WRAPDEBUG > 0
-		printf("Ratio: %f\n", (float)newRatio);
+		printf("New ratio: %f\n", (float)newRatio);
 #endif
 
 		if (newRatio < ratio)
@@ -673,7 +686,7 @@ bool CWrapEngine::WrapRecursive(std::vector<wxWindow*>& windows, double ratio, c
 			if (ratio - newRatio < bestRatioDiff)
 			{
 				bestRatioDiff = ratio - newRatio;
-				bestWidth = actualWidth;
+				bestWidth = std::max(desiredWidth, actualWidth);
 			}
 
 			if (min >= actualWidth)
@@ -687,39 +700,31 @@ bool CWrapEngine::WrapRecursive(std::vector<wxWindow*>& windows, double ratio, c
 			if (newRatio - ratio < bestRatioDiff)
 			{
 				bestRatioDiff = newRatio - ratio;
-				bestWidth = actualWidth;
+				bestWidth = std::max(desiredWidth, actualWidth);
 			}
 
 			if (max == actualWidth)
 				break;
-			max = actualWidth;
+			max = std::max(desiredWidth, actualWidth);
 		}
 		else
 		{
 			UnwrapRecursive_Wrapped(didwrap, windows);
 
 			bestRatioDiff = ratio - newRatio;
-			bestWidth = actualWidth;
+			max = std::max(desiredWidth, actualWidth);
 			break;
 		}
 
 		if (max - min < 2)
 			break;
 		desiredWidth = (min + max) / 2;
-
-		currentRatio = newRatio;
 	}
 #if WRAPDEBUG > 0
 		printf("Performing final wrap with bestwidth %d\n", bestWidth);
 #endif
-#ifdef __WXMAC__
-			const int offset = 6;
-#elif defined(__WXGTK__)
-			const int offset = 0;
-#else
-			const int offset = 0;
-#endif
-	for (std::vector<wxWindow*>::iterator iter = all_windows.begin(); iter != all_windows.end(); ++iter)
+
+	for (auto iter = all_windows.begin(); iter != all_windows.end(); ++iter)
 	{
 		wxSizer *pSizer = (*iter)->GetSizer();
 
@@ -730,9 +735,9 @@ bool CWrapEngine::WrapRecursive(std::vector<wxWindow*>& windows, double ratio, c
 			pSizer->Layout();
 			pSizer->Fit(*iter);
 		}
-#ifdef __WXDEBUG__
+#if WRAPDEBUG
 		size = pSizer->GetMinSize();
-		wxASSERT(size.x <= bestWidth);
+		WRAPASSERT(size.x <= bestWidth);
 #endif
 	}
 
@@ -744,7 +749,6 @@ bool CWrapEngine::WrapRecursive(std::vector<wxWindow*>& windows, double ratio, c
 wxString CWrapEngine::UnwrapText(const wxString& text)
 {
 	wxString unwrapped;
-#if wxUSE_UNICODE
 	int lang = wxGetApp().GetCurrentLanguage();
 	if (lang == wxLANGUAGE_CHINESE || lang == wxLANGUAGE_CHINESE_SIMPLIFIED ||
 		lang == wxLANGUAGE_CHINESE_TRADITIONAL || lang == wxLANGUAGE_CHINESE_HONGKONG ||
@@ -777,7 +781,6 @@ wxString CWrapEngine::UnwrapText(const wxString& text)
 		}
 	}
 	else
-#endif
 	{
 		unwrapped = text;
 
@@ -788,7 +791,7 @@ wxString CWrapEngine::UnwrapText(const wxString& text)
 			if (unwrapped[pos - 1] == ' ')
 				unwrapped = unwrapped.Left(pos + 2) + _T(" ") + unwrapped.Mid(pos + 3);
 			else
-				unwrapped = unwrapped.Left(pos + 2) + unwrapped.Mid(pos + 3);				
+				unwrapped = unwrapped.Left(pos + 2) + unwrapped.Mid(pos + 3);
 		}
 
 		unwrapped.Replace(_T("\n"), _T(" "));
@@ -846,20 +849,14 @@ int CWrapEngine::GetWidthFromCache(const char* name)
 	if (!name || !*name)
 		return 0;
 
-	// We have to synchronize access to layout.xml so that multiple processed don't write
+	// We have to synchronize access to layout.xml so that multiple processes don't write
 	// to the same file or one is reading while the other one writes.
 	CInterProcessMutex mutex(MUTEX_LAYOUT);
 
-	wxFileName file(COptions::Get()->GetOption(OPTION_DEFAULT_SETTINGSDIR), _T("layout.xml"));
-	TiXmlElement* pDocument = GetXmlFile(file);
-
-	if (!pDocument)
-		return 0;
-
-	TiXmlElement* pElement = pDocument->FirstChildElement("Layout");
-	if (!pElement)
-	{
-		delete pDocument->GetDocument();
+	CXmlFile xml(wxGetApp().GetSettingsFile(_T("layout")));
+	TiXmlElement* root = xml.Load();
+	TiXmlElement* pElement = root ? root->FirstChildElement("Layout") : 0;
+	if (!pElement) {
 		return 0;
 	}
 
@@ -868,23 +865,17 @@ int CWrapEngine::GetWidthFromCache(const char* name)
 		language = _T("default");
 
 	TiXmlElement* pLanguage = FindElementWithAttribute(pElement, "Language", "id", language.mb_str());
-	if (!pLanguage)
-	{
-		delete pDocument->GetDocument();
+	if (!pLanguage) {
 		return 0;
 	}
 
 	TiXmlElement* pDialog = FindElementWithAttribute(pLanguage, "Dialog", "name", name);
-	if (!pDialog)
-	{
-		delete pDocument->GetDocument();
+	if (!pDialog) {
 		return 0;
 	}
 
 	int value = GetAttributeInt(pDialog, "width");
 
-	delete pDocument->GetDocument();
-
 	return value;
 }
 
@@ -896,20 +887,14 @@ void CWrapEngine::SetWidthToCache(const char* name, int width)
 	if (!name || !*name)
 		return;
 
-	// We have to synchronize access to layout.xml so that multiple processed don't write
+	// We have to synchronize access to layout.xml so that multiple processes don't write
 	// to the same file or one is reading while the other one writes.
 	CInterProcessMutex mutex(MUTEX_LAYOUT);
 
-	wxFileName file(COptions::Get()->GetOption(OPTION_DEFAULT_SETTINGSDIR), _T("layout.xml"));
-	TiXmlElement* pDocument = GetXmlFile(file);
-
-	if (!pDocument)
-		return;
-
-	TiXmlElement* pElement = pDocument->FirstChildElement("Layout");
-	if (!pElement)
-	{
-		delete pDocument->GetDocument();
+	CXmlFile xml(wxGetApp().GetSettingsFile(_T("layout")));
+	TiXmlElement* root = xml.Load();
+	TiXmlElement* pElement = root ? root->FirstChildElement("Layout") : 0;
+	if (!pElement) {
 		return;
 	}
 
@@ -918,24 +903,18 @@ void CWrapEngine::SetWidthToCache(const char* name, int width)
 		language = _T("default");
 
 	TiXmlElement* pLanguage = FindElementWithAttribute(pElement, "Language", "id", language.mb_str());
-	if (!pLanguage)
-	{
-		delete pDocument->GetDocument();
+	if (!pLanguage) {
 		return;
 	}
 
 	TiXmlElement* pDialog = FindElementWithAttribute(pLanguage, "Dialog", "name", name);
-	if (!pDialog)
-	{
+	if (!pDialog) {
 		pDialog = pLanguage->LinkEndChild(new TiXmlElement("Dialog"))->ToElement();
 		pDialog->SetAttribute("name", name);
 	}
 
 	pDialog->SetAttribute("width", width);
-	wxString error;
-	SaveXmlFile(file, pDocument, &error);
-
-	delete pDocument->GetDocument();
+	xml.Save(false);
 }
 
 CWrapEngine::CWrapEngine()
@@ -943,10 +922,6 @@ CWrapEngine::CWrapEngine()
 	CheckLanguage();
 }
 
-CWrapEngine::~CWrapEngine()
-{
-}
-
 static wxString GetLocaleFile(const wxString& localesDir, wxString name)
 {
 	if (wxFileName::FileExists(localesDir + name + _T("/filezilla.mo")))
@@ -974,23 +949,22 @@ static wxString GetLocaleFile(const wxString& localesDir, wxString name)
 			return name + _T("/LC_MESSAGES");
 	}
 
-	return _T("");
+	return wxString();
 }
 
 bool CWrapEngine::LoadCache()
 {
-	// We have to synchronize access to layout.xml so that multiple processed don't write
+	// We have to synchronize access to layout.xml so that multiple processes don't write
 	// to the same file or one is reading while the other one writes.
 	CInterProcessMutex mutex(MUTEX_LAYOUT);
 
-	wxFileName file(COptions::Get()->GetOption(OPTION_DEFAULT_SETTINGSDIR), _T("layout.xml"));
-	CXmlFile xml(file);
+	CXmlFile xml(wxGetApp().GetSettingsFile(_T("layout")));
 	TiXmlElement* pDocument = xml.Load();
 
 	if (!pDocument)
 	{
 		m_use_cache = false;
-		wxMessageBox(xml.GetError(), _("Error loading xml file"), wxICON_ERROR);
+		wxMessageBoxEx(xml.GetError(), _("Error loading xml file"), wxICON_ERROR);
 
 		return false;
 	}
@@ -1022,18 +996,19 @@ bool CWrapEngine::LoadCache()
 	if (!pResources)
 		pResources = pElement->LinkEndChild(new TiXmlElement("Resources"))->ToElement();
 
-	wxString resourceDir = wxGetApp().GetResourceDir();
-	wxDir dir(resourceDir);
+	CLocalPath resourceDir = wxGetApp().GetResourceDir();
+	resourceDir.AddSegment(_T("xrc"));
+	wxDir dir(resourceDir.GetPath());
 
 	wxLogNull log;
 
 	wxString xrc;
 	for (bool found = dir.GetFirst(&xrc, _T("*.xrc")); found; found = dir.GetNext(&xrc))
 	{
-		if (!wxFileName::FileExists(resourceDir + xrc))
+		if (!wxFileName::FileExists(resourceDir.GetPath() + xrc))
 			continue;
 
-		wxFileName fn(resourceDir + xrc);
+		wxFileName fn(resourceDir.GetPath() + xrc);
 		wxDateTime date = fn.GetModificationTime();
 		wxLongLong ticks = date.GetTicks();
 
@@ -1070,7 +1045,7 @@ bool CWrapEngine::LoadCache()
 
 	// Get current language
 	wxString language = wxGetApp().GetCurrentLanguageCode();
-	if (language == _T(""))
+	if (language.empty())
 		language = _T("default");
 
 	TiXmlElement* languageElement = FindElementWithAttribute(pElement, "Language", "id", language.mb_str());
@@ -1112,12 +1087,12 @@ bool CWrapEngine::LoadCache()
 	pFrame->Destroy();
 
 	// Get language file
-	const wxString& localesDir = wxGetApp().GetLocalesDir();
-	wxString name = GetLocaleFile(localesDir, language);
+	CLocalPath const localesDir = wxGetApp().GetLocalesDir();
+	wxString name = GetLocaleFile(localesDir.GetPath(), language);
 
-	if (name != _T(""))
+	if (!name.empty())
 	{
-		wxFileName fn(localesDir + name + _T("/filezilla.mo"));
+		wxFileName fn(localesDir.GetPath() + name + _T("/filezilla.mo"));
 		wxDateTime date = fn.GetModificationTime();
 		wxLongLong ticks = date.GetTicks();
 
@@ -1143,25 +1118,17 @@ bool CWrapEngine::LoadCache()
 		return true;
 	}
 
-	wxString error;
-	if (!xml.Save(&error))
-	{
+	if (!xml.Save(true)) {
 		m_use_cache = false;
-
-		wxString msg = wxString::Format(_("Could not write \"%s\": %s"), file.GetFullPath().c_str(), error.c_str());
-		wxMessageBox(msg, _("Error writing xml file"), wxICON_ERROR);
 	}
-
-
 	return true;
 }
 
 void CWrapEngine::ClearCache()
 {
-	// We have to synchronize access to layout.xml so that multiple processed don't write
+	// We have to synchronize access to layout.xml so that multiple processes don't write
 	// to the same file or one is reading while the other one writes.
 	CInterProcessMutex mutex(MUTEX_LAYOUT);
-
 	wxFileName file(COptions::Get()->GetOption(OPTION_DEFAULT_SETTINGSDIR), _T("layout.xml"));
 	if (file.FileExists())
 		wxRemoveFile(file.GetFullPath());
@@ -1169,10 +1136,6 @@ void CWrapEngine::ClearCache()
 
 void CWrapEngine::CheckLanguage()
 {
-#if wxUSE_UNICODE
-	// Just don't bother with wrapping on anything other than UCS-2
-	// FIXME: Use charset conversion routines to convert into UCS-2 and back into
-	//        local charset if not using unicode.
 	int lang = wxGetApp().GetCurrentLanguage();
 	if (lang == wxLANGUAGE_CHINESE || lang == wxLANGUAGE_CHINESE_SIMPLIFIED ||
 		lang == wxLANGUAGE_CHINESE_TRADITIONAL || lang == wxLANGUAGE_CHINESE_HONGKONG ||
@@ -1184,7 +1147,6 @@ void CWrapEngine::CheckLanguage()
 		m_noWrapChars = noWrapChars_Chinese;
 	}
 	else
-#endif
 	{
 		m_wrapOnEveryChar = false;
 		m_noWrapChars = 0;
