@@ -50,6 +50,9 @@ std::vector<t_entry> CDirectoryListingParserTest::m_entries;
 
 CPPUNIT_TEST_SUITE_REGISTRATION(CDirectoryListingParserTest);
 
+typedef CRefcountObject<wxString> R;
+typedef CSparseOptional<wxString> O;
+
 static int calcYear(int month, int day)
 {
 	const int cur_year = wxDateTime::GetCurrentYear();
@@ -68,7 +71,7 @@ static int calcYear(int month, int day)
 void CDirectoryListingParserTest::InitEntries()
 {
 	// Unix-style listings
-        // -------------------
+	// -------------------
 
 	// We start with a perfect example of a unix style directory listing without anomalies.
 	m_entries.push_back((t_entry){
@@ -76,11 +79,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("01-unix-std dir"),
 				512,
-				_T("dr-xr-xr-x"),
-				_T("root other"),
-				CDirentry::flag_dir | CDirentry::flag_timestamp_date,
-				_T(""),
-				wxDateTime(8, wxDateTime::Apr, 1994)
+				R(_T("dr-xr-xr-x")),
+				R(_T("root other")),
+				CDirentry::flag_dir,
+				O(),
+				CDateTime(1994, 4, 8)
 			},
 			DEFAULT
 		});
@@ -91,11 +94,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("02-unix-std file"),
 				531,
-				_T("-rw-r--r--"),
-				_T("root other"),
-				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
-				_T(""),
-				wxDateTime(29, wxDateTime::Mar, calcYear(3, 29), 3, 26)
+				R(_T("-rw-r--r--")),
+				R(_T("root other")),
+				0,
+				O(),
+				CDateTime(calcYear(3, 29), 3, 29, 3, 26)
 			},
 			DEFAULT
 		});
@@ -106,11 +109,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("03-unix-nogroup dir"),
 				512,
-				_T("dr-xr-xr-x"),
-				_T("root"),
-				CDirentry::flag_dir | CDirentry::flag_timestamp_date,
-				_T(""),
-				wxDateTime(8, wxDateTime::Apr, 1994)
+				R(_T("dr-xr-xr-x")),
+				R(_T("root")),
+				CDirentry::flag_dir,
+				O(),
+				CDateTime(1994, 4, 8)
 			},
 			DEFAULT
 		});
@@ -121,11 +124,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("04-unix-std link"),
 				7,
-				_T("lrwxrwxrwx"),
-				_T("root other"),
-				CDirentry::flag_dir | CDirentry::flag_link | CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
-				_T("usr/bin"),
-				wxDateTime(25, wxDateTime::Jan, calcYear(1, 25), 0, 17)
+				R(_T("lrwxrwxrwx")),
+				R(_T("root other")),
+				CDirentry::flag_dir | CDirentry::flag_link,
+				O(_T("usr/bin")),
+				CDateTime(calcYear(1, 25), 1, 25, 0, 17)
 			},
 			DEFAULT
 		});
@@ -138,11 +141,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("05-unix-date file"),
 				531,
-				_T("-rw-r--r--"),
-				_T("root other"),
-				CDirentry::flag_timestamp_date,
-				_T(""),
-				wxDateTime(26, wxDateTime::Sep, 2000)
+				R(_T("-rw-r--r--")),
+				R(_T("root other")),
+				0,
+				O(),
+				CDateTime(2000, 9, 26)
 			},
 			DEFAULT
 		});
@@ -152,11 +155,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("06-unix-date file"),
 				531,
-				_T("-rw-r--r--"),
-				_T("root other"),
-				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
-				_T(""),
-				wxDateTime(26, wxDateTime::Sep, calcYear(9, 26), 13, 45)
+				R(_T("-rw-r--r--")),
+				R(_T("root other")),
+				0,
+				O(),
+				CDateTime(calcYear(9, 26), 9, 26, 13, 45)
 			},
 			DEFAULT
 		});
@@ -166,11 +169,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("07-unix-date file"),
 				531,
-				_T("-rw-r--r--"),
-				_T("root other"),
-				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
-				_T(""),
-				wxDateTime(7, wxDateTime::Jun, 2005, 21, 22)
+				R(_T("-rw-r--r--")),
+				R(_T("root other")),
+				0,
+				O(),
+				CDateTime(2005, 6, 7, 21, 22)
 			},
 			DEFAULT
 		});
@@ -182,11 +185,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("08-unix-namedsize file"),
 				335 * 1024 / 10,
-				_T("-rw-r--r--"),
-				_T("root other"),
-				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
-				_T(""),
-				wxDateTime(5, wxDateTime::Oct, calcYear(10, 5), 21, 22)
+				R(_T("-rw-r--r--")),
+				R(_T("root other")),
+				0,
+				O(),
+				CDateTime(calcYear(10, 5), 10, 5, 21, 22)
 			},
 			DEFAULT
 		});
@@ -199,11 +202,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("09-netware dir"),
 				512,
-				_T("d [R----F--]"),
-				_T("supervisor"),
-				CDirentry::flag_dir | CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
-				_T(""),
-				wxDateTime(16, wxDateTime::Jan, calcYear(1, 16), 18, 53)
+				R(_T("d [R----F--]")),
+				R(_T("supervisor")),
+				CDirentry::flag_dir,
+				O(),
+				CDateTime(calcYear(1, 16), 1, 16, 18, 53)
 			},
 			DEFAULT
 		});
@@ -213,11 +216,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("10-netware file"),
 				214059,
-				_T("- [R----F--]"),
-				_T("rhesus"),
-				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
-				_T(""),
-				wxDateTime(20, wxDateTime::Oct, calcYear(10, 20), 15, 27)
+				R(_T("- [R----F--]")),
+				R(_T("rhesus")),
+				0,
+				O(),
+				CDateTime(calcYear(10, 20), 10, 20, 15, 27)
 			},
 			DEFAULT
 		});
@@ -234,11 +237,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("11-netpresenz file"),
 				1392298,
-				_T("-------r--"),
-				_T("1391972"),
-				CDirentry::flag_timestamp_date,
-				_T(""),
-				wxDateTime(22, wxDateTime::Nov, 1995)
+				R(_T("-------r--")),
+				R(_T("1391972")),
+				0,
+				O(),
+				CDateTime(1995, 11, 22)
 			},
 			DEFAULT
 		});
@@ -248,11 +251,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("12-netpresenz dir"),
 				2,
-				_T("drwxrwxr-x"),
-				_T("folder"),
-				CDirentry::flag_dir | CDirentry::flag_timestamp_date,
-				_T(""),
-				wxDateTime(10, wxDateTime::May, 1996)
+				R(_T("drwxrwxr-x")),
+				R(_T("folder")),
+				CDirentry::flag_dir,
+				O(),
+				CDateTime(1996, 5, 10)
 			},
 			DEFAULT
 		});
@@ -263,11 +266,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("13-unix-domain file"),
 				531,
-				_T("-rw-r--r--"),
-				_T("group domain user"),
-				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
-				_T(""),
-				wxDateTime(29, wxDateTime::Jan, calcYear(1, 29), 3, 26)
+				R(_T("-rw-r--r--")),
+				R(_T("group domain user")),
+				0,
+				O(),
+				CDateTime(calcYear(1, 29), 1, 29, 3, 26)
 			},
 			DEFAULT
 		});
@@ -275,7 +278,7 @@ void CDirectoryListingParserTest::InitEntries()
 	// EPLF directory listings
 	// -----------------------
 
-	// See http://cr.yp.to/ftp/list/eplf.html (mirrored at http://filezilla-project.org/specs/eplf.html)
+	// See http://cr.yp.to/ftp/list/eplf.html (mirrored at https://filezilla-project.org/specs/eplf.html)
 
 	wxDateTime utc(1, wxDateTime::Mar, 1996, 22, 15, 3);
 	utc.MakeFromTimezone(wxDateTime::UTC);
@@ -284,11 +287,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("14-eplf file"),
 				280,
-				_T("755"),
-				_T(""),
-				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time | CDirentry::flag_timestamp_seconds,
-				_T(""),
-				utc
+				R(_T("755")),
+				R(),
+				0,
+				O(),
+				CDateTime(utc, CDateTime::seconds)
 			},
 			DEFAULT
 		});
@@ -296,15 +299,15 @@ void CDirectoryListingParserTest::InitEntries()
 	utc = wxDateTime(13, wxDateTime::Feb, 1996, 23, 58, 27);
 	utc.MakeFromTimezone(wxDateTime::UTC);
 	m_entries.push_back((t_entry){
-        	"+i8388621.50690,m824255907,/,\t15-eplf dir",
+			"+i8388621.50690,m824255907,/,\t15-eplf dir",
 			{
 				_T("15-eplf dir"),
 				-1,
-				_T(""),
-				_T(""),
-				CDirentry::flag_dir | CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time | CDirentry::flag_timestamp_seconds,
-				_T(""),
-				utc
+				R(),
+				R(),
+				CDirentry::flag_dir | 0,
+				O(),
+				CDateTime(utc, CDateTime::seconds)
 			},
 			DEFAULT
 		});
@@ -317,11 +320,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("16-dos-dateambiguous dir"),
 				-1,
-				_T(""),
-				_T(""),
-				CDirentry::flag_dir | CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
-				_T(""),
-				wxDateTime(27, wxDateTime::Apr, 2000, 12, 9)
+				R(),
+				R(),
+				CDirentry::flag_dir,
+				O(),
+				CDateTime(2000, 4, 27, 12, 9)
 			},
 			DEFAULT
 		});
@@ -332,11 +335,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("17-dos-dateambiguous file"),
 				589,
-				_T(""),
-				_T(""),
-				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
-				_T(""),
-				wxDateTime(6, wxDateTime::Apr, 2000, 15, 47)
+				R(),
+				R(),
+				0,
+				O(),
+				CDateTime(2000, 4, 6, 15, 47)
 			},
 			DEFAULT
 		});
@@ -346,11 +349,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("18-dos-longyear dir"),
 				-1,
-				_T(""),
-				_T(""),
-				CDirentry::flag_dir | CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
-				_T(""),
-				wxDateTime(2, wxDateTime::Sep, 2002, 18, 48)
+				R(),
+				R(),
+				CDirentry::flag_dir,
+				O(),
+				CDateTime(2002, 9, 2, 18, 48)
 			},
 			DEFAULT
 		});
@@ -360,11 +363,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("19-dos-longyear file"),
 				9730,
-				_T(""),
-				_T(""),
-				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
-				_T(""),
-				wxDateTime(2, wxDateTime::Sep, 2002, 19, 6)
+				R(),
+				R(),
+				0,
+				O(),
+				CDateTime(2002, 9, 2, 19, 6)
 			},
 			DEFAULT
 		});
@@ -377,11 +380,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("20-unix-numerical file"),
 				12345,
-				_T("0100644"),
-				_T("500 101"),
-				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time | CDirentry::flag_timestamp_seconds,
-				_T(""),
-				utc
+				R(_T("0100644")),
+				R(_T("500 101")),
+				0,
+				O(),
+				CDateTime(utc, CDateTime::seconds)
 			},
 			DEFAULT
 		});
@@ -394,11 +397,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("21-vshell-old file"),
 				206876,
-				_T(""),
-				_T(""),
-				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
-				_T(""),
-				wxDateTime(4, wxDateTime::Apr, 2000, 21, 6)
+				R(),
+				R(),
+				0,
+				O(),
+				CDateTime(2000, 4, 4, 21, 6)
 			},
 			DEFAULT
 		});
@@ -408,11 +411,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("22-vshell-old dir"),
 				0,
-				_T(""),
-				_T(""),
-				CDirentry::flag_dir | CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
-				_T(""),
-				wxDateTime(12, wxDateTime::Dec, 2002, 2, 13)
+				R(),
+				R(),
+				CDirentry::flag_dir,
+				O(),
+				CDateTime(2002, 12, 12, 2, 13)
 			},
 			DEFAULT
 		});
@@ -424,11 +427,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("23-vshell-new file"),
 				9,
-				_T("-rwxr-xr-x"),
-				_T("user group"),
-				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
-				_T(""),
-				wxDateTime(8, wxDateTime::Oct, 2002, 9, 47)
+				R(_T("-rwxr-xr-x")),
+				R(_T("user group")),
+				0,
+				O(),
+				CDateTime(2002, 10, 8, 9, 47)
 			},
 			DEFAULT
 		});
@@ -442,11 +445,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("24-os2 file"),
 				36611,
-				_T(""),
-				_T(""),
-				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
-				_T(""),
-				wxDateTime(23, wxDateTime::Apr, 2003, 10, 57)
+				R(),
+				R(),
+				0,
+				O(),
+				CDateTime(2003, 4, 23, 10, 57)
 			},
 			DEFAULT
 		});
@@ -456,11 +459,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("25-os2 file"),
 				1123,
-				_T(""),
-				_T(""),
-				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
-				_T(""),
-				wxDateTime(14, wxDateTime::Jul, 1999, 12, 37)
+				R(),
+				R(),
+				0,
+				O(),
+				CDateTime(1999, 7, 14, 12, 37)
 			},
 			DEFAULT
 		});
@@ -471,11 +474,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("26-os2 dir"),
 				0,
-				_T(""),
-				_T(""),
-				CDirentry::flag_dir | CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
-				_T(""),
-				wxDateTime(11, wxDateTime::Feb, 2003, 16, 15)
+				R(),
+				R(),
+				CDirentry::flag_dir,
+				O(),
+				CDateTime(2003, 2, 11, 16, 15)
 			},
 			DEFAULT
 		});
@@ -486,11 +489,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("27-os2 dir"),
 				1123,
-				_T(""),
-				_T(""),
-				CDirentry::flag_dir | CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
-				_T(""),
-				wxDateTime(5, wxDateTime::Oct, 2000, 23, 38)
+				R(),
+				R(),
+				CDirentry::flag_dir,
+				O(),
+				CDateTime(2000, 10, 5, 23, 38)
 			},
 			DEFAULT
 		});
@@ -503,11 +506,25 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("28-datetest-ger dir"),
 				2235,
-				_T("dr-xr-xr-x"),
-				_T("root other"),
-				CDirentry::flag_dir | CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
-				_T(""),
-				wxDateTime(26, wxDateTime::Jul, calcYear(7, 26), 20, 10)
+				R(_T("dr-xr-xr-x")),
+				R(_T("root other")),
+				CDirentry::flag_dir,
+				O(),
+				CDateTime(calcYear(7, 26), 7, 26, 20, 10)
+			},
+			DEFAULT
+		});
+
+	m_entries.push_back((t_entry){
+			"dr-xr-xr-x   2 root     other      2235 szept 26 20:10 28b-datetest-hungarian dir",
+			{
+				_T("28b-datetest-hungarian dir"),
+				2235,
+				R(_T("dr-xr-xr-x")),
+				R(_T("root other")),
+				CDirentry::flag_dir,
+				O(),
+				CDateTime(calcYear(9, 26), 9, 26, 20, 10)
 			},
 			DEFAULT
 		});
@@ -517,11 +534,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("29-datetest-ger file"),
 				2235,
-				_T("-r-xr-xr-x"),
-				_T("root other"),
-				CDirentry::flag_timestamp_date,
-				_T(""),
-				wxDateTime(2, wxDateTime::Oct, 2003)
+				R(_T("-r-xr-xr-x")),
+				R(_T("root other")),
+				0,
+				O(),
+				CDateTime(2003, 10, 2)
 			},
 			DEFAULT
 		});
@@ -531,11 +548,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("30-datetest file"),
 				2235,
-				_T("-r-xr-xr-x"),
-				_T("root other"),
-				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
-				_T(""),
-				wxDateTime(12, wxDateTime::Oct, 1999, 17, 12)
+				R(_T("-r-xr-xr-x")),
+				R(_T("root other")),
+				0,
+				O(),
+				CDateTime(1999, 10, 12, 17, 12)
 			},
 			DEFAULT
 		});
@@ -545,11 +562,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("31-datetest file"),
 				2235,
-				_T("-r-xr-xr-x"),
-				_T("root other"),
-				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
-				_T(""),
-				wxDateTime(24, wxDateTime::Apr, 2003, 17, 12)
+				R(_T("-r-xr-xr-x")),
+				R(_T("root other")),
+				0,
+				O(),
+				CDateTime(2003, 4, 24, 17, 12)
 			},
 			DEFAULT
 		});
@@ -562,11 +579,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("32-datatest-japanese file"),
 				8473,
-				_T("-rw-r--r--"),
-				_T("root sys"),
-				CDirentry::flag_timestamp_date,
-				_T(""),
-				wxDateTime(18, wxDateTime::Apr, 2003)
+				R(_T("-rw-r--r--")),
+				R(_T("root sys")),
+				0,
+				O(),
+				CDateTime(2003, 4, 18)
 			},
 			DEFAULT
 		});
@@ -578,11 +595,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("33-asian date file"),
 				0,
-				_T("-rwxrwxrwx"),
-				_T("root staff"),
-				CDirentry::flag_timestamp_date,
-				_T(""),
-				wxDateTime(20, wxDateTime::Mar, 2003)
+				R(_T("-rwxrwxrwx")),
+				R(_T("root staff")),
+				0,
+				O(),
+				CDateTime(2003, 3, 20)
 			},
 			DEFAULT
 		});
@@ -592,11 +609,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("34-asian date file"),
 				2096,
-				_T("-r--r--r--"),
-				_T("root root"),
-				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
-				_T(""),
-				wxDateTime(17, wxDateTime::Aug, calcYear(8, 17), 8, 52)
+				R(_T("-r--r--r--")),
+				R(_T("root root")),
+				0,
+				O(),
+				CDateTime(calcYear(8, 17), 8, 17, 8, 52)
 			},
 			DEFAULT
 		});
@@ -606,11 +623,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("35-dotted-date file"),
 				96,
-				_T("-r-xr-xr-x"),
-				_T("root root"),
-				CDirentry::flag_timestamp_date,
-				_T(""),
-				wxDateTime(15, wxDateTime::Jul, 2004)
+				R(_T("-r-xr-xr-x")),
+				R(_T("root root")),
+				0,
+				O(),
+				CDateTime(2004, 7, 15)
 			},
 			DEFAULT
 		});
@@ -623,11 +640,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("36-vms-dir"),
 				512,
-				_T("RWE,RWE,RE,RE"),
-				_T("root,root"),
-				CDirentry::flag_dir | CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
-				_T(""),
-				wxDateTime(19, wxDateTime::Nov, 2001, 21, 41)
+				R(_T("RWE,RWE,RE,RE")),
+				R(_T("root,root")),
+				CDirentry::flag_dir,
+				O(),
+				CDateTime(2001, 11, 19, 21, 41)
 			},
 			DEFAULT
 		});
@@ -638,11 +655,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("37-vms-file;1"),
 				79360,
-				_T(""),
-				_T(""),
-				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time | CDirentry::flag_timestamp_seconds,
-				_T(""),
-				wxDateTime(2, wxDateTime::Jul, 2003, 10, 30, 13)
+				R(),
+				R(),
+				0,
+				O(),
+				CDateTime(2003, 7, 2, 10, 30, 13)
 			},
 			DEFAULT
 		});
@@ -653,11 +670,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("38-vms-notime-file;1"),
 				1024,
-				_T("RWED,RWED,RE,"),
-				_T("IV2_XXX"),
-				CDirentry::flag_timestamp_date,
-				_T(""),
-				wxDateTime(7, wxDateTime::Jan, 2000)
+				R(_T("RWED,RWED,RE,")),
+				R(_T("IV2_XXX")),
+				0,
+				O(),
+				CDateTime(2000, 1, 7)
 			},
 			DEFAULT
 		});
@@ -668,11 +685,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("39-vms-notime-file;1"),
 				3072,
-				_T("RWED,RWED,RE,"),
-				_T("PRONAS"),
-				CDirentry::flag_timestamp_date,
-				_T(""),
-				wxDateTime(15, wxDateTime::Jul, 2002)
+				R(_T("RWED,RWED,RE,")),
+				R(_T("PRONAS")),
+				0,
+				O(),
+				CDateTime(2002, 7, 15)
 			},
 			DEFAULT
 		});
@@ -682,11 +699,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("40-vms-multiline-file;1"),
 				87436288,
-				_T("RWED,RWED,RE,"),
-				_T("FTP_CLIENT,SCOT"),
-				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time | CDirentry::flag_timestamp_seconds,
-				_T(""),
-				wxDateTime(24, wxDateTime::Apr, 2003, 8, 16, 15)
+				R(_T("RWED,RWED,RE,")),
+				R(_T("FTP_CLIENT,SCOT")),
+				0,
+				O(),
+				CDateTime(2003, 4, 24, 8, 16, 15)
 			},
 			DEFAULT
 		});
@@ -696,11 +713,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("41-vms-multiline-file;1"),
 				5120,
-				_T("RWED,RWED,RE,"),
-				_T("FTP_CLIENT,SCOT"),
-				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time | CDirentry::flag_timestamp_seconds,
-				_T(""),
-				wxDateTime(2, wxDateTime::Jul, 2003, 10, 30, 8)
+				R(_T("RWED,RWED,RE,")),
+				R(_T("FTP_CLIENT,SCOT")),
+				0,
+				O(),
+				CDateTime(2003, 7, 2, 10, 30, 8)
 			},
 			DEFAULT
 		});
@@ -711,11 +728,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("42-vms-alternate-field-order-file;1"),
 				512,
-				_T("RWE,RWE,RE,"),
-				_T("SUMMARY"),
-				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
-				_T(""),
-				wxDateTime(2, wxDateTime::Aug, 2006, 13, 5)
+				R(_T("RWE,RWE,RE,")),
+				R(_T("SUMMARY")),
+				0,
+				O(),
+				CDateTime(2006, 8, 2, 13, 5)
 			},
 			DEFAULT
 		});
@@ -725,11 +742,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("43-vms-alternate-field-order-file;1"),
 				3229696,
-				_T("RWED,RWED,R,"),
-				_T(""),
-				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time | CDirentry::flag_timestamp_seconds,
-				_T(""),
-				wxDateTime(17, wxDateTime::Jun, 1994, 17, 25, 37)
+				R(_T("RWED,RWED,R,")),
+				R(),
+				0,
+				O(),
+				CDateTime(1994, 6, 17, 17, 25, 37)
 			},
 			DEFAULT
 		});
@@ -743,11 +760,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("44-ibm-as400 dir"),
 				77824,
-				_T(""),
-				_T("QSYS"),
-				CDirentry::flag_dir | CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time | CDirentry::flag_timestamp_seconds,
-				_T(""),
-				wxDateTime(23, wxDateTime::Feb, 2000, 15, 9, 55)
+				R(),
+				R(_T("QSYS")),
+				CDirentry::flag_dir | 0,
+				O(),
+				CDateTime(2000, 2, 23, 15, 9, 55)
 			},
 			DEFAULT
 		});
@@ -757,11 +774,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("45-ibm-as400-date file"),
 				77824,
-				_T(""),
-				_T("QSYS"),
-				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time | CDirentry::flag_timestamp_seconds,
-				_T(""),
-				wxDateTime(23, wxDateTime::Feb, 2000, 15, 9, 55)
+				R(),
+				R(_T("QSYS")),
+				0,
+				O(),
+				CDateTime(2000, 2, 23, 15, 9, 55)
 			},
 			DEFAULT
 		});
@@ -772,11 +789,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("46-unix-concatsize file"),
 				123456,
-				_T("-r-xr-xr-x"),
-				_T("longowner longgroup"),
-				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
-				_T(""),
-				wxDateTime(12, wxDateTime::Feb, calcYear(2, 12), 17, 20)
+				R(_T("-r-xr-xr-x")),
+				R(_T("longowner longgroup")),
+				0,
+				O(),
+				CDateTime(calcYear(2, 12), 2, 12, 17, 20)
 			},
 			DEFAULT
 		});
@@ -787,11 +804,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("47_unix_shortdatemonth file"),
 				4512,
-				_T("-r-xr-xr-x"),
-				_T("owner group"),
-				CDirentry::flag_timestamp_date,
-				_T(""),
-				wxDateTime(1, wxDateTime::Jun, 1999)
+				R(_T("-r-xr-xr-x")),
+				R(_T("owner group")),
+				0,
+				O(),
+				CDateTime(1999, 6, 1)
 			},
 			DEFAULT
 		});
@@ -802,11 +819,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("48-nortel-wfftp-file"),
 				1014196,
-				_T(""),
-				_T(""),
-				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time | CDirentry::flag_timestamp_seconds,
-				_T(""),
-				wxDateTime(3, wxDateTime::Jun, 2004, 10, 20, 3)
+				R(),
+				R(),
+				0,
+				O(),
+				CDateTime(2004, 6, 3, 10, 20, 3)
 			},
 			DEFAULT
 		});
@@ -817,11 +834,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("49-nortel-vxworks dir"),
 				2048,
-				_T(""),
-				_T(""),
-				CDirentry::flag_dir | CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time | CDirentry::flag_timestamp_seconds,
-				_T(""),
-				wxDateTime(28, wxDateTime::Feb, 1998, 5, 23, 30)
+				R(),
+				R(),
+				CDirentry::flag_dir | 0,
+				O(),
+				CDateTime(1998, 2, 28, 5, 23, 30)
 			},
 			DEFAULT
 		});
@@ -832,11 +849,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("50-conent file"),
 				1294495,
-				_T("-C--E-----FTP"),
-				_T("B BCC3I1 7670"),
-				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
-				_T(""),
-				wxDateTime(13, wxDateTime::Jan, calcYear(1, 13), 7, 42)
+				R(_T("-C--E-----FTP")),
+				R(_T("B BCC3I1 7670")),
+				0,
+				O(),
+				CDateTime(calcYear(1, 13), 1, 13, 7, 42)
 			},
 			DEFAULT
 		});
@@ -848,11 +865,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("51-OS-9 dir"),
 				85920,
-				_T("d-ewrewr"),
-				_T("20.20"),
-				CDirentry::flag_dir | CDirentry::flag_timestamp_date,
-				_T(""),
-				wxDateTime(29, wxDateTime::Mar, 2007)
+				R(_T("d-ewrewr")),
+				R(_T("20.20")),
+				CDirentry::flag_dir,
+				O(),
+				CDateTime(2007, 3, 29)
 			},
 			DEFAULT
 		});
@@ -863,11 +880,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("52-swapped-daymonth dir"),
 				512,
-				_T("drwxr-xr-x"),
-				_T("user group"),
-				CDirentry::flag_dir | CDirentry::flag_timestamp_date,
-				_T(""),
-				wxDateTime(1, wxDateTime::Oct, 2004)
+				R(_T("drwxr-xr-x")),
+				R(_T("user group")),
+				CDirentry::flag_dir,
+				O(),
+				CDateTime(2004, 10, 1)
 			},
 			DEFAULT
 		});
@@ -877,11 +894,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("53-noownergroup file"),
 				12,
-				_T("-r--r--r--"),
-				_T(""),
-				CDirentry::flag_timestamp_date,
-				_T(""),
-				wxDateTime(11, wxDateTime::Nov, 2005)
+				R(_T("-r--r--r--")),
+				R(),
+				0,
+				O(),
+				CDateTime(2005, 11, 11)
 			},
 			DEFAULT
 		});
@@ -892,11 +909,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("54-asian date year first dir"),
 				512,
-				_T("drwxr-xr-x"),
-				_T("root sys"),
-				CDirentry::flag_dir | CDirentry::flag_timestamp_date,
-				_T(""),
-				wxDateTime(6, wxDateTime::Jan, 2005)
+				R(_T("drwxr-xr-x")),
+				R(_T("root sys")),
+				CDirentry::flag_dir,
+				O(),
+				CDateTime(2005, 1, 6)
 			},
 			DEFAULT
 		});
@@ -907,11 +924,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("55-AS400.FILE"),
 				36864,
-				_T(""),
-				_T("QPGMR"),
-				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time | CDirentry::flag_timestamp_seconds,
-				_T(""),
-				wxDateTime(18, wxDateTime::Sep, 2006, 14, 21, 26)
+				R(),
+				R(_T("QPGMR")),
+				0,
+				O(),
+				CDateTime(2006, 9, 18, 14, 21, 26)
 			},
 			DEFAULT
 		});
@@ -922,11 +939,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("56-VMS-complex-size;1"),
 				2048,
-				_T(""),
-				_T(""),
-				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time | CDirentry::flag_timestamp_seconds,
-				_T(""),
-				wxDateTime(23, wxDateTime::Sep, 2005, 14, 57, 7)
+				R(),
+				R(),
+				0,
+				O(),
+				CDateTime(2005, 9, 23, 14, 57, 7)
 			},
 			DEFAULT
 		});
@@ -937,11 +954,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("57-HP_NonStop"),
 				528,
-				_T("\"oooo\""),
-				_T("255, 0"),
-				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time | CDirentry::flag_timestamp_seconds,
-				_T(""),
-				wxDateTime(6, wxDateTime::Apr, 2007, 14, 21, 18)
+				R(_T("\"oooo\"")),
+				R(_T("255, 0")),
+				0,
+				O(),
+				CDateTime(2007, 4, 6, 14, 21, 18)
 			},
 			HPNONSTOP
 		});
@@ -952,11 +969,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("58-HP_NonStop"),
 				528,
-				_T("\"oooo\""),
-				_T("255,255"),
-				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time | CDirentry::flag_timestamp_seconds,
-				_T(""),
-				wxDateTime(6, wxDateTime::Apr, 2007, 14, 21, 18)
+				R(_T("\"oooo\"")),
+				R(_T("255,255")),
+				0,
+				O(),
+				CDateTime(2007, 4, 6, 14, 21, 18)
 			},
 			HPNONSTOP
 		});
@@ -967,11 +984,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("59-localized-date-dir"),
 				1024,
-				_T("drwxr-xr-x"),
-				_T("user sys"),
-				CDirentry::flag_dir | CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
-				_T(""),
-				wxDateTime(30, wxDateTime::Jan, calcYear(1, 30), 12, 40)
+				R(_T("drwxr-xr-x")),
+				R(_T("user sys")),
+				CDirentry::flag_dir,
+				O(),
+				CDateTime(calcYear(1, 30), 1, 30, 12, 40)
 			},
 			DEFAULT
 		});
@@ -990,11 +1007,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("60-MVS.FILE"),
 				100,
-				_T(""),
-				_T(""),
-				CDirentry::flag_timestamp_date,
-				_T(""),
-				wxDateTime(21, wxDateTime::May, 2003)
+				R(),
+				R(),
+				0,
+				O(),
+				CDateTime(2003, 5, 21)
 		},
 		DEFAULT
 	});
@@ -1004,11 +1021,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("61-MVS.DATASET"),
 				-1,
-				_T(""),
-				_T(""),
-				CDirentry::flag_dir | CDirentry::flag_timestamp_date,
-				_T(""),
-				wxDateTime(04, wxDateTime::Mar, 2004)
+				R(),
+				R(),
+				CDirentry::flag_dir,
+				O(),
+				CDateTime(2004, 3, 04)
 		},
 		DEFAULT
 	});
@@ -1018,11 +1035,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("62-MVS-NONEDATE.DATASET"),
 				-1,
-				_T(""),
-				_T(""),
+				R(),
+				R(),
 				CDirentry::flag_dir,
-				_T(""),
-				wxDateTime()
+				O(),
+				CDateTime()
 		},
 		DEFAULT
 	});
@@ -1032,11 +1049,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("63-MVS.DATASET"),
 				-1,
-				_T(""),
-				_T(""),
-				CDirentry::flag_dir | CDirentry::flag_timestamp_date,
-				_T(""),
-				wxDateTime(6, wxDateTime::Jun, 2005)
+				R(),
+				R(),
+				CDirentry::flag_dir,
+				O(),
+				CDateTime(2005, 6, 6)
 		},
 		DEFAULT
 	});
@@ -1046,32 +1063,32 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("64-mvs-file"),
 				-1,
-				_T(""),
-				_T(""),
+				R(),
+				R(),
 				0,
-				_T(""),
-				wxDateTime()
+				O(),
+				CDateTime()
 		},
 		DEFAULT
 	});
 
 	// MVS Dataset members
-	// 
+	//
 	// As common with IBM misdesign, multiple styles exist.
-	
+
 	// Speciality: Some members have no attributes at all.
-	// Requires servertype to be MVS or it won't be parsed, as 
+	// Requires servertype to be MVS or it won't be parsed, as
 	// it would conflict with lots of other servers.
 	m_entries.push_back((t_entry){
 			"65-MVS-PDS-MEMBER",
 			{
 				_T("65-MVS-PDS-MEMBER"),
 				-1,
-				_T(""),
-				_T(""),
+				R(),
+				R(),
 				0,
-				_T(""),
-				wxDateTime()
+				O(),
+				CDateTime()
 		},
 		MVS
 	});
@@ -1082,11 +1099,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("66-MVSPDSMEMBER"),
 				128,
-				_T(""),
-				_T(""),
-				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
-				_T(""),
-				wxDateTime(22, wxDateTime::Jun, 2004, 16, 32)
+				R(),
+				R(),
+				0,
+				O(),
+				CDateTime(2004, 6, 22, 16, 32)
 		},
 		MVS
 	});
@@ -1097,11 +1114,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("67-MVSPDSMEMBER2"),
 				45832,
-				_T(""),
-				_T(""),
+				R(),
+				R(),
 				0,
-				_T(""),
-				wxDateTime()
+				O(),
+				CDateTime()
 		},
 		MVS
 	});
@@ -1111,11 +1128,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("68-MVSPDSMEMBER3"),
 				45832,
-				_T(""),
-				_T(""),
+				R(),
+				R(),
 				0,
-				_T(""),
-				wxDateTime()
+				O(),
+				CDateTime()
 			},
 			MVS
 		});
@@ -1126,18 +1143,18 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("69-SOME.FILE"),
 				-1,
-				_T(""),
-				_T(""),
+				R(),
+				R(),
 				0,
-				_T(""),
-				wxDateTime()
+				O(),
+				CDateTime()
 			},
 			MVS
 		});
 
 	// z/VM, another IBM abomination. Description by Alexandre Charbey
 	// Requires type set to ZVM or it cannot be parsed.
-	// 
+	//
 	// 70-ZVMFILE
 	//   is a filename
 	// TRACE
@@ -1158,11 +1175,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("70-ZVMFILE.TRACE"),
 				6955,
-				_T(""),
-				_T("060191"),
-				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time | CDirentry::flag_timestamp_seconds,
-				_T(""),
-				wxDateTime(4, wxDateTime::Oct, 2005, 15, 28, 42)
+				R(),
+				R(_T("060191")),
+				0,
+				O(),
+				CDateTime(2005, 10, 4, 15, 28, 42)
 			},
 			ZVM
 		});
@@ -1172,11 +1189,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("71-unix-dateless"),
 				512,
-				_T("drwxr-xr-x"),
-				_T("slopri devlab"),
+				R(_T("drwxr-xr-x")),
+				R(_T("slopri devlab")),
 				CDirentry::flag_dir,
-				_T(""),
-				wxDateTime()
+				O(),
+				CDateTime()
 			},
 			DEFAULT
 		});
@@ -1188,11 +1205,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("72-MLSD-file"),
 				1234,
-				_T(""),
-				_T(""),
-				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time | CDirentry::flag_timestamp_seconds,
-				_T(""),
-				utc
+				R(),
+				R(),
+				0,
+				O(),
+				CDateTime(utc, CDateTime::seconds)
 			},
 			DEFAULT
 		});
@@ -1206,11 +1223,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("73-MSV-TAPE.FILE"),
 				-1,
-				_T(""),
-				_T(""),
+				R(),
+				R(),
 				0,
-				_T(""),
-				wxDateTime()
+				O(),
+				CDateTime()
 			},
 			MVS
 		});
@@ -1220,11 +1237,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("74-MLSD-whitespace trailing\t "),
 				-1,
-				_T(""),
-				_T(""),
+				R(),
+				R(),
 				0,
-				_T(""),
-				wxDateTime()
+				O(),
+				CDateTime()
 			},
 			DEFAULT
 		});
@@ -1234,11 +1251,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("\t 75-MLSD-whitespace leading"),
 				-1,
-				_T(""),
-				_T(""),
+				R(),
+				R(),
 				0,
-				_T(""),
-				wxDateTime()
+				O(),
+				CDateTime()
 			},
 			DEFAULT
 		});
@@ -1250,28 +1267,42 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("75 MLSD file with empty permissions"),
 				65718921,
-				_T("00"),
-				_T("1179 1179"),
-				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time | CDirentry::flag_timestamp_seconds,
-				_T(""),
-				utc
+				R(_T("00")),
+				R(_T("1179 1179")),
+				0,
+				O(),
+				CDateTime(utc, CDateTime::seconds)
 			},
 			DEFAULT
-		});	
+		});
 
 		m_entries.push_back((t_entry){
 			"type=OS.unix=slink:/foo; 76 MLSD symlink",
 			{
 				_T("76 MLSD symlink"),
 				-1,
-				_T(""),
-				_T(""),
+				R(),
+				R(),
 				CDirentry::flag_dir | CDirentry::flag_link,
-				_T("/foo"),
-				wxDateTime()
+				O(_T("/foo")),
+				CDateTime()
 			},
 			DEFAULT
-		});	
+		});
+
+		m_entries.push_back((t_entry){
+			"type=OS.UNIX=symlink; 76b MLSD symlink",
+			{
+				_T("76b MLSD symlink"),
+				-1,
+				R(),
+				R(),
+				CDirentry::flag_dir | CDirentry::flag_link,
+				O(),
+				CDateTime()
+			},
+			DEFAULT
+		});
 
 		// Old ietf draft for MLST earlier than mlst-07 has no trailing semicolon after facts
 		m_entries.push_back((t_entry){
@@ -1279,11 +1310,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("77 MLSD file no trailing semicolon after facts < mlst-07"),
 				-1,
-				_T(""),
-				_T(""),
+				R(),
+				R(),
 				0,
-				_T(""),
-				wxDateTime()
+				O(),
+				CDateTime()
 			},
 			DEFAULT
 		});
@@ -1293,11 +1324,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("77 MLSD symlink notarget"),
 				-1,
-				_T(""),
-				_T(""),
+				R(),
+				R(),
 				CDirentry::flag_dir | CDirentry::flag_link,
-				_T(""),
-				wxDateTime()
+				O(),
+				CDateTime()
 			},
 			DEFAULT
 		});
@@ -1309,11 +1340,11 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("adsl TV 2009-07-22 08-25-10 78 mlsd file that can get parsed as unix.file"),
 				1365694195,
-				_T(""),
-				_T(""),
-				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time | CDirentry::flag_timestamp_seconds,
-				_T(""),
-				utc
+				R(),
+				R(),
+				0,
+				O(),
+				CDateTime(utc, CDateTime::seconds)
 			},
 			DEFAULT
 		});
@@ -1325,45 +1356,45 @@ void CDirectoryListingParserTest::InitEntries()
 			{
 				_T("79-MVS.FILE"),
 				100,
-				_T(""),
-				_T(""),
-				CDirentry::flag_timestamp_date,
-				_T(""),
-				wxDateTime(21, wxDateTime::May, 2003)
+				R(),
+				R(),
+				0,
+				O(),
+				CDateTime(2003, 5, 21)
 		},
 		DEFAULT
 	});
 
 	// MVS entry with a large number of used blocks:
-	// http://forum.filezilla-project.org/viewtopic.php?t=21667
+	// https://forum.filezilla-project.org/viewtopic.php?t=21667
 	// Volume Unit    Referred Ext Used Recfm Lrecl BlkSz Dsorg Dsname
 	m_entries.push_back((t_entry){
 			"GISBWI 3390   2011/08/25  2 ++++  FB     904 18080  PS  80-MVS.FILE",
 			{
 				_T("80-MVS.FILE"),
 				100,
-				_T(""),
-				_T(""),
-				CDirentry::flag_timestamp_date,
-				_T(""),
-				wxDateTime(25, wxDateTime::Aug, 2011)
+				R(),
+				R(),
+				0,
+				O(),
+				CDateTime(2011, 8, 25)
 		},
 		DEFAULT
 	});
 
 	// MVS entry with PO-E Dsorg indicating direrctory. See
-	// http://forum.filezilla-project.org/viewtopic.php?t=19374 for reference.
+	// https://forum.filezilla-project.org/viewtopic.php?t=19374 for reference.
 	// Volume Unit    Referred Ext Used Recfm Lrecl BlkSz Dsorg Dsname
 	m_entries.push_back((t_entry){
 			"WYOSPT 3420   2003/05/21  1 3 U 6447    6447  PO-E 81-MVS.DIR",
 			{
 				_T("81-MVS.DIR"),
 				-1,
-				_T(""),
-				_T(""),
-				CDirentry::flag_dir | CDirentry::flag_timestamp_date,
-				_T(""),
-				wxDateTime(21, wxDateTime::May, 2003)
+				R(),
+				R(),
+				CDirentry::flag_dir,
+				O(),
+				CDateTime(2003, 5, 21)
 		},
 		DEFAULT
 	});
@@ -1380,7 +1411,7 @@ void CDirectoryListingParserTest::InitEntries()
 */
 
 	// Fix line endings
-	for (std::vector<t_entry>::iterator iter = m_entries.begin(); iter != m_entries.end(); iter++)
+	for (auto iter = m_entries.begin(); iter != m_entries.end(); iter++)
 		iter->data += "\r\n";
 }
 
